unit Add2dfnc;

(*
Copyright 1999-2011 Greg VanMullem
This program is distributed under the terms of the GNU General Public License Version 3

This file is part of MathGV.

MathGV is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

MathGV is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MathGV.  If not, see <http://www.gnu.org/licenses/>.
*)

interface

uses WinTypes, WinProcs, Classes, Graphics, Forms, Controls, Buttons,
  Spin, dialogs, sysutils, Clsample, ComnDlgs, MathCalc, MathFunc,
  Func2D, editmath, CentrDlg, Grids, ValueLst, 
  StdCtrls, ComCtrls, ExtCtrls;

type
  TAdd2DFuncDlg = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    HelpBtn: TBitBtn;
    PageControl1: TPageControl;
    FuncDefTabSheet: TTabSheet;
    SpecialValTabSheet: TTabSheet;
    FuncStrBox: TGroupBox;
    Label1: TLabel;
    Label3: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    Label2: TLabel;
    Label4: TLabel;
    EUserFuncStr: TEdit;
    ELoBoundStr: TEditMathConst;
    EHiBoundStr: TEditMathConst;
    EAsymFixUp: TCheckBox;
    EExtraUndefAccuracy: TCheckBox;
    PropertiesBox: TGroupBox;
    EFuncColorSample: TColorSample;
    Label5: TLabel;
    LineWidthLabel: TLabel;
    ColorButton1: TButton;
    ELineWidth: TScrollBar;
    SpecialValuesGrid: TStringGrid;
    LeftSideGroupBox: TGroupBox;
    AtValueGroupBox: TGroupBox;
    ELeftBehavior: TComboBox;
    RightSideGroupBox: TGroupBox;
    ERightBehavior: TComboBox;
    Label10: TLabel;
    Label11: TLabel;
    Label12: TLabel;
    EAtValueAutoDetect: TRadioButton;
    EAtValueUndefined: TRadioButton;
    AddButton: TButton;
    InsertButton: TButton;
    DeleteButton: TButton;
    MoveUpButton: TButton;
    MoveDownButton: TButton;
    ShowValuesButton: TButton;
    Label6: TLabel;
    Label9: TLabel;
    Label13: TLabel;
    Label14: TLabel;
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure ELineWidthChange(Sender: TObject);
    procedure ColorButton1Click(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure SpecialValuesGridSelectCell(Sender: TObject; ACol,
      ARow: Integer; var CanSelect: Boolean);
    procedure SpecialValuesGridExit(Sender: TObject);
    procedure ELeftChange(Sender: TObject);
    procedure ERightChange(Sender: TObject);
    procedure EAtValueClick(Sender: TObject);
    procedure AddButtonClick(Sender: TObject);
    procedure InsertButtonClick(Sender: TObject);
    procedure DeleteButtonClick(Sender: TObject);
    procedure MoveUpButtonClick(Sender: TObject);
    procedure MoveDownButtonClick(Sender: TObject);
    procedure SpecialValuesGridKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure ShowValuesButtonClick(Sender: TObject);
  private
    { Private declarations }
    SettingAttribFields : boolean;
    LastGridRow, LastGridCol : integer;
    SpecialValueFunc : TMathFunction;
    MathConstant : TMathFunction;
    FFuncCalc : TMath2DFunc;
    function VerifyTheFields : boolean;
    procedure SetFocusGridEditCell(TheCol, TheRow : integer);
    procedure CalcSetLeftSideCell(TheRowIndex : integer);
    procedure CalcSetAtPointCell(TheRowIndex : integer);
    procedure CalcSetRightSideCell(TheRowIndex : integer);
    function VerifyCalcValueRow(TheRowIndex : integer) : boolean;
    procedure DoAddValueRow;
    procedure DoInsertValueRow;
    procedure DoDeleteValueRow;
    procedure DoSwapValueRows(TheRowNdx1, TheRowNdx2 : integer);
    procedure DoMoveUpValueRow;
    procedure DoMoveDownValueRow;
    procedure SetLeftSideAttribs(LeftSideBehave : TFromSideBehave);
    procedure SetAtPointAttribs(AtPointBehave : TAtPointBehave);
    procedure SetRightSideAttribs(RightSideBehave : TFromSideBehave);
    procedure SetCurrRowAttribs(RowNdx : integer);
    procedure AddBlankRowToEndIfNeeded;
  public
    { Public declarations }
    IsGetNewFunction : boolean;
    procedure SetTheFields(TheFunc2DObj : TFunc2DObj);
    procedure GetTheFields(TheFunc2DObj : TFunc2DObj);
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  end;

function GetNew2DDlg(TheFunc2DObj : TFunc2DObj; TheCenterOnForm : TForm): boolean;
function Modify2DDlg(TheFunc2DObj : TFunc2DObj; TheCenterOnForm : TForm) : boolean;

implementation

const
  nxFirst     = 0;
  nxSecond    = 1;
  nxEnd       = 2;
  nxFunction  = 3;
  nxLeftSide  = 4;
  nxAtPoint   = 5;
  nxRightSide = 6;

{$R *.DFM}

{---------------------------------------------------------------------}

function GetNew2DDlg(TheFunc2DObj : TFunc2DObj; TheCenterOnForm : TForm): boolean;
var
  Add2DFuncDlg: TAdd2DFuncDlg;
begin {function GetNew2DDlg}
  Result := false;
  Add2DFuncDlg := TAdd2DFuncDlg.Create(Application);
  try
    Add2DFuncDlg.PageControl1.ActivePageIndex := 0;
    SetDlgDefaultPos(Add2DFuncDlg, TheCenterOnForm);
    Add2DFuncDlg.IsGetNewFunction := true;
    Add2DFuncDlg.Caption := 'Add 2D Function';
    Add2DFuncDlg.SetTheFields(TheFunc2DObj);
    if Add2DFuncDlg.ShowModal = IDOK then
      begin
        Result := true;
        Add2DFuncDlg.GetTheFields(TheFunc2DObj);
        TheFunc2DObj.DidFieldsChange := true;
      end { if Result = true then}
  finally
    Add2DFuncDlg.Free;
  end; {try...Finally}
end; {function GetNew3DDlg}

{---------------------------------------------------------------------}

function Modify2DDlg(TheFunc2DObj : TFunc2DObj; TheCenterOnForm : TForm) : boolean;
var
  Add2DFuncDlg: TAdd2DFuncDlg;
begin {function Modify2DDlg}
  Result := false;
  Add2DFuncDlg := TAdd2DFuncDlg.Create(Application);
  try
    Add2DFuncDlg.PageControl1.ActivePageIndex := 0;
    SetDlgDefaultPos(Add2DFuncDlg, TheCenterOnForm);
    Add2DFuncDlg.IsGetNewFunction := false;
    Add2DFuncDlg.Caption := 'Modify 2D Function';
    Add2DFuncDlg.SetTheFields(TheFunc2DObj);
    if Add2DFuncDlg.ShowModal = IDOK then
      begin
        Result := true;
        Add2DFuncDlg.GetTheFields(TheFunc2DObj);
        TheFunc2DObj.DidFieldsChange := true;
      end { if Result = true then}
  finally
    Add2DFuncDlg.Free;
  end; {try...Finally}
end; {function Modify2DDlg}

{----------------------------------------------------------------------}

function FromSideToString(FromSideBehave : TFromSideBehave) : string;
begin {function FromSideToString}
  case FromSideBehave of
    sbAutomatic : Result := 'Auto';
    sbExtraAccuracy : Result := 'Extr';
    sbNoExtraAccuracy : Result := 'NoEx';
    sbDrawPosInfinity : Result := 'PosI';
    sbDrawNegInfinity : Result := 'NegI';
    sbNotContinuous : Result := 'NotC';
    else
      Result := 'Auto';
  end; {case}
end; {function FromSideToString}

{----------------------------------------------------------------------}

function CleanGridStr(const TheStr : string) : string;
var
  I : integer;
begin {function CleanGridStr}
  Result := '';
  for I := 1 to Length(TheStr) do //0=nul; 9=TAB; 10=LF; 13=CR;
    if not (TheStr[I] in [' ', Char(0), Char(9), Char(10), Char(13)]) then
      Result := Result + UpCase(TheStr[I]);
end; {function CleanGridStr}

{----------------------------------------------------------------------}

function StringToFromSide(const TheStr : string) : TFromSideBehave;
var
  S : string;
begin {function StringToFromSide}
  Result := sbAutomatic;
  S := CleanGridStr(TheStr);
  if Length(S) = 4 then
    begin
      if S = 'AUTO' then Result := sbAutomatic else
      if S = 'EXTR' then Result := sbExtraAccuracy else
      if S = 'NOEX' then Result := sbNoExtraAccuracy else
      if S = 'POSI' then Result := sbDrawPosInfinity else
      if S = 'NEGI' then Result := sbDrawNegInfinity else
      if S = 'NOTC' then Result := sbNotContinuous;
    end; {if}
end; {function StringToFromSide}

{----------------------------------------------------------------------}

function AtPointToString(AtPointBehave : TAtPointBehave) : string;
begin {function AtPointToString}
  case AtPointBehave of
    pbAutoDetect: Result := 'Auto';
    pbUndefined: Result := 'Undef';
    else
      Result := 'Auto';
  end; {case}
end; {function AtPointToString}

{----------------------------------------------------------------------}

function StringToAtPoint(const TheStr : string) : TAtPointBehave;
var
  S : string;
begin {function StringToAtPoint}
  Result := pbAutoDetect;
  S := CleanGridStr(TheStr);
  if Length(S) >= 1 then
    begin
      if S[1] = 'A' then
        Result := pbAutoDetect
      else
        if S[1] = 'U' then
          Result := pbUndefined;
    end; {if}
end; {function StringToAtPoint}

{----------------------------------------------------------------------}

function TAdd2DFuncDlg.VerifyTheFields : boolean;
begin {function TAdd2DFuncDlg.VerifyTheFields}
  Result := false;
  if ELoBoundStr.ValidateMathStr then
    if EHiBoundStr.ValidateMathStr then
      begin
        if (not ELoBoundStr.LastWasEmpty) and (not EHiBoundStr.LastWasEmpty)
           and (ELoBoundStr.LastValue > EHiBoundStr.LastValue) then
          begin
            MessageDlgCenter('Lower Bound must be <= Upper Bound.',
                             mtError, [mbOK], 0, Self);
            PageControl1.ActivePageIndex := 0;
            ELoBoundStr.SetFocus;
            exit;
          end; {if}
        FFuncCalc.FuncString := EUserFuncStr.Text;
        Result := FFuncCalc.VerifyMathFunc;
        if not Result then
          begin
            ShowFuncErrorMsg(FFuncCalc.GetLastErrorString,
                             'Function Definition', self);
            PageControl1.ActivePageIndex := 0;
            EUserFuncStr.SetFocus;
          end; {if}
      end; {if}
end; {function TAdd2DFuncDlg.VerifyTheFields}

{----------------------------------------------------------------------}

procedure TAdd2DFuncDlg.SetTheFields(TheFunc2DObj : TFunc2DObj);
var
  I, ItemCount : integer;
  TmpItemObj : TFunc2DSpecialItem;
begin {procedure TAdd2DFuncDlg.SetTheFields}
  with TheFunc2DObj do
    begin
      FFuncCalc.Assign(FuncCalc);
      EUserFuncStr.Text := FuncCalc.FuncString;
      ELoBoundStr.text := LoBoundStr;
      EHiBoundStr.text := HiBoundStr;
      EAsymFixUp.Checked := AsymFixUp;
      EExtraUndefAccuracy.Checked := ExtraUndefAccuracy;
      EFuncColorSample.ColorValue := FuncColor;
      ELineWidth.Position := LineWidth;
      LineWidthLabel.Caption := IntToStr(LineWidth);
      //add to grid
      ItemCount := Func2DSpecialValues.ItemsList.Count;
      if ItemCount <= 0 then
        begin
          with SpecialValuesGrid do
            begin
              RowCount := 2;
              Objects[nxFirst, 1] := nil;
              Cells[nxFirst, 1] := '';
              Cells[nxSecond, 1] := '';
              Cells[nxEnd, 1] := '';
              Cells[nxFunction, 1] := '';
              Cells[nxLeftSide, 1] := 'Auto';
              Cells[nxAtPoint, 1] := 'Auto';
              Cells[nxRightSide, 1] := 'Auto';
            end; {with}
        end
      else
        begin
          SpecialValuesGrid.RowCount := ItemCount + 1;
          for I := 1 to ItemCount do
            begin
              TmpItemObj := TFunc2DSpecialItem.Create;
              if not Assigned(TmpItemObj) then
                begin
                  SpecialValuesGrid.RowCount := I;
                  Break;
                end; {if}
              TmpItemObj.Assign(Func2DSpecialValues.ItemsList.Items[I-1]);
              SpecialValuesGrid.Objects[nxFirst, I] := TmpItemObj;
              with SpecialValuesGrid do
              with TmpItemObj do
                begin
                  Cells[nxFirst, I] := FirstStr;
                  Cells[nxSecond, I] := SecondStr;
                  Cells[nxEnd, I] := EndStr;
                  Cells[nxFunction, I] := FunctionStr;
                  Cells[nxLeftSide, I] := FromSideToString(LeftSideBehave);
                  Cells[nxAtPoint, I] := AtPointToString(AtPointBehave);
                  Cells[nxRightSide, I] := FromSideToString(RightSideBehave);
                end; {with}
            end; {for}
          AddBlankRowToEndIfNeeded;
        end; {else}
      SetCurrRowAttribs(1);
    end; {with}
end; {procedure TAdd2DFuncDlg.SetTheFields}

{---------------------------------------------------------------------}

procedure TAdd2DFuncDlg.GetTheFields(TheFunc2DObj : TFunc2DObj);
var
  I : integer;
  TmpObj : TObject;
begin {procedure TAdd2DFuncDlg.GetTheFields}
  with TheFunc2DObj do
    begin
      FuncCalc.Assign(FFuncCalc);
      LoBoundStr := ELoBoundStr.text;
      HiBoundStr := EHiBoundStr.text;
      LoBlank := ELoBoundStr.LastWasEmpty;
      HiBlank := EHiBoundStr.LastWasEmpty;
      LoBoundVal := ELoBoundStr.LastMathValue;
      HiBoundVal := EHiBoundStr.LastMathValue;
      AsymFixUp := EAsymFixUp.Checked;
      ExtraUndefAccuracy := EExtraUndefAccuracy.Checked;
      FuncColor := EFuncColorSample.ColorValue;
      LineWidth := ELineWidth.Position;
      //get from grid
      Func2DSpecialValues.Clear;
      //set capacity
      if (SpecialValuesGrid.RowCount -2) > 0 then
        Func2DSpecialValues.ItemsList.Capacity := (SpecialValuesGrid.RowCount -2);
      //get the objects
      for I := 1 to (SpecialValuesGrid.RowCount -1) do
        begin
          TmpObj := SpecialValuesGrid.Objects[nxFirst,I];
          if Assigned(TmpObj) then
            Func2DSpecialValues.ItemsList.Add(TmpObj);
          SpecialValuesGrid.Objects[nxFirst,I] := nil;
        end; {for}
      //calc points
      if not Func2DSpecialValues.CalcPointsArray(SpecialValueFunc) then
        MessageDlgCenter('Error calculating special X values to plot.', mtError, [mbOK], 0, nil);
    end; {with}
end; {procedure TAdd2DFuncDlg.GetTheFields}

{---------------------------------------------------------------------}

procedure TAdd2DFuncDlg.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  if ModalResult = idOK then
    CanClose := VerifyTheFields
  else
    CanClose := true;
end;

{---------------------------------------------------------------------}

procedure TAdd2DFuncDlg.ELineWidthChange(Sender: TObject);
begin
  LineWidthLabel.Caption := IntToStr(ELineWidth.Position);
end;

{---------------------------------------------------------------------}

procedure TAdd2DFuncDlg.ColorButton1Click(Sender: TObject);
begin
  EFuncColorSample.ColorValue := ReturnNewColorComnDlg(EFuncColorSample.ColorValue, Self);
end;

{---------------------------------------------------------------------}

procedure TAdd2DFuncDlg.FormShow(Sender: TObject);
begin
  PageControl1.ActivePageIndex := 0;
  if IsGetNewFunction then
    EUserFuncStr.SetFocus;
end;

{---------------------------------------------------------------------}

constructor TAdd2DFuncDlg.Create(AOwner: TComponent);
begin {constructor TAdd2DFuncDlg.Create}
  inherited Create(AOwner);
  SettingAttribFields := false;
  LastGridRow := 1;
  LastGridCol := 0;
  SpecialValueFunc := TMathFunction.Create;
  SpecialValueFunc.VarCount := 1;
  SpecialValueFunc.VarStrings[0] := 'X';
  MathConstant := TMathFunction.Create;
  FFuncCalc := TMath2DFunc.Create;
  with SpecialValuesGrid do
    begin
      Objects[0, 0] := nil;
      Objects[0, 1] := nil;
      Cells[0, 0] := 'First Value';
      Cells[1, 0] := ', Second Value';
      Cells[2, 0] := '.. End Value';
      Cells[3, 0] := 'Function of X f(x)=';
      Cells[4, 0] := 'Left';
      Cells[5, 0] := 'At Value';
      Cells[6, 0] := 'Right';
    end; {with}
end; {constructor TAdd2DFuncDlg.Create}

{---------------------------------------------------------------------}

destructor TAdd2DFuncDlg.Destroy;
var
  I : integer;
  TmpObj : TFunc2DSpecialItem;
begin {destructor TAdd2DFuncDlg.Destroy;}
  SpecialValueFunc.Free;
  FFuncCalc.Free;
  MathConstant.Free;
  for I := 1 to (SpecialValuesGrid.RowCount - 1) do
    begin
      TmpObj := TFunc2DSpecialItem(SpecialValuesGrid.Objects[nxFirst, I]);
      if Assigned(TmpObj) then
        TmpObj.Free;
      SpecialValuesGrid.Objects[nxFirst, I] := nil;
    end; {for}
  inherited Destroy;
end; {destructor TAdd2DFuncDlg.Destroy;}

{---------------------------------------------------------------------}

procedure TAdd2DFuncDlg.SetFocusGridEditCell(TheCol, TheRow: integer);
var
  GR : TGridRect;
begin {procedure TAdd2DFuncDlg.SetFocusGridEditCell}
  LastGridRow := TheRow;
  LastGridCol := TheCol;
  PageControl1.ActivePageIndex := 1;
  GR.Left := TheCol;
  GR.Right := TheCol;
  GR.Top := TheRow;
  GR.Bottom := TheRow;
  SpecialValuesGrid.Selection := GR;
  SpecialValuesGrid.EditorMode := true;
  SpecialValuesGrid.SetFocus;
end; {procedure TAdd2DFuncDlg.SetFocusGridEditCell}

{---------------------------------------------------------------------}

procedure TAdd2DFuncDlg.CalcSetLeftSideCell(TheRowIndex : integer);
var
  TmpObj : TFunc2DSpecialItem;
  TmpLeftSideStr : string;
  TmpLeftSideBehave : TFromSideBehave;
begin {procedure TAdd2DFuncDlg.CalcSetLeftSideCell}
  TmpLeftSideStr := SpecialValuesGrid.Cells[nxLeftSide, TheRowIndex];
  TmpLeftSideBehave := StringToFromSide(TmpLeftSideStr);
  TmpLeftSideStr := FromSideToString(TmpLeftSideBehave);
  SpecialValuesGrid.Cells[nxLeftSide, TheRowIndex] := TmpLeftSideStr;
  TmpObj := TFunc2DSpecialItem(SpecialValuesGrid.Objects[nxFirst, TheRowIndex]);
  if Assigned(TmpObj) then
    TmpObj.LeftSideBehave := TmpLeftSideBehave;
  SetLeftSideAttribs(TmpLeftSideBehave);
end; {procedure TAdd2DFuncDlg.CalcSetLeftSideCell}

{---------------------------------------------------------------------}

procedure TAdd2DFuncDlg.CalcSetAtPointCell(TheRowIndex : integer);
var
  TmpAtPointStr : string;
  TmpAtPointBehave : TAtPointBehave;
  TmpObj : TFunc2DSpecialItem;
begin {procedure TAdd2DFuncDlg.CalcSetAtPointCell}
  TmpAtPointStr := SpecialValuesGrid.Cells[nxAtPoint, TheRowIndex];
  TmpAtPointBehave := StringToAtPoint(TmpAtPointStr);
  TmpAtPointStr := AtPointToString(TmpAtPointBehave);
  SpecialValuesGrid.Cells[nxAtPoint, TheRowIndex] := TmpAtPointStr;
  TmpObj := TFunc2DSpecialItem(SpecialValuesGrid.Objects[nxAtPoint, TheRowIndex]);
  if Assigned(TmpObj) then
    begin
      TmpObj.AtPointBehave := TmpAtPointBehave;
    end; {if}
  SetAtPointAttribs(TmpAtPointBehave);
end; {procedure TAdd2DFuncDlg.CalcSetAtPointCell}

{---------------------------------------------------------------------}

procedure TAdd2DFuncDlg.CalcSetRightSideCell(TheRowIndex : integer);
var
  TmpObj : TFunc2DSpecialItem;
  TmpRightSideStr : string;
  TmpRightSideBehave : TFromSideBehave;
begin {procedure TAdd2DFuncDlg.CalcSetRightSideCell}
  TmpRightSideStr := SpecialValuesGrid.Cells[nxRightSide, TheRowIndex];
  TmpRightSideBehave := StringToFromSide(TmpRightSideStr);
  TmpRightSideStr := FromSideToString(TmpRightSideBehave);
  SpecialValuesGrid.Cells[nxRightSide, TheRowIndex] := TmpRightSideStr;
  TmpObj := TFunc2DSpecialItem(SpecialValuesGrid.Objects[nxFirst, TheRowIndex]);
  if Assigned(TmpObj) then
    TmpObj.RightSideBehave := TmpRightSideBehave;
  SetRightSideAttribs(TmpRightSideBehave);
end; {procedure TAdd2DFuncDlg.CalcSetRightSideCell}

{---------------------------------------------------------------------}

function TAdd2DFuncDlg.VerifyCalcValueRow(TheRowIndex: integer): boolean;
var
  TmpFirstStr, TmpSecondStr, TmpEndStr : string;
  TmpFunctionStr : string;
  TmpLeftSideStr : string;
  TmpAtPointStr : string;
  TmpRightSideStr : string;
  TmpFirstVal, TmpSecondVal, TmpEndVal : TMathValue;
  TmpLeftSideBehave : TFromSideBehave;
  TmpAtPointBehave : TAtPointBehave;
  TmpRightSideBehave : TFromSideBehave;
  TmpSeriesPointCount : integer;
  TmpItemType : TItemType;
  TmpObj : TFunc2DSpecialItem;

{-------------------------------------}

function IsMaxPointItemsCount : boolean;
var
  I : integer;
  ItemCount : integer;
  LocalObj : TFunc2DSpecialItem;
begin {function IsMaxPointItemsCount}
  if (SpecialValuesGrid.RowCount - 1) < MaxPointItemsCount then
    Result := false
  else
    begin
      ItemCount := 0;
      for I := 1 to (SpecialValuesGrid.RowCount - 1) do
        begin
          LocalObj := TFunc2DSpecialItem(SpecialValuesGrid.Cells[nxFirst, I]);
          if Assigned(LocalObj) then
            if LocalObj.ItemType = itSingleValue then
              inc(ItemCount);
        end; {for}
      if ItemCount < MaxPointItemsCount then
        Result := false
      else
        Result := true;
    end; {else}
end; {function IsMaxPointItemsCount}

{-------------------------------------}

function IsMaxSeriesItemsCount : boolean;
var
  I : integer;
  ItemCount : integer;
  LocalObj : TFunc2DSpecialItem;
begin {function IsMaxSeriesItemsCount}
  if (SpecialValuesGrid.RowCount - 1) < MaxSeriesItemsCount then
    Result := false
  else
    begin
      ItemCount := 0;
      for I := 1 to (SpecialValuesGrid.RowCount - 1) do
        begin
          LocalObj := TFunc2DSpecialItem(SpecialValuesGrid.Cells[nxFirst, I]);
          if Assigned(LocalObj) then
            if LocalObj.ItemType in [itSeries, itFuncSeries] then
              inc(ItemCount);
        end; {for}
      if ItemCount < MaxSeriesItemsCount then
        Result := false
      else
        Result := true;
    end; {else}
end; {function IsMaxSeriesItemsCount}

{-------------------------------------}

procedure SetEditCell(TheCol : integer);
begin {procedure SetEditCell}
  SetFocusGridEditCell(TheCol, TheRowIndex);
end; {procedure SetEditCell}

{-------------------------------------}

begin {function TAdd2DFuncDlg.VerifyCalcValueRow}
  //begin clean attribute cells
  CalcSetLeftSideCell(TheRowIndex);
  CalcSetAtPointCell(TheRowIndex);
  CalcSetRightSideCell(TheRowIndex);
  //end clean attribute cells
  Result := false;
  TmpSeriesPointCount := 1;
  with SpecialValuesGrid do
    begin
      TmpFirstStr     := Cells[nxFirst, TheRowIndex];
      TmpSecondStr    := Cells[nxSecond, TheRowIndex];
      TmpEndStr       := Cells[nxEnd, TheRowIndex];
      TmpFunctionStr  := Cells[nxFunction, TheRowIndex];
      TmpLeftSideStr  := Cells[nxLeftSide, TheRowIndex];
      TmpAtPointStr   := Cells[nxAtPoint, TheRowIndex];
      TmpRightSideStr := Cells[nxRightSide, TheRowIndex];
    end; {with}
  //if row is empty then delete object
  if (TmpFirstStr = '')and(TmpSecondStr = '')and(TmpEndStr = '')and(TmpFunctionStr = '') then
    begin //empty row
      TmpObj := TFunc2DSpecialItem(SpecialValuesGrid.Objects[nxAtPoint, TheRowIndex]);
      if Assigned(TmpObj) then
        TmpObj.Free;
      SpecialValuesGrid.Objects[nxAtPoint, TheRowIndex] := nil;
      Result := true;
      Exit;
    end; {if}
  if (TmpFirstStr = '') then
    begin
      MessageDlgCenter('First Value can not be empty.', mtError, [mbOK], 0, Self);
      SetEditCell(nxFirst);
      Exit;
    end; {if}
  if (TmpSecondStr <> '') or (TmpEndStr <> '') or
     (TmpFunctionStr <> '') then
    begin //is a series or function series
      if (TmpSecondStr = '') then
        begin
          MessageDlgCenter('Second Value in a series can not be empty.', mtError, [mbOK], 0, Self);
          SetEditCell(nxSecond);
          Exit;
        end; {if}
      if (TmpEndStr = '') then
        begin
          MessageDlgCenter('End Value in a series can not be empty.', mtError, [mbOK], 0, Self);
          SetEditCell(nxEnd);
          Exit;
        end; {if}
    end; {if}
  //determine item type
  if (TmpFirstStr <> '')and(TmpSecondStr <> '')and(TmpEndStr <> '')and(TmpFunctionStr <> '') then
    TmpItemType := itFuncSeries
  else
    if (TmpFirstStr <> '')and(TmpSecondStr <> '')and(TmpEndStr <> '')and(TmpFunctionStr = '') then
      TmpItemType := itSeries
    else //if (TmpFirstStr <> '')and(TmpSecondStr = '')and(TmpEndStr = '')and(TmpFunctionStr = '') then
      TmpItemType := itSingleValue;
  //validate and calc math values
  InitMathValue(TmpFirstVal);
  InitMathValue(TmpSecondVal);
  InitMathValue(TmpEndVal);
  //validate first value
  if TmpFirstStr <> '' then
    begin
      MathConstant.FuncString := TmpFirstStr;
      if not MathConstant.VerifyMathFunc then
        begin
          ShowFuncErrorMsg(MathConstant.GetLastErrorString, 'First Value', self);
          SetEditCell(nxFirst);
          Exit;
        end; {if}
      TmpFirstVal := MathConstant.MathResultValue;
    end; {if}
  //validate second value
  if TmpSecondStr <> '' then
    begin
      MathConstant.FuncString := TmpSecondStr;
      if not MathConstant.VerifyMathFunc then
        begin
          ShowFuncErrorMsg(MathConstant.GetLastErrorString, 'Second Value', self);
          SetEditCell(nxSecond);
          Exit;
        end; {if}
      TmpSecondVal := MathConstant.MathResultValue;
    end; {if}
  //validate end value
  if TmpEndStr <> '' then
    begin
      MathConstant.FuncString := TmpEndStr;
      if not MathConstant.VerifyMathFunc then
        begin
          ShowFuncErrorMsg(MathConstant.GetLastErrorString, 'End Value', self);
          SetEditCell(nxEnd);
          Exit;
        end; {if}
      TmpEndVal := MathConstant.MathResultValue;
    end; {if}
  //validate function
  if TmpFunctionStr <> '' then
    begin
      SpecialValueFunc.FuncString := TmpFunctionStr;
      if not SpecialValueFunc.VerifyMathFunc then
        begin
          ShowFuncErrorMsg(SpecialValueFunc.GetLastErrorString, 'Function of X f(x)=', self);
          SetEditCell(nxFunction);
          Exit;
        end; {if}
      if not SpecialValueFunc.FoundVarInFunc then
        begin
          MessageDlgCenter('Function in field "Function of X f(x)=" must contain an "X" variable.', mtError, [mbOK], 0, Self);
          SetEditCell(nxFunction);
          Exit;
        end; {if}
    end; {if}
  //make sure series is not to big
  if (TmpItemType = itFuncSeries) or (TmpItemType = itSeries) then
    begin //is a series or function series
      if not VerifySeriesPoints(TmpFirstVal, TmpSecondVal, TmpEndVal) then
        begin
          MessageDlgCenter('Error in series value order, Valid orders are: (First < Second < End) or (First > Second > End).', mtError, [mbOK], 0, Self);
          SetEditCell(nxFirst);
          Exit;
        end; {if}
      if not CalcSeriesPointCount(TmpFirstVal, TmpSecondVal, TmpEndVal, TmpSeriesPointCount) then
        begin
          MessageDlgCenter('Series can not contain more than ' + IntToStr(MaxPointsPerSeries) + ' values.', mtError, [mbOK], 0, Self);
          SetEditCell(nxFirst);
          Exit;
        end; {if}
    end; {else}
  //get and calc behavior values
  //get Left side behavior
  TmpLeftSideBehave := StringToFromSide(TmpLeftSideStr);
  //get At Point behavior
  TmpAtPointBehave := StringToAtPoint(TmpAtPointStr);
  //get Right side behavior
  TmpRightSideBehave := StringToFromSide(TmpRightSideStr);
  //Assign Values to object
  if Assigned(SpecialValuesGrid.Objects[nxFirst, TheRowIndex]) then
    TmpObj := TFunc2DSpecialItem(SpecialValuesGrid.Objects[nxFirst, TheRowIndex])
  else
    TmpObj := TFunc2DSpecialItem.Create;
  if not Assigned(TmpObj) then
    begin
      MessageDlgErrorOutOfMemoryNoWait;
      SetEditCell(nxFirst);
      Exit;
    end; {if}
  with TmpObj do
    begin
      FirstStr := TmpFirstStr;
      SecondStr := TmpSecondStr;
      EndStr := TmpEndStr;
      FirstVal := TmpFirstVal;
      SecondVal := TmpSecondVal;
      EndVal := TmpEndVal;
      FunctionStr := TmpFunctionStr;
      LeftSideBehave := TmpLeftSideBehave;
      AtPointBehave := TmpAtPointBehave;
      RightSideBehave := TmpRightSideBehave;
      SeriesPointCount := TmpSeriesPointCount;
      ItemType := TmpItemType;
    end; {with}
  SpecialValuesGrid.Objects[nxFirst, TheRowIndex] := TmpObj;
  //make sure they have not entered too many points or series rows
  if TmpItemType = itSingleValue then
    begin //is a series or function series
      if IsMaxPointItemsCount then
        begin
          MessageDlgCenter('There can not be more than ' + IntToStr(MaxPointItemsCount) +
                           ' point row values.', mtError, [mbOK], 0, Self);
          SetEditCell(nxFirst);
          Exit;
        end; {if}
    end
  else //if (TmpItemType = itFuncSeries) or (TmpItemType = itSeries) then
    begin //is a series or function series
      if IsMaxSeriesItemsCount then
        begin
          MessageDlgCenter('There can not be more than ' + IntToStr(MaxSeriesItemsCount) +
                           ' series rows.', mtError, [mbOK], 0, Self);
          SetEditCell(nxFirst);
          Exit;
        end; {if}
    end; {else}
  //successful if it gets here
  Result := true;
end; {function TAdd2DFuncDlg.VerifyCalcValueRow}

{---------------------------------------------------------------------}

procedure TAdd2DFuncDlg.SpecialValuesGridSelectCell(Sender: TObject; ACol,
  ARow: Integer; var CanSelect: Boolean);
begin
  CanSelect := true;
  if LastGridRow <> ARow then //if row changed
    begin
      CanSelect := VerifyCalcValueRow(LastGridRow);
      if CanSelect then
        SetCurrRowAttribs(ARow);
    end
  else
    if LastGridCol <> ACol then //if col changed
      begin
        if LastGridCol = nxLeftSide then
          CalcSetLeftSideCell(LastGridRow)
        else
        if LastGridCol = nxAtPoint then
          CalcSetAtPointCell(LastGridRow)
        else
        if LastGridCol = nxRightSide then
          CalcSetRightSideCell(LastGridRow);
      end; {if}
  if CanSelect then
    begin
      LastGridRow := ARow;
      LastGridCol := ACol;
    end; {if}
  AddBlankRowToEndIfNeeded;
end;

{---------------------------------------------------------------------}

procedure TAdd2DFuncDlg.SpecialValuesGridExit(Sender: TObject);
begin
  VerifyCalcValueRow(LastGridRow);
  AddBlankRowToEndIfNeeded;
end;

{---------------------------------------------------------------------}

procedure TAdd2DFuncDlg.SetCurrRowAttribs(RowNdx: integer);
var
  LeftSideBehave : TFromSideBehave;
  AtPointBehave : TAtPointBehave;
  RightSideBehave : TFromSideBehave;
begin {procedure TAdd2DFuncDlg.SetCurrRowAttribs}
  //convert strings to values
  LeftSideBehave := StringToFromSide(SpecialValuesGrid.Cells[nxLeftSide, RowNdx]);
  AtPointBehave := StringToAtPoint(SpecialValuesGrid.Cells[nxAtPoint, RowNdx]);
  RightSideBehave := StringToFromSide(SpecialValuesGrid.Cells[nxRightSide, RowNdx]);
  //set values
  SetLeftSideAttribs(LeftSideBehave);
  SetAtPointAttribs(AtPointBehave);
  SetRightSideAttribs(RightSideBehave);
end; {procedure TAdd2DFuncDlg.SetCurrRowAttribs}

{---------------------------------------------------------------------}

procedure TAdd2DFuncDlg.AddBlankRowToEndIfNeeded;
var
  LastRowNdx : integer;
begin {procedure TAdd2DFuncDlg.AddBlankRowToEndIfNeeded;}
  with SpecialValuesGrid do
    begin
      LastRowNdx := RowCount - 1;
      if ((Cells[nxFirst, LastRowNdx] <> '') or (Cells[nxSecond, LastRowNdx] <> '') or
          (Cells[nxEnd, LastRowNdx] <> '') or (Cells[nxFunction, LastRowNdx] <> '')) or
          (RowCount = 1) then
        begin
          RowCount := RowCount + 1;
          LastRowNdx := RowCount - 1;
          Objects[nxFirst, LastRowNdx] := nil;
          Cells[nxFirst, LastRowNdx] := '';
          Cells[nxSecond, LastRowNdx] := '';
          Cells[nxEnd, LastRowNdx] := '';
          Cells[nxFunction, LastRowNdx] := '';
          Cells[nxLeftSide, LastRowNdx] := 'Auto';
          Cells[nxAtPoint, LastRowNdx] := 'Auto';
          Cells[nxRightSide, LastRowNdx] := 'Auto';
        end; {if}
    end; {with}
end; {procedure TAdd2DFuncDlg.AddBlankRowToEndIfNeeded;}

{---------------------------------------------------------------------}

procedure TAdd2DFuncDlg.SetAtPointAttribs(AtPointBehave: TAtPointBehave);
begin {procedure TAdd2DFuncDlg.SetAtPointAttribs}
  SettingAttribFields := true;
  //At Point
  case AtPointBehave of
    pbAutoDetect: EAtValueAutoDetect.Checked := True;
    pbUndefined: EAtValueUndefined.Checked := True;
  end; {case}
  SettingAttribFields := false;
end; {procedure TAdd2DFuncDlg.SetAtPointAttribs}

{---------------------------------------------------------------------}

procedure TAdd2DFuncDlg.SetLeftSideAttribs(LeftSideBehave: TFromSideBehave);
begin {procedure TAdd2DFuncDlg.SetLeftSideAttribs}
  SettingAttribFields := true;
  //Left Side
  case LeftSideBehave of
    sbAutomatic: ELeftBehavior.ItemIndex := 0;
    sbExtraAccuracy: ELeftBehavior.ItemIndex := 1;
    sbNoExtraAccuracy: ELeftBehavior.ItemIndex := 2;
    sbDrawPosInfinity: ELeftBehavior.ItemIndex := 3;
    sbDrawNegInfinity: ELeftBehavior.ItemIndex := 4;
    sbNotContinuous: ELeftBehavior.ItemIndex := 5;
  end; {case}
  SettingAttribFields := false;
end; {procedure TAdd2DFuncDlg.SetLeftSideAttribs}

{---------------------------------------------------------------------}

procedure TAdd2DFuncDlg.SetRightSideAttribs(RightSideBehave: TFromSideBehave);
begin {procedure TAdd2DFuncDlg.SetRightSideAttribs}
  SettingAttribFields := true;
  //Right Side
  case RightSideBehave of
    sbAutomatic: ERightBehavior.ItemIndex := 0;
    sbExtraAccuracy: ERightBehavior.ItemIndex := 1;
    sbNoExtraAccuracy: ERightBehavior.ItemIndex := 2;
    sbDrawPosInfinity: ERightBehavior.ItemIndex := 3;
    sbDrawNegInfinity: ERightBehavior.ItemIndex := 4;
    sbNotContinuous: ERightBehavior.ItemIndex := 5;
  end; {case}
  SettingAttribFields := false;
end; {procedure TAdd2DFuncDlg.SetRightSideAttribs}

{---------------------------------------------------------------------}

procedure TAdd2DFuncDlg.ELeftChange(Sender: TObject);
var
  TmpObj : TFunc2DSpecialItem;
  CurrRow : integer;
  TmpLeftSideBehave : TFromSideBehave;
begin
  if not SettingAttribFields then
    begin
      CurrRow := SpecialValuesGrid.Row;
      if CurrRow >= 1 then
        begin
          //get values
          case ELeftBehavior.ItemIndex of
            0: TmpLeftSideBehave := sbAutomatic;
            1: TmpLeftSideBehave := sbExtraAccuracy;
            2: TmpLeftSideBehave := sbNoExtraAccuracy;
            3: TmpLeftSideBehave := sbDrawPosInfinity;
            4: TmpLeftSideBehave := sbDrawNegInfinity;
            5: TmpLeftSideBehave := sbNotContinuous;
            else
              TmpLeftSideBehave := sbAutomatic;
          end; {case}
          //assign values to object
          TmpObj := TFunc2DSpecialItem(SpecialValuesGrid.Objects[nxFirst, CurrRow]);
          if Assigned(TmpObj) then
            TmpObj.LeftSideBehave := TmpLeftSideBehave;
          //assign values to grid
          SpecialValuesGrid.Cells[nxLeftSide, CurrRow] := FromSideToString(TmpLeftSideBehave);
        end; {if}
    end; {if}
end;

{---------------------------------------------------------------------}

procedure TAdd2DFuncDlg.ERightChange(Sender: TObject);
var
  TmpObj : TFunc2DSpecialItem;
  CurrRow : integer;
  TmpRightSideBehave : TFromSideBehave;
begin
  if not SettingAttribFields then
    begin
      CurrRow := SpecialValuesGrid.Row;
      if CurrRow >= 1 then
        begin
          //get values
          case ERightBehavior.ItemIndex of
            0: TmpRightSideBehave := sbAutomatic;
            1: TmpRightSideBehave := sbExtraAccuracy;
            2: TmpRightSideBehave := sbNoExtraAccuracy;
            3: TmpRightSideBehave := sbDrawPosInfinity;
            4: TmpRightSideBehave := sbDrawNegInfinity;
            5: TmpRightSideBehave := sbNotContinuous;
            else
              TmpRightSideBehave := sbAutomatic;
          end; {case}
          //assign values to object
          TmpObj := TFunc2DSpecialItem(SpecialValuesGrid.Objects[nxFirst, CurrRow]);
          if Assigned(TmpObj) then
            TmpObj.RightSideBehave := TmpRightSideBehave;
          //assign values to grid
          SpecialValuesGrid.Cells[nxRightSide, CurrRow] := FromSideToString(TmpRightSideBehave);
        end; {if}
    end; {if}
end;

{---------------------------------------------------------------------}

procedure TAdd2DFuncDlg.EAtValueClick(Sender: TObject);
var
  TmpObj : TFunc2DSpecialItem;
  CurrRow : integer;
  TmpAtPointBehave : TAtPointBehave;
begin
  if not SettingAttribFields then
    begin
      CurrRow := SpecialValuesGrid.Row;
      if CurrRow >= 1 then
        begin
          //get values
          if EAtValueUndefined.Checked then
            TmpAtPointBehave := pbUndefined
          else //if EAtValueAutoDetect.Checked then
            TmpAtPointBehave := pbAutoDetect;
          //assign values to object
          TmpObj := TFunc2DSpecialItem(SpecialValuesGrid.Objects[nxFirst, CurrRow]);
          if Assigned(TmpObj) then
            TmpObj.AtPointBehave := TmpAtPointBehave;
          //assign values to grid
          SpecialValuesGrid.Cells[nxAtPoint, CurrRow] := AtPointToString(TmpAtPointBehave);
        end; {if}
    end; {if}
end;

{---------------------------------------------------------------------}

procedure TAdd2DFuncDlg.DoAddValueRow;
var
  LastRowIndex : integer;
begin {procedure TAdd2DFuncDlg.DoAddValueRow;}
  if VerifyCalcValueRow(SpecialValuesGrid.Row) then
    begin
      AddBlankRowToEndIfNeeded;
      LastRowIndex := SpecialValuesGrid.RowCount - 1;
      SetFocusGridEditCell(nxFirst, LastRowIndex);
      SetCurrRowAttribs(LastRowIndex);
    end; {if}
end; {procedure TAdd2DFuncDlg.DoAddValueRow;}

{---------------------------------------------------------------------}

procedure TAdd2DFuncDlg.DoInsertValueRow;
var
  I : integer;
  FromNdx, ToNdx : integer;
  TheRowIndex: integer;
begin {procedure TAdd2DFuncDlg.DoInsertValueRow;}
  if VerifyCalcValueRow(SpecialValuesGrid.Row) then
    begin
      with SpecialValuesGrid do
        begin
          if RowCount > MaxGridEditorRows then
            Exit; //don't do insert if too many rows
          //get current row
          TheRowIndex := Row;
          //do make hole
          RowCount := RowCount + 1;
          for I := (RowCount - 1) downto (TheRowIndex + 1) do
            begin
              FromNdx := I - 1;
              ToNdx := I;
              Objects[nxFirst, ToNdx] := Objects[nxFirst, FromNdx];
              Cells[nxFirst, ToNdx] := Cells[nxFirst, FromNdx];
              Cells[nxSecond, ToNdx] := Cells[nxSecond, FromNdx];
              Cells[nxEnd, ToNdx] := Cells[nxEnd, FromNdx];
              Cells[nxFunction, ToNdx] := Cells[nxFunction, FromNdx];
              Cells[nxLeftSide, ToNdx] := Cells[nxLeftSide, FromNdx];
              Cells[nxAtPoint, ToNdx] := Cells[nxAtPoint, FromNdx];
              Cells[nxRightSide, ToNdx] := Cells[nxRightSide, FromNdx];
            end; {for}
          //add blank to empth hole
          Objects[nxFirst, TheRowIndex] := nil;
          Cells[nxFirst, TheRowIndex] := '';
          Cells[nxSecond, TheRowIndex] := '';
          Cells[nxEnd, TheRowIndex] := '';
          Cells[nxFunction, TheRowIndex] := '';
          Cells[nxLeftSide, TheRowIndex] := 'Auto';
          Cells[nxAtPoint, TheRowIndex] := 'Auto';
          Cells[nxRightSide, TheRowIndex] := 'Auto';
          AddBlankRowToEndIfNeeded;
          SetCurrRowAttribs(TheRowIndex);
          SetFocusGridEditCell(nxFirst, TheRowIndex);
        end; {with}
    end; {if}
end; {procedure TAdd2DFuncDlg.DoInsertValueRow;}

{---------------------------------------------------------------------}

procedure TAdd2DFuncDlg.DoDeleteValueRow;
var
  I : integer;
  TheRowIndex: integer;
begin {procedure TAdd2DFuncDlg.DoDeleteValueRow;}
  if VerifyCalcValueRow(SpecialValuesGrid.Row) then
    begin
      with SpecialValuesGrid do
        begin
          //get current row
          TheRowIndex := Row;
          //do delete
          if Assigned(Objects[nxFirst, TheRowIndex]) then
            TFunc2DSpecialItem(Objects[nxFirst, TheRowIndex]).Free;
          Objects[nxFirst, TheRowIndex] := nil;
          if TheRowIndex = (RowCount - 1) then
            begin
              Cells[nxFirst, TheRowIndex] := '';
              Cells[nxSecond, TheRowIndex] := '';
              Cells[nxEnd, TheRowIndex] := '';
              Cells[nxFunction, TheRowIndex] := '';
              Cells[nxLeftSide, TheRowIndex] := 'Auto';
              Cells[nxAtPoint, TheRowIndex] := 'Auto';
              Cells[nxRightSide, TheRowIndex] := 'Auto';
            end
          else
            begin
              for I := TheRowIndex to (RowCount - 2) do
                begin
                  Objects[nxFirst, I] := Objects[nxFirst, (I + 1)];
                  Cells[nxFirst, I] := Cells[nxFirst, (I + 1)];
                  Cells[nxSecond, I] := Cells[nxSecond, (I + 1)];
                  Cells[nxEnd, I] := Cells[nxEnd, (I + 1)];
                  Cells[nxFunction, I] := Cells[nxFunction, (I + 1)];
                  Cells[nxLeftSide, I] := Cells[nxLeftSide, (I + 1)];
                  Cells[nxAtPoint, I] := Cells[nxAtPoint, (I + 1)];
                  Cells[nxRightSide, I] := Cells[nxRightSide, (I + 1)];
                end; {for}
              RowCount := RowCount - 1;
            end; {if}
        end; {with}
      AddBlankRowToEndIfNeeded;
      SetCurrRowAttribs(TheRowIndex);
      SetFocusGridEditCell(nxFirst, TheRowIndex);
    end; {if}
end; {procedure TAdd2DFuncDlg.DoDeleteValueRow;}

{---------------------------------------------------------------------}

procedure TAdd2DFuncDlg.DoSwapValueRows(TheRowNdx1, TheRowNdx2: integer);
var
  TmpObj : TFunc2DSpecialItem;
  TmpFirstStr, TmpSecondStr, TmpEndStr : string;
  TmpFunctionStr : string;
  TmpLeftSideStr : string;
  TmpAtPointStr : string;
  TmpRightSideStr : string;
begin {procedure TAdd2DFuncDlg.DoSwapValueRows}
  with SpecialValuesGrid do
    begin
      //save to tmp variables; Tmp := Ndx1
      TmpObj := TFunc2DSpecialItem(Objects[nxFirst, TheRowNdx1]);
      TmpFirstStr := Cells[nxFirst, TheRowNdx1];
      TmpSecondStr := Cells[nxSecond, TheRowNdx1];
      TmpEndStr := Cells[nxEnd, TheRowNdx1];
      TmpFunctionStr := Cells[nxFunction, TheRowNdx1];
      TmpLeftSideStr := Cells[nxLeftSide, TheRowNdx1];
      TmpAtPointStr := Cells[nxAtPoint, TheRowNdx1];
      TmpRightSideStr := Cells[nxRightSide, TheRowNdx1];
      //Ndx1 := Ndx2
      Objects[nxFirst, TheRowNdx1] := Objects[nxFirst, TheRowNdx2];
      Cells[nxFirst, TheRowNdx1] := Cells[nxFirst, TheRowNdx2];
      Cells[nxSecond, TheRowNdx1] := Cells[nxSecond, TheRowNdx2];
      Cells[nxEnd, TheRowNdx1] := Cells[nxEnd, TheRowNdx2];
      Cells[nxFunction, TheRowNdx1] := Cells[nxFunction, TheRowNdx2];
      Cells[nxLeftSide, TheRowNdx1] := Cells[nxLeftSide, TheRowNdx2];
      Cells[nxAtPoint, TheRowNdx1] := Cells[nxAtPoint, TheRowNdx2];
      Cells[nxRightSide, TheRowNdx1] := Cells[nxRightSide, TheRowNdx2];
      //Ndx2 := Tmp
      Objects[nxFirst, TheRowNdx2] := TmpObj;
      Cells[nxFirst, TheRowNdx2] := TmpFirstStr;
      Cells[nxSecond, TheRowNdx2] := TmpSecondStr;
      Cells[nxEnd, TheRowNdx2] := TmpEndStr;
      Cells[nxFunction, TheRowNdx2] := TmpFunctionStr;
      Cells[nxLeftSide, TheRowNdx2] := TmpLeftSideStr;
      Cells[nxAtPoint, TheRowNdx2] := TmpAtPointStr;
      Cells[nxRightSide, TheRowNdx2] := TmpRightSideStr;
    end; {with}
end; {procedure TAdd2DFuncDlg.DoSwapValueRows}

{---------------------------------------------------------------------}

procedure TAdd2DFuncDlg.DoMoveDownValueRow;
var
  TheRowIndex: integer;
begin {procedure TAdd2DFuncDlg.DoMoveDownValueRow;}
  if VerifyCalcValueRow(SpecialValuesGrid.Row) then
    begin
      AddBlankRowToEndIfNeeded;
      with SpecialValuesGrid do
        begin
          //get current row
          TheRowIndex := Row;
          if TheRowIndex < (RowCount - 2) then //if not last row or, next to last row
            begin
              DoSwapValueRows(TheRowIndex, (TheRowIndex + 1));
              TheRowIndex := TheRowIndex + 1;
            end; {if}
          SetCurrRowAttribs(TheRowIndex);
          SetFocusGridEditCell(nxFirst, TheRowIndex);
        end; {with}
    end; {if}
end; {procedure TAdd2DFuncDlg.DoMoveDownValueRow;}

{---------------------------------------------------------------------}

procedure TAdd2DFuncDlg.DoMoveUpValueRow;
var
  TheRowIndex: integer;
begin {procedure TAdd2DFuncDlg.DoMoveUpValueRow;}
  if VerifyCalcValueRow(SpecialValuesGrid.Row) then
    begin
      AddBlankRowToEndIfNeeded;
      with SpecialValuesGrid do
        begin
          //get current row
          TheRowIndex := Row;
          //if not first row or, last row
          if (TheRowIndex >= 2) and (TheRowIndex < (RowCount - 1)) then
            begin
              DoSwapValueRows(TheRowIndex, (TheRowIndex - 1));
              TheRowIndex := TheRowIndex - 1;
            end; {if}
          SetCurrRowAttribs(TheRowIndex);
          SetFocusGridEditCell(nxFirst, TheRowIndex);
        end; {with}
    end; {if}
end; {procedure TAdd2DFuncDlg.DoMoveUpValueRow;}

{---------------------------------------------------------------------}

procedure TAdd2DFuncDlg.AddButtonClick(Sender: TObject);
begin
  DoAddValueRow;
end;

procedure TAdd2DFuncDlg.InsertButtonClick(Sender: TObject);
begin
  DoInsertValueRow;
end;

procedure TAdd2DFuncDlg.DeleteButtonClick(Sender: TObject);
begin
  DoDeleteValueRow;
end;

procedure TAdd2DFuncDlg.MoveUpButtonClick(Sender: TObject);
begin
  DoMoveUpValueRow;
end;

procedure TAdd2DFuncDlg.MoveDownButtonClick(Sender: TObject);
begin
  DoMoveDownValueRow;
end;

procedure TAdd2DFuncDlg.SpecialValuesGridKeyUp(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  AddBlankRowToEndIfNeeded;
end;

procedure TAdd2DFuncDlg.ShowValuesButtonClick(Sender: TObject);
var
  TmpObj : TFunc2DSpecialItem;
begin
  if VerifyCalcValueRow(SpecialValuesGrid.Row) then
    begin
      TmpObj := TFunc2DSpecialItem(SpecialValuesGrid.Objects[nxFirst, SpecialValuesGrid.Row]);
      if Assigned(TmpObj) then
        ShowSpec2DValuesRowDlg(TmpObj, SpecialValueFunc, Self);
    end; {if}
end;

end.
