unit Add3ddlg;

(*
Copyright 1999-2011 Greg VanMullem
This program is distributed under the terms of the GNU General Public License Version 3

This file is part of MathGV.

MathGV is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

MathGV is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MathGV.  If not, see <http://www.gnu.org/licenses/>.
*)

interface

uses WinTypes, WinProcs, Classes, Graphics, Forms, Controls, Buttons,
  StdCtrls, Spin, ExtCtrls, Func3D, MathCalc, MathFunc, FuncObj,
  Dialogs, ComnDlgs, CentrDlg, Clsample, editmath;

type
  TAdd3DDialog = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    HelpBtn: TBitBtn;
    FuncBox: TGroupBox;
    EDependantVar: TComboBox;
    EUserFuncString: TEdit;
    Var1GroupBox: TGroupBox;
    EVar1Lines: TSpinEdit;
    EVar1Detail: TSpinEdit;
    EVar1LinesLabel: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Var2GroupBox: TGroupBox;
    EVar2LinesLabel: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    EVar2Lines: TSpinEdit;
    EVar2Detail: TSpinEdit;
    EFuncSample: TColorSample;
    RoundValuesBox: TGroupBox;
    ERadioRoundDown: TRadioButton;
    ERadioDontDraw: TRadioButton;
    EColor: TButton;
    Label1: TLabel;
    EBeginVar1Str: TEditMathConst;
    EEndVar1Str: TEditMathConst;
    EBeginVar2Str: TEditMathConst;
    EEndVar2Str: TEditMathConst;
    procedure EDependantVarExit(Sender: TObject);
    procedure EColorClick(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure EVar1LinesExit(Sender: TObject);
    procedure EVar2LinesExit(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
  private
    { Private declarations }
    FTheDependantVar : TGraph3DFuncType;
    FFuncCalc : TMath3DFunc;
    procedure SetTheDependantVar(Value : TGraph3DFuncType);
    function GetVarDomainName(WhichVar : byte) : Char;
    function GetVerifyFuncStr : boolean;
  public
    { Public declarations }
    IsGetNewFunction : boolean;
    property TheDependantVar : TGraph3DFuncType read FTheDependantVar
                                                write SetTheDependantVar;
    constructor Create(AOwner : TComponent) ; override;
    destructor Destroy; override;
    procedure SetTheC3dFuncFields(TheFunc3DObj : TFunc3DObj);
    procedure GetTheC3dFuncFields(TheFunc3DObj : TFunc3DObj);
  end;

function GetNewC3DDlg(TheFunc3DObj : TFunc3DObj; TheCenterOnForm : TForm) : boolean;
function ModifyC3DDlg(TheFunc3DObj : TFunc3DObj; TheCenterOnForm : TForm) : boolean;

implementation

{$R *.DFM}

uses
  Fg3DLine;

{---------------------------------------------------------------------}

function GetNewC3DDlg(TheFunc3DObj : TFunc3DObj; TheCenterOnForm : TForm) : boolean;
var
  Add3DDialog: TAdd3DDialog;
begin {function GetNewC3DDlg}
  Result := false;
  Add3DDialog := TAdd3DDialog.Create(Application);
  try
    SetDlgDefaultPos(Add3DDialog, TheCenterOnForm);
    Add3DDialog.IsGetNewFunction := true;
    Add3DDialog.Caption := 'Add 3D Function';
    Add3DDialog.SetTheC3dFuncFields(TheFunc3DObj);
    if Add3DDialog.ShowModal = IDOK then
      begin
        Result := true;
        Add3DDialog.GetTheC3dFuncFields(TheFunc3DObj);
        TheFunc3DObj.DidFieldsChange := true;
      end; {if}
  finally
    Add3DDialog.free;
  end; {try...finally}
end; {function GetNewC3DDlg}

{---------------------------------------------------------------------}

function ModifyC3DDlg(TheFunc3DObj : TFunc3DObj; TheCenterOnForm : TForm) : boolean;
var
  Add3DDialog: TAdd3DDialog;
begin {function ModifyC3DDlg}
  Result := false;
  Add3DDialog := TAdd3DDialog.Create(Application);
  try
    SetDlgDefaultPos(Add3DDialog, TheCenterOnForm);
    Add3DDialog.IsGetNewFunction := false;
    Add3DDialog.Caption := 'Modify 3D Function';
    Add3DDialog.SetTheC3dFuncFields(TheFunc3DObj);
    if Add3DDialog.ShowModal = IDOK then
      begin
        Result := true;
        Add3DDialog.GetTheC3dFuncFields(TheFunc3DObj);
        TheFunc3DObj.DidFieldsChange := true;
      end; {if}
  finally
    Add3DDialog.free;
  end; {try...finally}
end; {function ModifyC3DDlg}

{---------------------------------------------------------------------}

function TAdd3DDialog.GetVarDomainName(WhichVar : byte) : char;
begin {function TAdd3DDialog.GetVarDomainName}
  Result := 'X';
     case FTheDependantVar of
        ft_3DX : begin
                   if WhichVar = 1 then
                     Result := 'Y'
                   else
                     Result := 'Z';
                 end;
        ft_3DY : begin
                   if WhichVar = 1 then
                     Result := 'X'
                   else
                     Result := 'Z';
                 end;
        ft_3DZ : begin
                   if WhichVar = 1 then
                     Result := 'X'
                   else
                     Result := 'Y';
                 end;
      end; {case}
end; {function TAdd3DDialog.GetVarDomainName}

{---------------------------------------------------------------------}

function TAdd3DDialog.GetVerifyFuncStr : boolean;
var
  TheDomainVar : char;
begin
  Result := false;
  EBeginVar1Str.emCtrlErrorName := GetVarDomainName(1)+' Lower Bound';
  EEndVar1Str.emCtrlErrorName := GetVarDomainName(1)+' Upper Bound';
  EBeginVar2Str.emCtrlErrorName := GetVarDomainName(2)+' Lower Bound';
  EEndVar2Str.emCtrlErrorName := GetVarDomainName(2)+' Upper Bound';
  if EBeginVar1Str.ValidateMathStr then
  if EEndVar1Str.ValidateMathStr then
  if EBeginVar2Str.ValidateMathStr then
  if EEndVar2Str.ValidateMathStr then
    begin
      if (EBeginVar1Str.LastValue > EEndVar1Str.LastValue) then
        begin
          TheDomainVar := GetVarDomainName(1);
          MessageDlgCenter(TheDomainVar+' Lower Bound must be <= '+
                           TheDomainVar+' Upper Bound.', mtError, [mbOK], 0, Self);
          EBeginVar1Str.SetFocus;
          exit;
        end; {if}
      if (EBeginVar2Str.LastValue > EEndVar2Str.LastValue) then
        begin
          TheDomainVar := GetVarDomainName(2);
          MessageDlgCenter(TheDomainVar+' Lower Bound must be <= '+
                           TheDomainVar+' Upper Bound.', mtError, [mbOK], 0, Self);
          EBeginVar2Str.SetFocus;
          exit;
        end; {if}
      FFuncCalc.FuncString := EUserFuncString.Text;
      FFuncCalc.Graph3DFuncType := TheDependantVar;
      Result := FFuncCalc.VerifyMathFunc;
      if not Result then
        begin
          ShowFuncErrorMsg(FFuncCalc.GetLastErrorString,
                           'Function Definition', self);
          EUserFuncString.SetFocus;
        end; {if}
    end; {if}
end; {function TAdd3DDialog.GetVerifyFuncStr}

{---------------------------------------------------------------------}

procedure TAdd3DDialog.SetTheDependantVar(Value : TGraph3DFuncType);
const
  DispVarStr : string[9] = ' Variable';
  LabelVarStr : string[23] = 'Wire Lines Parallel to ';
begin
  if Value <> FTheDependantVar then
    begin
      if Value in [ft_3DX, ft_3DY, ft_3DZ] then
        FTheDependantVar := Value
      else
        FTheDependantVar := ft_3DZ;
      case FTheDependantVar of
        ft_3DX : begin
                   Var1GroupBox.Caption := 'Y'+DispVarStr;
                   Var2GroupBox.Caption := 'Z'+DispVarStr;
                   EVar1LinesLabel.Caption := LabelVarStr + 'Y';
                   EVar2LinesLabel.Caption := LabelVarStr + 'Z';
                   EDependantVar.Text := EDependantVar.Items[0];
                 end;
        ft_3DY : begin
                   Var1GroupBox.Caption := 'X'+DispVarStr;
                   Var2GroupBox.Caption := 'Z'+DispVarStr;
                   EVar1LinesLabel.Caption := LabelVarStr + 'X';
                   EVar2LinesLabel.Caption := LabelVarStr + 'Z';
                   EDependantVar.Text := EDependantVar.Items[1];
                 end;
        ft_3DZ : begin
                   Var1GroupBox.Caption := 'X'+DispVarStr;
                   Var2GroupBox.Caption := 'Y'+DispVarStr;
                   EVar1LinesLabel.Caption := LabelVarStr + 'X';
                   EVar2LinesLabel.Caption := LabelVarStr + 'Y';
                   EDependantVar.Text := EDependantVar.Items[2];
                 end;
      end; {case}
    end; {if}
end; {procedure TAdd3DDialog.SetTheDependantVar}

{---------------------------------------------------------------------}

constructor TAdd3DDialog.Create(AOwner : TComponent);
{----------------}
procedure InitalizeMathEdLine(var TheMathEdLine : TEditMathConst);
begin {procedure InitalizeMathEdLine}
  with TheMathEdLine do
    begin
      emBoundsCheck := [CheckLoBound,LoBoundCanEqual,
                        CheckHiBound,HiBoundCanEqual];
      emLoBoundVal := Min3DDomain;
      emLoBoundStr := Min3DDomainStr;
      emHiBoundVal := Max3DDomain;
      emHiBoundStr := Max3DDomainStr;
    end; {with}
end; {procedure InitalizeMathEdLine}
{----------------}
begin
  inherited Create(AOwner);
  FFuncCalc := TMath3DFunc.Create;
  FTheDependantVar := ft_3DX;
  TheDependantVar := ft_3DZ;
  InitalizeMathEdLine(EBeginVar1Str);
  InitalizeMathEdLine(EEndVar1Str);
  InitalizeMathEdLine(EBeginVar2Str);
  InitalizeMathEdLine(EEndVar2Str);
end;

{---------------------------------------------------------------------}

destructor TAdd3DDialog.Destroy;
begin {destructor TAdd3DDialog.Destroy;}
  FFuncCalc.Free;
  inherited Destroy;
end; {destructor TAdd3DDialog.Destroy;}

{---------------------------------------------------------------------}

procedure TAdd3DDialog.SetTheC3dFuncFields(TheFunc3DObj : TFunc3DObj);
begin
  with TheFunc3DObj do
    begin
      TheDependantVar := FuncCalc.Graph3DFuncType;
      EUserFuncString.text := FuncCalc.FuncString;
      FFuncCalc.Assign(FuncCalc);
      EFuncSample.ColorValue :=FuncColor;
      {Var1}
      EBeginVar1Str.text := BeginVar1Str;
      EEndVar1Str.text := EndVar1Str;
      EVar1Lines.Value := Var1Lines;
      EVar1Detail.MaxValue := CalcHighestDetail(Var2Lines);
      EVar1Detail.Value := Var1Detail;
      {Var2}
      EBeginVar2Str.text := BeginVar2Str;
      EEndVar2Str.text := EndVar2Str;
      EVar2Lines.Value := Var2Lines;
      EVar2Detail.MaxValue := CalcHighestDetail(Var1Lines);
      EVar2Detail.Value := Var2Detail;
      if RoundLargeValues then
        ERadioRoundDown.Checked := true
      else
        ERadioDontDraw.Checked := true;
    end; {with}
end; {procedure TAdd3DDialog.SetTheC3dFuncFields}

{---------------------------------------------------------------------}

procedure TAdd3DDialog.GetTheC3dFuncFields(TheFunc3DObj : TFunc3DObj);
begin
  with TheFunc3DObj do
    begin
      FuncCalc.Assign(FFuncCalc);
      FuncColor := EFuncSample.ColorValue;
      {Var1}
      BeginVar1Str := EBeginVar1Str.text;
      EndVar1Str := EEndVar1Str.text;
      BeginVar1Val := EBeginVar1Str.LastMathValue;
      EndVar1Val := EEndVar1Str.LastMathValue;
      Var1Lines := EVar1Lines.Value;
      Var1Detail := EVar1Detail.Value;
      {Var2}
      BeginVar2Str := EBeginVar2Str.text;
      EndVar2Str := EEndVar2Str.text;
      BeginVar2Val := EBeginVar2Str.LastMathValue;
      EndVar2Val := EEndVar2Str.LastMathValue;
      Var2Lines := EVar2Lines.Value;
      Var2Detail := EVar2Detail.Value;
      RoundLargeValues := ERadioRoundDown.Checked;
    end; {with}
end; {procedure TAdd3DDialog.GetTheC3dFuncFields}

{---------------------------------------------------------------------}

procedure TAdd3DDialog.EDependantVarExit(Sender: TObject);
var
  FirstChar : char;
begin
  if length(EDependantVar.Text) >= 1 then
    FirstChar := upcase(EDependantVar.Text[1])
  else
    FirstChar := '='; //set to invalid char if string is null
  case FirstChar of
    'X': TheDependantVar := ft_3DX;
    'Y': TheDependantVar := ft_3DY;
    'Z': TheDependantVar := ft_3DZ;
    else
      begin
        case TheDependantVar of
          ft_3DX : EDependantVar.Text := EDependantVar.Items[0];
          ft_3DY : EDependantVar.Text := EDependantVar.Items[1];
          ft_3DZ : EDependantVar.Text := EDependantVar.Items[2];
        end; {nested case}
      end;
  end; {case}
end;

procedure TAdd3DDialog.EColorClick(Sender: TObject);
begin
  EDependantVarExit(nil);
  EVar1LinesExit(nil);
  EVar2LinesExit(nil);
  EFuncSample.ColorValue := ReturnNewColorComnDlg(EFuncSample.ColorValue, Self);
end;

procedure TAdd3DDialog.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  EDependantVarExit(nil);
  EVar1LinesExit(nil);
  EVar2LinesExit(nil);
  if ModalResult = IDOK then
    CanClose := GetVerifyFuncStr
  else
    CanClose := true;
end;

procedure TAdd3DDialog.EVar1LinesExit(Sender: TObject);
var
  Tmp : integer;
begin
  Tmp := CalcHighestDetail(EVar1Lines.Value);
  if EVar2Detail.Value > Tmp then
    EVar2Detail.Value := Tmp;
  if Tmp <= 0 then
    EVar2Detail.ReadOnly := true
  else
    begin
      EVar2Detail.ReadOnly := false;
      EVar2Detail.MaxValue := Tmp;
    end; {else}
end;

procedure TAdd3DDialog.EVar2LinesExit(Sender: TObject);
var
  Tmp : integer;
begin
  Tmp := CalcHighestDetail(EVar2Lines.Value);
  if EVar1Detail.Value > Tmp then
    EVar1Detail.Value := Tmp;
  if Tmp <= 0 then
    EVar1Detail.ReadOnly := true
  else
    begin
      EVar1Detail.ReadOnly := false;
      EVar1Detail.MaxValue := Tmp;
    end; {else}
end;

procedure TAdd3DDialog.FormShow(Sender: TObject);
begin
  if IsGetNewFunction then
    EUserFuncString.SetFocus;
end;

procedure TAdd3DDialog.FormKeyPress(Sender: TObject; var Key: Char);
begin
  FormKeyPressSpinEditClose(Self, Key);
end;

end.
