unit Adrotdlg;

(*
Copyright 1999-2011 Greg VanMullem
This program is distributed under the terms of the GNU General Public License Version 3

This file is part of MathGV.

MathGV is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

MathGV is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MathGV.  If not, see <http://www.gnu.org/licenses/>.
*)

interface

uses WinTypes, WinProcs, Classes, Graphics, Forms, Controls, Buttons,
  StdCtrls, ExtCtrls, Spin, ComnDlgs, CentrDlg, MathCalc, MathFunc, Func3D, FuncObj,
  Dialogs, fg3DLine, Clsample, editmath;

type
  TAddRotateDlg = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    HelpBtn: TBitBtn;
    SLineBox: TGroupBox;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    AppearBox: TGroupBox;
    ECircles: TSpinEdit;
    ELines: TSpinEdit;
    ECircleDetail: TSpinEdit;
    Label9: TLabel;
    Label10: TLabel;
    Label11: TLabel;
    FuncBox: TGroupBox;
    EUserFuncStr: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    EFuncColor: TButton;
    Label4: TLabel;
    ELineDetail: TSpinEdit;
    Label12: TLabel;
    EFuncSample: TColorSample;
    RoundValuesBox: TGroupBox;
    ERoundVal: TRadioButton;
    EDontVal: TRadioButton;
    RotAxisBox: TGroupBox;
    ERotX: TRadioButton;
    ERotY: TRadioButton;
    ERotSpecified: TRadioButton;
    EBeginXStr: TEditMathConst;
    EEndXStr: TEditMathConst;
    ELineP1XStr: TEditMathConst;
    ELineP1YStr: TEditMathConst;
    ELineP2XStr: TEditMathConst;
    ELineP2YStr: TEditMathConst;
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure EFuncColorClick(Sender: TObject);
    procedure ELinesExit(Sender: TObject);
    procedure ECirclesExit(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
  private
    { Private declarations }
    FFuncCalc : TMathRotateFunc;
  public
    { Public declarations }
    IsGetNewFunction : boolean;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function VerifyTheFields : boolean;
    procedure GetTheFields(TheFuncRotateObj : TFuncRotateObj);
    procedure SetTheFields(TheFuncRotateObj : TFuncRotateObj);
  end;

function GetNewRotDlg(TheFuncRotateObj : TFuncRotateObj; TheCenterOnForm : TForm): boolean;
function ModifyRotDlg(TheFuncRotateObj : TFuncRotateObj; TheCenterOnForm : TForm) : boolean;

implementation

{$R *.DFM}

{---------------------------------------------------------------------}

function GetNewRotDlg(TheFuncRotateObj : TFuncRotateObj; TheCenterOnForm : TForm) : boolean;
var
  AddRotateDlg: TAddRotateDlg;
begin {function GetNewRotDlg}
  Result := false;
  AddRotateDlg := TAddRotateDlg.Create(Application);
  try
    SetDlgDefaultPos(AddRotateDlg, TheCenterOnForm);
    AddRotateDlg.IsGetNewFunction := true;
    AddRotateDlg.Caption := 'Add 2D Function Rotated Into 3D';
    AddRotateDlg.SetTheFields(TheFuncRotateObj);
    if AddRotateDlg.ShowModal = IDOK then
      begin
        Result := true;
        AddRotateDlg.GetTheFields(TheFuncRotateObj);
        TheFuncRotateObj.DidFieldsChange := true;
      end { if Result = true then}
  finally
    AddRotateDlg.Free;
  end; {try...Finally}
end; {function GetNewRotDlg}

{---------------------------------------------------------------------}

function ModifyRotDlg(TheFuncRotateObj : TFuncRotateObj; TheCenterOnForm : TForm) : boolean;
var
  AddRotateDlg: TAddRotateDlg;
begin {function ModifyRotDlg}
  Result := false;
  AddRotateDlg := TAddRotateDlg.Create(Application);
  try
    SetDlgDefaultPos(AddRotateDlg, TheCenterOnForm);
    AddRotateDlg.IsGetNewFunction := false;
    AddRotateDlg.Caption := 'Modify 2D Function Rotated Into 3D';
    AddRotateDlg.SetTheFields(TheFuncRotateObj);
    if AddRotateDlg.ShowModal = IDOK then
      begin
        Result := true;
        AddRotateDlg.GetTheFields(TheFuncRotateObj);
        TheFuncRotateObj.DidFieldsChange := true;
      end { if Result = true then}
  finally
    AddRotateDlg.Free;
  end; {try...Finally}
end; {function ModifyRotDlg}

{---------------------------------------------------------------------}

constructor TAddRotateDlg.Create(AOwner: TComponent);
begin {constructor TAddRotateDlg.Create}
  inherited Create(AOwner);
  FFuncCalc := TMathRotateFunc.Create;
  with EBeginXStr do
    begin
      emBoundsCheck := [CheckLoBound,LoBoundCanEqual,
                        CheckHiBound,HiBoundCanEqual];
      emLoBoundVal := Min3DDomain;
      emLoBoundStr := Min3DDomainStr;
      emHiBoundVal := Max3DDomain;
      emHiBoundStr := Max3DDomainStr;
    end; {with}
   with EEndXStr do
    begin
      emBoundsCheck := [CheckLoBound,LoBoundCanEqual,
                        CheckHiBound,HiBoundCanEqual];
      emLoBoundVal := Min3DDomain;
      emLoBoundStr := Min3DDomainStr;
      emHiBoundVal := Max3DDomain;
      emHiBoundStr := Max3DDomainStr;
    end; {with}
end; {constructor TAddRotateDlg.Create}

{----------------------------------------------------------------------}

destructor TAddRotateDlg.Destroy;
begin {destructor TAddRotateDlg.Destroy;}
  FFuncCalc.Free;
  inherited Destroy;
end; {destructor TAddRotateDlg.Destroy;}

{----------------------------------------------------------------------}

function TAddRotateDlg.VerifyTheFields : boolean;
begin {function TAddRotateDlg.VerifyTheFields}
  Result := false;
  if EBeginXStr.ValidateMathStr then
  if EEndXStr.ValidateMathStr then
  if ELineP1XStr.ValidateMathStr then
  if ELineP1YStr.ValidateMathStr then
  if ELineP2XStr.ValidateMathStr then
  if ELineP2YStr.ValidateMathStr then
    begin
      if EBeginXStr.LastValue > EEndXStr.LastValue then
        begin
          MessageDlgCenter('Lower Bound must be <= Upper Bound.',mtError,[mbOK],0,self);
          EBeginXStr.SetFocus;
        end
      else
        begin
          FFuncCalc.FuncString := EUserFuncStr.Text;
          Result := FFuncCalc.VerifyMathFunc;
          if not Result then
            begin
              ShowFuncErrorMsg(FFuncCalc.GetLastErrorString,
                               'Function Definition', self);
              EUserFuncStr.SetFocus;
            end; {if}
        end; {else}
    end; {if}
end; {function TAddRotateDlg.VerifyTheFields}

{----------------------------------------------------------------------}

procedure TAddRotateDlg.GetTheFields(TheFuncRotateObj : TFuncRotateObj);
begin {procedure TAddRotateDlg.GetTheFields;}
  with TheFuncRotateObj do
    begin
      FuncCalc.Assign(FFuncCalc);
      BeginXStr := EBeginXStr.text;
      EndXStr := EEndXStr.text;
      BeginXVal := EBeginXStr.LastMathValue;
      EndXVal := EEndXStr.LastMathValue;
      if ERotX.Checked then RotAxis := Rot_XAxis else
        if ERotY.Checked then RotAxis := Rot_YAxis else
          RotAxis := Rot_Line;
      LineP1XStr := ELineP1XStr.text;
      LineP1YStr := ELineP1YStr.text;
      LineP2XStr := ELineP2XStr.text;
      LineP2YStr := ELineP2YStr.text;
      LineP1XVal := ELineP1XStr.LastMathValue;
      LineP1YVal := ELineP1YStr.LastMathValue;
      LineP2XVal := ELineP2XStr.LastMathValue;
      LineP2YVal := ELineP2YStr.LastMathValue;
      Circles := ECircles.Value;
      CircleDetail := ECircleDetail.Value;
      Lines := ELines.Value;
      LineDetail := ELineDetail.Value;
      FuncColor := EFuncSample.ColorValue;
      if ERoundVal.Checked then
        RoundLargeValues := true
      else
        RoundLargeValues := false;
    end; {with}
end; {procedure TAddRotateDlg.GetTheFields;}

{-----------------------------------------------------------------------}

procedure TAddRotateDlg.SetTheFields(TheFuncRotateObj : TFuncRotateObj);

begin {procedure TAddRotateDlg.SetTheFields}
  with TheFuncRotateObj do
    begin
      FFuncCalc.Assign(FuncCalc);
      EUserFuncStr.Text := FuncCalc.FuncString;
      EBeginXStr.Text := BeginXStr;
      EEndXStr.Text := EndXStr;
      case RotAxis of
          Rot_XAxis : ERotX.Checked := true;
          Rot_YAxis : ERotY.Checked := true;
          Rot_Line  : ERotSpecified.Checked := true;
        end; {case}
      ELineP1XStr.Text := LineP1XStr;
      ELineP1YStr.Text := LineP1YStr;
      ELineP2XStr.Text := LineP2XStr;
      ELineP2YStr.Text := LineP2YStr;
      ECircles.Value := Circles;
      ECircleDetail.MaxValue := CalcHighestDetail(Lines);
      ECircleDetail.Value := CircleDetail;
      ELines.Value := Lines;
      ELineDetail.MaxValue := CalcHighestDetail(Circles);
      ELineDetail.Value := LineDetail;
      EFuncSample.ColorValue := FuncColor;
      if RoundLargeValues then
        ERoundVal.Checked := true
      else
        EDontVal.Checked := true;
    end; {with}
end; {procedure TAddRotateDlg.SetTheFields}

{----------------------------------------------------------------------}

procedure TAddRotateDlg.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  ELinesExit(nil);
  ECirclesExit(nil);
  if ModalResult = idOK then
    CanClose := VerifyTheFields
  else
    CanClose := true;
end;

procedure TAddRotateDlg.EFuncColorClick(Sender: TObject);
begin
   ELinesExit(nil);
   ECirclesExit(nil);
   EFuncSample.ColorValue := ReturnNewColorComnDlg(EFuncSample.ColorValue, Self);
end;

procedure TAddRotateDlg.ELinesExit(Sender: TObject);
var
  Tmp : integer;
begin
  Tmp := CalcHighestDetail(ELines.Value);
  if ECircleDetail.Value > Tmp then
    ECircleDetail.Value := Tmp;
  if Tmp <= 0 then
    ECircleDetail.ReadOnly := true
  else
    begin
      ECircleDetail.ReadOnly := false;
      ECircleDetail.MaxValue := Tmp;
    end; {else}
end;

procedure TAddRotateDlg.ECirclesExit(Sender: TObject);
var
  Tmp : integer;
begin
  Tmp := CalcHighestDetail(ECircles.Value);
  if ELineDetail.Value > Tmp then
    ELineDetail.Value := Tmp;
  if Tmp <= 0 then
    ELineDetail.ReadOnly := true
  else
    begin
      ELineDetail.ReadOnly := false;
      ELineDetail.MaxValue := Tmp;
    end; {else}
end;

procedure TAddRotateDlg.FormShow(Sender: TObject);
begin
  if IsGetNewFunction then
    EUserFuncStr.SetFocus;
end;

procedure TAddRotateDlg.FormKeyPress(Sender: TObject; var Key: Char);
begin
  FormKeyPressSpinEditClose(Self, Key);
end;

end.
