unit Axislist;

(*
Copyright 1999-2011 Greg VanMullem
This program is distributed under the terms of the GNU General Public License Version 3

This file is part of MathGV.

MathGV is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

MathGV is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MathGV.  If not, see <http://www.gnu.org/licenses/>.
*)

interface

uses WinTypes, WinProcs, Classes, Graphics, Forms, Controls, Buttons,
  StdCtrls, ExtCtrls, SetPoObj, dialogs, MathCalc, MathFunc, FloatStr,
  editmath, CentrDlg;

type
  TStdAxisListDlg = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    HelpBtn: TBitBtn;
    QuickListBox: TGroupBox;
    ERadio90: TRadioButton;
    ERadio45: TRadioButton;
    ERadioNone: TRadioButton;
    ERadio30: TRadioButton;
    GroupBox1: TGroupBox;
    ERadioIntervals: TRadioButton;
    EIntervalAmt: TEditMathConst;
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
  private
    { Private declarations }
  public
    { Public declarations }
    FIntervalAmount : TMathFloat;
  end;

function ShowStandardAxisListDlg(var TheAxisCount : integer;
                        var TheAxisAngles :TPolarAxisAngles;
                        TheCenterOnForm : TForm) : boolean;

implementation

{$R *.DFM}

{------------------------------------------------------------------}

function ShowStandardAxisListDlg(var TheAxisCount : integer;
                        var TheAxisAngles :TPolarAxisAngles;
                        TheCenterOnForm : TForm) : boolean;
var
  StdAxisListDlg: TStdAxisListDlg;
  I : integer;
  CurrAngle : TMathFloat;
  LoopAgain : boolean;
begin {function ShowStandardAxisListDlg}
  Result := false;
  StdAxisListDlg := TStdAxisListDlg.Create(Application);
  try
    SetDlgDefaultPos(StdAxisListDlg, TheCenterOnForm);
    StdAxisListDlg.FIntervalAmount := 15;
    StdAxisListDlg.EIntervalAmt.text := '15';
    if StdAxisListDlg.ShowModal = idOK then
      begin
        Result := true;
        if StdAxisListDlg.ERadioNone.Checked then
          TheAxisCount := 0
        else
        if StdAxisListDlg.ERadio30.Checked then
          begin
            TheAxisCount := 12;
            for I := 1 to 12 do
              TheAxisAngles[I] := (I-1) * 30;
          end
        else
        if StdAxisListDlg.ERadio45.Checked then
          begin
            TheAxisCount := 8;
            for I := 1 to 8 do
              TheAxisAngles[I] := (I-1) * 45;
          end
        else
        if StdAxisListDlg.ERadio90.Checked then
          begin
            TheAxisCount := 4;
            for I := 1 to 4 do
              TheAxisAngles[I] := (I-1) * 90;
          end
        else {if ERadioIntervals.Checked then}
          begin
            I := 0;
            LoopAgain := true;
            while (I < MaxPolarAxisCount)and (LoopAgain) do
              begin
                CurrAngle := StdAxisListDlg.FIntervalAmount * (I);
                if CurrAngle >= 360 then
                  LoopAgain := false
                else
                  begin
                    inc(I);
                    TheAxisAngles[I] := CurrAngle;
                  end; {else}
              end; {while}
            TheAxisCount := I;
          end; {else}
      end; {if}
  finally
    StdAxisListDlg.free;
  end; {try..finally}
end; {function ShowStandardAxisListDlg}

{------------------------------------------------------------------}

procedure TStdAxisListDlg.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  CanClose := true;
  if (ModalResult = IDOK)and(ERadioIntervals.Checked) then
    begin
      CanClose := EIntervalAmt.ValidateMathStr;
      if CanClose then
        FIntervalAmount := EIntervalAmt.LastValue;
    end; {if}
end;

{------------------------------------------------------------------}

end.
