unit Addpolar;

(*
Copyright 1999-2011 Greg VanMullem
This program is distributed under the terms of the GNU General Public License Version 3

This file is part of MathGV.

MathGV is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

MathGV is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MathGV.  If not, see <http://www.gnu.org/licenses/>.
*)

interface

uses SysUtils, WinTypes, WinProcs, Classes, Graphics, dialogs,
     Forms, Controls, Buttons, StdCtrls, ExtCtrls, Clsample, Spin,
     MathCalc, MathFunc, FuncObj, FuncPolr, ComnDlgs, CentrDlg, editmath;

type
  TAddPolarFuncDlg = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    HelpBtn: TBitBtn;
    FuncStrBox: TGroupBox;
    Label1: TLabel;
    EUserFuncStr: TEdit;
    Label2: TLabel;
    Label3: TLabel;
    GroupBox1: TGroupBox;
    ColorButton1: TButton;
    EFuncColorSample: TColorSample;
    Label4: TLabel;
    ESamples: TSpinEdit;
    ELineWidth: TScrollBar;
    Label5: TLabel;
    LineWidthLabel: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    ELoBoundStr: TEditMathConst;
    EHiBoundStr: TEditMathConst;
    procedure ELineWidthChange(Sender: TObject);
    procedure ColorButton1Click(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormShow(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
  private
    { Private declarations }
    FFuncCalc : TMathPolarFunc;
    function VerifyTheFields : boolean;
  public
    { Public declarations }
    IsGetNewFunction : boolean;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SetTheFields(TheFuncPolarObj : TFuncPolarObj);
    procedure GetTheFields(TheFuncPolarObj : TFuncPolarObj);
  end;

function GetNewPolarDlg(TheFuncPolarObj : TFuncPolarObj; TheCenterOnForm : TForm) : boolean;
function ModifyPolarDlg(TheFuncPolarObj : TFuncPolarObj; TheCenterOnForm : TForm) : boolean;

implementation

{$R *.DFM}

{---------------------------------------------------------------------}

function GetNewPolarDlg(TheFuncPolarObj : TFuncPolarObj; TheCenterOnForm : TForm) : boolean;
var
  AddPolarFuncDlg: TAddPolarFuncDlg;
begin {function GetNewPolarDlg}
  Result := false;
  AddPolarFuncDlg := TAddPolarFuncDlg.Create(Application);
  try
    SetDlgDefaultPos(AddPolarFuncDlg, TheCenterOnForm);
    AddPolarFuncDlg.IsGetNewFunction := true;
    AddPolarFuncDlg.Caption := 'Add Polar Function';
    AddPolarFuncDlg.SetTheFields(TheFuncPolarObj);
    if AddPolarFuncDlg.ShowModal = IDOK then
      begin
        Result := true;
        AddPolarFuncDlg.GetTheFields(TheFuncPolarObj);
        TheFuncPolarObj.DidFieldsChange := true;
      end { if Result = true then}
  finally
    AddPolarFuncDlg.Free;
  end; {try...Finally}
end; {function GetNewPolarDlg}

{---------------------------------------------------------------------}

function ModifyPolarDlg(TheFuncPolarObj : TFuncPolarObj; TheCenterOnForm : TForm) : boolean;
var
  AddPolarFuncDlg: TAddPolarFuncDlg;
begin {function ModifyPolarDlg}
  Result := false;
  AddPolarFuncDlg := TAddPolarFuncDlg.Create(Application);
  try
    SetDlgDefaultPos(AddPolarFuncDlg, TheCenterOnForm);
    AddPolarFuncDlg.IsGetNewFunction := false;
    AddPolarFuncDlg.Caption := 'Modify Polar Function';
    AddPolarFuncDlg.SetTheFields(TheFuncPolarObj);
    if AddPolarFuncDlg.ShowModal = IDOK then
      begin
        Result := true;
        AddPolarFuncDlg.GetTheFields(TheFuncPolarObj);
        TheFuncPolarObj.DidFieldsChange := true;
      end { if Result = true then}
  finally
    AddPolarFuncDlg.Free;
  end; {try...Finally}
end; {function ModifyPolarDlg}

{----------------------------------------------------------------------}

constructor TAddPolarFuncDlg.Create(AOwner: TComponent);
begin {constructor TAddPolarFuncDlg.Create}
  inherited Create(AOwner);
  FFuncCalc := TMathPolarFunc.Create;
  { Add/sub 0.0001 to compensate for float comparison errors }
  with ELoBoundStr do
    begin
      emBoundsCheck := [CheckLoBound,LoBoundCanEqual,
                        CheckHiBound,HiBoundCanEqual];
      emLoBoundVal := (MinPolarDomain-0.0001);
      emLoBoundStr := MinPolarDomainStr;
      emHiBoundVal := (MaxPolarDomain+0.0001);
      emHiBoundStr := MaxPolarDomainStr;
    end; {with}
  with EHiBoundStr do
    begin
      emBoundsCheck := [CheckLoBound,LoBoundCanEqual,
                        CheckHiBound,HiBoundCanEqual];
      emLoBoundVal := (MinPolarDomain-0.0001);
      emLoBoundStr := MinPolarDomainStr;
      emHiBoundVal := (MaxPolarDomain+0.0001);
      emHiBoundStr := MaxPolarDomainStr;
    end; {with}
end; {constructor TAddPolarFuncDlg.Create}

{----------------------------------------------------------------------}

destructor TAddPolarFuncDlg.Destroy;
begin {destructor TAddPolarFuncDlg.Destroy;}
  FFuncCalc.Free;
  inherited Destroy;
end; {destructor TAddPolarFuncDlg.Destroy;}

{----------------------------------------------------------------------}

function TAddPolarFuncDlg.VerifyTheFields : boolean;
begin {function TAddPolarFuncDlg.VerifyTheFields}
  Result := false;
  if ELoBoundStr.ValidateMathStr then
    if EHiBoundStr.ValidateMathStr then
      begin
        if ELoBoundStr.LastValue > EHiBoundStr.LastValue then
          begin
            MessageDlgCenter('Lower Bound must be <= Upper Bound.',
                       mtError, [mbOK], 0, Self);
            ELoBoundStr.SetFocus;
          end
        else
          begin
            FFuncCalc.FuncString := EUserFuncStr.Text;
            Result := FFuncCalc.VerifyMathFunc;
            if not Result then
              begin
                ShowFuncErrorMsg(FFuncCalc.GetLastErrorString,
                                 'Function Definition', self);
                EUserFuncStr.SetFocus;
              end; {if}
          end; {else}
      end; {if}
end; {function TAddPolarFuncDlg.VerifyTheFields}

{---------------------------------------------------------------------}

procedure TAddPolarFuncDlg.SetTheFields(TheFuncPolarObj : TFuncPolarObj);
begin {procedure TAddPolarFuncDlg.SetTheFields}
  with TheFuncPolarObj do
    begin
      FFuncCalc.Assign(FuncCalc);
      EUserFuncStr.Text := FuncCalc.FuncString;
      ELoBoundStr.text := LoBoundStr;
      EHiBoundStr.text := HiBoundStr;
      ESamples.Value := Samples;
      EFuncColorSample.ColorValue := FuncColor;
      ELineWidth.Position := LineWidth;
      LineWidthLabel.Caption := IntToStr(LineWidth);
    end; {with}
end; {procedure TAddPolarFuncDlg.SetTheFields}

{---------------------------------------------------------------------}

procedure TAddPolarFuncDlg.GetTheFields(TheFuncPolarObj : TFuncPolarObj);
begin {procedure TAddPolarFuncDlg.GetTheFields}
  with TheFuncPolarObj do
    begin
      FuncCalc.Assign(FFuncCalc);
      LoBoundStr := ELoBoundStr.text;
      HiBoundStr := EHiBoundStr.text;
      LoBoundVal := ELoBoundStr.LastMathValue;
      HiBoundVal := EHiBoundStr.LastMathValue;
      Samples := ESamples.Value;
      FuncColor := EFuncColorSample.ColorValue;
      LineWidth := ELineWidth.Position;
    end; {with}
end; {procedure TAddPolarFuncDlg.GetTheFields}

{---------------------------------------------------------------------}

procedure TAddPolarFuncDlg.ELineWidthChange(Sender: TObject);
begin
  LineWidthLabel.Caption := IntToStr(ELineWidth.Position);
end;

procedure TAddPolarFuncDlg.ColorButton1Click(Sender: TObject);
begin
  EFuncColorSample.ColorValue := ReturnNewColorComnDlg(EFuncColorSample.ColorValue, Self);
end;

procedure TAddPolarFuncDlg.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  if ModalResult = idOK then
    begin
      CanClose := VerifyTheFields;
    end
  else
    CanClose := true;
end;

procedure TAddPolarFuncDlg.FormShow(Sender: TObject);
begin
  if IsGetNewFunction then
    EUserFuncStr.SetFocus;
end;

procedure TAddPolarFuncDlg.FormKeyPress(Sender: TObject; var Key: Char);
begin
  FormKeyPressSpinEditClose(Self, Key);
end;

end.
