unit Calculat;

(*
Copyright 1999-2011 Greg VanMullem
This program is distributed under the terms of the GNU General Public License Version 3

This file is part of MathGV.

MathGV is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

MathGV is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MathGV.  If not, see <http://www.gnu.org/licenses/>.
*)

interface

uses WinTypes, WinProcs, Classes, Graphics, Forms, Controls, Buttons,
  StdCtrls, dialogs, sysutils, MathCalc, MathFunc, FuncObj, CentrDlg,
  FloatStr;

type
  TCaculatorDlg = class(TForm)
    OKBtn: TBitBtn;
    HelpBtn: TBitBtn;
    FDefBox: TGroupBox;
    Label1: TLabel;
    EFunctionStr: TEdit;
    EVar1Value: TEdit;
    EVar2Value: TEdit;
    Label4: TLabel;
    Label5: TLabel;
    ResultBox: TGroupBox;
    Label6: TLabel;
    EFunctionResult: TEdit;
    FormatBox: TGroupBox;
    EDecimal: TRadioButton;
    EScientific: TRadioButton;
    CalculateButton: TBitBtn;
    EFraction: TRadioButton;
    EVar3Value: TEdit;
    Label2: TLabel;
    EVar1Name: TComboBox;
    Label3: TLabel;
    EVar2Name: TComboBox;
    EVar3Name: TComboBox;
    Label7: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    Label10: TLabel;
    Label11: TLabel;
    EMixedFrac: TRadioButton;
    EIntegerFraction: TRadioButton;
    procedure FormShow(Sender: TObject);
    procedure CalculateButtonClick(Sender: TObject);
    procedure ResultFormatMouseUp(Sender: TObject;
      Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
  private
    { Private declarations }
    MathConstant,
    MathXZFunc : TMathFunction;
    function VerifyVariablesAndValues : boolean;
    function GetVerifyAddVariable(TheVarName : TComboBox; TheVarValue: TEdit;
                                  const VarNumStr : string;
                                  var ResultName : string;
                                  var ResultValue : TMathValue) : boolean;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  end;

procedure ShowCaculatorDlg;

implementation

{$R *.DFM}

{---------------------------------------------------------------------}

procedure ShowCaculatorDlg;
var
  CaculatorDlg: TCaculatorDlg;
begin {procedure ShowCaculatorDlg;}
  CaculatorDlg := TCaculatorDlg.Create(Application);
  try
    SetDlgDefaultPos(CaculatorDlg, nil);
    CaculatorDlg.ShowModal;
  finally
    CaculatorDlg.free;
  end; {try..finally}
end; {procedure ShowCaculatorDlg;}

{---------------------------------------------------------------------}

procedure TCaculatorDlg.FormShow(Sender: TObject);
begin {procedure TCaculatorDlg.FormShow}
  EFunctionStr.SetFocus;
end; {procedure TCaculatorDlg.FormShow}

{---------------------------------------------------------------------}

procedure TCaculatorDlg.CalculateButtonClick(Sender: TObject);
var
  TempHoleNum, TempRemainder : Int64;
begin {procedure TCaculatorDlg.CalculateButtonClick}
  if not VerifyVariablesAndValues then
    Exit;
  MathXZFunc.FuncString := EFunctionStr.Text;
  if not MathXZFunc.VerifyMathFunc then
    begin
      EFunctionResult.text := 'Function Definition: ' +
                              MathXZFunc.GetLastErrorString;
      EFunctionStr.SetFocus;
      Exit;
    end; {if}
  with MathXZFunc do
    begin
      if EvaluateMathFunc then
        begin
          if EIntegerFraction.Checked then
            begin
              if MathResultValue.ValueType = vtFraction then
                begin
                  if MathResultValue.Denominator = 1 then
                    EFunctionResult.text := IntToStr(MathResultValue.Numerator)
                  else
                    EFunctionResult.text := IntToStr(MathResultValue.Numerator) + '/' +
                                            IntToStr(MathResultValue.Denominator);
                end
              else
                EFunctionResult.text := FloatToStrF(FloatResultValue, ffFixed, 18, 18);
            end
          else
          if EDecimal.Checked then
            EFunctionResult.text := FloatToStrF(FloatResultValue, ffFixed, 18, 18)
          else
          if EScientific.Checked then
            EFunctionResult.text := FloatToStrF(FloatResultValue, ffExponent, 18, 18)
          else
          if EFraction.Checked then
            begin
              if MathResultValue.ValueType = vtFraction then
                EFunctionResult.text := IntToStr(MathResultValue.Numerator) + '/' +
                                        IntToStr(MathResultValue.Denominator)
              else
                EFunctionResult.text := FloatToStrF(FloatResultValue, ffFixed, 18, 18);
            end
          else //if EMixedFrac.Checked then
            begin
              if MathResultValue.ValueType = vtFraction then
                begin
                  if (abs(MathResultValue.Numerator) >= abs(MathResultValue.Denominator)) and
                     (MathResultValue.Denominator <> 0) then
                    begin
                      TempHoleNum := MathResultValue.Numerator div MathResultValue.Denominator;
                      TempRemainder := abs(MathResultValue.Numerator mod MathResultValue.Denominator);
                      if TempRemainder = 0 then
                        EFunctionResult.text := IntToStr(TempHoleNum)
                      else
                        EFunctionResult.text := IntToStr(TempHoleNum) + ' and ' +
                          IntToStr(TempRemainder) + '/' + IntToStr(abs(MathResultValue.Denominator));
                    end
                  else
                    EFunctionResult.text := IntToStr(MathResultValue.Numerator) + '/' +
                                            IntToStr(MathResultValue.Denominator);
                end
              else
                EFunctionResult.text := FloatToStrF(FloatResultValue, ffFixed, 18, 18);
            end; {else}
        end
      else
        EFunctionResult.text := 'Error: ' + GetLastErrorString;
    end; {with}
end; {procedure TCaculatorDlg.CalculateButtonClick}

{---------------------------------------------------------------------}

constructor TCaculatorDlg.Create(AOwner: TComponent);
begin {constructor TCaculatorDlg.Create}
  inherited Create(AOwner);
  MathConstant := TMathFunction.Create;
  MathXZFunc := TMathFunction.Create;
end; {constructor TCaculatorDlg.Create}

{---------------------------------------------------------------------}

destructor TCaculatorDlg.Destroy;
begin {destructor TCaculatorDlg.Destroy;}
  MathConstant.free;
  MathXZFunc.free;
  inherited Destroy;
end; {destructor TCaculatorDlg.Destroy;}

{---------------------------------------------------------------------}

function TCaculatorDlg.VerifyVariablesAndValues: boolean;
var
  RName1, RName2, RName3 : string;
  RValue1, RValue2, RValue3 : TMathValue;
{------------------------------------}
function CompareIsNameStrGreater(const S1, S2 : string) : integer;
begin {function CompareIsNameStrGreater}
  if Length(S1) > Length(S2) then
    Result := 1
  else
  if Length(S1) < Length(S2) then
    Result := -1
  else
    Result := CompareText(S1, S2);
end; {function CompareIsNameStrGreater}
{------------------------------------}
procedure SwapNamesAndValues(var S1, S2 : string; var V1, V2 : TMathValue);
var
  TmpS : string;
  TmpV : TMathValue;
begin {procedure SwapNamesAndValues}
  //Swap string
  TmpS := S1;
  S1 := S2;
  S2 := TmpS;
  //swap TMathValue
  TmpV := V1;
  V1 := V2;
  V2 := TmpV;
end; {procedure SwapNamesAndValues}
{------------------------------------}
begin {function TCaculatorDlg.VerifyVariablesAndValues}
  Result := False;
  MathXZFunc.VarCount := 0;
  if GetVerifyAddVariable(EVar1Name, EVar1Value, '1', RName1, RValue1) then
  if GetVerifyAddVariable(EVar2Name, EVar2Value, '2', RName2, RValue2) then
  if GetVerifyAddVariable(EVar3Name, EVar3Value, '3', RName3, RValue3) then
    begin
      Result := True;
      //Eliminate Duplicates
      if (CompareText(RName3, RName1) = 0) or (CompareText(RName3, RName2) = 0) then
        RName3 := '';
      if CompareText(RName2, RName1) = 0 then
        RName2 := '';
      //Sort Values
      if CompareIsNameStrGreater(RName1, RName2) < 0 then
        SwapNamesAndValues(RName1, RName2, RValue1, RValue2);
      if CompareIsNameStrGreater(RName2, RName3) < 0 then
        SwapNamesAndValues(RName2, RName3, RValue2, RValue3);
      if CompareIsNameStrGreater(RName1, RName2) < 0 then
        SwapNamesAndValues(RName1, RName2, RValue1, RValue2);
      //assign values of not empty
      if RName1 <> '' then
        begin
          MathXZFunc.VarCount := MathXZFunc.VarCount + 1;
          MathXZFunc.VarStrings[MathXZFunc.VarCount - 1] := RName1;
          MathXZFunc.MathVarValues[MathXZFunc.VarCount - 1] := RValue1;
        end; {if}
      if RName2 <> '' then
        begin
          MathXZFunc.VarCount := MathXZFunc.VarCount + 1;
          MathXZFunc.VarStrings[MathXZFunc.VarCount - 1] := RName2;
          MathXZFunc.MathVarValues[MathXZFunc.VarCount - 1] := RValue2;
        end; {if}
      if RName3 <> '' then
        begin
          MathXZFunc.VarCount := MathXZFunc.VarCount + 1;
          MathXZFunc.VarStrings[MathXZFunc.VarCount - 1] := RName3;
          MathXZFunc.MathVarValues[MathXZFunc.VarCount - 1] := RValue3;
        end; {if}
    end; {if}
end; {function TCaculatorDlg.VerifyVariablesAndValues}

{---------------------------------------------------------------------}

function TCaculatorDlg.GetVerifyAddVariable(TheVarName: TComboBox;
  TheVarValue: TEdit; const VarNumStr: string;
  var ResultName : string; var ResultValue : TMathValue): boolean;
begin {function TCaculatorDlg.GetVerifyAddVariable}
  Result := False;
  ResultName := '';
  InitMathValue(ResultValue);
  if TheVarName.Text <> '' then
    begin
      if not IsValidVariableString(TheVarName.Text) then
        begin
          EFunctionResult.text := 'Error in Variable Name ' + VarNumStr;
          TheVarName.SetFocus;
          Exit;
        end; {if}
      MathConstant.FuncString := TheVarValue.text;
      if MathConstant.VerifyMathFunc then
        begin
          ResultName := TheVarName.text;
          ResultValue := MathConstant.MathResultValue;
        end
      else
        begin
          EFunctionResult.text := 'Variable Value ' + VarNumStr + ': ' + MathConstant.GetLastErrorString;
          TheVarValue.SetFocus;
          Exit;
        end; {else}
    end; {if}
  Result := True;
end; {function TCaculatorDlg.GetVerifyAddVariable}

{---------------------------------------------------------------------}

procedure TCaculatorDlg.ResultFormatMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin {procedure TCaculatorDlg.ResultFormatMouseUp}
  CalculateButtonClick(Sender);
end; {procedure TCaculatorDlg.ResultFormatMouseUp}

{---------------------------------------------------------------------}

end.
