unit Clipline;

(*
Copyright 1999-2011 Greg VanMullem
This program is distributed under the terms of the GNU General Public License Version 3

This file is part of MathGV.

MathGV is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

MathGV is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MathGV.  If not, see <http://www.gnu.org/licenses/>.
*)

interface

uses sysutils;

function ClipLineToBmp(var X0, X1, Y0, Y1 : integer;
                       InBottomRightY, InBottomRightX : integer) : boolean;

implementation

function ClipLineToBmp(var X0, X1, Y0, Y1 : integer;
                       InBottomRightY, InBottomRightX : integer) : boolean;
{Returns true if cliped line is on bitmap}
{Assumes Bitmap Cordinates where Y increases as we go down the BMP}
const
  TopLeftX = 0;
  TopLeftY = 0;
type
  PClipRec = ^TClipRec;
  TClipRec = record
    X, Y : integer;
  end; {TClipRec}
var
  IsOnBmpP0, IsOnBmpP1 : boolean;
  PHi, PLo : PClipRec;
  ClipP0, ClipP1 : TClipRec;
  BottomRightX, BottomRightY : integer;

{------------------------}

    procedure ClipAginstVert(ClipedPoint : PClipRec; RectXVal: integer);
    var
      MSlope : Extended;
    begin
      try
        MSlope := (ClipP0.Y-ClipP1.Y)/(ClipP0.X-ClipP1.X);
        with ClipedPoint^ do
          begin
            Y := trunc(Y + (RectXVal - X)*(MSlope));
            X := RectXVal;
          end; {with}
      except
        ClipedPoint^.Y := 0;
        ClipedPoint^.X := RectXVal;
      end; {try..except}
    end; {procedure ClipAginstVert}

{------------------------}

    procedure ClipAginstHorz(ClipedPoint : PClipRec; RectYVal: integer);
    var
      MSlope : Extended;
    begin
      try
        MSlope := (ClipP0.X-ClipP1.X)/(ClipP0.Y-ClipP1.Y);
        with ClipedPoint^ do
          begin
            X := trunc(X + (RectYVal - Y)*(MSlope));
            Y := RectYVal;
          end; {with}
      except
        ClipedPoint^.X := 0;
        ClipedPoint^.Y := RectYVal;
      end; {try..except}
    end; {procedure ClipAginstHorz}

{------------------------}

begin {function ClipLineToBmp}
  Result := false;
  BottomRightX := InBottomRightX;
  BottomRightY := InBottomRightY;
  ClipP0.X := X0;
  ClipP0.Y := Y0;
  ClipP1.X := X1;
  ClipP1.Y := Y1;
  IsOnBmpP0 := (((ClipP0.X >= TopLeftX)and(ClipP0.X <= BottomRightX))and
                ((ClipP0.Y >= TopLeftY)and(ClipP0.Y <= BottomRightY)));
  IsOnBmpP1 := (((ClipP1.X >= TopLeftX)and(ClipP1.X <= BottomRightX))and
                ((ClipP1.Y >= TopLeftY)and(ClipP1.Y <= BottomRightY)));
  if (IsOnBmpP0 and IsOnBmpP1) then
    begin
      Result := true;
      exit;
    end
  else
    if ((not IsOnBmpP0) and (not IsOnBmpP1)) then
      begin
        if ((ClipP0.X < TopLeftX)and(ClipP1.X < TopLeftX))or
           ((ClipP0.X > BottomRightX)and(ClipP1.X > BottomRightX))or
           ((ClipP0.Y < TopLeftY)and(ClipP1.Y < TopLeftY))or
           ((ClipP0.Y > BottomRightY)and(ClipP1.Y > BottomRightY))then
          exit; {Line does not cross BMP, Result := false;}
      end; {else}
  if ClipP0.X = ClipP1.X then
    begin
      if ClipP0.Y < ClipP1.Y then
        begin
          PLo := @ClipP0;
          PHi := @ClipP1;
        end
      else
        begin
          PHi := @ClipP0;
          PLo := @ClipP1;
        end; {else}
      if (PHi^.Y >= TopLeftY)and(PLo^.Y < TopLeftY) then
        PLo^.Y := TopLeftY;
      if (PHi^.Y > BottomRightY)and(PLo^.Y <= BottomRightY) then
        PHi^.Y := BottomRightY;
    end
  else
  if ClipP0.Y = ClipP1.Y then
    begin
      if ClipP0.X < ClipP1.X then
        begin {swap them}
          PLo := @ClipP0;
          PHi := @ClipP1;
        end
      else
        begin
          PHi := @ClipP0;
          PLo := @ClipP1;
        end; {else}
      if (PHi^.X >= TopLeftX)and(PLo^.X < TopLeftX) then
        PLo^.X := TopLeftX;
      if (PHi^.X > BottomRightX)and(PLo^.X <= BottomRightX) then
        PHi^.X := BottomRightX;
    end
  else
    begin
      if ClipP0.X < ClipP1.X then
        begin {swap them}
          PLo := @ClipP0;
          PHi := @ClipP1;
        end
      else
        begin
          PHi := @ClipP0;
          PLo := @ClipP1;
        end; {else}
      if (PHi^.X >= TopLeftX)and(PLo^.X < TopLeftX) then
        ClipAginstVert(PLo, TopLeftX);
      if (PHi^.X > BottomRightX)and(PLo^.X <= BottomRightX) then
        ClipAginstVert(PHi, BottomRightX);
      if ClipP0.Y < ClipP1.Y then
        begin
          PLo := @ClipP0;
          PHi := @ClipP1;
        end
      else
        begin
          PHi := @ClipP0;
          PLo := @ClipP1;
        end; {else}
      if (PHi^.Y >= TopLeftY)and(PLo^.Y < TopLeftY) then
        ClipAginstHorz(PLo, TopLeftY);
      if (PHi^.Y > BottomRightY)and(PLo^.Y <= BottomRightY) then
        ClipAginstHorz(PHi, BottomRightY);
    end; {else}
  X0 := ClipP0.X;
  Y0 := ClipP0.Y;
  X1 := ClipP1.X;
  Y1 := ClipP1.Y;
  Result := (((ClipP0.X >= TopLeftX)and(ClipP0.X <= BottomRightX))and
             ((ClipP0.Y >= TopLeftY)and(ClipP0.Y <= BottomRightY)))
           and
            (((ClipP1.X >= TopLeftX)and(ClipP1.X <= BottomRightX))and
             ((ClipP1.Y >= TopLeftY)and(ClipP1.Y <= BottomRightY)));
end; {function ClipLineToBmp}

end.
