unit CentrDlg;

(*
Copyright 1999-2011 Greg VanMullem
This program is distributed under the terms of the GNU General Public License Version 3

This file is part of MathGV.

MathGV is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

MathGV is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MathGV.  If not, see <http://www.gnu.org/licenses/>.
*)

interface

uses
  Windows, Forms, Dialogs, Spin, SysUtils, GrGlobal;

type
  TDefaultDlgPosition = (dpScreenCenter, dpDesktopCenter,
                         dpMainFormCenter, dpSpecifiedFormCenter);

var
  RePositionCommonDlgs : boolean = true;
  DefaultDlgPosition : TDefaultDlgPosition = dpSpecifiedFormCenter;

procedure SetDlgDefaultPos(TheNewForm : TForm; TheCenterOnForm : TForm);

function MessageDlgCenter(const Msg: string; DlgType: TMsgDlgType;
  Buttons: TMsgDlgButtons; HelpCtx: Longint; TheCenterOnForm : TForm): Integer;

procedure SetCommonDlgDefPosOnShow(TheNewCommonDlg : TCommonDialog;
                                   TheCenterOnForm : TForm);

function FormKeyPressSpinEditClose(TheForm: TForm; var Key: Char) : boolean;

function MessageDlgOverMaxFuncCount(MaxFuncCount : Integer): Integer;

procedure MessageDlgErrorOutOfMemoryNoWait;

procedure MessageDlgUnknownErrorNoWait;

implementation

uses
  CentrErr;

const
  ScrnEdgeDist = 10;

{---------------------------------------------------------------------}

function CalcChildCenterCords(var CTop, CLeft : integer; CHeight, CWidth : integer;
                              PTop, PLeft, PHeight, PWidth : integer) : boolean;
var
  TmpScreenWidth, TmpScreenHeight, NewCFormLeft, NewCFormTop : integer;
begin {function CalcChildCenterCords}
  Result := false;
  if (CWidth <= 0) or (CHeight <= 0) then
    Exit;
  if (PWidth <= 0) or (PHeight <= 0) then
    Exit;
  TmpScreenWidth := Screen.Width;   //GetSystemMetrics(SM_CXSCREEN);
  TmpScreenHeight := Screen.Height; //GetSystemMetrics(SM_CYSCREEN);
  if (TmpScreenWidth <= 0) or (TmpScreenHeight <= 0) then
    Exit;
  if (CWidth >= TmpScreenWidth) or (CHeight >= TmpScreenHeight) then
    Exit;
  //set left
  if CWidth >= TmpScreenWidth then
    NewCFormLeft := 0
  else
    begin
      NewCFormLeft := PLeft + ((PWidth - CWidth) div 2);
      //here's where we ensure the form doesn't fall off the screen.
      if (NewCFormLeft + CWidth) > TmpScreenWidth then
        NewCFormLeft := TmpScreenWidth - (CWidth + ScrnEdgeDist);
      if NewCFormLeft < 0 then
        NewCFormLeft := ScrnEdgeDist;
    end; {else}
  //set top
  if CHeight >= TmpScreenHeight then
    NewCFormTop := 0
  else
    begin
      NewCFormTop := PTop + ((PHeight - CHeight) div 2);
      //here's where we ensure the form doesn't fall off the screen.
      if (NewCFormTop + CHeight) > TmpScreenHeight then
        NewCFormTop := TmpScreenHeight - (CHeight + ScrnEdgeDist);
      if NewCFormTop < 0 then
        NewCFormTop := ScrnEdgeDist;
    end; {else}
  CLeft := NewCFormLeft;
  CTop := NewCFormTop;
  Result := true;
end; {function CalcChildCenterCords}

{---------------------------------------------------------------------}

procedure DoCenterOverScreen(CForm : TForm);
begin {procedure DoCenterOverScreen}
  if Assigned(CForm) then
    CForm.Position := poScreenCenter;
end; {procedure DoCenterOverScreen}

{---------------------------------------------------------------------}

procedure DoCenterOverDesktop(CForm : TForm);
begin {procedure DoCenterOverDesktop}
  if Assigned(CForm) then
    CForm.Position := poDesktopCenter;
end; {procedure DoCenterOverDesktop}

{---------------------------------------------------------------------}

procedure DoCenterOverMainForm(CForm : TForm);
var
  NewCFormLeft, NewCFormTop : integer;
begin {procedure DoCenterOverMainForm}
  if Assigned(CForm) then
    begin
      if not Assigned(Application) then
        begin
          DoCenterOverScreen(CForm);
          exit;
        end; {if}
      if not Assigned(Application.MainForm) then
        begin
          DoCenterOverScreen(CForm);
          exit;
        end; {if}
      if CalcChildCenterCords(NewCFormTop, NewCFormLeft,
                              CForm.Height, CForm.Width,
                              Application.MainForm.Top, Application.MainForm.Left,
                              Application.MainForm.Height, Application.MainForm.Width) then
        begin
          CForm.Position := poDesigned;
          CForm.Top := NewCFormTop;
          CForm.Left := NewCFormLeft;
        end
      else
        DoCenterOverScreen(CForm);
    end; {if}
end; {procedure DoCenterOverMainForm}

{---------------------------------------------------------------------}

procedure DoCenterOverSpecifiedDlg(CForm, PForm : TForm);
var
  NewCFormLeft, NewCFormTop : integer;
begin {procedure DoCenterOverSpecifiedDlg}
  if Assigned(CForm) then
    begin
      if not Assigned(PForm) then
        begin
          DoCenterOverMainForm(CForm);
          exit;
        end; {if}
      if CalcChildCenterCords(NewCFormTop, NewCFormLeft,
                              CForm.Height, CForm.Width,
                              PForm.Top, PForm.Left,
                              PForm.Height, PForm.Width) then
        begin
          CForm.Position := poDesigned;
          CForm.Top := NewCFormTop;
          CForm.Left := NewCFormLeft;
        end
      else
        DoCenterOverScreen(CForm);
    end; {if}
end; {procedure DoCenterOverSpecifiedDlg}

{---------------------------------------------------------------------}

procedure SetDlgDefaultPos(TheNewForm : TForm; TheCenterOnForm : TForm);
begin {procedure SetDlgDefaultPos}
  try
    case DefaultDlgPosition of
      dpScreenCenter : DoCenterOverScreen(TheNewForm);
      dpDesktopCenter : DoCenterOverDesktop(TheNewForm);
      dpMainFormCenter : DoCenterOverMainForm(TheNewForm);
      dpSpecifiedFormCenter : DoCenterOverSpecifiedDlg(TheNewForm, TheCenterOnForm);
    end; {case}
  except
    TheNewForm.Position := poDesktopCenter;
  end; {try..except}
end; {procedure SetDlgDefaultPos}

{---------------------------------------------------------------------}

function MessageDlgCenter(const Msg: string; DlgType: TMsgDlgType;
  Buttons: TMsgDlgButtons; HelpCtx: Longint; TheCenterOnForm : TForm): Integer;
var
  TmpMsgDlg : TForm;
begin {function MessageDlgCenter}
  TmpMsgDlg := CreateMessageDialog(Msg, DlgType, Buttons);
  try
    TmpMsgDlg.HelpContext := HelpCtx;
    SetDlgDefaultPos(TmpMsgDlg, TheCenterOnForm);
    Result := TmpMsgDlg.ShowModal;
  finally
    TmpMsgDlg.Free;
  end;
end; {function MessageDlgCenter}

{---------------------------------------------------------------------}

procedure CalcComnDlgScreenDeskTop(TheNewCommonDlg : TCommonDialog;
            TmpScreenWidth, TmpScreenHeight : integer);
var
  TmpHandle : HWnd;
  Rect : TRect;
  ComnDlgWidth, ComnDlgHeight : integer;
  NewLeft, NewTop : integer;
begin {procedure CalcComnDlgScreenDeskTop}
  if (TmpScreenWidth <= 0) or (TmpScreenHeight <= 0) then
    Exit;
  TmpHandle := TheNewCommonDlg.Handle;
  if not GetWindowRect(TmpHandle, Rect) then
    Exit;
  ComnDlgWidth := Rect.Right - Rect.Left;
  ComnDlgHeight := Rect.Bottom - Rect.Top;
  if (ComnDlgWidth <= 0) or (ComnDlgHeight <= 0) then
    Exit;
  if (ComnDlgWidth >= TmpScreenWidth) or (ComnDlgHeight >= TmpScreenHeight) then
    Exit;
  NewLeft := (TmpScreenWidth  - ComnDlgWidth) div 2;
  NewTop := (TmpScreenHeight - ComnDlgHeight) div 2;
  //Make sure dialog does not fall off desktop
  //fix Left
  if (NewLeft + ComnDlgWidth) > TmpScreenWidth then
    NewLeft := TmpScreenWidth - (ComnDlgWidth + ScrnEdgeDist);
  if NewLeft < 0 then
    NewLeft := ScrnEdgeDist;
  //fix Top
  if (NewTop + ComnDlgHeight) > TmpScreenHeight then
    NewTop := TmpScreenHeight - (ComnDlgHeight + ScrnEdgeDist);
  if NewTop < 0 then
    NewTop := ScrnEdgeDist;
  SetWindowPos(TmpHandle, 0, Newleft, Newtop, 0, 0, (SWP_NOSIZE + SWP_NOZORDER));
end; {procedure CalcComnDlgScreenDeskTop}

{---------------------------------------------------------------------}

procedure DoComnDlgCenterScreen(TheNewCommonDlg : TCommonDialog);
begin {procedure DoComnDlgCenterScreen}
  if Assigned(TheNewCommonDlg) then
    CalcComnDlgScreenDeskTop(TheNewCommonDlg, Screen.Width, Screen.Height);
end; {procedure DoComnDlgCenterScreen}

{---------------------------------------------------------------------}

procedure DoComnDlgCenterDesktop(TheNewCommonDlg : TCommonDialog);
begin {procedure DoComnDlgCenterDesktop}
  if Assigned(TheNewCommonDlg) then
    CalcComnDlgScreenDeskTop(TheNewCommonDlg, Screen.DeskTopWidth, Screen.DeskTopHeight);
end; {procedure DoComnDlgCenterDesktop}

{---------------------------------------------------------------------}

procedure CalcComnDlgCenterOnForm(TheNewCommonDlg : TCommonDialog;
                                  TheCenterOnForm : TForm);
var
  TmpHandle : HWnd;
  TmpScreenWidth, TmpScreenHeight, NewFormLeft, NewFormTop : integer;
  Rect : TRect;
  ComnDlgWidth, ComnDlgHeight : integer;
begin {procedure CalcComnDlgCenterOnForm}
  if not Assigned(TheCenterOnForm) then
    begin
      DoComnDlgCenterScreen(TheNewCommonDlg);
      exit;
    end; {if}
  TmpScreenWidth := Screen.Width;   //GetSystemMetrics(SM_CXSCREEN);
  TmpScreenHeight := Screen.Height; //GetSystemMetrics(SM_CYSCREEN);
  if (TmpScreenWidth <= 0) or (TmpScreenHeight <= 0) then
    Exit;
  TmpHandle := TheNewCommonDlg.Handle;
  if not GetWindowRect(TmpHandle, Rect) then
    Exit;
  ComnDlgWidth := Rect.Right - Rect.Left;
  ComnDlgHeight := Rect.Bottom - Rect.Top;
  if (ComnDlgWidth <= 0) or (ComnDlgHeight <= 0) then
    Exit;
  if (ComnDlgWidth >= TmpScreenWidth) or (ComnDlgHeight >= TmpScreenHeight) then
    Exit;
  //set left
  NewFormLeft := TheCenterOnForm.Left + ((TheCenterOnForm.Width - ComnDlgWidth) div 2);
  //here's where we ensure the form doesn't fall off the screen.
  if (NewFormLeft + ComnDlgWidth) > TmpScreenWidth then
    NewFormLeft := TmpScreenWidth - (ComnDlgWidth + ScrnEdgeDist);
  if NewFormLeft < 0 then
    NewFormLeft := ScrnEdgeDist;
  //set top
  NewFormTop := TheCenterOnForm.Top + ((TheCenterOnForm.Height - ComnDlgHeight) div 2);
  //here's where we ensure the form doesn't fall off the screen.
  if (NewFormTop + ComnDlgHeight) > TmpScreenHeight then
    NewFormTop := TmpScreenHeight - (ComnDlgHeight + ScrnEdgeDist);
  if NewFormTop < 0 then
    NewFormTop := ScrnEdgeDist;
  SetWindowPos(TmpHandle, 0, NewFormLeft, NewFormTop, 0, 0, (SWP_NOSIZE + SWP_NOZORDER));
end; {procedure CalcComnDlgCenterOnForm}

{---------------------------------------------------------------------}

procedure DoComnDlgCenterMainForm(TheNewCommonDlg : TCommonDialog);
begin {procedure DoComnDlgCenterMainForm}
  if Assigned(TheNewCommonDlg) then
    CalcComnDlgCenterOnForm(TheNewCommonDlg, Application.MainForm);
end; {procedure DoComnDlgCenterMainForm}

{---------------------------------------------------------------------}

procedure DoComnDlgCenterSpecifiedDlg(TheNewCommonDlg : TCommonDialog;
                                      TheCenterOnForm : TForm);
begin {procedure DoComnDlgCenterSpecifiedDlg}
  if Assigned(TheNewCommonDlg) then
    begin
      if Assigned(TheCenterOnForm) then
        CalcComnDlgCenterOnForm(TheNewCommonDlg, TheCenterOnForm)
      else
        DoComnDlgCenterMainForm(TheNewCommonDlg);
    end; {if}
end; {procedure DoComnDlgCenterSpecifiedDlg}

{---------------------------------------------------------------------}

procedure SetCommonDlgDefPosOnShow(TheNewCommonDlg : TCommonDialog;
                                   TheCenterOnForm : TForm);
begin {procedure SetCommonDlgDefPosOnShow}
  try
    if RePositionCommonDlgs then
      case DefaultDlgPosition of
        dpScreenCenter : DoComnDlgCenterScreen(TheNewCommonDlg);
        dpDesktopCenter : DoComnDlgCenterDesktop(TheNewCommonDlg);
        dpMainFormCenter : DoComnDlgCenterMainForm(TheNewCommonDlg);
        dpSpecifiedFormCenter : DoComnDlgCenterSpecifiedDlg(TheNewCommonDlg, TheCenterOnForm)
      end; {case}
  except
  end; {try..except}
end; {procedure SetCommonDlgDefPosOnShow}

{---------------------------------------------------------------------}

function FormKeyPressSpinEditClose(TheForm: TForm; var Key: Char) : boolean;
begin {function FormKeyPressSpinEditClose}
  Result := false;
  try
    if (key = #13) or (key = #27) then
      if Assigned(TheForm) then
        if TheForm.Active then
          if Assigned(TheForm.ActiveControl) then
            if (TheForm.ActiveControl is TSpinEdit) then
              begin
                Result := true;
                if key = #13 then
                  TheForm.ModalResult := idOK  //closes form
                else
                  TheForm.ModalResult := idCancel; //closes form
                Key := char(0);
              end; {if}
  except
    Result := false;
  end; {try..except}
end; {function FormKeyPressSpinEditClose}

{---------------------------------------------------------------------}

function MessageDlgOverMaxFuncCount(MaxFuncCount : Integer): Integer;
begin {function MessageDlgOverMaxFuncCount}
  Result := MessageDlgCenter('This window can not accept more than '
              + IntToStr(MaxFuncCount) + ' functions.', mtError, [mbOK], 0, nil);
end; {function MessageDlgOverMaxFuncCount}

{---------------------------------------------------------------------}

procedure MessageDlgErrorOutOfMemoryNoWait;
begin {procedure MessageDlgErrorOutOfMemoryNoWait;}
  try
    ErrorDlgCenterShowNoWait(ErrorStr_OutOfMemory, nil);
  except
  end; {try..except}
end; {procedure MessageDlgErrorOutOfMemoryNoWait;}

{---------------------------------------------------------------------}

procedure MessageDlgUnknownErrorNoWait;
begin {procedure MessageDlgUnknownErrorNoWait;}
  try
    ErrorDlgCenterShowNoWait(ErrorStr_UnknownError, nil);
  except
  end; {try..except}
end; {procedure MessageDlgUnknownErrorNoWait;}

{---------------------------------------------------------------------}

end.
