unit DefFont;

(*
Copyright 1999-2011 Greg VanMullem
This program is distributed under the terms of the GNU General Public License Version 3

This file is part of MathGV.

MathGV is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

MathGV is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MathGV.  If not, see <http://www.gnu.org/licenses/>.
*)

interface

uses SyncObjs, Graphics;

type
  TDefaultFontHolder = class
  private
    { Private declarations }
    FDefFont : TFont;
    CriticalSection : TCriticalSection;
  public
    { Public declarations }
    constructor Create;
    destructor Destroy; override;
    function NewTFontClass : TFont;
    procedure GetFontAttributes(TheFont : TFont);
    procedure SetFontAttributes(TheFont : TFont);
    procedure ResetToDefault;
  end; {TDefaultFontHolder}

var
  DefaultFontHolder : TDefaultFontHolder;

function CreateANewTFontClass : TFont;
function AreTFontClassesEqual(TheFont1, TheFont2 : TFont) : boolean;

implementation

{---------------------------------------------------------------------}

function CreateANewTFontClass : TFont;
begin {function CreateANewTFontClass}
  Result := DefaultFontHolder.NewTFontClass;
end; {function CreateANewTFontClass}

{---------------------------------------------------------------------}

function AreTFontClassesEqual(TheFont1, TheFont2 : TFont) : boolean;
begin {function AreTFontClassesEqual}
  Result := false;
  if TheFont1 = TheFont2 then
    Result := true
  else
    begin
      if (TheFont1 <> nil) and (TheFont2 <> nil) then
        begin
          if (TheFont1.Name = TheFont2.Name) and
             (TheFont1.Color = TheFont2.Color) and
             (TheFont1.Style = TheFont2.Style) and
             (TheFont1.Size = TheFont2.Size) then
            Result := true;
        end; {if}
    end; {else}
end; {function AreTFontClassesEqual}

{---------------------------------------------------------------------}

{ TDefaultFontHolder }

constructor TDefaultFontHolder.Create;
begin {constructor TDefaultFontHolder.Create;}
  inherited;
  CriticalSection := TCriticalSection.Create;
  FDefFont := TFont.Create;
end; {constructor TDefaultFontHolder.Create;}

{---------------------------------------------------------------------}

function TDefaultFontHolder.NewTFontClass: TFont;
begin {function TDefaultFontHolder.NewTFontClass}
  CriticalSection.Acquire;
  try
    Result := TFont.Create;
    Result.Assign(FDefFont);
  finally
    CriticalSection.Release;
  end; {try..finally}
end; {function TDefaultFontHolder.NewTFontClass}

{---------------------------------------------------------------------}

destructor TDefaultFontHolder.Destroy;
begin {destructor TDefaultFontHolder.Destroy;}
  FDefFont.Free;
  CriticalSection.Free;
  inherited;
end; {destructor TDefaultFontHolder.Destroy;}

{---------------------------------------------------------------------}

procedure TDefaultFontHolder.GetFontAttributes(TheFont: TFont);
begin {procedure TDefaultFontHolder.GetFontAttributes}
  Assert(Assigned(TheFont), 'Null value in TDefaultFontHolder.GetFontAttributes()');
  CriticalSection.Acquire;
  try
    TheFont.Assign(FDefFont);
  finally
    CriticalSection.Release;
  end; {try..finally}
end; {procedure TDefaultFontHolder.GetFontAttributes}

{---------------------------------------------------------------------}

procedure TDefaultFontHolder.ResetToDefault;
var
  TmpFont : TFont;
begin {procedure TDefaultFontHolder.ResetToDefault;}
  CriticalSection.Acquire;
  try
    TmpFont := TFont.Create;
    try
      if Assigned(TmpFont) then
        FDefFont.Assign(TmpFont);
    finally
      TmpFont.Free;
    end; {try..finally}
  finally
    CriticalSection.Release;
  end; {try..finally}
end; {procedure TDefaultFontHolder.ResetToDefault;}

{---------------------------------------------------------------------}

procedure TDefaultFontHolder.SetFontAttributes(TheFont: TFont);
begin {procedure TDefaultFontHolder.SetFontAttributes}
  Assert(Assigned(TheFont), 'Null value in TDefaultFontHolder.SetFontAttributes()');
  CriticalSection.Acquire;
  try
    FDefFont.Assign(TheFont);
  finally
    CriticalSection.Release;
  end; {try..finally}
end; {procedure TDefaultFontHolder.SetFontAttributes}

{---------------------------------------------------------------------}

initialization
  DefaultFontHolder := TDefaultFontHolder.Create;

finalization
  DefaultFontHolder.Free;

{---------------------------------------------------------------------}

end.
