unit DgGlobal;

(*
Copyright 1999-2011 Greg VanMullem
This program is distributed under the terms of the GNU General Public License Version 3

This file is part of MathGV.

MathGV is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

MathGV is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MathGV.  If not, see <http://www.gnu.org/licenses/>.
*)

interface

{This unit contains code used by the MathGV GUI}

uses Windows, Graphics, Classes, Forms, FuncObj;

const
  MaxOpenGraphWindowCount = 12; //was 16 in version 3.1, set to 12 in version 4.
  TrueTypeFontSizes = '08'#13'09'#13'10'#13'11'#13'12'#13'14'#13'16'#13'18'#13+
                      '20'#13'22'#13'24'#13'26'#13'28'#13'36'#13'48'#13'72';

function NewUserFunctionObj(FuncObj : TBaseFuncObj; DefaultColor : TColor;
                            TheCenterOnForm : TForm) : boolean;
function ModifyUserFunctionObj(FuncObj : TBaseFuncObj; TheCenterOnForm : TForm) : boolean;

function GetScreenBMPPixelFormat : TPixelFormat;

implementation

uses Func2D, Add2DFnc, FuncPara, AddPara,
  FuncPolr, Addpolar, Func3D, AdRotDlg, Add3DDlg;

{---------------------------------------------------------------------}

function NewUserFunctionObj(FuncObj : TBaseFuncObj; DefaultColor : TColor;
                            TheCenterOnForm : TForm) : boolean;
begin {function NewUserFunctionObj}
  Result := false;
  try
    if FuncObj is TFunc2DObj then
      begin
        TFunc2DObj(FuncObj).FuncColor := DefaultColor;
        Result := GetNew2DDlg(TFunc2DObj(FuncObj), TheCenterOnForm);
      end
    else
    if FuncObj is TFuncParametricObj then
      begin
        TFuncParametricObj(FuncObj).FuncColor := DefaultColor;
        Result := GetNewParametricDlg(TFuncParametricObj(FuncObj), TheCenterOnForm);
      end
    else
    if FuncObj is TFuncPolarObj then
      begin
        TFuncPolarObj(FuncObj).FuncColor := DefaultColor;
        Result := GetNewPolarDlg(TFuncPolarObj(FuncObj), TheCenterOnForm);
      end
    else
    if FuncObj is TFuncRotateObj then
      begin
        TFuncRotateObj(FuncObj).FuncColor := DefaultColor;
        Result := GetNewRotDlg(TFuncRotateObj(FuncObj), TheCenterOnForm);
      end
    else
    if FuncObj is TFunc3DObj then
      begin
        TFunc3DObj(FuncObj).FuncColor := DefaultColor;
        Result := GetNewC3DDlg(TFunc3DObj(FuncObj), TheCenterOnForm);
      end;
  except
    Result := False;
  end; {try..except}
end; {function NewUserFunctionObj}

{---------------------------------------------------------------------}

function ModifyUserFunctionObj(FuncObj : TBaseFuncObj; TheCenterOnForm : TForm) : boolean;
begin {function ModifyUserFunctionObj}
  Result := false;
  try
    if FuncObj is TFunc2DObj then
      Result := Modify2DDlg(TFunc2DObj(FuncObj), TheCenterOnForm)
    else
    if FuncObj is TFuncParametricObj then
      Result := ModifyParametricDlg(TFuncParametricObj(FuncObj), TheCenterOnForm)
    else
    if FuncObj is TFuncPolarObj then
      Result := ModifyPolarDlg(TFuncPolarObj(FuncObj), TheCenterOnForm)
    else
    if FuncObj is TFuncRotateObj then
      Result := ModifyRotDlg(TFuncRotateObj(FuncObj), TheCenterOnForm)
    else
    if FuncObj is TFunc3DObj then
      Result := ModifyC3DDlg(TFunc3DObj(FuncObj), TheCenterOnForm);
  except
    Result := False;
  end; {try..except}
end; {function ModifyUserFunctionObj}

{---------------------------------------------------------------------}

function GetScreenBMPPixelFormat : TPixelFormat;
var
  BitCount, TmpBitsPixel, TmpPlanes : integer;
  TmpHDC : HDC;
begin {function GetScreenBMPPixelFormat}
  try
    TmpHDC := GetDC(0);
    TmpBitsPixel := GetDeviceCaps(TmpHDC, BITSPIXEL);
    TmpPlanes := GetDeviceCaps(TmpHDC, PLANES);
    ReleaseDC(0, TmpHDC);
    BitCount := TmpBitsPixel * TmpPlanes;
    case BitCount of
      4: Result := pf4bit;
      8: Result := pf8bit;
      15: Result := pf15bit;
      16: Result := pf16bit;
      24: Result := pf24bit;
      32: Result := pf32bit;
      else
        Result := pf8bit;
    end; {case}
  except
    Result := pf8bit;
  end; {try..except}
end; {function GetScreenBMPPixelFormat}

{---------------------------------------------------------------------}

end.
