unit Floatstr;

(*
Copyright 1999-2011 Greg VanMullem
This program is distributed under the terms of the GNU General Public License Version 3

This file is part of MathGV.

MathGV is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

MathGV is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MathGV.  If not, see <http://www.gnu.org/licenses/>.
*)

interface

uses sysutils, MathCalc, MathFunc;

type
  TFloatNumFormat = (lfMaxDecimal, lfFixedDecimal,
                     lfScientific, lfMaxCurrency, lfFixedCurrency);
const
  NumLabelFormatCount = 5;
  NumLabelFormatStrArray : array[1..NumLabelFormatCount] of string = (
    'Max Decimal', 'Fixed Decimal', 'Scientific',
    'Currency Max Dec', 'Currency Fixed Dec');

function ComboToTFloatNumFormat(TheComboNDX : integer) : TFloatNumFormat;
function TFloatNumFormatToCombo(TheNLFormat : TFloatNumFormat) : integer;
function FloatCordToFormatStr(TheFloatCord : TMathFloat;
           TheFloatNumFormat : TFloatNumFormat; ShowComaChar : boolean;
           ShowPlusSignChar : boolean; DecimalPlaces : integer) : string;
procedure GetFloatCordDispOptions(TheFloatNumFormat : TFloatNumFormat;
           var ComaCharEnabled : boolean; var PlusSignCharEnabled : boolean;
           var DecimalPlacesLabelText : string);
function GetFloatCordDecPlacesDefault(
                         TheFloatNumFormat : TFloatNumFormat) : integer;

procedure CleanCurrencyString(var TheStr : string);
procedure CleanFloatString(var TheStr : string);

function FloatCordToStrF(TheFloatCord : TMathFloat): string;
function Float2DZoomToStrF(TheFloatCord : TMathFloat): string;

function StrToFloatCord(const TheFloatString : string;
                        var TheFloatCord : TMathFloat) : boolean;

function RotateNumTo360D(TheRotateNum : TMathFloat) : TMathFloat;
function RotateNumToStr(TheRotateNum : TMathFloat): string;
{function StrToRotateNum(const TheStr : string; var TheRotateNum : TMathFloat) : boolean;}
function PositiveRotToStr(TheRotateNum : TMathFloat): string;
function StrToPositiveRot(const TheStr : string;
                        var TheRotateNum : TMathFloat) : boolean;

implementation

{---------------------------------------------------------------------}

function ComboToTFloatNumFormat(TheComboNDX : integer) : TFloatNumFormat;
{ This function assumes the following combo box string list.
Max Decimal Places
Fixed Decimal Places
Scientific
Currency Max Decimal Places
Currency Fixed Decimal Places
}
begin {function ComboToTFloatNumFormat}
  case TheComboNDX of
    0: Result := lfMaxDecimal;
    1: Result := lfFixedDecimal;
    2: Result := lfScientific;
    3: Result := lfMaxCurrency;
    4: Result := lfFixedCurrency;
  else
    Result := lfMaxDecimal;
  end; {case}
end; {function ComboToTFloatNumFormat}

{---------------------------------------------------------------------}

function TFloatNumFormatToCombo(TheNLFormat : TFloatNumFormat) : integer;
begin {function TFloatNumFormatToCombo}
  case TheNLFormat of
    lfMaxDecimal: Result := 0;
    lfFixedDecimal: Result := 1;
    lfScientific: Result := 2;
    lfMaxCurrency: Result := 3;
    lfFixedCurrency: Result := 4;
  else
    Result := 0;
  end; {case}
end; {function TFloatNumFormatToCombo}

{---------------------------------------------------------------------}

function FloatCordToFormatStr(TheFloatCord : TMathFloat;
           TheFloatNumFormat : TFloatNumFormat; ShowComaChar : boolean;
           ShowPlusSignChar : boolean; DecimalPlaces : integer) : string;
begin {function FloatCordToFormatStr}
  if DecimalPlaces < 0 then
    DecimalPlaces := 0;
  if DecimalPlaces > 18 then
    DecimalPlaces := 18;
  case TheFloatNumFormat of
    lfMaxDecimal: begin
      if ShowComaChar then
        Result := FloatToStrF(TheFloatCord, ffNumber, 18, DecimalPlaces)
      else
        Result := FloatToStrF(TheFloatCord, ffFixed, 18, DecimalPlaces);
      CleanFloatString(Result);
    end; {lfMaxDecimal:}
    lfFixedDecimal: begin
      if ShowComaChar then
        Result := FloatToStrF(TheFloatCord, ffNumber, 18, DecimalPlaces)
      else
        Result := FloatToStrF(TheFloatCord, ffFixed, 18, DecimalPlaces);
    end; {lfFixedDecimal:}
    lfScientific: begin
      Result := FloatToStrF(TheFloatCord, ffExponent, DecimalPlaces, 1);
    end; {lfScientific:}
    lfMaxCurrency: begin
      Result := FloatToStrF(TheFloatCord, ffCurrency, 18, DecimalPlaces);
      CleanCurrencyString(Result);
    end; {lfMaxCurrency:}
    lfFixedCurrency: begin
      Result := FloatToStrF(TheFloatCord, ffCurrency, 18, DecimalPlaces);
    end; {lfFixedCurrency:}
  else
    Result := FloatToStr(TheFloatCord);
  end; {case}
  if Result = '' then
    Result := '0';
  if ShowPlusSignChar and
     (not(TheFloatNumFormat in [lfMaxCurrency,lfFixedCurrency])) then
    if not (Result[1] in ['+','-']) then
      begin
        if TheFloatCord < 0 then
          Result := '-' + Result
        else
          if TheFloatCord > 0 then
            Result := '+' + Result;
      end; {if}
end; {function FloatCordToFormatStr}

{---------------------------------------------------------------------}

procedure GetFloatCordDispOptions(TheFloatNumFormat : TFloatNumFormat;
           var ComaCharEnabled : boolean; var PlusSignCharEnabled : boolean;
           var DecimalPlacesLabelText : string);
const
  TheLableText_DecPlac : string = 'Decimal Places';
  TheLableText_Precision : string = 'Precision';
begin {procedure GetFloatCordDispOptions}
  case TheFloatNumFormat of
    lfMaxDecimal, lfFixedDecimal:
      begin
        ComaCharEnabled := true;
        PlusSignCharEnabled := true;
        DecimalPlacesLabelText := TheLableText_DecPlac;
      end;
    lfScientific:
      begin
        ComaCharEnabled := false;
        PlusSignCharEnabled := true;
        DecimalPlacesLabelText := TheLableText_Precision;
      end;
    lfMaxCurrency, lfFixedCurrency:
      begin
        ComaCharEnabled := false;
        PlusSignCharEnabled := false;
        DecimalPlacesLabelText := TheLableText_DecPlac;
      end;
  else
    ComaCharEnabled := true;
    PlusSignCharEnabled := true;
    DecimalPlacesLabelText := TheLableText_DecPlac;
  end; {case}
end; {procedure GetFloatCordDispOptions}

{---------------------------------------------------------------------}

function GetFloatCordDecPlacesDefault(
                         TheFloatNumFormat : TFloatNumFormat) : integer;
begin {function GetFloatCordDecPlacesDefault}
  case TheFloatNumFormat of
    lfScientific: Result := 3;
    lfMaxDecimal, lfFixedDecimal: Result := 1;
    lfMaxCurrency, lfFixedCurrency: Result := 2;
  else
    Result := 1;
  end; {case..else}
end; {function GetFloatCordDecPlacesDefault}

{---------------------------------------------------------------------}

procedure CleanCurrencyString(var TheStr : string);
{This procedure assumes that TheStr is a properly formated currency string}
var
  {CurrChar : char;}
  I : integer;
  ScientificNote : boolean;
  DecimalPointCount : integer;
  FirstDigitPos, LastDigitPos : integer;
begin {procedure CleanCurrencyString}
  {Begin: remove all blank spaces from TheStr}
  I := 1;
  while I <= length(TheStr) do
    begin
      if TheStr[I] = ' ' then
        delete(TheStr, I, 1)
      else
        inc(I);
    end; {while}
  {End: remove all blank spaces from TheStr}
  ScientificNote := false;
  DecimalPointCount := 0;
  FirstDigitPos := 0;
  LastDigitPos := 0;
  {scan the string for important information}
  for I := 1 to length(TheStr) do
    begin
      if (TheStr[I] in ['0'..'9']) then
        begin
          if FirstDigitPos = 0 then
            FirstDigitPos := I;
          LastDigitPos := I;
        end
      else
        if (TheStr[I] = DecimalSeparator) then //DecimalSeparator = '.'
          begin
            inc(DecimalPointCount);
            if FirstDigitPos = 0 then
              FirstDigitPos := I;
            LastDigitPos := I;
          end
        else
          begin
            if TheStr[I] in ['E','e'] then
              ScientificNote := true
          end; {else}
    end; {for}
  if (ScientificNote = false) and (DecimalPointCount = 1) and
     (LastDigitPos <> 0) then
     begin
       {Strip traling zeroes}
       while (LastDigitPos > FirstDigitPos)and(TheStr[LastDigitPos] = '0')do
         begin
           delete(TheStr, LastDigitPos, 1);
           dec(LastDigitPos);
         end; {while}
       if (Length(TheStr) >= 1) and
          (TheStr[LastDigitPos] = DecimalSeparator) then
         begin
           if (Length(TheStr) = 1) then
             TheStr[1] := '0'
           else
             begin
               if (TheStr[LastDigitPos-1] in ['0'..'9']) then
                 delete(TheStr, LastDigitPos, 1)
               else
                 TheStr[LastDigitPos] := '0';
             end; {else}
         end; {if}
     end; {if}
  {make sure string is a valid number if it = ''}
  if TheStr = '' then
    TheStr := '0';
end; {procedure CleanCurrencyString}

{---------------------------------------------------------------------}

procedure CleanFloatString(var TheStr : string);
var
  I : integer;
  ScientificNote : boolean;
  DecimalPointCount : integer;
begin {procedure CleanFloatString}
  {Begin: remove all blank spaces from TheStr}
  I := 1;
  while I <= length(TheStr) do
    begin
      if TheStr[I] = ' ' then
        delete(TheStr, I, 1)
      else
        inc(I);
    end; {while}
  {End: remove all blank spaces from TheStr}
  ScientificNote := false;
  DecimalPointCount := 0;
  {scan the string for important information}
  for I := 1 to length(TheStr) do
    begin
      if (TheStr[I] = DecimalSeparator) then //DecimalSeparator = '.'
        inc(DecimalPointCount)
      else
        if TheStr[I] in ['E','e'] then
          ScientificNote := true;
    end; {for}
  {Strip traling zeroes}
  if (length(TheStr) >= 1) then
    begin
      if (not ScientificNote) and (DecimalPointCount = 1) then
        begin
          while (Length(TheStr) >= 1) and (TheStr[Length(TheStr)] = '0') do
            SetLength(TheStr, Length(TheStr)-1); //dec(TheStr[0])
          if (Length(TheStr) >= 1) and
             (TheStr[Length(TheStr)] = DecimalSeparator) then
            begin
              if (Length(TheStr) = 1) then
                TheStr[1] := '0'
              else
                begin
                  if (TheStr[Length(TheStr)-1] in ['0'..'9']) then
                    SetLength(TheStr, Length(TheStr)-1) //dec(TheStr[0])
                  else
                    TheStr[Length(TheStr)] := '0';
                end; {else}
            end; {if}
        end; {if}
    end; {if}
  {make sure string is a valid number if it = ''}
  if TheStr = '' then
    TheStr := '0'
  else
    begin
      {make sure string <> '+0' or '-0'}
      if (Length(TheStr) = 2) then
        if ((TheStr[1] = '+') or (TheStr[1] = '-')) and (TheStr[2] = '0') then
          TheStr := '0';
    end; {else}
end; {procedure CleanFloatString}

{---------------------------------------------------------------------}

function FloatCordToStrF(TheFloatCord : TMathFloat): string;
begin {function FloatCordToStrF(TheFloatCord : TMathFloat)}
  //old conversion; str(TheFloatCord:15:15,Result);
  Result := FloatToStrF(TheFloatCord, ffFixed, 30, 30);
  CleanFloatString(Result);
end; {function FloatCordToStrF(TheFloatCord : TMathFloat)}

{---------------------------------------------------------------------}

function Float2DZoomToStrF(TheFloatCord : TMathFloat): string;
begin {function Float2DZoomToStrF}
  //old conversion; str(TheFloatCord:8:6,Result);
  Result := FloatToStrF(TheFloatCord, ffFixed, 30, 6);
  CleanFloatString(Result);
end; {function Float2DZoomToStrF}

{---------------------------------------------------------------------}

function StrToFloatCord(const TheFloatString : string;
                        var TheFloatCord : TMathFloat) : boolean;
begin {function StrToFloatCord}
  Result := false;
  try
    if TheFloatString <> '' then
      begin
        TheFloatCord := StrToFloat(TheFloatString);
        Result := true;
      end; {if}
  except
    Result := false;
  end; {try..except}
end; {function StrToFloatCord}

{---------------------------------------------------------------------}

function RotateNumTo360D(TheRotateNum : TMathFloat) : TMathFloat;
var
  NumRevolutions : TMathFloat;
begin {function RotateNumTo360D}
  {force -360 < TheRotateNum < 360}
  NumRevolutions := int(TheRotateNum/(360));
  if abs(NumRevolutions) >= 1 then
    Result := TheRotateNum - (NumRevolutions*(360))
  else
    Result := TheRotateNum;
end; {function RotateNumTo360D}

{---------------------------------------------------------------------}

function RotateNumToStr(TheRotateNum : TMathFloat): string;
var
  NumRevolutions : TMathFloat;
begin {procedure RotateNumToStr}
  {force -360 < TheRotateNum < 360}
  NumRevolutions := int(TheRotateNum/(360));
  if abs(NumRevolutions) >= 1 then
    TheRotateNum := TheRotateNum - (NumRevolutions*(360));
  //old conversion; str(TheRotateNum:11:6, Result);
  Result := FloatToStrF(TheRotateNum, ffFixed, 30, 6);
  CleanFloatString(Result);
end; {procedure RotateNumToStr}

{---------------------------------------------------------------------}
(*
function StrToRotateNum(const TheStr : string;
                        var TheRotateNum : TMathFloat) : boolean;
var
  TmpRot : TMathFloat;
  NumRevolutions : TMathFloat;
begin {function StrToRotateNum}
  Result := true;
  if not StrToFloatCord(TheStr, TmpRot) then
    begin
      Result := false;
      TheRotateNum := 0;
    end
  else
    begin
      {force -360 < TheRotateNum < 360}
      NumRevolutions := int(TmpRot/(360));
      if abs(NumRevolutions) >= 1 then
        TheRotateNum := TmpRot - (NumRevolutions*(360))
      else
        TheRotateNum := TmpRot;
    end; {else}
end; {function StrToRotateNum}
*)
{---------------------------------------------------------------------}

function PositiveRotToStr(TheRotateNum : TMathFloat): string;
var
  NumRevolutions : TMathFloat;
begin {function PositiveRotToStr}
  {force 0 <= TheRotateNum < 360}
  NumRevolutions := int(TheRotateNum/(360));
  if abs(NumRevolutions) >= 1 then
    TheRotateNum := TheRotateNum - (NumRevolutions*(360));
  if TheRotateNum < 0 then
    TheRotateNum := TheRotateNum + 360;
  //old conversion; str(TheRotateNum:11:6, Result);
  Result := FloatToStrF(TheRotateNum, ffFixed, 30, 6);
  CleanFloatString(Result);
end; {function PositiveRotToStr}

{---------------------------------------------------------------------}

function StrToPositiveRot(const TheStr : string;
                          var TheRotateNum : TMathFloat) : boolean;
var
  TmpRot : TMathFloat;
  NumRevolutions : TMathFloat;
begin {function StrToPositiveRot}
  Result := true;
  if not StrToFloatCord(TheStr, TmpRot) then
    begin
      Result := false;
      TheRotateNum := 0;
    end
  else
    begin
      {force 0 <= TheRotateNum < 360}
      NumRevolutions := int(TmpRot/(360));
      if abs(NumRevolutions) >= 1 then
        TheRotateNum := TmpRot - (NumRevolutions*(360))
      else
        TheRotateNum := TmpRot;
      if TheRotateNum < 0 then
        TheRotateNum := TheRotateNum + 360;
    end; {else}
end; {function StrToPositiveRot}

{---------------------------------------------------------------------}

end.
