unit Flistdlg;

(*
Copyright 1999-2011 Greg VanMullem
This program is distributed under the terms of the GNU General Public License Version 3

This file is part of MathGV.

MathGV is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

MathGV is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MathGV.  If not, see <http://www.gnu.org/licenses/>.
*)

interface

uses WinTypes, WinProcs, Classes, Graphics, Forms, Controls, Buttons,
  StdCtrls, ExtCtrls, Spin, FuncObj, CentrDlg, DgGlobal;

type
  TFuncListDlg = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    HelpBtn: TBitBtn;
    FListBox: TListBox;
    SpinButton1: TSpinButton;
    ModifyButton: TBitBtn;
    DeleteButton: TBitBtn;
    procedure SpinButton1DownClick(Sender: TObject);
    procedure SpinButton1UpClick(Sender: TObject);
    procedure ModifyButtonClick(Sender: TObject);
    procedure DeleteButtonClick(Sender: TObject);
    procedure FListBoxDblClick(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
  private
    { Private declarations }
    DeleteFuncList : TList;
  public
    { Public declarations }
    FDidModify,
    FDidDelete,
    FDidReOrder: boolean;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  end;

procedure DisplayFuncListDlg(FuncObjList : TList;
                             var DidModify,DidDelete,DidReOrder: boolean);

implementation

{$R *.DFM}

{------------------------------------------------------------------------}

procedure DisplayFuncListDlg(FuncObjList : TList;
                             var DidModify,DidDelete,DidReOrder: boolean);
var
  I : integer;
  FuncListDlg: TFuncListDlg;
begin {procedure DisplayFuncListDlg}
  DidModify := false;
  DidDelete := false;
  DidReOrder := false;
  FuncListDlg := TFuncListDlg.Create(Application);
  try
    SetDlgDefaultPos(FuncListDlg, nil);
    FuncListDlg.FListBox.Items.Clear;
    if FuncObjList.Count > 0 then
      begin
        for I := 0 to FuncObjList.Count -1 do
          FuncListDlg.FListBox.Items.AddObject(
            TBaseFuncObj(FuncObjList[I]).ListFuncStr, FuncObjList[I]);
        FuncListDlg.FListBox.ItemIndex := 0;
      end; {if}
    if FuncListDlg.ShowModal=idOK then
      begin
        DidDelete := FuncListDlg.FDidDelete;
        DidReOrder := FuncListDlg.FDidReOrder;
        if DidDelete or DidReOrder then
          begin
            FuncObjList.Clear;
            if FuncListDlg.FListBox.Items.Count > 0 then
              for I := 0 to (FuncListDlg.FListBox.Items.Count -1) do
                 FuncObjList.Add(FuncListDlg.FListBox.Items.objects[I]);
          end; {if}
      end; {if}
    DidModify := FuncListDlg.FDidModify;
  finally
    FuncListDlg.Free;
  end; {try...Finally}
end; {procedure DisplayFuncListDlg}

{------------------------------------------------------------------------}

constructor TFuncListDlg.Create(AOwner: TComponent);
begin {constructor TFuncListDlg.Create}
  inherited Create(AOwner);
  DeleteFuncList := TList.Create;
  FDidModify := false;
  FDidDelete := false;
  FDidReOrder := false;
end; {constructor TFuncListDlg.Create}

{------------------------------------------------------------------------}

destructor TFuncListDlg.Destroy;
var
  I : integer;
begin {destructor TFuncListDlg.Destroy;}
  if (ModalResult = IDOK)and(FDidDelete) then
    for I := 0 to (DeleteFuncList.Count -1) do
       TBaseFuncObj(DeleteFuncList[I]).free;
  DeleteFuncList.Free;
  inherited Destroy;
end; {destructor TFuncListDlg.Destroy;}

{------------------------------------------------------------------------}

procedure TFuncListDlg.ModifyButtonClick(Sender: TObject);
var
  I : integer;
  TempGFObj : TBaseFuncObj;
begin
  I := FListBox.ItemIndex;
  if I > -1 then
    begin
      TempGFObj := TBaseFuncObj(FListBox.Items.objects[I]);
      if ModifyUserFunctionObj(TempGFObj, Self) then
        begin
          FDidModify := true;
          FListBox.Items.Strings[I] := TempGFObj.ListFuncStr;
          if FListBox.Items.Count >= 1 then
            FListBox.ItemIndex := I;
        end; {if}
    end; {if}
end;

procedure TFuncListDlg.DeleteButtonClick(Sender: TObject);
var
  I : integer;
begin
  I := FListBox.ItemIndex;
  if I > -1 then
    begin
      FDidDelete := true;
      DeleteFuncList.Add(FListBox.Items.objects[I]);
      FListBox.Items.Delete(I);
      if FListBox.Items.Count >= 1 then
        begin
          if I > (FListBox.Items.Count-1) then
            FListBox.ItemIndex := I-1
          else
            FListBox.ItemIndex := I;
        end;
    end; {if}
end;

procedure TFuncListDlg.SpinButton1DownClick(Sender: TObject);
var
  I : integer;
begin
  I := FListBox.ItemIndex;
  if I > -1 then
    if I < (FListBox.Items.Count-1) then
      begin
        FDidReOrder := true;
        FListBox.Items.Exchange(I, (I+1));
        FListBox.ItemIndex := I+1;
      end; {if}
end;

procedure TFuncListDlg.SpinButton1UpClick(Sender: TObject);
var
  I : integer;
begin
  I := FListBox.ItemIndex;
  if I > 0 then
    begin
      FDidReOrder := true;
      FListBox.Items.Exchange(I, (I-1));
      FListBox.ItemIndex := I-1;
    end; {if}
end;

procedure TFuncListDlg.FListBoxDblClick(Sender: TObject);
begin
  ModifyButtonClick(Sender);
end;

procedure TFuncListDlg.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (Key = VK_DELETE) then
    DeleteButtonClick(Sender);
end;

end.
