unit GapSort;

(*
Copyright 1999-2011 Greg VanMullem
This program is distributed under the terms of the GNU General Public License Version 3

This file is part of MathGV.

MathGV is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

MathGV is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MathGV.  If not, see <http://www.gnu.org/licenses/>.
*)

interface

type
  TGapSortSwap = procedure(Item1, Item2: integer) of object;
  TGapSortCompare = procedure(Item1, Item2: integer; var CompResult : integer) of object;
  {Compare is a comparison function that indicates how the items are to be ordered.}
  {Compare returns < 0 if Item1 is less and Item2,}
  {  0 if they are equal and > 0 if Item1 is greater than Item2.}

procedure GapSortZeroBasedArray(RecCount : integer; Compare: TGapSortCompare; Swap : TGapSortSwap);

implementation

procedure GapSortZeroBasedArray(RecCount : integer; Compare: TGapSortCompare; Swap : TGapSortSwap);
var
  CompResult : integer;
  I, Gap : Integer;
  Exchange : boolean;
begin {procedure GapSortZeroBasedArray}
  if RecCount <= 1 then
    Exit;
  if RecCount = 2 then
    begin
      Compare(0, 1, CompResult);
      if CompResult > 0 then
        Swap(0, 1);
      Exit;
    end; {if}
  if RecCount = 3 then
    begin
      //0, 1
      Compare(0, 1, CompResult);
      if CompResult > 0 then
        Swap(0, 1);
      //1, 2
      Compare(1, 2, CompResult);
      if CompResult > 0 then
        Swap(1, 2);
      //0, 1
      Compare(0, 1, CompResult);
      if CompResult > 0 then
        Swap(0, 1);
      Exit;
    end; {if}
  //do gap sort
  {Gap := RecCount div 2;}
  Gap := RecCount shr 1;
  dec(RecCount); {Fixup for 0 based arrays}
  repeat
    repeat
      Exchange := false;
      for I := 0 to (RecCount - Gap) do{start at 0 not 1; 0 based arrays}
        begin
          Compare(I, (I + Gap), CompResult);
          if CompResult > 0 then
            begin
              Swap(I, (I + Gap));
              Exchange := true;
            end; {if}
        end; {for}
    until not(Exchange);
    {Gap := Gap div 2;}
    Gap := Gap shr 1;
  until (Gap < 1);
end; {procedure GapSortZeroBasedArray}

end.
