unit GenOptns;

(*
Copyright 1999-2011 Greg VanMullem
This program is distributed under the terms of the GNU General Public License Version 3

This file is part of MathGV.

MathGV is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

MathGV is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MathGV.  If not, see <http://www.gnu.org/licenses/>.
*)

interface

uses
  Graphics, CentrDlg, MathCalc, MathFunc, inifiles, DefFont;

type
  TGeneralOptionsObj = class
  private
    { Private declarations }
  public
    { Public declarations }
    EnableRepTBButtons : boolean;
    RepeatDelay : integer;
    RepeatRate : integer;
    DefaultBmpWidth : integer;
    DefaultBmpHeight : integer;
    DefaultColorDepth : TPixelFormat;
    FlatToolBars : boolean;
    CenterDlgsOnPosition : TDefaultDlgPosition;
    CenterCommonDlgs : boolean;
    constructor Create;
    procedure ResetDefaults;
    function ReadFromIniFile(TheIniFile : TMemIniFile) : boolean;
    function SaveToIniFile(TheIniFile : TMemIniFile) : boolean;
  end; {TGeneralOptionsObj}

function ReadDefaultFontFromIni(TheIniFile : TMemIniFile) : boolean;
function WriteDefaultFontToIni(TheIniFile : TMemIniFile) : boolean;

implementation

const
  DefRepeatDelay = 600;
  DefRepeatRate = 150;
  DefBmpWidth = 1000;
  DefBmpHeight = 1000;

const
  //TGeneralOptionsObj
  sct_General = 'General';
    key_DefFontName = 'DefFontName';
    key_DefFontColor = 'DefFontColor';
    key_DefFontStyleBold = 'DefFontStyleBold';
    key_DefFontStyleItalic = 'DefFontStyleItalic';
    key_DefFontStyleUnderline = 'DefFontStyleUnderline';
    key_DefFontStyleStrikeOut = 'DefFontStyleStrikeOut';
    key_DefFontSize = 'DefFontSize';
    key_EnableRepTBButtons = 'EnableRepTBButtons';
    key_RepeatDelay = 'RepeatDelay';
    key_RepeatRate = 'RepeatRate';
    key_DefaultBmpWidth = 'DefaultBmpWidth';
    key_DefaultBmpHeight = 'DefaultBmpHeight';
    key_DefaultColorDepth = 'DefaultColorDepth';
      enu_pfDevice = 'pfDevice';
      enu_pf4bit = 'pf4bit';
      enu_pf8bit = 'pf8bit';
      enu_pf15bit = 'pf15bit';
      enu_pf16bit = 'pf16bit';
      enu_pf24bit = 'pf24bit';
      enu_pf32bit = 'pf32bit'; //new default, used to be device
    key_FlatToolBars = 'FlatToolBars';
    key_CenterDlgsOnPosition = 'CenterDlgsOnPosition';
      enu_dpScreenCenter = 'dpScreenCenter';
      enu_dpDesktopCenter = 'dpDesktopCenter';
      enu_dpMainFormCenter = 'dpMainFormCenter';
      enu_dpSpecifiedFormCenter = 'dpSpecifiedFormCenter';
    key_CenterCommonDlgs = 'CenterCommonDlgs';

{---------------------------------------------------------------------}

procedure ReadFontFromIniFile(TheIniFile : TMemIniFile; TheFont : TFont;
                        const TheGeneral, TheDefFontName, TheDefFontColor,
                              TheDefFontStyleBold, TheDefFontStyleItalic,
                              TheDefFontStyleUnderline, TheDefFontStyleStrikeOut,
                              TheDefFontSize : string);
var
  TmpName : string;
  TmpColor : TColor;
  TmpStyle : TFontStyles;
  TmpSize : Longint;
begin {procedure ReadFontFromIniFile}
  //read values
  TmpName := TheIniFile.ReadString(TheGeneral, TheDefFontName, TheFont.Name);
  TmpColor := TheIniFile.ReadInteger(TheGeneral, TheDefFontColor, TheFont.Color);
  TmpStyle := [];
  if TheIniFile.ReadBool(TheGeneral, TheDefFontStyleBold, (fsBold in TheFont.Style)) then
    TmpStyle := TmpStyle + [fsBold];
  if TheIniFile.ReadBool(TheGeneral, TheDefFontStyleItalic, (fsItalic in TheFont.Style)) then
    TmpStyle := TmpStyle + [fsItalic];
  if TheIniFile.ReadBool(TheGeneral, TheDefFontStyleUnderline, (fsUnderline in TheFont.Style)) then
    TmpStyle := TmpStyle + [fsUnderline];
  if TheIniFile.ReadBool(TheGeneral, TheDefFontStyleStrikeOut, (fsStrikeOut in TheFont.Style)) then
    TmpStyle := TmpStyle + [fsStrikeOut];
  TmpSize := TheIniFile.ReadInteger(TheGeneral, TheDefFontSize, TheFont.Size);
  //set values
  TheFont.Name := TmpName;
  TheFont.Color := TmpColor;
  TheFont.Style := TmpStyle;
  TheFont.Size := TmpSize;
end; {procedure ReadFontFromIniFile}

{---------------------------------------------------------------------}

procedure WriteFontToIniFile(TheIniFile : TMemIniFile; TheFont : TFont;
                        const TheGeneral, TheDefFontName, TheDefFontColor,
                              TheDefFontStyleBold, TheDefFontStyleItalic,
                              TheDefFontStyleUnderline, TheDefFontStyleStrikeOut,
                              TheDefFontSize : string);
var
  TmpName : string;
begin {procedure WriteFontToIniFile}
  TmpName := TheFont.Name;
  UniqueString(TmpName);
  TheIniFile.WriteString(TheGeneral, TheDefFontName, TmpName);
  TheIniFile.WriteInteger(TheGeneral, TheDefFontColor, TheFont.Color);
  TheIniFile.WriteBool(TheGeneral, TheDefFontStyleBold, (fsBold in TheFont.Style));
  TheIniFile.WriteBool(TheGeneral, TheDefFontStyleItalic, (fsItalic in TheFont.Style));
  TheIniFile.WriteBool(TheGeneral, TheDefFontStyleUnderline, (fsUnderline in TheFont.Style));
  TheIniFile.WriteBool(TheGeneral, TheDefFontStyleStrikeOut, (fsStrikeOut in TheFont.Style));
  TheIniFile.WriteInteger(TheGeneral, TheDefFontSize, TheFont.Size);
end; {procedure WriteFontToIniFile}

{---------------------------------------------------------------------}

function ReadDefaultFontFromIni(TheIniFile : TMemIniFile) : boolean;
var
  TheFont : TFont;
begin {function ReadDefaultFontFromIni}
  Result := true;
  try
    TheFont := TFont.Create;
    try
      ReadFontFromIniFile(TheIniFile, TheFont, sct_General, key_DefFontName,
                          key_DefFontColor, key_DefFontStyleBold, key_DefFontStyleItalic,
                          key_DefFontStyleUnderline, key_DefFontStyleStrikeOut, key_DefFontSize);
      DefaultFontHolder.SetFontAttributes(TheFont);
    finally
      TheFont.Free;
    end; {try..finally}
  except
    Result := false;
  end; {try..except}
end; {function ReadDefaultFontFromIni}

{---------------------------------------------------------------------}

function WriteDefaultFontToIni(TheIniFile : TMemIniFile) : boolean;
var
  TheFont : TFont;
begin {function WriteDefaultFontToIni}
  Result := true;
  try
    TheFont := TFont.Create;
    try
      DefaultFontHolder.GetFontAttributes(TheFont);
      WriteFontToIniFile(TheIniFile, TheFont, sct_General, key_DefFontName,
                         key_DefFontColor, key_DefFontStyleBold, key_DefFontStyleItalic,
                         key_DefFontStyleUnderline, key_DefFontStyleStrikeOut, key_DefFontSize);
    finally
      TheFont.Free;
    end; {try..finally}
  except
    Result := false;
  end; {try..except}
end; {function WriteDefaultFontToIni}

{---------------------------------------------------------------------}

{ TGeneralOptionsObj }

constructor TGeneralOptionsObj.Create;
begin {constructor TGeneralOptionsObj.Create;}
  inherited Create;
  EnableRepTBButtons := true;
  RepeatDelay := DefRepeatDelay;
  RepeatRate := DefRepeatRate;
  DefaultBmpWidth := DefBmpWidth;
  DefaultBmpHeight := DefBmpHeight;
  DefaultColorDepth := pf32bit;
  FlatToolBars := false;
  CenterDlgsOnPosition := dpSpecifiedFormCenter;
  CenterCommonDlgs := true;
end; {constructor TGeneralOptionsObj.Create;}

{---------------------------------------------------------------------}

function TGeneralOptionsObj.ReadFromIniFile(
  TheIniFile: TMemIniFile): boolean;
var
  TmpStr : string;
begin {function TGeneralOptionsObj.ReadFromIniFile}
  Result := true;
  try
    EnableRepTBButtons := TheIniFile.ReadBool(sct_General, key_EnableRepTBButtons, true);
    RepeatDelay := TheIniFile.ReadInteger(sct_General, key_RepeatDelay, DefRepeatDelay);
    RepeatRate := TheIniFile.ReadInteger(sct_General, key_RepeatRate, DefRepeatRate);
    DefaultBmpWidth := TheIniFile.ReadInteger(sct_General, key_DefaultBmpWidth, DefBmpWidth);
    DefaultBmpHeight := TheIniFile.ReadInteger(sct_General, key_DefaultBmpHeight, DefBmpHeight);
    TmpStr := TheIniFile.ReadString(sct_General, key_DefaultColorDepth, '');
    if TmpStr = enu_pfDevice then DefaultColorDepth := pfDevice else
    if TmpStr = enu_pf4bit then DefaultColorDepth := pf4bit else
    if TmpStr = enu_pf8bit then DefaultColorDepth := pf8bit else
    if TmpStr = enu_pf15bit then DefaultColorDepth := pf15bit else
    if TmpStr = enu_pf16bit then DefaultColorDepth := pf16bit else
    if TmpStr = enu_pf24bit then DefaultColorDepth := pf24bit else
    if TmpStr = enu_pf32bit then DefaultColorDepth := pf32bit else
      DefaultColorDepth := pf32bit;
    FlatToolBars := TheIniFile.ReadBool(sct_General, key_FlatToolBars, false);
    TmpStr := TheIniFile.ReadString(sct_General, key_CenterDlgsOnPosition, '');
    if TmpStr = enu_dpScreenCenter then CenterDlgsOnPosition := dpScreenCenter else
    if TmpStr = enu_dpDesktopCenter then CenterDlgsOnPosition := dpDesktopCenter else
    if TmpStr = enu_dpMainFormCenter then CenterDlgsOnPosition := dpMainFormCenter else
    if TmpStr = enu_dpSpecifiedFormCenter then CenterDlgsOnPosition := dpSpecifiedFormCenter else
      CenterDlgsOnPosition := dpSpecifiedFormCenter;
    CenterCommonDlgs := TheIniFile.ReadBool(sct_General, key_CenterCommonDlgs, true);
  except
    Result := false;
  end; {try..except}
end; {function TGeneralOptionsObj.ReadFromIniFile}

{---------------------------------------------------------------------}

procedure TGeneralOptionsObj.ResetDefaults;
begin {procedure TGeneralOptionsObj.ResetDefaults;}
  EnableRepTBButtons := true;
  RepeatDelay := DefRepeatDelay;
  RepeatRate := DefRepeatRate;
  DefaultBmpWidth := DefBmpWidth;
  DefaultBmpHeight := DefBmpHeight;
  DefaultColorDepth := pf32bit;
  FlatToolBars := false;
  CenterDlgsOnPosition := dpSpecifiedFormCenter;
  CenterCommonDlgs := true;
end; {procedure TGeneralOptionsObj.ResetDefaults;}

{---------------------------------------------------------------------}

function TGeneralOptionsObj.SaveToIniFile(
  TheIniFile: TMemIniFile): boolean;
var
  TmpStr : string;
begin {function TGeneralOptionsObj.SaveToIniFile}
  Result := true;
  try
    TheIniFile.WriteBool(sct_General, key_EnableRepTBButtons, EnableRepTBButtons);
    TheIniFile.WriteInteger(sct_General, key_RepeatDelay, RepeatDelay);
    TheIniFile.WriteInteger(sct_General, key_RepeatRate, RepeatRate);
    TheIniFile.WriteInteger(sct_General, key_DefaultBmpWidth, DefaultBmpWidth);
    TheIniFile.WriteInteger(sct_General, key_DefaultBmpHeight, DefaultBmpHeight);
    case DefaultColorDepth of
      pfDevice: TmpStr := enu_pfDevice;
      pf4bit: TmpStr := enu_pf4bit;
      pf8bit: TmpStr := enu_pf8bit;
      pf15bit: TmpStr := enu_pf15bit;
      pf16bit: TmpStr := enu_pf16bit;
      pf24bit: TmpStr := enu_pf24bit;
      pf32bit: TmpStr := enu_pf32bit;
      else TmpStr := enu_pf32bit;
    end; {case}
    TheIniFile.WriteString(sct_General, key_DefaultColorDepth, TmpStr);
    TheIniFile.WriteBool(sct_General, key_FlatToolBars, FlatToolBars);
    case CenterDlgsOnPosition of
      dpScreenCenter: TmpStr := enu_dpScreenCenter;
      dpDesktopCenter: TmpStr := enu_dpDesktopCenter;
      dpMainFormCenter: TmpStr := enu_dpMainFormCenter;
      dpSpecifiedFormCenter: TmpStr := enu_dpSpecifiedFormCenter;
      else TmpStr := enu_dpSpecifiedFormCenter;
    end; {case}
    TheIniFile.WriteString(sct_General, key_CenterDlgsOnPosition, TmpStr);
    TheIniFile.WriteBool(sct_General, key_CenterCommonDlgs, CenterCommonDlgs);
  except
    Result := false;
  end; {try..except}
end; {function TGeneralOptionsObj.SaveToIniFile}

{---------------------------------------------------------------------}

end.
