unit GrDraw;

(*
Copyright 1999-2011 Greg VanMullem
This program is distributed under the terms of the GNU General Public License Version 3

This file is part of MathGV.

MathGV is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

MathGV is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MathGV.  If not, see <http://www.gnu.org/licenses/>.
*)

interface

uses
  Windows, Graphics, DefFont;

const
  MaxSelectDistance = 5;
  MaxTextLabelLength = 250;
  {constants used to identify labels when saved to file}
  dl_FileType_Line = 1;
  dl_FileType_FloodFill = 2;
  dl_FileType_Rectangle = 3;
  dl_FileType_Ellipse = 4;
  dl_FileType_RoundRect = 5;
  dl_FileType_Text = 6;
  dl_FileType_FreeHand = 7;

const
  MaxLabelListObjs = 2000;
  MaxFloodFillObjs = 2000;
  MinSelectRegionSize = 5;
  MinSelectRegionSizeStr = '5';

const
  lbHiddenLabelNone = -1;
  lbHiddenLabelDoNotChange = -2;

type
  TLabelWindowStatus = (lwNormal, lwEditingText, lwDrawingNewLabel,
    lwLabelSelected, lwDragResizeLabel, lwFloodFillsVisible,
    lwDrawingRegion, lwRegionSelected);

  TDrawingTool = (dtOff, dtLine, dtFloodFill, dtRectangle, dtEllipse,
                  dtRoundRect, dtText, dtFreeHand, dtSelectRegion);

type
  TLabelWinStatusChangeEvent = procedure(Sender: TObject; LabelWindowStatus : TLabelWindowStatus) of object;
  TUpDateStatusLineEvent = procedure(Sender: TObject; StatusStr1, StatusStr2, StatusStr3 : string) of object;
  TGetDrawingToolEvent = procedure(out TheDrawingTool : TDrawingTool) of object;
  TGetResizeLabelsModeEvent = procedure(out TheResizeLabelsMode : boolean) of object;
  TShowLabelPopupMenuEvent =  procedure(ScreenX, ScreenY : integer;
                                        ShowInPlaceEditorItem : boolean) of object;

type
  TBitmapAttribRec = record
    BackgroundColor : TColor;
    BmpWidth, BmpHeight: Integer;
  end; {TBitmapAttribRec = record}

const
  Dflt_ToolBar_BrushColor = clBlue;
  Dflt_ToolBar_BrushStyle = bsClear;
  Dflt_ToolBar_PenColor = clBlack;
  Dflt_ToolBar_PenStyle = psSolid;
  Dflt_ToolBar_PenWidth = 1;

type
  TToolBarAttribClass = class
  protected
    FFont : TFont;
  public
    BrushColor : TColor;
    BrushStyle : TBrushStyle;
    PenColor : TColor;
    PenStyle : TPenStyle;
    PenWidth : integer;
    property Font : TFont read FFont;
    constructor Create;
    destructor Destroy; override;
  end; {TToolBarAttribClass}

  TGetToolBarAttribEvent = procedure(out TheToolBarAttribClass : TToolBarAttribClass) of object;

procedure FindTopLeftPoint(var TheTopLeft, TheBottomRight : TPoint);
function IsPointOnCanvas(TheX, TheY, TheWidth, TheHeight: Integer) : boolean;

implementation

{----------------------------------------------------------------------}

procedure FindTopLeftPoint(var TheTopLeft, TheBottomRight : TPoint);
var
  SwapCord : integer;
begin {procedure FindTopLeftPoint}
  if TheTopLeft.X > TheBottomRight.X then
    begin
      SwapCord := TheTopLeft.X;
      TheTopLeft.X := TheBottomRight.X;
      TheBottomRight.X := SwapCord;
    end; {if}
  if TheTopLeft.Y > TheBottomRight.Y then
    begin
      SwapCord := TheTopLeft.Y;
      TheTopLeft.Y := TheBottomRight.Y;
      TheBottomRight.Y := SwapCord;
    end; {if}
end; {procedure FindTopLeftPoint}

{----------------------------------------------------------------------}

function IsPointOnCanvas(TheX, TheY, TheWidth, TheHeight: Integer) : boolean;
begin {function IsPointOnCanvas}
  Result := (TheX >= 0) and (TheX < TheWidth) and
            (TheY >= 0) and (TheY < TheHeight);
end; {function IsPointOnCanvas}

{----------------------------------------------------------------------}

constructor TToolBarAttribClass.Create;
begin {constructor TToolBarAttribClass.Create;}
  inherited;
  BrushColor := Dflt_ToolBar_BrushColor;
  BrushStyle := Dflt_ToolBar_BrushStyle;
  PenColor := Dflt_ToolBar_PenColor;
  PenStyle := Dflt_ToolBar_PenStyle;
  PenWidth := Dflt_ToolBar_PenWidth;
  FFont := CreateANewTFontClass;
end; {constructor TToolBarAttribClass.Create;}

{----------------------------------------------------------------------}

destructor TToolBarAttribClass.Destroy;
begin {Destructor TToolBarAttribClass.Destroy;}
  FFont.Free;
  inherited;
end; {Destructor TToolBarAttribClass.Destroy;}

{----------------------------------------------------------------------}

end.
