unit GrGlobal;

(*
Copyright 1999-2011 Greg VanMullem
This program is distributed under the terms of the GNU General Public License Version 3

This file is part of MathGV.

MathGV is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

MathGV is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MathGV.  If not, see <http://www.gnu.org/licenses/>.
*)

interface

{This unit contains code used by the MathGV graphing components}

uses classes, Graphics, MathCalc, MathFunc;

const
  MaxBmpHeight = 2000;
  MaxBmpWidth = 2000;
  MaxBmpDim = 2000;
  MinBmpHeight = 16;
  MinBmpWidth = 16;
  MinBmpDim = 16;

const
  SupportedBmpColorDepths = [pf4bit, pf8bit, pf15bit, pf16bit, pf24bit, pf32bit];

const
  //floating point errors
  ErrorStr_OverFlow = 'Floating point over flow error';
  ErrorStr_UnderFlow = 'Floating point under flow error';
  ErrorStr_ZeroDivide = 'Floating point divide-by-zero error';
  ErrorStr_InvalidOp = 'Invalid floating point operation error';
  ErrorStr_MathError = 'Floating point math error';
  //integer errors
  ErrorStr_DivByZero = 'Integer divide-by-zero error';
  ErrorStr_IntOverflow = 'Integer over flow error';
  ErrorStr_IntError = 'Integer calculation error';
  //other errors
  ErrorStr_OutOfMemory = 'Out of memory error';
  ErrorStr_InvalidPointer = 'Invalid pointer operations error';
  ErrorStr_ConvertError = 'String or object conversion error';
  //more errors
  ErrorStr_UnknownError = 'Unknown error';

type
  //this event must return immediately, it can't display a dialog and wait.
  TPlotErrorEvent = procedure(Sender: TObject; const ImagePlotErrorString : string) of object;

type
  TGraphBackgroundTileMode = (btCenter, btStretchToFit, btTile,
                              btUpperLeft, btUpperRight, LowerLeft, LowerRight);

procedure ConvCartesianToPolar(CX, CY : TMathFloat;
                               var PR, PTheta : TMathFloat);
procedure ConvPolarToCartesian(PR, PTheta : TMathFloat;
                               var CX, CY : TMathFloat);
function ConvDegToRad(TheAngleInDegrees : TMathFloat) : TMathFloat;
function ConvRadToDeg(TheAngleInRadians : TMathFloat) : TMathFloat;

function DoesStringEndWith(StrToTest, StrToFindAtEnd : string) : boolean;

implementation

{---------------------------------------------------------------------}

procedure ConvCartesianToPolar(CX, CY : TMathFloat;
                               var PR, PTheta : TMathFloat);
begin {procedure ConvCartesianToPolar}
  try
    PR := sqrt((CX*CX)+(CY*CY));
    if CX = 0 then
      begin
        if CY = 0 then
          PTheta := 0
        else
          if CY > 0 then
            PTheta := (pi/2)
          else
            PTheta := ((3*pi)/2);
      end {if}
    else
    if CY = 0 then
      begin
        if CX >= 0 then
          PTheta := 0
        else
          PTheta := pi;
      end {if}
    else
      begin
        PTheta := ArcTan(Abs(CY)/Abs(CX));
        if (CX < 0)and(CY > 0) then {quadrant 2}
          PTheta := pi - PTheta
        else
          if (CX < 0)and(CY < 0) then {quadrant 3}
            PTheta := PTheta + (pi)
          else
            if (CX > 0)and(CY < 0) then {quadrant 4}
              PTheta := (2*pi) - PTheta;
      end; {else}
  except
    PR := 0;
    PTheta := 0;
  end; {except}
end; {procedure ConvCartesianToPolar}

{---------------------------------------------------------------------}

procedure ConvPolarToCartesian(PR, PTheta : TMathFloat;
                               var CX, CY : TMathFloat);
begin {procedure ConvPolarToCartesian}
  try
    CX := PR * cos(PTheta);
    CY := PR * sin(PTheta);
  except
    CX := 0;
    CY := 0;
  end; {except}
end; {procedure ConvPolarToCartesian}

{---------------------------------------------------------------------}

function ConvDegToRad(TheAngleInDegrees : TMathFloat) : TMathFloat;
begin {function ConvDegToRad}
  try
    Result := TheAngleInDegrees*(pi/180);
  except
    Result := 0;
  end; {except}
end; {function ConvDegToRad}

{---------------------------------------------------------------------}

function ConvRadToDeg(TheAngleInRadians : TMathFloat) : TMathFloat;
begin {function ConvRadToDeg}
  try
    Result := TheAngleInRadians*(180/pi);
  except
    Result := 0;
  end; {except}
end; {function ConvRadToDeg}

{---------------------------------------------------------------------}

function DoesStringEndWith(StrToTest, StrToFindAtEnd : string) : boolean;
begin {function DoesStringEndWith}
  Result := false;
  if (Length(StrToTest) > 0) and (Length(StrToFindAtEnd) > 0) then
    if Length(StrToTest) >= Length(StrToFindAtEnd) then
      if Pos(StrToFindAtEnd, StrToTest) = (Length(StrToTest) - Length(StrToFindAtEnd) + 1) then
        Result := true;
end; {function DoesStringEndWith}

{---------------------------------------------------------------------}

end.
