unit HotKey;

(*
Copyright 1999-2011 Greg VanMullem
This program is distributed under the terms of the GNU General Public License Version 3

This file is part of MathGV.

MathGV is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

MathGV is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MathGV.  If not, see <http://www.gnu.org/licenses/>.
*)

interface

uses
  MathCalc, MathFunc, inifiles;

type
  TZoomRangeFloatArray = array[1..6] of TMathFloat;

  THotKeyObj = class
  private
    { Private declarations }
  protected
    { Protected declarations }
    function DoReadFromIniFile(TheIniFile : TMemIniFile; const GraphTypeStr : string) : boolean;
    function DoSaveToIniFile(TheIniFile : TMemIniFile; const GraphTypeStr : string) : boolean;
    function DoCalcNewZoomIn(var TheViewZoomVar : TMathFloat; MaxViewZoom : TMathFloat) : boolean;
    function DoCalcNewZoomOut(var TheViewZoomVar : TMathFloat; MinViewZoom : TMathFloat) : boolean;
  public
    { Public declarations }
    GraphScrollViewAmount : integer;
    ZoomRangeFloatArray : TZoomRangeFloatArray;
    constructor Create;
    procedure ResetDefaults;
  end; {THotKeyObj}


implementation

const
  //key strings
  sct_HotKey = 'HotKey';
    key_GrScrollAmt = 'GrScrollAmt';
    key_Zoom1Range = 'Zoom1Range';
    key_Zoom2Range = 'Zoom2Range';
    key_Zoom3Range = 'Zoom3Range';
    key_Zoom4Range = 'Zoom4Range';
    key_Zoom5Range = 'Zoom5Range';
    key_Zoom6Range = 'Zoom6Range';

{ THotKey2DObj }

{---------------------------------------------------------------------}

constructor THotKeyObj.Create;
begin {constructor THotKeyObj.Create;}
  inherited Create;
  ResetDefaults;
end; {constructor THotKeyObj.Create;}

{---------------------------------------------------------------------}

function THotKeyObj.DoReadFromIniFile(TheIniFile: TMemIniFile; const GraphTypeStr : string): boolean;
var
  TmpSectStr : string;
begin {function THotKeyObj.DoReadFromIniFile}
  Result := true;
  try
    TmpSectStr := sct_HotKey + GraphTypeStr;
    GraphScrollViewAmount := TheIniFile.ReadInteger(TmpSectStr, key_GrScrollAmt + GraphTypeStr, 10);
    ZoomRangeFloatArray[1] := TheIniFile.ReadFloat(TmpSectStr, key_Zoom1Range + GraphTypeStr, 500);
    ZoomRangeFloatArray[2] := TheIniFile.ReadFloat(TmpSectStr, key_Zoom2Range + GraphTypeStr, 25);
    ZoomRangeFloatArray[3] := TheIniFile.ReadFloat(TmpSectStr, key_Zoom3Range + GraphTypeStr, 5);
    ZoomRangeFloatArray[4] := TheIniFile.ReadFloat(TmpSectStr, key_Zoom4Range + GraphTypeStr, 2);
    ZoomRangeFloatArray[5] := TheIniFile.ReadFloat(TmpSectStr, key_Zoom5Range + GraphTypeStr, 0.02);
    ZoomRangeFloatArray[6] := TheIniFile.ReadFloat(TmpSectStr, key_Zoom6Range + GraphTypeStr, 0.0002);
  except
    Result := false;
  end; {try..except}
end; {function THotKeyObj.DoReadFromIniFile}

{---------------------------------------------------------------------}

procedure THotKeyObj.ResetDefaults;
begin {procedure THotKeyObj.ResetDefaults;}
  GraphScrollViewAmount := 10;
  ZoomRangeFloatArray[1] := 500;
  ZoomRangeFloatArray[2] := 25;
  ZoomRangeFloatArray[3] := 5;
  ZoomRangeFloatArray[4] := 2;
  ZoomRangeFloatArray[5] := 0.02;
  ZoomRangeFloatArray[6] := 0.0002;
end; {procedure THotKeyObj.ResetDefaults;}

{---------------------------------------------------------------------}

function THotKeyObj.DoSaveToIniFile(TheIniFile: TMemIniFile; const GraphTypeStr : string): boolean;
var
  TmpSectStr : string;
begin {function THotKeyObj.DoSaveToIniFile}
  Result := true;
  try
    TmpSectStr := sct_HotKey + GraphTypeStr;
    TheIniFile.WriteInteger(TmpSectStr, key_GrScrollAmt + GraphTypeStr, GraphScrollViewAmount);
    TheIniFile.WriteFloat(TmpSectStr, key_Zoom1Range + GraphTypeStr, ZoomRangeFloatArray[1]);
    TheIniFile.WriteFloat(TmpSectStr, key_Zoom2Range + GraphTypeStr, ZoomRangeFloatArray[2]);
    TheIniFile.WriteFloat(TmpSectStr, key_Zoom3Range + GraphTypeStr, ZoomRangeFloatArray[3]);
    TheIniFile.WriteFloat(TmpSectStr, key_Zoom4Range + GraphTypeStr, ZoomRangeFloatArray[4]);
    TheIniFile.WriteFloat(TmpSectStr, key_Zoom5Range + GraphTypeStr, ZoomRangeFloatArray[5]);
    TheIniFile.WriteFloat(TmpSectStr, key_Zoom6Range + GraphTypeStr, ZoomRangeFloatArray[6]);
  except
    Result := false;
  end; {try..except}
end; {function THotKeyObj.DoSaveToIniFile}

{---------------------------------------------------------------------}

function THotKeyObj.DoCalcNewZoomIn(var TheViewZoomVar : TMathFloat; MaxViewZoom : TMathFloat) : boolean;
var
  I : integer;
  OldZoom, Bound : TMathFloat;
begin {function THotKeyObj.DoCalcNewZoomIn}
  OldZoom := TheViewZoomVar;
  if TheViewZoomVar >= MaxViewZoom then
    begin
      TheViewZoomVar := MaxViewZoom;
      Result := (OldZoom <> TheViewZoomVar);
      exit;
    end; {if}
  if TheViewZoomVar < 0.01 then
    begin
      I := 6;
      Bound := 0.01;
    end
  else
  if TheViewZoomVar < 1 then
    begin
      I := 5;
      Bound := 1;
    end
  else
  if TheViewZoomVar < 20 then
    begin
      I := 4;
      Bound := 20;
    end
  else
  if TheViewZoomVar < 100 then
    begin
      I := 3;
      Bound := 100;
    end
  else
  if TheViewZoomVar < 1000 then
    begin
      I := 2;
      Bound := 1000;
    end
  else
    begin {if TheViewZoomVar < 10,000 then}
      I := 1;
      Bound := 10000;
    end; {else}
  TheViewZoomVar := TheViewZoomVar + ZoomRangeFloatArray[I];
  if TheViewZoomVar > Bound then
    TheViewZoomVar := Bound;
  Result := (OldZoom <> TheViewZoomVar);
end; {function THotKeyObj.DoCalcNewZoomIn}

{---------------------------------------------------------------------}

function THotKeyObj.DoCalcNewZoomOut(var TheViewZoomVar : TMathFloat; MinViewZoom : TMathFloat) : boolean;
var
  I : integer;
  OldZoom, Bound : TMathFloat;
begin {function THotKeyObj.DoCalcNewZoomOut}
  OldZoom := TheViewZoomVar;
  if TheViewZoomVar <= MinViewZoom then
    begin
      TheViewZoomVar := MinViewZoom;
      Result := (OldZoom <> TheViewZoomVar);
      exit;
    end; {if}
  if TheViewZoomVar > 1000 then
    begin
      I := 1;
      Bound := 1000;
    end
  else
  if TheViewZoomVar > 100 then
    begin
      I := 2;
      Bound := 100;
    end
  else
  if TheViewZoomVar > 20 then
    begin
      I := 3;
      Bound := 20;
    end
  else
  if TheViewZoomVar > 1 then
    begin
      I := 4;
      Bound := 1;
    end
  else
  if TheViewZoomVar > 0.01 then
    begin
      I := 5;
      Bound := 0.01;
    end
  else
    begin {if TheViewZoomVar > 0.0001 then}
      I := 6;
      Bound := 0.0001;
    end; {else}
  TheViewZoomVar := TheViewZoomVar - ZoomRangeFloatArray[I];
  if TheViewZoomVar < Bound then
    TheViewZoomVar := Bound;
  Result := (OldZoom <> TheViewZoomVar);
end; {function THotKeyObj.DoCalcNewZoomOut}

{---------------------------------------------------------------------}

end.
