unit HotKey3D;

(*
Copyright 1999-2011 Greg VanMullem
This program is distributed under the terms of the GNU General Public License Version 3

This file is part of MathGV.

MathGV is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

MathGV is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MathGV.  If not, see <http://www.gnu.org/licenses/>.
*)

interface

uses
  MathCalc, MathFunc, Vw3dObj, IniFiles;

type
  TZoomRange3DIntArray = array[1..4] of integer;

type
  THotKey3DObj = class
  private
    { Private declarations }
  public
    { Public declarations }
    Rotation3DIncAmt : TMathFloat;
    ZoomRange3DIntArray : TZoomRange3DIntArray;
    constructor Create;
    procedure ResetDefaults;
    function ReadFromIniFile(TheIniFile : TMemIniFile) : boolean;
    function SaveToIniFile(TheIniFile : TMemIniFile) : boolean;
    function Calc3DZoomIn(var TheViewZoomVar : integer) : boolean;
    function Calc3DZoomOut(var TheViewZoomVar : integer) : boolean;
  end; {THotKey3DObj}

implementation

const
  //key strings
  sct_HotKey3D = 'HotKey3D';
    key_Rotation3DIncAmt = 'Rotation3DIncAmt';
    key_Zoom1Range3D = 'Zoom1Range3D';
    key_Zoom2Range3D = 'Zoom2Range3D';
    key_Zoom3Range3D = 'Zoom3Range3D';
    key_Zoom4Range3D = 'Zoom4Range3D';

{ THotKey3DObj }

{--------------------------------------------------------------------}

constructor THotKey3DObj.Create;
begin {constructor THotKey3DObj.Create;}
  inherited Create;
  ResetDefaults;
end; {constructor THotKey3DObj.Create;}

{--------------------------------------------------------------------}

function THotKey3DObj.ReadFromIniFile(TheIniFile: TMemIniFile): boolean;
begin {function THotKey3DObj.ReadFromIniFile}
  Result := true;
  try
    Rotation3DIncAmt := TheIniFile.ReadFloat(sct_HotKey3D, key_Rotation3DIncAmt, 3);
    ZoomRange3DIntArray[1] := TheIniFile.ReadInteger(sct_HotKey3D, key_Zoom1Range3D, 20);
    ZoomRange3DIntArray[2] := TheIniFile.ReadInteger(sct_HotKey3D, key_Zoom2Range3D, 10);
    ZoomRange3DIntArray[3] := TheIniFile.ReadInteger(sct_HotKey3D, key_Zoom3Range3D, 5);
    ZoomRange3DIntArray[4] := TheIniFile.ReadInteger(sct_HotKey3D, key_Zoom4Range3D, 2);
  except
    Result := false;
  end; {try..except}
end; {function THotKey3DObj.ReadFromIniFile}

{--------------------------------------------------------------------}

procedure THotKey3DObj.ResetDefaults;
begin {procedure THotKey3DObj.ResetDefaults;}
  Rotation3DIncAmt := 3;
  ZoomRange3DIntArray[1] := 20;
  ZoomRange3DIntArray[2] := 10;
  ZoomRange3DIntArray[3] := 5;
  ZoomRange3DIntArray[4] := 2;
end; {procedure THotKey3DObj.ResetDefaults;}

{--------------------------------------------------------------------}

function THotKey3DObj.SaveToIniFile(TheIniFile: TMemIniFile): boolean;
begin {function THotKey3DObj.SaveToIniFile}
  Result := true;
  try
    TheIniFile.WriteFloat(sct_HotKey3D, key_Rotation3DIncAmt, Rotation3DIncAmt);
    TheIniFile.WriteInteger(sct_HotKey3D, key_Zoom1Range3D, ZoomRange3DIntArray[1]);
    TheIniFile.WriteInteger(sct_HotKey3D, key_Zoom2Range3D, ZoomRange3DIntArray[2]);
    TheIniFile.WriteInteger(sct_HotKey3D, key_Zoom3Range3D, ZoomRange3DIntArray[3]);
    TheIniFile.WriteInteger(sct_HotKey3D, key_Zoom4Range3D, ZoomRange3DIntArray[4]);
  except
    Result := false;
  end; {try..except}
end; {function THotKey3DObj.SaveToIniFile}

{--------------------------------------------------------------------}

function THotKey3DObj.Calc3DZoomIn(var TheViewZoomVar : integer) : boolean;
var
  I : integer;
  OldZoom : integer;
begin {function THotKey3DObj.Calc3DZoomIn}
  OldZoom := TheViewZoomVar;
  if TheViewZoomVar >= Max3DViewZoom then
    begin
      TheViewZoomVar := Max3DViewZoom;
      Result := (OldZoom <> TheViewZoomVar);
      exit;
    end; {if}
  if TheViewZoomVar < 20 then
    I := 4
  else
  if TheViewZoomVar < 200 then
    I := 3
  else
  if TheViewZoomVar < 500 then
    I := 2
  else {if TheViewZoomVar < 1000 then}
    I := 1;
  TheViewZoomVar := TheViewZoomVar + abs(ZoomRange3DIntArray[I]);
  if TheViewZoomVar > Max3DViewZoom then
    TheViewZoomVar := Max3DViewZoom
  else
    if TheViewZoomVar < Min3DViewZoom then
      TheViewZoomVar := Min3DViewZoom;
  Result := (OldZoom <> TheViewZoomVar);
end; {function THotKey3DObj.Calc3DZoomIn}

{--------------------------------------------------------------------}

function THotKey3DObj.Calc3DZoomOut(var TheViewZoomVar : integer) : boolean;
var
  I : integer;
  OldZoom : integer;
begin {function THotKey3DObj.Calc3DZoomOut}
  OldZoom := TheViewZoomVar;
  if TheViewZoomVar <= Min3DViewZoom then
    begin
      TheViewZoomVar := Min3DViewZoom;
      Result := (OldZoom <> TheViewZoomVar);
      exit;
    end; {if}
  if TheViewZoomVar > 500 then
    I := 1
  else
  if TheViewZoomVar > 200 then
    I := 2
  else
  if TheViewZoomVar > 20 then
    I := 3
  else {if TheViewZoomVar > 1 then}
    I := 4;
  TheViewZoomVar := TheViewZoomVar - abs(ZoomRange3DIntArray[I]);
  if TheViewZoomVar > Max3DViewZoom then
    TheViewZoomVar := Max3DViewZoom
  else
    if TheViewZoomVar < Min3DViewZoom then
      TheViewZoomVar := Min3DViewZoom;
  Result := (OldZoom <> TheViewZoomVar);
end; {function THotKey3DObj.Calc3DZoomOut}

{--------------------------------------------------------------------}

end.
