unit PtGr2d;

(*
Copyright 1999-2011 Greg VanMullem
This program is distributed under the terms of the GNU General Public License Version 3

This file is part of MathGV.

MathGV is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

MathGV is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MathGV.  If not, see <http://www.gnu.org/licenses/>.
*)

interface

uses
  Windows, //eventually delete this, use Delphi call instead of API call.
  Classes, Graphics, MathCalc, MathFunc, PtGrList, PtShared, Vw2dObj, Set2dObj,
  func2d, funcpara, FloatStr;

const
  Max2DFuncCount = 252;

type
  TPlotGraph2D = class(TPlotGraphFuncList)
  private
    { Private declarations }
    LinePointArray : TLinePointArray;
    FView2DZoomObj : TView2DZoomObj;
    FSetting2DObj : TSetting2DObj;
    procedure GraphWinPaint;
  protected
    { Protected declarations }
  public
    { Public declarations }
    property View2DZoomObj : TView2DZoomObj read FView2DZoomObj;
    property Setting2DObj : TSetting2DObj read FSetting2DObj;
    constructor Create;
    destructor Destroy; override;
    function FullGraphPaint : string; override;
    procedure RestartAnimatedGraphPaint; override;
    function StepAnimatedGraphPaint : boolean; override;
  end; {TPlotGraph2D}

implementation

{ TPlotGraph2D }

{---------------------------------------------------------------------}

constructor TPlotGraph2D.Create;
begin {constructor TPlotGraph2D.Create}
  inherited;
  FView2DZoomObj := TView2DZoomObj.Create;
  FSetting2DObj := TSetting2DObj.Create;
end; {constructor TPlotGraph2D.Create}

{---------------------------------------------------------------------}

destructor TPlotGraph2D.Destroy;
begin {destructor TPlotGraph2D.Destroy;}
  FView2DZoomObj.free;
  FSetting2DObj.free;
  inherited;
end; {destructor TPlotGraph2D.Destroy;}

{---------------------------------------------------------------------}

function TPlotGraph2D.FullGraphPaint : string;
begin {function TPlotGraph2D.FullGraphPaint}
  Result := '';
  GraphWinPaint;
end; {function TPlotGraph2D.FullGraphPaint}

{=====================================================================}

procedure TPlotGraph2D.GraphWinPaint;
type
  TVirtualPix = record
    X,Y : TMathFloat;
  end;
  TBmpPix = record
    X,Y : integer;
  end;
const
  MinTickPixDistance = 5;
  TickMLabelDistance = 1;
var
  { All Bmp Variables use the TopLeft corner = (0,0) standard}
  BmpCenter : TBmpPix;
  BmpHeightSub1,
  BmpWidthSub1 : integer;
  { All View Variables use a cartesian type cordinate system
      with the Center at (0,0) if View2DZoomObj.ViewCenterX = 0 and
      View2DZoomObj.ViewCenterY = 0}
  ViewCenterPix, ViewTopLeftPix, ViewBottomRightPix : TVirtualPix;
  BmpLoEnd, BmpHiEnd : TMathFloat;
  ClipTop, ClipBottom : TMathFloat;
var
  LocalXViewZoom : TMathFloat;
  LocalYViewZoom : TMathFloat;

{----------------------------------------------------------------------}

procedure VirtualToBmpPix(const TheVirRec : TVirtualPix;
                          var TheBmpRec : TBmpPix);
begin {procedure VirtualToBmpPix}
  TheBmpRec.X :=Round(TheVirRec.X-ViewCenterPix.X) + BmpCenter.X;
  TheBmpRec.Y :=Round(ViewCenterPix.Y - TheVirRec.Y) + BmpCenter.Y;
end; {procedure VirtualToBmpPix}

{----------------------------------------------------------------------}

function VirFloatXToBmpX(TheVir : TMathFloat) : integer;
begin {function VirFloatXToBmpX}
  Result := Round(TheVir - ViewCenterPix.X) + BmpCenter.X;
end; {function VirFloatXToBmpX}

{----------------------------------------------------------------------}

function VirFloatYToBmpY(TheVir : TMathFloat) : integer;
begin {function VirFloatYToBmpY}
  Result := Round(ViewCenterPix.Y - TheVir) + BmpCenter.Y;
end; {function VirFloatYToBmpY}

{----------------------------------------------------------------------}

procedure InitialCalculations;
begin {procedure InitialCalculations}
  {Begin Set Local Var's}
  LocalXViewZoom := View2DZoomObj.XViewZoom;
  LocalYViewZoom := View2DZoomObj.YViewZoom;
  {End Set Local Var's}
  BmpHeightSub1 := BMPHeight -1;
  BmpWidthSub1 := BMPWidth -1;
  BmpCenter.X := (BMPWidth div 2);
  BmpCenter.Y := (BMPHeight div 2);
  with ViewCenterPix do
    begin
      X := View2DZoomObj.ViewCenterX;
      Y := View2DZoomObj.ViewCenterY;
    end; {with}
  if DoIsTerminated then Exit;
  with ViewTopLeftPix do
    begin
      X := ViewCenterPix.X - BmpCenter.X;
      Y := ViewCenterPix.Y + BmpCenter.Y;
    end; {with}
  with ViewBottomRightPix do
    begin
      X := ViewCenterPix.X + (BmpWidthSub1 - BmpCenter.X);
      Y := ViewCenterPix.Y - (BmpHeightSub1 - BmpCenter.Y);
    end; {with}
  BmpLoEnd := ViewTopLeftPix.X;
  BmpHiEnd := ViewBottomRightPix.X;
  ClipTop := ViewTopLeftPix.Y + 2500;
  ClipBottom := ViewBottomRightPix.Y - 2500;
end; {procedure InitialCalculations}

{-----------------------------------------------------------------------}

procedure DrawTheAxisLines;
var
  XAxisInView, YAxisInView : boolean;
  XLimitsInView, YLimitsInView : boolean;
  VirtualXLimitLoAxis, VirtualXLimitHiAxis,
  VirtualYLimitLoAxis, VirtualYLimitHiAxis : TMathFloat;
  XAxisLineWasDrawn, YAxisLineWasDrawn : boolean;
  XTicksWereDrawn, YTicksWereDrawn : boolean;

{---------------------------------------}

procedure InitalizeAxisLineVars;
begin {procedure InitalizeAxisLineVars;}
  XAxisInView := (ViewTopLeftPix.Y >= 0)and(ViewBottomRightPix.Y <= 0);
  YAxisInView := (ViewTopLeftPix.X <= 0)and(ViewBottomRightPix.X >= 0);
  {Init X Vars}
  VirtualXLimitLoAxis := 0;
  VirtualXLimitHiAxis := 0;
  if Setting2DObj.XLimitLoAxisEmpty then
    VirtualXLimitLoAxis := ViewTopLeftPix.X
  else
    begin
      VirtualXLimitLoAxis := Setting2DObj.XLimitLoAxisVal.FloatValue * LocalXViewZoom;
      if VirtualXLimitLoAxis <= ViewTopLeftPix.X then
        VirtualXLimitLoAxis := ViewTopLeftPix.X;
    end; {else}
  if Setting2DObj.XLimitHiAxisEmpty then
    VirtualXLimitHiAxis := ViewBottomRightPix.X
  else
    begin
      VirtualXLimitHiAxis := Setting2DObj.XLimitHiAxisVal.FloatValue * LocalXViewZoom;
      if VirtualXLimitHiAxis >= ViewBottomRightPix.X then
        VirtualXLimitHiAxis := ViewBottomRightPix.X;
    end; {else}
  XLimitsInView := (VirtualXLimitLoAxis <= VirtualXLimitHiAxis) and
                   (VirtualXLimitLoAxis <= ViewBottomRightPix.X) and
                   (VirtualXLimitHiAxis >= ViewTopLeftPix.X);
  if DoIsTerminated then Exit;
  {Init Y Vars}
  VirtualYLimitLoAxis := 0;
  VirtualYLimitHiAxis := 0;
  if Setting2DObj.YLimitLoAxisEmpty then
    VirtualYLimitLoAxis := ViewBottomRightPix.Y
  else
    begin
      VirtualYLimitLoAxis := Setting2DObj.YLimitLoAxisVal.FloatValue * LocalYViewZoom;
      if VirtualYLimitLoAxis <= ViewBottomRightPix.Y then
        VirtualYLimitLoAxis := ViewBottomRightPix.Y;
    end; {else}
  if Setting2DObj.YLimitHiAxisEmpty then
    VirtualYLimitHiAxis := ViewTopLeftPix.Y
  else
    begin
      VirtualYLimitHiAxis := Setting2DObj.YLimitHiAxisVal.FloatValue * LocalYViewZoom;
      if VirtualYLimitHiAxis >= ViewTopLeftPix.Y then
        VirtualYLimitHiAxis := ViewTopLeftPix.Y;
    end; {else}
  YLimitsInView := (VirtualYLimitLoAxis <= VirtualYLimitHiAxis) and
                   (VirtualYLimitLoAxis <= ViewTopLeftPix.Y) and
                   (VirtualYLimitHiAxis >= ViewBottomRightPix.Y);
end; {procedure InitalizeAxisLineVars;}

{---------------------------------------}

procedure DrawXAxisLine;
var
  TmpBmpCord : integer;
begin {procedure DrawXAxisLine;}
  if XAxisInView and XLimitsInView and Setting2DObj.XShowAxisLines then
    begin {draw the line and the tick marks}
      with Setting2DObj do
        begin
          GraphBitmap.Canvas.Pen.Color := XAxisColor;
          GraphBitmap.Canvas.Pen.Width := XAxisWidth;
          GraphBitmap.Canvas.Pen.Style := XAxisStyle;
        end; {with}
      TmpBmpCord := VirFloatYToBmpY(0);
      if DoIsTerminated then Exit;
      if VirtualXLimitLoAxis = ViewTopLeftPix.X then
        GraphBitmap.Canvas.MoveTo(0, TmpBmpCord)
      else
        GraphBitmap.Canvas.MoveTo(VirFloatXToBmpX(VirtualXLimitLoAxis), TmpBmpCord);
      if VirtualXLimitHiAxis = ViewBottomRightPix.X then
        GraphBitmap.Canvas.LineTo(BMPWidth, TmpBmpCord)
      else
        GraphBitmap.Canvas.LineTo(VirFloatXToBmpX(VirtualXLimitHiAxis)+1, TmpBmpCord);
      XAxisLineWasDrawn := true;
    end; {if}
end; {procedure DrawXAxisLine;}

{---------------------------------------}

procedure DrawYAxisLine;
var
  TmpBmpCord : integer;
begin {procedure DrawYAxisLine;}
  if YAxisInView and YLimitsInView and Setting2DObj.YShowAxisLines then
    begin {draw the line and the tick marks}
      with Setting2DObj do
        begin
          GraphBitmap.Canvas.Pen.Color := YAxisColor;
          GraphBitmap.Canvas.Pen.Width := YAxisWidth;
          GraphBitmap.Canvas.Pen.Style := YAxisStyle;
        end; {with}
      TmpBmpCord := VirFloatXToBmpX(0);
      if DoIsTerminated then Exit;
      if VirtualYLimitHiAxis = ViewTopLeftPix.Y then
        GraphBitmap.Canvas.MoveTo(TmpBmpCord, 0)
      else
        GraphBitmap.Canvas.MoveTo(TmpBmpCord, VirFloatYToBmpY(VirtualYLimitHiAxis));
      if VirtualYLimitLoAxis = ViewBottomRightPix.Y then
        GraphBitmap.Canvas.LineTo(TmpBmpCord, BMPHeight)
      else
        GraphBitmap.Canvas.LineTo(TmpBmpCord, VirFloatYToBmpY(VirtualYLimitLoAxis)+1);
      YAxisLineWasDrawn := true;
    end; {if}
end; {procedure DrawYAxisLine;}

{---------------------------------------}

procedure DrawGridMarks;
var
  XTickMPixInc, XCurrTickInc, XTickMCounter : TMathFloat;
  FirstYTickMPixInc, FirstYCurrTickInc, FirstYTickMCounter : TMathFloat;
  YTickMPixInc, YCurrTickInc, YTickMCounter : TMathFloat;
  XBmp, YBmp : integer;

{----------------}

procedure DrawAGridPoint;
begin {procedure DrawAGridPoint;}
  if Setting2DObj.XTickMarkType = tmGridMarks then
    begin
      with Setting2DObj do
      with GraphBitmap.Canvas do
        begin
          Pen.Color := XTickMarkColor;
          Pen.Width := XAxisWidth;
          Pen.Style := psSolid;
          MoveTo(XBmp, YBmp - XTickMarkLength);
          lineTo(XBmp, YBmp + XTickMarkLength);
          Pixels[XBmp, YBmp + XTickMarkLength] := XTickMarkColor;
        end; {with}
    end; {if}
  if Setting2DObj.YTickMarkType = tmGridMarks then
    begin
      with Setting2DObj do
      with GraphBitmap.Canvas do
        begin
          Pen.Color := Setting2DObj.YTickMarkColor;
          Pen.Width := Setting2DObj.YAxisWidth;
          Pen.Style := psSolid;
          MoveTo(XBmp - YTickMarkLength, YBmp);
          lineTo(XBmp + YTickMarkLength, YBmp);
          Pixels[XBmp + YTickMarkLength, YBmp] := YTickMarkColor;
        end; {with}
    end; {if}
end; {procedure DrawAGridPoint;}

{----------------}

begin {procedure DrawGridMarks;}
  if ((Setting2DObj.XTickMarkType = tmGridMarks) or
      (Setting2DObj.YTickMarkType = tmGridMarks)) and
     XLimitsInView and (Setting2DObj.XTickMarkIncVal.FloatValue > 0) and
     YLimitsInView and (Setting2DObj.YTickMarkIncVal.FloatValue > 0) then
    begin
      {init X vars}
      XTickMPixInc := Setting2DObj.XTickMarkIncVal.FloatValue * LocalXViewZoom;
      if MinTickPixDistance > XTickMPixInc then
        Exit;
      XTickMCounter := int(VirtualXLimitLoAxis / XTickMPixInc);
      XCurrTickInc := XTickMPixInc * XTickMCounter;
      if XCurrTickInc < VirtualXLimitLoAxis then
        begin
          XTickMCounter := XTickMCounter + 1;
          XCurrTickInc := XTickMPixInc * XTickMCounter;
        end; {if}
      if DoIsTerminated then Exit;
      {init Y vars}
      FirstYTickMPixInc := Setting2DObj.YTickMarkIncVal.FloatValue * LocalYViewZoom;
      if MinTickPixDistance > FirstYTickMPixInc then
        Exit;
      FirstYTickMCounter := int(VirtualYLimitLoAxis / FirstYTickMPixInc);
      FirstYCurrTickInc := FirstYTickMPixInc * FirstYTickMCounter;
      if FirstYCurrTickInc < VirtualYLimitLoAxis then
        begin
          FirstYTickMCounter := FirstYTickMCounter + 1;
          FirstYCurrTickInc := FirstYTickMPixInc * FirstYTickMCounter;
        end; {if}
      if DoIsTerminated then Exit;
      {drawing loop}
      while XCurrTickInc <= VirtualXLimitHiAxis do
        begin
          if (XCurrTickInc <> 0) or (not Setting2DObj.YShowAxisLines) then
            begin
              XBmp := VirFloatXToBmpX(XCurrTickInc);
              YTickMPixInc := FirstYTickMPixInc;
              YCurrTickInc := FirstYCurrTickInc;
              YTickMCounter := FirstYTickMCounter;
              while YCurrTickInc <= VirtualYLimitHiAxis do
                begin
                  if (YCurrTickInc <> 0) or (not Setting2DObj.XShowAxisLines) then
                    begin
                      YBmp := VirFloatYToBmpY(YCurrTickInc);
                      DrawAGridPoint;
                    end; {if}
                  YTickMCounter := YTickMCounter + 1;
                  YCurrTickInc := YTickMPixInc * YTickMCounter;
                  if DoIsTerminated then Exit;
                end; {while}
            end; {if}
          XTickMCounter := XTickMCounter + 1;
          XCurrTickInc := XTickMPixInc * XTickMCounter;
          if DoIsTerminated then Exit;
        end; {while}
    end; {if}
end; {procedure DrawGridMarks;}

{---------------------------------------}

procedure DrawXTicks;
var
  TickMPixInc : TMathFloat;
  CurrTickInc : TMathFloat;
  TickMCounter : TMathFloat;
  XBmp, YBmp, TmpYBmp : integer;
  BmpYMoveTo, BmpYLineTo : integer;
begin {procedure DrawXTicks;}
  if XLimitsInView and (Setting2DObj.XTickMarkIncVal.FloatValue > 0) then
  if (Setting2DObj.XTickMarkType = tmGridLines) or
     ((Setting2DObj.XTickMarkType in [tmTickMarks, tmGridMarks]) and
       XAxisLineWasDrawn) then
    begin
      TickMPixInc := Setting2DObj.XTickMarkIncVal.FloatValue * LocalXViewZoom;
      if MinTickPixDistance > TickMPixInc then
        Exit;
      TickMCounter := int(VirtualXLimitLoAxis / TickMPixInc);
      CurrTickInc := TickMPixInc * TickMCounter;
      if CurrTickInc < VirtualXLimitLoAxis then
        begin
          TickMCounter := TickMCounter + 1;
          CurrTickInc := TickMPixInc * TickMCounter;
        end; {if}
      with GraphBitmap.Canvas.Pen do
        begin
          Color := Setting2DObj.XTickMarkColor;
          Width := Setting2DObj.XAxisWidth;
          if Setting2DObj.XTickMarkType = tmGridLines then
            Style := Setting2DObj.XGridLineStyle
          else
            Style := psSolid;
        end; {with}
      if DoIsTerminated then Exit;
      if Setting2DObj.XTickMarkType = tmGridLines then
        begin
          if YLimitsInView then
            begin
              if VirtualYLimitHiAxis = ViewTopLeftPix.Y then
                BmpYMoveTo := 0
              else
                BmpYMoveTo := VirFloatYToBmpY(VirtualYLimitHiAxis);
              if VirtualYLimitLoAxis = ViewBottomRightPix.Y then
                BmpYLineTo := BMPHeight
              else
                BmpYLineTo := VirFloatYToBmpY(VirtualYLimitLoAxis) + 1;
              while CurrTickInc <= VirtualXLimitHiAxis do
                begin
                  if DoIsTerminated then Exit;
                  if (CurrTickInc <> 0) or (not Setting2DObj.YShowAxisLines) then
                    begin
                      XBmp := VirFloatXToBmpX(CurrTickInc);
                      GraphBitmap.Canvas.MoveTo(XBmp, BmpYMoveTo);
                      GraphBitmap.Canvas.lineTo(XBmp, BmpYLineTo);
                    end; {if}
                  TickMCounter := TickMCounter + 1;
                  CurrTickInc := TickMPixInc * TickMCounter;
                end; {while}
            end; {if}
        end
      else
        begin
          TmpYBmp := VirFloatYToBmpY(0);
          while CurrTickInc <= VirtualXLimitHiAxis do
            begin
              if DoIsTerminated then Exit;
              if (CurrTickInc <> 0) or (not Setting2DObj.YShowAxisLines) then
                begin
                  XBmp := VirFloatXToBmpX(CurrTickInc);
                  YBmp := TmpYBmp;
                  GraphBitmap.Canvas.MoveTo(XBmp, YBmp-Setting2DObj.XTickMarkLength);
                  YBmp := YBmp + Setting2DObj.XTickMarkLength;
                  GraphBitmap.Canvas.lineTo(XBmp, YBmp);
                  GraphBitmap.Canvas.Pixels[XBmp, YBmp] := Setting2DObj.XTickMarkColor;
                end; {if}
              TickMCounter := TickMCounter + 1;
              CurrTickInc := TickMPixInc * TickMCounter;
            end; {while}
        end; {else}
      XTicksWereDrawn := true;
    end; {if}
end; {procedure DrawXTicks;}

{---------------------------------------}

procedure DrawYTicks;
var
  TickMPixInc : TMathFloat;
  CurrTickInc : TMathFloat;
  TickMCounter : TMathFloat;
  XBmp, YBmp, TmpXBmp : integer;
  BmpXMoveTo, BmpXLineTo : integer;
begin {procedure DrawYTicks;}
  if YLimitsInView and (Setting2DObj.YTickMarkIncVal.FloatValue > 0) then
  if (Setting2DObj.YTickMarkType = tmGridLines) or
     ((Setting2DObj.YTickMarkType in [tmTickMarks, tmGridMarks]) and
       YAxisLineWasDrawn) then
    begin
      TickMPixInc := Setting2DObj.YTickMarkIncVal.FloatValue * LocalYViewZoom;
      if MinTickPixDistance > TickMPixInc then
        Exit;
      TickMCounter := int(VirtualYLimitLoAxis / TickMPixInc);
      CurrTickInc := TickMPixInc * TickMCounter;
      if CurrTickInc < VirtualYLimitLoAxis then
        begin
          TickMCounter := TickMCounter + 1;
          CurrTickInc := TickMPixInc * TickMCounter;
        end; {if}
      with GraphBitmap.Canvas.Pen do
        begin
          Color := Setting2DObj.YTickMarkColor;
          Width := Setting2DObj.YAxisWidth;
          if Setting2DObj.YTickMarkType = tmGridLines then
            Style := Setting2DObj.YGridLineStyle
          else
            Style := psSolid;
        end; {with}
      if DoIsTerminated then Exit;
      if Setting2DObj.YTickMarkType = tmGridLines then
        begin
          if XLimitsInView then
            begin
              if VirtualXLimitLoAxis = ViewTopLeftPix.X then
                BmpXMoveTo := 0
              else
                BmpXMoveTo := VirFloatXToBmpX(VirtualXLimitLoAxis);
              if VirtualXLimitHiAxis = ViewBottomRightPix.X then
                BmpXLineTo := BMPWidth
              else
                BmpXLineTo := VirFloatXToBmpX(VirtualXLimitHiAxis) + 1;
              while CurrTickInc <= VirtualYLimitHiAxis do
                begin
                  if DoIsTerminated then Exit;
                  if (CurrTickInc <> 0) or (not Setting2DObj.XShowAxisLines) then
                    begin
                      YBmp := VirFloatYToBmpY(CurrTickInc);
                      GraphBitmap.Canvas.MoveTo(BmpXMoveTo, YBmp);
                      GraphBitmap.Canvas.lineTo(BmpXLineTo, YBmp);
                    end; {if}
                  TickMCounter := TickMCounter + 1;
                  CurrTickInc := TickMPixInc * TickMCounter;
                end; {while}
            end; {if}
        end
      else
        begin
          TmpXBmp := VirFloatXToBmpX(0);
          while CurrTickInc <= VirtualYLimitHiAxis do
            begin
              if DoIsTerminated then Exit;
              if (CurrTickInc <> 0) or (not Setting2DObj.XShowAxisLines) then
                begin
                  XBmp := TmpXBmp;
                  YBmp := VirFloatYToBmpY(CurrTickInc);
                  GraphBitmap.Canvas.MoveTo(XBmp-Setting2DObj.YTickMarkLength, YBmp);
                  XBmp := XBmp + Setting2DObj.YTickMarkLength;
                  GraphBitmap.Canvas.lineTo(XBmp, YBmp);
                  GraphBitmap.Canvas.Pixels[XBmp, YBmp] := Setting2DObj.YTickMarkColor;
                end; {if}
              TickMCounter := TickMCounter + 1;
              CurrTickInc := TickMPixInc * TickMCounter;
            end; {while}
        end; {else}
      YTicksWereDrawn := true;
    end; {if}
end; {procedure DrawYTicks;}

{---------------------------------------}

procedure DrawXLabels;
var
  TickMPixInc : TMathFloat;
  CurrTickInc : TMathFloat;
  TickMCounter : TMathFloat;
  TmpYBmp : integer;

{----------------}

procedure XDrawLabelAtGrid(XPos, YPos : integer; TheTMCount : TMathFloat);
var
  TMLabelVal : TMathFloat;
  TMLabelStr : string;
begin {procedure XDrawLabelAtGrid}
  with Setting2DObj do
    begin
      TMLabelVal := TheTMCount * XTickMarkIncVal.FloatValue;
      TMLabelStr := FloatCordToFormatStr(TMLabelVal,
                    XTMLabelFormat, XShowTMLabelComa,
                    XShowTMLabelPlus, XTMLabelDecPlaces);
    end; {with}
  if TMLabelVal >= 0 then
    XPos := XPos + TickMLabelDistance
  else
    XPos := XPos - abs(GraphBitmap.Canvas.TextWidth(TMLabelStr)) -
            TickMLabelDistance;
  GraphBitmap.Canvas.TextOut(XPos, YPos, TMLabelStr);
end; {procedure XDrawLabelAtGrid}

{----------------}

procedure XDrawLabelAtDash(XPos, YPos : integer; TheTMCount : TMathFloat);
var
  TMLabelStr : string;
begin {procedure XDrawLabelAtDash}
  with Setting2DObj do
    TMLabelStr := FloatCordToFormatStr(TheTMCount*XTickMarkIncVal.FloatValue,
                  XTMLabelFormat, XShowTMLabelComa,
                  XShowTMLabelPlus, XTMLabelDecPlaces);
  XPos := XPos - (abs(GraphBitmap.Canvas.TextWidth(TMLabelStr)) div 2);
  GraphBitmap.Canvas.TextOut(XPos, YPos, TMLabelStr);
end; {procedure XDrawLabelAtDash}

{----------------}

begin {procedure DrawXLabels;}
  if Setting2DObj.XShowTMLabels and XTicksWereDrawn and XAxisLineWasDrawn then
    begin
      GraphBitmap.Canvas.Font.Assign(Setting2DObj.XLabelFont);
      if DoIsTerminated then Exit;
      TickMPixInc := Setting2DObj.XTickMarkIncVal.FloatValue * LocalXViewZoom;
      if MinTickPixDistance > TickMPixInc then
        Exit;
      TickMCounter := int(VirtualXLimitLoAxis / TickMPixInc);
      CurrTickInc := TickMPixInc * TickMCounter;
      if CurrTickInc < VirtualXLimitLoAxis then
        begin
          TickMCounter := TickMCounter + 1;
          CurrTickInc := TickMPixInc * TickMCounter;
        end; {if}
      TmpYBmp := VirFloatYToBmpY(0);
      if DoIsTerminated then Exit;
      if Setting2DObj.XTickMarkType = tmGridLines then
        begin
          while CurrTickInc <= VirtualXLimitHiAxis do
            begin
              if DoIsTerminated then Exit;
              if (CurrTickInc <> 0) or (not Setting2DObj.YShowAxisLines) then
                XDrawLabelAtGrid(VirFloatXToBmpX(CurrTickInc),
                                 TmpYBmp + TickMLabelDistance, TickMCounter);
              TickMCounter := TickMCounter + 1;
              CurrTickInc := TickMPixInc * TickMCounter;
            end; {while}
        end
      else
        begin
          while CurrTickInc <= VirtualXLimitHiAxis do
            begin
              if DoIsTerminated then Exit;
              if (CurrTickInc <> 0) or (not Setting2DObj.YShowAxisLines) then
                XDrawLabelAtDash(VirFloatXToBmpX(CurrTickInc),
                   (TmpYBmp + Setting2DObj.XTickMarkLength + TickMLabelDistance),
                   TickMCounter);
              TickMCounter := TickMCounter + 1;
              CurrTickInc := TickMPixInc * TickMCounter;
            end; {while}
        end; {else}
    end; {if}
end; {procedure DrawXLabels;}

{---------------------------------------}

procedure DrawYLabels;
var
  TickMPixInc : TMathFloat;
  CurrTickInc : TMathFloat;
  TickMCounter : TMathFloat;
  TmpXBmp : integer;

{----------------}

procedure YDrawLabelAtGrid(XPos, YPos : integer; TheTMCount : TMathFloat);
var
  TMLabelVal : TMathFloat;
  TMLabelStr : string;
begin {procedure YDrawLabelAtGrid}
  with Setting2DObj do
    begin
      TMLabelVal := TheTMCount * YTickMarkIncVal.FloatValue;
      TMLabelStr := FloatCordToFormatStr(TMLabelVal,
                    YTMLabelFormat, YShowTMLabelComa,
                    YShowTMLabelPlus, YTMLabelDecPlaces);
    end; {with}
  if TMLabelVal >= 0 then
    YPos := YPos - abs(GraphBitmap.Canvas.TextHeight(TMLabelStr)) - TickMLabelDistance
  else
    YPos := YPos + TickMLabelDistance;
  GraphBitmap.Canvas.TextOut(XPos, YPos, TMLabelStr);
end; {procedure YDrawLabelAtGrid}

{----------------}

procedure YDrawLabelAtDash(XPos, YPos : integer; TheTMCount : TMathFloat);
var
  TMLabelVal : TMathFloat;
  TMLabelStr : string;
begin {procedure YDrawLabelAtDash}
  with Setting2DObj do
    begin
      TMLabelVal := TheTMCount * YTickMarkIncVal.FloatValue;
      TMLabelStr := FloatCordToFormatStr(TMLabelVal,
                    YTMLabelFormat, YShowTMLabelComa,
                    YShowTMLabelPlus, YTMLabelDecPlaces);
    end; {with}
  if TMLabelVal < 0 then
    XPos := XPos + 3;
  YPos := YPos - (abs(GraphBitmap.Canvas.TextHeight(TMLabelStr)) div 2);
  GraphBitmap.Canvas.TextOut(XPos, YPos, TMLabelStr);
end; {procedure YDrawLabelAtDash}

{----------------}

begin {procedure DrawYLabels;}
  if Setting2DObj.YShowTMLabels and YTicksWereDrawn and YAxisLineWasDrawn then
    begin
      GraphBitmap.Canvas.Font.Assign(Setting2DObj.YLabelFont);
      if DoIsTerminated then Exit;
      TickMPixInc := Setting2DObj.YTickMarkIncVal.FloatValue * LocalYViewZoom;
      if MinTickPixDistance > TickMPixInc then
        Exit;
      TickMCounter := int(VirtualYLimitLoAxis / TickMPixInc);
      CurrTickInc := TickMPixInc * TickMCounter;
      if CurrTickInc < VirtualYLimitLoAxis then
        begin
          TickMCounter := TickMCounter + 1;
          CurrTickInc := TickMPixInc * TickMCounter;
        end; {if}
      TmpXBmp := VirFloatXToBmpX(0);
      if DoIsTerminated then Exit;
      if Setting2DObj.YTickMarkType = tmGridLines then
        begin
          while CurrTickInc <= VirtualYLimitHiAxis do
            begin
              if DoIsTerminated then Exit;
              if (CurrTickInc <> 0) or (not Setting2DObj.XShowAxisLines) then
                YDrawLabelAtGrid(TmpXBmp + TickMLabelDistance,
                                 VirFloatYToBmpY(CurrTickInc), TickMCounter);
              TickMCounter := TickMCounter + 1;
              CurrTickInc := TickMPixInc * TickMCounter;
            end; {while}
        end
      else
        begin
          while CurrTickInc <= VirtualYLimitHiAxis do
            begin
              if DoIsTerminated then Exit;
              if (CurrTickInc <> 0) or (not Setting2DObj.XShowAxisLines) then
                YDrawLabelAtDash((TmpXBmp + Setting2DObj.YTickMarkLength +
                  TickMLabelDistance), VirFloatYToBmpY(CurrTickInc), TickMCounter);
              TickMCounter := TickMCounter + 1;
              CurrTickInc := TickMPixInc * TickMCounter;
            end; {while}
        end; {else}
    end; {if}
end; {procedure DrawYLabels;}

{---------------------------------------}

begin {procedure DrawTheAxisLines;}
  try
    XAxisLineWasDrawn := false;
    YAxisLineWasDrawn := false;
    XTicksWereDrawn := false;
    YTicksWereDrawn := false;
    if DoIsTerminated then Exit;
    InitalizeAxisLineVars;
    if DoIsTerminated then Exit;
    DrawXAxisLine;
    if DoIsTerminated then Exit;
    DrawYAxisLine;
    if DoIsTerminated then Exit;
    try
      DrawGridMarks;
    except
    end; {try..except}
    if DoIsTerminated then Exit;
    try
      DrawXTicks;
    except
      XTicksWereDrawn := false;
    end; {try..except}
    if DoIsTerminated then Exit;
    try
      DrawYTicks;
    except
      YTicksWereDrawn := false;
    end; {try..except}
    if DoIsTerminated then Exit;
    try
      DrawXLabels;
    except
    end; {try..except}
    if DoIsTerminated then Exit;
    try
      DrawYLabels;
    except
    end; {try..except}
  except
  end; {try..except}
end; {procedure DrawTheAxisLines;}

{-----------------------------------------------------------------------}

function VirFloatToBmpLo(TheVir : TMathFloat) : integer;
var
  TmpCord : TMathFloat;
begin {function VirFloatToBmpLo}
  TmpCord := TheVir - ViewCenterPix.X;
  if frac(TmpCord) <> 0 then
    TmpCord := int(TmpCord) + 1;
  Result := trunc(TmpCord) + BmpCenter.X;
end; {function VirFloatToBmpLo}

{-----------------------------------------------------------------------}

function VirFloatToBmpHi(TheVir : TMathFloat) : integer;
var
  TmpCord : TMathFloat;
begin {function VirFloatToBmpHi}
  TmpCord := TheVir - ViewCenterPix.X;
  if frac(TmpCord) <> 0 then
    TmpCord := int(TmpCord) - 1;
  Result := trunc(TmpCord) + BmpCenter.X;
end; {function VirFloatToBmpHi}

{----------------------------------------------------------------------}

function BmpXToVirFloatX(TheBmpX : integer) : TMathFloat;
begin {function BmpXToVirFloatX}
  Result := ((TheBmpX - BmpCenter.X) + ViewCenterPix.X);
end; {function BmpXToVirFloatX}

{----------------------------------------------------------------------}

function BmpXToFuncX(TheBmpX : integer) : TMathFloat;
begin {function BmpXToFuncX}
  Result := ((TheBmpX - BmpCenter.X) + ViewCenterPix.X) / LocalXViewZoom;
end; {function BmpXToFuncX}

{-----------------------------------------------------------------------}

function FuncYToClipBmpY(TheFuncY : TMathFloat) : integer;
var
  TmpVirPixY : TMathFloat;
begin {function FuncYToClipBmpY}
  try
    TmpVirPixY := TheFuncY * LocalYViewZoom;
  except
    if TheFuncY >= 0 then
      TmpVirPixY := ClipTop
    else
      TmpVirPixY := ClipBottom;
  end; {try..Except}
  if TmpVirPixY > ClipTop then
    TmpVirPixY := ClipTop
  else
    if TmpVirPixY < ClipBottom then
      TmpVirPixY := ClipBottom;
  try
    {InLine Replacement BmpY := VirFloatYToBmpY(CurrPixY);}
    Result := Round(ViewCenterPix.Y - TmpVirPixY) + BmpCenter.Y;
  except
    Result := 0; {Should never happen}
  end; {try.except}
end; {function FuncYToClipBmpY}

{-----------------------------------------------------------------------}

procedure Draw2DCartesianFunc(CurrFuncObj : TFunc2DObj);

type
  TSlopeStatus = (ssNegative, ssPositive, ssZero, ssUndefinedPoint);
  TPointDefinedStatus = (dsEmpty, dsDefined, dsNotDefined);
  TSidePointBehave = (psNoExtraAccuracy, psExtraAccuracy,
                      psDrawPosInfinity, psDrawNegInfinity, psNotContinuous);
  TPlotPointRec = record
    DefinedStatus : TPointDefinedStatus;
    LeftSidePointBehave, RightSidePointBehave : TSidePointBehave;
    BmpX, BmpY : integer;
    FuncX, FuncY : TMathFloat;
  end; {TPlotPointRec}

var
  {ploting queues}
  SlopeFrom1To2Que, SlopeFrom2To3Que, SlopeFrom3To4Que : TSlopeStatus;
  PlotPointQue : array[1..4] of TPlotPointRec;
  DrawFrom1To2Que, DrawFrom2To3Que, DrawFrom3To4Que : boolean;
  GotSpPoint1To2Que, GotSpPoint2To3Que, GotSpPoint3To4Que : boolean;
  {other variables}
  CurrLoFuncBoundX,
  CurrHiFuncBoundX : TMathFloat;
  BmpLo, BmpHi, CurrBmpX : integer;
  LinePointCount : integer;
  PointsCurrForceNdx,
  GapsCurrForceNdx : integer;
  {Local declarations}
  LocalFuncCalc : TMath2DFunc;
  LocalSpecialPointArray : TSpecialPointArray;
  LocalAsymFixUp : boolean;
  LocalExtraUndefAccuracy : boolean;

{----------------------------}

function InitDrawAFunc : boolean;
var
  CurrLoBlank,
  CurrHiBlank : boolean;
  CurrLoBoundVal,
  CurrHiBoundVal : TMathFloat;
begin {function InitDrawAFunc}
  Result := false;
  LinePointCount := 0;
  CurrLoBoundVal := 0; //initalize to stop compiler warning
  CurrHiBoundVal := 0; //initalize to stop compiler warning
  with CurrFuncObj do
    begin
      LocalFuncCalc := FuncCalc;
      LocalSpecialPointArray := Func2DSpecialValues.SpecialPointArray;
      LocalAsymFixUp := AsymFixUp;
      LocalExtraUndefAccuracy := ExtraUndefAccuracy;
      GraphBitmap.Canvas.Pen.Color := FuncColor;
      GraphBitmap.Canvas.Pen.Width := LineWidth;
      CurrLoBlank := LoBlank;
      CurrHiBlank := HiBlank;
      try
        if not CurrLoBlank then
          CurrLoBoundVal := LoBoundVal.FloatValue * LocalXViewZoom;
      except
        CurrLoBlank := true;
      end; {try..except}
      try
        if not CurrHiBlank then
          CurrHiBoundVal := HiBoundVal.FloatValue * LocalXViewZoom;
      except
        CurrHiBlank := true;
      end; {try..Except}
    end; {with}
  if CurrLoBlank then
    begin
      BmpLo := -1;
      CurrLoFuncBoundX := BmpXToFuncX(-1);
    end
  else
    begin
      if CurrLoBoundVal <= BmpHiEnd then
        begin
          CurrLoFuncBoundX := CurrFuncObj.LoBoundVal.FloatValue;
          if CurrLoBoundVal <= BmpLoEnd then
            begin
              if CurrLoBoundVal <= BmpXToVirFloatX(-1) then
                BmpLo := -1
              else
                BmpLo := 0;
            end
          else
            BmpLo := VirFloatToBmpLo(CurrLoBoundVal);
        end
      else
        Exit; {Function is not visible}
    end; {else}
  if CurrHiBlank then
    begin
      BmpHi := BmpWidth;
      CurrHiFuncBoundX := BmpXToFuncX(BmpHi);
    end
  else
    begin
      if CurrHiBoundVal >= BmpLoEnd then
        begin
          CurrHiFuncBoundX := CurrFuncObj.HiBoundVal.FloatValue;
          if CurrHiBoundVal >= BmpHiEnd then
            begin
              if CurrHiBoundVal >= BmpXToVirFloatX(BmpWidth) then
                BmpHi := BmpWidth
              else
                BmpHi := BmpWidthSub1;
            end
          else
            BmpHi := VirFloatToBmpHi(CurrHiBoundVal);
        end
      else
        Exit; {Function is not visible}
    end; {else}
  if CurrLoFuncBoundX > CurrHiFuncBoundX then
    Exit; //an error occured
  //clip BmpLo and BmpHi
  if BmpLo < (-1) then
    BmpLo := (-1);
  if BmpLo > BmpWidth then
    BmpLo := BmpWidth;
  if BmpHi < (-1) then
    BmpHi := (-1);
  if BmpHi > BmpWidth then
    BmpHi := BmpWidth;
  if BmpLo >= BmpHi then //only plot if function has 2 or more inc points.
    Exit;
  CurrBmpX := BmpLo;
  PointsCurrForceNdx := 0; //Low(LocalSpecialPointArray);
  GapsCurrForceNdx := 0;   //Low(LocalSpecialPointArray);
  Result := true;
end; {function InitDrawAFunc}

{----------------------------}

procedure InitTPlotPointRec(var ThePlotPointRec : TPlotPointRec);
begin {procedure InitTPlotPointRec}
  with ThePlotPointRec do
    begin
      DefinedStatus := dsEmpty;
      LeftSidePointBehave := psNoExtraAccuracy;
      RightSidePointBehave := psNoExtraAccuracy;
    end; {with}
end; {procedure InitTPlotPointRec}

{----------------------------}

procedure InitQueues;
begin {procedure InitQueues;}
  //Init SlopeQue
  SlopeFrom1To2Que := ssUndefinedPoint;
  SlopeFrom2To3Que := ssUndefinedPoint;
  SlopeFrom3To4Que := ssUndefinedPoint;
  //Init PlotPointQue
  InitTPlotPointRec(PlotPointQue[1]);
  InitTPlotPointRec(PlotPointQue[2]);
  InitTPlotPointRec(PlotPointQue[3]);
  InitTPlotPointRec(PlotPointQue[4]);
  //Init DrawFormToQue
  DrawFrom1To2Que := true;
  DrawFrom2To3Que := true;
  DrawFrom3To4Que := true;
  //Init GotSpPointToQue
  GotSpPoint1To2Que := false;
  GotSpPoint2To3Que := false;
  GotSpPoint3To4Que := false;
end; {procedure InitQueues;}

{----------------------------}

procedure RotateQueues;
begin {procedure RotateQueues;}
  //Rotate SlopeQue
  SlopeFrom1To2Que := SlopeFrom2To3Que;
  SlopeFrom2To3Que := SlopeFrom3To4Que;
  SlopeFrom3To4Que := ssUndefinedPoint;
  //Rotate PlotPointQue
  PlotPointQue[1] := PlotPointQue[2];
  PlotPointQue[2] := PlotPointQue[3];
  PlotPointQue[3] := PlotPointQue[4];
  InitTPlotPointRec(PlotPointQue[4]);
  //Rotate DrawFormToQue
  DrawFrom1To2Que := DrawFrom2To3Que;
  DrawFrom2To3Que := DrawFrom3To4Que;
  DrawFrom3To4Que := true;
  //Rotate GotSpPointToQue
  GotSpPoint1To2Que := GotSpPoint2To3Que;
  GotSpPoint2To3Que := GotSpPoint3To4Que;
  GotSpPoint3To4Que := false;
end; {procedure RotateQueues;}

{----------------------------}

procedure CalcIncPoint(TheBmpX : integer; var ThePlotPointRec : TPlotPointRec);
begin {procedure CalcIncPoint}
  InitTPlotPointRec(ThePlotPointRec);
  with ThePlotPointRec do
    begin
      BmpX := TheBmpX;
      FuncX := BmpXToFuncX(TheBmpX);
      LocalFuncCalc.FloatVarValues[0] := FuncX;
      if LocalFuncCalc.EvaluateMathFunc then
        begin
          DefinedStatus := dsDefined;
          FuncY := LocalFuncCalc.FloatResultValue;
          BmpY := FuncYToClipBmpY(FuncY);
        end
      else
        DefinedStatus := dsNotDefined;
    end; {with}
end; {procedure CalcIncPoint}

{----------------------------}

function ConvFromSideToSidePoint(TheFromSideBehave : TFromSideBehave) : TSidePointBehave;
{ TSidePointBehave = (psNoExtraAccuracy, psExtraAccuracy,
                      psDrawPosInfinity, psDrawNegInfinity, psNotContinuous);
  TFromSideBehave = (sbAutomatic, sbExtraAccuracy, sbNoExtraAccuracy,
                     sbDrawPosInfinity, sbDrawNegInfinity, sbNotContinuous);}
begin {function ConvFromSideToSidePoint}
  case TheFromSideBehave of
    sbAutomatic:
      begin
        if CurrFuncObj.ExtraUndefAccuracy then
          Result := psExtraAccuracy
        else
          Result := psNoExtraAccuracy;
      end;
    sbExtraAccuracy: Result := psExtraAccuracy;
    sbNoExtraAccuracy: Result := psNoExtraAccuracy;
    sbDrawPosInfinity: Result := psDrawPosInfinity;
    sbDrawNegInfinity: Result := psDrawNegInfinity;
    sbNotContinuous: Result := psNotContinuous;
    else
      Result := psNoExtraAccuracy;
  end; {case}
end; {function ConvFromSideToSidePoint}

{----------------------------}

procedure DetermineNextPointAttribs(var ThePlotPointRec : TPlotPointRec);
var
  DidFindSpecialPointMatch : boolean;
begin {procedure DetermineNextPointAttribs}
  Assert(ThePlotPointRec.DefinedStatus <> dsEmpty, 'Can not determine empty point attributes');
  //advance PointsCurrForceNdx
  while (PointsCurrForceNdx < Length(LocalSpecialPointArray)) do
    begin
      if LocalSpecialPointArray[PointsCurrForceNdx].PointValue.FloatValue < ThePlotPointRec.FuncX then
        inc(PointsCurrForceNdx)
      else
        break;
    end; {while}
  DidFindSpecialPointMatch := false;
  if PointsCurrForceNdx < Length(LocalSpecialPointArray) then
    if LocalSpecialPointArray[PointsCurrForceNdx].PointValue.FloatValue = ThePlotPointRec.FuncX then
      DidFindSpecialPointMatch := true;
  if DidFindSpecialPointMatch then
    begin
      //next point will use SpecialPoint attributes
      if LocalSpecialPointArray[PointsCurrForceNdx].AtPointBehave = pbUndefined then
        ThePlotPointRec.DefinedStatus := dsNotDefined;
      //set Left side behavior
      ThePlotPointRec.LeftSidePointBehave := ConvFromSideToSidePoint(LocalSpecialPointArray[PointsCurrForceNdx].LeftSideBehave);
      //set Right side behavior
      ThePlotPointRec.RightSidePointBehave := ConvFromSideToSidePoint(LocalSpecialPointArray[PointsCurrForceNdx].RightSideBehave);
    end
  else
    begin
      //next point will use default attributes
      if CurrFuncObj.ExtraUndefAccuracy then
        begin
          ThePlotPointRec.LeftSidePointBehave := psExtraAccuracy;
          ThePlotPointRec.RightSidePointBehave := psExtraAccuracy;
        end
      else
        begin
          ThePlotPointRec.LeftSidePointBehave := psNoExtraAccuracy;
          ThePlotPointRec.RightSidePointBehave := psNoExtraAccuracy;
        end; {else}
    end; {else}
end; {procedure DetermineNextPointAttribs}

{----------------------------}

procedure CalcUndefinedLeft(LoPointFuncX, HiPointFuncX : TMathFloat;
                            var LoResultFuncY, HiResultFuncY : TMathFloat);
const
  MaxSearchCount = 1500;
  QuitWhenMatchCount = 20;
var
  FoundFuncCount, PrevYMatchCount : integer;
  PrevSearchY : TMathFloat;
  ValidSmallSearch : boolean;
  LowSearchX, MidSearchX, HighSearchX, SmallDivide : TMathFloat;
  I, LoopCount : integer;

{-----}

function StoreSearchPoint(TheSearchY : TMathFloat) : boolean;
begin {function StoreSearchPoint}
  Result := true;
  if TheSearchY < LoResultFuncY then
    LoResultFuncY := TheSearchY;
  if TheSearchY > HiResultFuncY then
    HiResultFuncY := TheSearchY;
  inc(FoundFuncCount);
  if FoundFuncCount > 1 then
    begin
      if TheSearchY = PrevSearchY then
        begin
          inc(PrevYMatchCount);
          if PrevYMatchCount >= QuitWhenMatchCount then
            Result := false;
        end
      else
        PrevYMatchCount := 0;
    end; {if}
  PrevSearchY := TheSearchY;
end; {function StoreSearchPoint}

{-----}

procedure ShrinkSmallDivide;
begin {procedure ShrinkSmallDivide;}
  try
    if SmallDivide >= 30 then
      SmallDivide := SmallDivide / 2;
  except
    SmallDivide := 15;
  end; {try..except}
end; {procedure ShrinkSmallDivide;}

{-----}

begin {procedure CalcUndefinedLeft;}
  FoundFuncCount := 0;
  PrevYMatchCount := 0;
  PrevSearchY := 0;
  LowSearchX := LoPointFuncX;
  HighSearchX := HiPointFuncX;
  MidSearchX := HighSearchX; //set to a value to stop compiler warnings
  SmallDivide := 3;
  LoopCount := 0;
  repeat
    inc(LoopCount);
    {begin half distance search loop}
    for I := 1 to 3 do
      begin
        try
          MidSearchX := HighSearchX - (abs(HighSearchX - LowSearchX) / 2);
        except
          Exit;
        end; {try..except}
        if (MidSearchX <= LowSearchX) or (MidSearchX >= HighSearchX) then
          Exit;
        LocalFuncCalc.FloatVarValues[0] := MidSearchX;
        if LocalFuncCalc.EvaluateMathFunc then
          begin
            if StoreSearchPoint(LocalFuncCalc.FloatResultValue) then
              HighSearchX := MidSearchX
            else
              Exit;
          end
        else
          LowSearchX := MidSearchX;
        if HighSearchX <= LowSearchX then
          Exit;
      end; {for}
    {end half distance search loop}
    {begin small distance search code}
    if FoundFuncCount >= 18 then
      begin
        ValidSmallSearch := true;
        try
          MidSearchX := LowSearchX + (abs(HighSearchX - LowSearchX) /
                                      SmallDivide);
        except
          ValidSmallSearch := false;
        end; {try..except}
        if ValidSmallSearch and (MidSearchX > LowSearchX) and
           (MidSearchX < HighSearchX) then
          begin
            LocalFuncCalc.FloatVarValues[0] := MidSearchX;
            if LocalFuncCalc.EvaluateMathFunc then
              begin
                if LocalFuncCalc.FloatResultValue <> PrevSearchY then
                  begin
                    StoreSearchPoint(LocalFuncCalc.FloatResultValue);
                    HighSearchX := MidSearchX;
                    try
                      SmallDivide := SmallDivide * 2
                    except
                      Exit;
                    end; {try..except}
                  end
                else
                  ShrinkSmallDivide;
              end
            else
              begin
                LowSearchX := MidSearchX;
                ShrinkSmallDivide;
              end; {else}
            if HighSearchX <= LowSearchX then
              Exit;
          end
        else
          ShrinkSmallDivide;
      end; {if}
   {end small distance search code}
  until (LoopCount >= MaxSearchCount);
end; {procedure CalcUndefinedLeft;}

{----------------------------}

procedure CalcUndefinedRight(LoPointFuncX, HiPointFuncX : TMathFloat;
                             var LoResultFuncY, HiResultFuncY : TMathFloat);

const
  MaxSearchCount = 1500;
  QuitWhenMatchCount = 20;
var
  FoundFuncCount, PrevYMatchCount : integer;
  PrevSearchY : TMathFloat;
  ValidSmallSearch : boolean;
  LowSearchX, MidSearchX, HighSearchX, SmallDivide : TMathFloat;
  I, LoopCount : integer;

{-----}

function StoreSearchPoint(TheSearchY : TMathFloat) : boolean;
begin {function StoreSearchPoint}
  Result := true;
  if TheSearchY < LoResultFuncY then
    LoResultFuncY := TheSearchY;
  if TheSearchY > HiResultFuncY then
    HiResultFuncY := TheSearchY;
  inc(FoundFuncCount);
  if FoundFuncCount > 1 then
    begin
      if TheSearchY = PrevSearchY then
        begin
          inc(PrevYMatchCount);
          if PrevYMatchCount >= QuitWhenMatchCount then
            Result := false;
        end
      else
        PrevYMatchCount := 0;
    end; {if}
  PrevSearchY := TheSearchY;
end; {function StoreSearchPoint}

{-----}

procedure ShrinkSmallDivide;
begin {procedure ShrinkSmallDivide;}
  try
    if SmallDivide >= 30 then
      SmallDivide := SmallDivide / 2;
  except
    SmallDivide := 15;
  end; {try..except}
end; {procedure ShrinkSmallDivide;}

{-----}

begin {procedure CalcUndefinedRight;}
  FoundFuncCount := 0;
  PrevYMatchCount := 0;
  PrevSearchY := 0;
  LowSearchX := LoPointFuncX;
  HighSearchX := HiPointFuncX;
  MidSearchX := HighSearchX; //set to a value to stop compiler warnings
  SmallDivide := 3;
  LoopCount := 0;
  repeat
    inc(LoopCount);
    {begin half distance search loop}
    for I := 1 to 3 do
      begin
        try
          MidSearchX := LowSearchX + (abs(HighSearchX - LowSearchX) / 2);
        except
          Exit;
        end; {try..except}
        if (MidSearchX <= LowSearchX) or (MidSearchX >= HighSearchX) then
          Exit;
        LocalFuncCalc.FloatVarValues[0] := MidSearchX;
        if LocalFuncCalc.EvaluateMathFunc then
          begin
            if StoreSearchPoint(LocalFuncCalc.FloatResultValue) then
              LowSearchX := MidSearchX
            else
              Exit;
          end
        else
          HighSearchX := MidSearchX;
        if HighSearchX <= LowSearchX then
          Exit;
      end; {for}
    {end half distance search loop}
    {begin small distance search code}
    if FoundFuncCount >= 18 then
      begin
        ValidSmallSearch := true;
        try
          MidSearchX := HighSearchX - (abs(HighSearchX - LowSearchX) /
                                       SmallDivide);
        except
          ValidSmallSearch := false;
        end; {try..except}
        if ValidSmallSearch and (MidSearchX > LowSearchX) and
           (MidSearchX < HighSearchX) then
          begin
            LocalFuncCalc.FloatVarValues[0] := MidSearchX;
            if LocalFuncCalc.EvaluateMathFunc then
              begin
                if LocalFuncCalc.FloatResultValue <> PrevSearchY then
                  begin
                    StoreSearchPoint(LocalFuncCalc.FloatResultValue);
                    LowSearchX := MidSearchX;
                    try
                      SmallDivide := SmallDivide * 2
                    except
                      Exit;
                    end; {try..except}
                  end
                else
                  ShrinkSmallDivide;
              end
            else
              begin
                HighSearchX := MidSearchX;
                ShrinkSmallDivide;
              end; {else}
            if HighSearchX <= LowSearchX then
              Exit;
          end
        else
          ShrinkSmallDivide;
      end; {if}
   {end small distance search code}
  until (LoopCount >= MaxSearchCount);
end; {procedure CalcUndefinedRight;}

{----------------------------}

procedure DrawDisconnectedLine(P1X, P1Y, P2X, P2Y : integer);
begin {procedure DrawDisconnectedLine}
  GraphBitmap.Canvas.MoveTo(P1X, P1Y);
  GraphBitmap.Canvas.LineTo(P2X, P2Y);
  GraphBitmap.Canvas.LineTo(P2X, P2Y);
  GraphBitmap.Canvas.Pixels[P2X, P2Y] := CurrFuncObj.FuncColor;
end; {procedure DrawDisconnectedLine}

{----------------------------}

procedure DrawDisconnectedPoint(PX, PY : integer);
begin {procedure DrawDisconnectedPoint}
  GraphBitmap.Canvas.MoveTo(PX, PY);
  GraphBitmap.Canvas.LineTo(PX, PY);
  GraphBitmap.Canvas.Pixels[PX, PY] := CurrFuncObj.FuncColor;
end; {procedure DrawDisconnectedPoint}

{----------------------------}

procedure CalcPlotBehaviorBetweenNextPoints(const LoPRec, HiPRec : TPlotPointRec;
                               LoShouldDrawPoint, HiShouldDrawPoint : boolean;
                               var DrawFromLoToHi, GotSpPointLoToHi : boolean);
var
  I, FirstSpecialPointNdx, LastSpecialPointNdx, CountSpecialPoints : integer;
  LoSpecialCDRec, HiSpecialCDRec : TPlotPointRec;

{------------}

procedure DrawPlotPointsUndefinedLoLeft(const TheLoPointRec, TheHiPointRec : TPlotPointRec);
var
  LowerFuncY, UpperFuncY : TMathFloat;
  LowerValueBmpY, UpperValueBmpY : integer;
begin {procedure DrawPlotPointsUndefinedLoLeft}
  case TheLoPointRec.RightSidePointBehave of
    psNoExtraAccuracy, psNotContinuous:
      begin
        with TheHiPointRec do
          DrawDisconnectedPoint(BmpX, BmpY);
      end; {psNoExtraAccuracy, psNotContinuous:}
    psExtraAccuracy:
      begin
        LowerFuncY := TheHiPointRec.FuncY;
        UpperFuncY := TheHiPointRec.FuncY;
        CalcUndefinedLeft(TheLoPointRec.FuncX, TheHiPointRec.FuncX, LowerFuncY, UpperFuncY);
        LowerValueBmpY := FuncYToClipBmpY(LowerFuncY);
        UpperValueBmpY := FuncYToClipBmpY(UpperFuncY);
        if LowerValueBmpY <> TheHiPointRec.BmpY then
          DrawDisconnectedLine(TheHiPointRec.BmpX, LowerValueBmpY,
                               TheHiPointRec.BmpX, TheHiPointRec.BmpY);
        if UpperValueBmpY <> TheHiPointRec.BmpY then
          DrawDisconnectedLine(TheHiPointRec.BmpX, UpperValueBmpY,
                               TheHiPointRec.BmpX, TheHiPointRec.BmpY);
      end; {psExtraAccuracy:}
    psDrawPosInfinity:
      begin
        if TheHiPointRec.BmpY > (-16) then //draw to 16 pixels off bitmap
          DrawDisconnectedLine(TheHiPointRec.BmpX, (-16),
                               TheHiPointRec.BmpX, TheHiPointRec.BmpY);
      end; {psDrawPosInfinity:}
    psDrawNegInfinity:
      begin
        if TheHiPointRec.BmpY < (BmpHeight + 16) then //draw to 16 pixels off bitmap
          DrawDisconnectedLine(TheHiPointRec.BmpX, (BmpHeight + 16),
                               TheHiPointRec.BmpX, TheHiPointRec.BmpY);
      end; {psDrawNegInfinity:}
  end; {case}
end; {procedure DrawPlotPointsUndefinedLoLeft}

{------------}

procedure DrawPlotPointsUndefinedHiRight(const TheLoPointRec, TheHiPointRec : TPlotPointRec);
var
  LowerFuncY, UpperFuncY : TMathFloat;
  LowerValueBmpY, UpperValueBmpY : integer;
begin {procedure DrawPlotPointsUndefinedHiRight}
  case TheHiPointRec.LeftSidePointBehave of
    psNoExtraAccuracy, psNotContinuous:
      begin
        with TheLoPointRec do
          DrawDisconnectedPoint(BmpX, BmpY);
      end; {psNoExtraAccuracy, psNotContinuous:}
    psExtraAccuracy:
      begin
        LowerFuncY := TheLoPointRec.FuncY;
        UpperFuncY := TheLoPointRec.FuncY;
        CalcUndefinedRight(TheLoPointRec.FuncX, TheHiPointRec.FuncX, LowerFuncY, UpperFuncY);
        LowerValueBmpY := FuncYToClipBmpY(LowerFuncY);
        UpperValueBmpY := FuncYToClipBmpY(UpperFuncY);
        if LowerValueBmpY <> TheLoPointRec.BmpY then
          DrawDisconnectedLine(TheLoPointRec.BmpX, LowerValueBmpY,
                               TheLoPointRec.BmpX, TheLoPointRec.BmpY);
        if UpperValueBmpY <> TheLoPointRec.BmpY then
          DrawDisconnectedLine(TheLoPointRec.BmpX, UpperValueBmpY,
                               TheLoPointRec.BmpX, TheLoPointRec.BmpY);
      end; {psExtraAccuracy:}
      psDrawPosInfinity:
        begin
          if TheLoPointRec.BmpY > (-16) then //draw to 16 pixels off bitmap
            DrawDisconnectedLine(TheLoPointRec.BmpX, (-16),
                                 TheLoPointRec.BmpX, TheLoPointRec.BmpY);
        end; {psDrawPosInfinity:}
      psDrawNegInfinity:
        begin
          if TheLoPointRec.BmpY < (BmpHeight + 16) then //draw to 16 pixels off bitmap
            DrawDisconnectedLine(TheLoPointRec.BmpX, (BmpHeight + 16),
                                 TheLoPointRec.BmpX, TheLoPointRec.BmpY);
        end; {psDrawNegInfinity:}
  end; {case}
end; {procedure DrawPlotPointsUndefinedHiRight}

{------------}

procedure CalcAndDrawSpecialPointsLine;
begin {procedure CalcAndDrawSpecialPointsLine}
  if (LoSpecialCDRec.DefinedStatus = dsNotDefined) and
     (HiSpecialCDRec.DefinedStatus = dsNotDefined) then
    Exit; //Nothing to draw
  if (LoSpecialCDRec.DefinedStatus = dsNotDefined) and
     (HiSpecialCDRec.DefinedStatus = dsDefined) then
    begin
      DrawPlotPointsUndefinedLoLeft(LoSpecialCDRec, HiSpecialCDRec);
    end
  else
  if (LoSpecialCDRec.DefinedStatus = dsDefined) and
     (HiSpecialCDRec.DefinedStatus = dsNotDefined) then
    begin
      DrawPlotPointsUndefinedHiRight(LoSpecialCDRec, HiSpecialCDRec);
    end
  else //if (LoSpecialCDRec.DefinedStatus = dsDefined) and (HiSpecialCDRec.DefinedStatus = dsDefined) then
    begin
      if (LoSpecialCDRec.RightSidePointBehave = psNotContinuous) or
         (HiSpecialCDRec.LeftSidePointBehave = psNotContinuous) then
        begin
          DrawDisconnectedPoint(LoSpecialCDRec.BmpX, LoSpecialCDRec.BmpY);
          DrawDisconnectedPoint(HiSpecialCDRec.BmpX, HiSpecialCDRec.BmpY);
        end
      else
        DrawDisconnectedLine(LoSpecialCDRec.BmpX, LoSpecialCDRec.BmpY,
                             HiSpecialCDRec.BmpX, HiSpecialCDRec.BmpY);
    end; {else}
end; {procedure CalcAndDrawSpecialPointsLine}

{------------}

function ConvCalcSpecialPointToPlotPoint(const SpecialPointRec : TSpecialPointRec) : TPlotPointRec;
begin {function ConvCalcSpecialPointToPlotPoint}
  Result.LeftSidePointBehave := ConvFromSideToSidePoint(SpecialPointRec.LeftSideBehave);
  Result.RightSidePointBehave := ConvFromSideToSidePoint(SpecialPointRec.RightSideBehave);
  if SpecialPointRec.AtPointBehave = pbUndefined then
    begin
      with Result do
        begin
          DefinedStatus := dsNotDefined;
          FuncX := SpecialPointRec.PointValue.FloatValue;
          FuncY := 0;
          if abs(FuncX - LoPRec.FuncX) <= abs(HiPRec.FuncX - FuncX) then
            BmpX := LoPRec.BmpX
          else
            BmpX := HiPRec.BmpX;
          BmpY := 0;
        end; {with}
    end
  else //if SpecialPointRec.AtPointBehave = pbAutoDetect then
    begin
      with Result do
        begin
          FuncX := SpecialPointRec.PointValue.FloatValue;
          if abs(FuncX - LoPRec.FuncX) <= abs(HiPRec.FuncX - FuncX) then
            BmpX := LoPRec.BmpX
          else
            BmpX := HiPRec.BmpX;
          LocalFuncCalc.FloatVarValues[0] := FuncX;
          if LocalFuncCalc.EvaluateMathFunc then
            begin
              DefinedStatus := dsDefined;
              FuncY := LocalFuncCalc.FloatResultValue;
              BmpY := FuncYToClipBmpY(FuncY);
            end
          else
            begin
              DefinedStatus := dsNotDefined;
              FuncY := 0;
              BmpY := 0;
            end; {else}
        end; {with}
    end; {else}
  //set DrawFromLoToHi to false if not continuous between inc points
  if Result.DefinedStatus = dsNotDefined then
    DrawFromLoToHi := false
  else
    if (Result.LeftSidePointBehave = psNotContinuous) or
       (Result.RightSidePointBehave = psNotContinuous) then
      DrawFromLoToHi := false;
end; {function ConvCalcSpecialPointToPlotPoint}

{------------}

begin {procedure CalcPlotBehaviorBetweenNextPoints}
  Assert(((LoPRec.DefinedStatus <> dsEmpty) and (HiPRec.DefinedStatus <> dsEmpty)),
         'Can not determine empty point attributes');
  Assert((LoShouldDrawPoint or HiShouldDrawPoint), 'Not enough points to calulate with.');
  GotSpPointLoToHi := false;
  //initialize DrawFromLoToHi return var
  if (LoPRec.DefinedStatus = dsNotDefined) or (HiPRec.DefinedStatus = dsNotDefined) then
    DrawFromLoToHi := false
  else
    begin
      if (LoPRec.RightSidePointBehave = psNotContinuous) or (HiPRec.LeftSidePointBehave = psNotContinuous) then
        DrawFromLoToHi := false
      else
        DrawFromLoToHi := true;
    end; {else}
  //advance GapsCurrForceNdx
  while (GapsCurrForceNdx < Length(LocalSpecialPointArray)) do
    begin
      if LocalSpecialPointArray[GapsCurrForceNdx].PointValue.FloatValue <= LoPRec.FuncX then
        inc(GapsCurrForceNdx)
      else
        break;
    end; {while}
  //Count the special points and find first and last indexes
  CountSpecialPoints := 0;
  FirstSpecialPointNdx := GapsCurrForceNdx;
  I := GapsCurrForceNdx;
  while (I < Length(LocalSpecialPointArray)) do
    begin
      if LocalSpecialPointArray[I].PointValue.FloatValue < HiPRec.FuncX then
        inc(CountSpecialPoints)
      else
        break;
      inc(I);
    end; {while}
  LastSpecialPointNdx := FirstSpecialPointNdx + (CountSpecialPoints - 1);
  //eliminate special points that are not in function bounds
  if CountSpecialPoints >= 1 then
    begin
      //eliminate low special points that are not in function bounds
      while (CountSpecialPoints >= 1) and (FirstSpecialPointNdx <= LastSpecialPointNdx) do
        begin
          if (LocalSpecialPointArray[FirstSpecialPointNdx].PointValue.FloatValue >= CurrLoFuncBoundX) and
             (LocalSpecialPointArray[FirstSpecialPointNdx].PointValue.FloatValue <= CurrHiFuncBoundX) then
            Break
          else
            begin
              inc(FirstSpecialPointNdx);
              dec(CountSpecialPoints);
            end; {else}
        end; {while}
      //eliminate high special points that are not in function bounds
      while (CountSpecialPoints >= 1) and (FirstSpecialPointNdx <= LastSpecialPointNdx) do
        begin
          if (LocalSpecialPointArray[LastSpecialPointNdx].PointValue.FloatValue >= CurrLoFuncBoundX) and
             (LocalSpecialPointArray[LastSpecialPointNdx].PointValue.FloatValue <= CurrHiFuncBoundX) then
            Break
          else
            begin
              dec(LastSpecialPointNdx);
              dec(CountSpecialPoints);
            end; {else}
        end; {while}
    end; {if}
  //calc behavior and draw
  if CountSpecialPoints <= 0 then
    begin //no special points between
      if LoShouldDrawPoint and HiShouldDrawPoint then
        begin
          if (LoPRec.DefinedStatus <> dsDefined) and (HiPRec.DefinedStatus = dsDefined) then
            DrawPlotPointsUndefinedLoLeft(LoPRec, HiPRec)
          else
            if (LoPRec.DefinedStatus = dsDefined) and (HiPRec.DefinedStatus <> dsDefined) then
              DrawPlotPointsUndefinedHiRight(LoPRec, HiPRec);
        end; {if}
    end
  else
    begin //1 or more special points between
      GotSpPointLoToHi := true;
      //Calc and Draw From LoPRec to first Special Point
      LoSpecialCDRec := LoPRec;
      HiSpecialCDRec := ConvCalcSpecialPointToPlotPoint(LocalSpecialPointArray[FirstSpecialPointNdx]);
      if LoShouldDrawPoint then
        CalcAndDrawSpecialPointsLine;
      //Calc and Draw From first Special Point to last Special Point
      if CountSpecialPoints > 1 then
        begin
          for I := (FirstSpecialPointNdx + 1) to LastSpecialPointNdx do
            begin
              LoSpecialCDRec := HiSpecialCDRec;
              HiSpecialCDRec := ConvCalcSpecialPointToPlotPoint(LocalSpecialPointArray[I]);
              CalcAndDrawSpecialPointsLine;
            end; {for}
        end; {if}
      //Calc and Draw From last Special Point to HiPRec
      LoSpecialCDRec := HiSpecialCDRec;
      HiSpecialCDRec := HiPRec;
      if HiShouldDrawPoint then
        CalcAndDrawSpecialPointsLine;
    end; {else}
end; {procedure CalcPlotBehaviorBetweenNextPoints}

{----------------------------}

function CalcSlopeStatus(const LoPRec, HiPRec : TPlotPointRec) : TSlopeStatus;
begin {function CalcSlopeStatus}
  Result := ssUndefinedPoint;
  if (LoPRec.DefinedStatus = dsDefined) and (HiPRec.DefinedStatus = dsDefined) then
    begin
      if LoPRec.FuncY < HiPRec.FuncY then
        Result := ssPositive
      else
        if LoPRec.FuncY > HiPRec.FuncY then
          Result := ssNegative
        else
          Result := ssZero;
    end; {if}
end; {function CalcSlopeStatus}

{----------------------------}

procedure CalcFromToSlopeFixupIfNecessary;
begin {procedure CalcFromToSlopeFixupIfNecessary;}
  if LocalAsymFixUp then
   if DrawFrom1To2Que and DrawFrom2To3Que and DrawFrom3To4Que then
     begin
      if (SlopeFrom1To2Que = ssNegative) and
         (SlopeFrom2To3Que = ssPositive) and
         (SlopeFrom3To4Que = ssNegative) then
        DrawFrom2To3Que := false
      else
        if (SlopeFrom1To2Que = ssPositive) and
           (SlopeFrom2To3Que = ssNegative) and
           (SlopeFrom3To4Que = ssPositive) then
          DrawFrom2To3Que := false;
    end; {if}
end; {procedure CalcFromToSlopeFixupIfNecessary;}

{----------------------------}

procedure DrawTwoIncPoints(const LoIncPointRec, HiIncPointRec : TPlotPointRec;
                           ShouldDrawLineBetween, GotSpPointBetween : boolean);
begin {procedure DrawTwoIncPoints}
  Assert((LoIncPointRec.DefinedStatus <> dsEmpty) and (HiIncPointRec.DefinedStatus <> dsEmpty), 'Error drawing empty increment point.');
  if ShouldDrawLineBetween and (not GotSpPointBetween) and
      (LoIncPointRec.DefinedStatus = dsDefined) and
      (HiIncPointRec.DefinedStatus = dsDefined) then
    DrawDisconnectedLine(LoIncPointRec.BmpX, LoIncPointRec.BmpY,
                         HiIncPointRec.BmpX, HiIncPointRec.BmpY)
  else
    begin
      if (LoIncPointRec.DefinedStatus = dsDefined) then
        DrawDisconnectedPoint(LoIncPointRec.BmpX, LoIncPointRec.BmpY);
      if (HiIncPointRec.DefinedStatus = dsDefined) then
        DrawDisconnectedPoint(HiIncPointRec.BmpX, HiIncPointRec.BmpY);
    end; {else}
end; {procedure DrawTwoIncPoints}

{----------------------------}

begin {procedure Draw2DCartesianFunc}
  if not InitDrawAFunc then
    Exit;
  if DoIsTerminated then Exit;
  InitQueues;
  if DoIsTerminated then Exit;
  //calculate previous out of bounds point
  CalcIncPoint((CurrBmpX - 1), PlotPointQue[1]);
  if DoIsTerminated then Exit;
  DetermineNextPointAttribs(PlotPointQue[1]);
  if DoIsTerminated then Exit;
  //calculate first point; guaranteed to have atleast 2 inc points.
  CalcIncPoint(CurrBmpX, PlotPointQue[2]);
  if DoIsTerminated then Exit;
  DetermineNextPointAttribs(PlotPointQue[2]);
  if DoIsTerminated then Exit;
  //calculate second point; guaranteed to have atleast 2 inc points.
  inc(CurrBmpX);
  if DoIsTerminated then Exit;
  CalcIncPoint(CurrBmpX, PlotPointQue[3]);
  if DoIsTerminated then Exit;
  DetermineNextPointAttribs(PlotPointQue[3]);
  if DoIsTerminated then Exit;
  //calculate third point; might be final out of bounds point
  inc(CurrBmpX);
  if DoIsTerminated then Exit;
  CalcIncPoint(CurrBmpX, PlotPointQue[4]);
  if DoIsTerminated then Exit;
  DetermineNextPointAttribs(PlotPointQue[4]);
  if DoIsTerminated then Exit;
  //calc and plot between inc points
  CalcPlotBehaviorBetweenNextPoints(PlotPointQue[1], PlotPointQue[2],
                               false, true, DrawFrom1To2Que, GotSpPoint1To2Que);
  if DoIsTerminated then Exit;
  CalcPlotBehaviorBetweenNextPoints(PlotPointQue[2], PlotPointQue[3],
                               true, true, DrawFrom2To3Que, GotSpPoint2To3Que);
  if DoIsTerminated then Exit;
  CalcPlotBehaviorBetweenNextPoints(PlotPointQue[3], PlotPointQue[4],
                 true, (CurrBmpX <= BmpHi), DrawFrom3To4Que, GotSpPoint3To4Que);
  if DoIsTerminated then Exit;
  //calculate slope values
  SlopeFrom1To2Que := CalcSlopeStatus(PlotPointQue[1], PlotPointQue[2]);
  if DoIsTerminated then Exit;
  SlopeFrom2To3Que := CalcSlopeStatus(PlotPointQue[2], PlotPointQue[3]);
  if DoIsTerminated then Exit;
  SlopeFrom3To4Que := CalcSlopeStatus(PlotPointQue[3], PlotPointQue[4]);
  if DoIsTerminated then Exit;
  CalcFromToSlopeFixupIfNecessary;
  if DoIsTerminated then Exit;
  //main plotting loop
  while CurrBmpX <= BmpHi do
    begin
      if DoIsTerminated then Exit;
      RotateQueues;
      if DoIsTerminated then Exit;
      inc(CurrBmpX);
      if DoIsTerminated then Exit;
      CalcIncPoint(CurrBmpX, PlotPointQue[4]);
      if DoIsTerminated then Exit;
      DetermineNextPointAttribs(PlotPointQue[4]);
      if DoIsTerminated then Exit;
      CalcPlotBehaviorBetweenNextPoints(PlotPointQue[3], PlotPointQue[4],
                 true, (CurrBmpX <= BmpHi), DrawFrom3To4Que, GotSpPoint3To4Que);
      if DoIsTerminated then Exit;
      SlopeFrom3To4Que := CalcSlopeStatus(PlotPointQue[3], PlotPointQue[4]);
      if DoIsTerminated then Exit;
      CalcFromToSlopeFixupIfNecessary;
      if DoIsTerminated then Exit;
      DrawTwoIncPoints(PlotPointQue[1], PlotPointQue[2], DrawFrom1To2Que, GotSpPoint1To2Que);
      if DoIsTerminated then Exit;
    end; {while}
  //finish plotting points left in que
  if DoIsTerminated then Exit;
  DrawTwoIncPoints(PlotPointQue[2], PlotPointQue[3], DrawFrom2To3Que, GotSpPoint2To3Que);
end; {procedure Draw2DCartesianFunc}

{-----------------------------------------------------------------------}

procedure DrawParametricFunc(CurrFuncObj : TFuncParametricObj);
type
  TParaPoint = record
    XVal, YVal : TMathFloat;
    Exists : boolean;
    OnBmp : boolean;
  end; {TParaPoint}
  PClipRec = ^TClipRec;
  TClipRec = record
    X, Y : TMathFloat;
  end; {TClipRec}
var
  I : integer;
  CurrLoBoundVal,
  CurrHiBoundVal : TMathFloat;
  BoundDiff, CurrSamplesFloat : TMathFloat;
  CurrPoint, PrevPoint : TParaPoint;
  CurrClip, PrevClip : TClipRec;
  ClipedLineCrossesBmp : boolean;
  LinePointCount : integer;
  DoFixUpPoint : boolean;
  {Local Variables}
  XLocalFuncCalc, YLocalFuncCalc : TMathParametricFunc;

{------------------------------------}

procedure DrawToClippedPoint(const TheClippedPoint : TClipRec);
begin {procedure DrawToClippedPoint}
  inc(LinePointCount);
  with LinePointArray[LinePointCount] do
    begin
      X := VirFloatXToBmpX(TheClippedPoint.X);
      Y := VirFloatYToBmpY(TheClippedPoint.Y);
    end; {with}
  if LinePointCount >= LinePointArraySize then
    begin
      LinePointCount := 1;
      PolyLine(GraphBitmap.Canvas.Handle,
               LinePointArray, LinePointArraySize);
      LinePointArray[1] := LinePointArray[LinePointArraySize];
    end; {if}
end; {procedure DrawToClippedPoint}

{------------------------------------}

procedure DrawToCurrPoint;
begin {procedure DrawToCurrPoint;}
  inc(LinePointCount);
  with LinePointArray[LinePointCount] do
    begin
      X := VirFloatXToBmpX(CurrPoint.XVal);
      Y := VirFloatYToBmpY(CurrPoint.YVal);
    end; {with}
  if LinePointCount >= LinePointArraySize then
    begin
      LinePointCount := 1;
      PolyLine(GraphBitmap.Canvas.Handle,
               LinePointArray, LinePointArraySize);
      LinePointArray[1] := LinePointArray[LinePointArraySize];
    end; {if}
end; {procedure DrawToCurrPoint;}

{------------------------------------}

procedure GotUnDefinedPoint;
begin {procedure GotUnDefinedPoint;}
  if LinePointCount > 0 then
    begin
      if LinePointCount = 1 then
        begin
          if DoFixUpPoint then
            with LinePointArray[1] do
              GraphBitmap.Canvas.Pixels[X, Y]:= CurrFuncObj.FuncColor;
        end
      else
        begin
          PolyLine(GraphBitmap.Canvas.Handle, LinePointArray, LinePointCount);
          if DoFixUpPoint then
            with LinePointArray[LinePointCount] do
              GraphBitmap.Canvas.Pixels[X, Y] := CurrFuncObj.FuncColor;
        end; {else}
      LinePointCount := 0;
    end; {if}
end; {procedure GotUnDefinedPoint;}

{------------------------------------}

procedure ClipCurrPrevLine;
  {This Procedure Assumes that the line has allready been tested}
  {     and that it requires clipping.}
var
  PHi, PLo : PClipRec;
  CurrClipOnBitmap, PrevClipOnBitmap : boolean;

{------------------------}

procedure ClipAginstVert(ClipedPoint : PClipRec;
                         RectXVal: TMathFloat);
var
  MSlope : TMathFloat;
begin
  MSlope := (CurrClip.Y-PrevClip.Y)/(CurrClip.X-PrevClip.X);
  with ClipedPoint^ do
    begin
      Y := (Y + (RectXVal - X)*(MSlope));
      X := RectXVal;
    end; {with}
end; {procedure ClipAginstVert}

{------------------------}

procedure ClipAginstHorz(ClipedPoint : PClipRec;
                         RectYVal: TMathFloat);
var
  MSlope : TMathFloat;
begin
  MSlope := (CurrClip.X-PrevClip.X)/(CurrClip.Y-PrevClip.Y);
  with ClipedPoint^ do
    begin
      X := (X + (RectYVal - Y)*(MSlope));
      Y := RectYVal;
    end; {with}
end; {procedure ClipAginstHorz}

{------------------------}

begin {procedure ClipCurrPrevLine;}
  CurrClip.X := CurrPoint.XVal;
  CurrClip.Y := CurrPoint.YVal;
  PrevClip.X := PrevPoint.XVal;
  PrevClip.Y := PrevPoint.YVal;
  if ((not CurrPoint.OnBmp) and (not PrevPoint.OnBmp)) then
    begin
      if((CurrClip.X < ViewTopLeftPix.X)and
         (PrevClip.X < ViewTopLeftPix.X))or
        ((CurrClip.X > ViewBottomRightPix.X)and
         (PrevClip.X > ViewBottomRightPix.X))or
        ((CurrClip.Y > ViewTopLeftPix.Y)and
         (PrevClip.Y > ViewTopLeftPix.Y))or
        ((CurrClip.Y < ViewBottomRightPix.Y)and
         (PrevClip.Y < ViewBottomRightPix.Y))then
           begin
             ClipedLineCrossesBmp := false;
             exit; {Line does not cross BMP}
           end; {if}
    end; {if}
  if CurrClip.X = PrevClip.X then
    begin
      if CurrClip.Y < PrevClip.Y then
        begin
          PLo := @CurrClip;
          PHi := @PrevClip;
        end
      else
        begin
          PHi := @CurrClip;
          PLo := @PrevClip;
        end; {else}
      if (PHi^.Y > ViewTopLeftPix.Y)and(PLo^.Y <= ViewTopLeftPix.Y) then
        PHi^.Y := ViewTopLeftPix.Y;
      if (PHi^.Y >= ViewBottomRightPix.Y)and(PLo^.Y < ViewBottomRightPix.Y) then
        PLo^.Y := ViewBottomRightPix.Y;
    end
  else
  if CurrClip.Y = PrevClip.Y then
    begin
      if CurrClip.X < PrevClip.X then
        begin {swap them}
          PLo := @CurrClip;
          PHi := @PrevClip;
        end
      else
        begin
          PHi := @CurrClip;
          PLo := @PrevClip;
        end; {else}
      if (PHi^.X >= ViewTopLeftPix.X)and(PLo^.X < ViewTopLeftPix.X) then
        PLo^.X := ViewTopLeftPix.X;
      if (PHi^.X > ViewBottomRightPix.X)and(PLo^.X <= ViewBottomRightPix.X) then
        PHi^.X := ViewBottomRightPix.X;
    end
  else
    begin
      if CurrClip.X < PrevClip.X then
        begin {swap them}
          PLo := @CurrClip;
          PHi := @PrevClip;
        end
      else
        begin
          PHi := @CurrClip;
          PLo := @PrevClip;
        end; {else}
      if (PHi^.X >= ViewTopLeftPix.X)and(PLo^.X < ViewTopLeftPix.X) then
        ClipAginstVert(PLo, ViewTopLeftPix.X);
      if (PHi^.X > ViewBottomRightPix.X)and(PLo^.X <= ViewBottomRightPix.X) then
        ClipAginstVert(PHi, ViewBottomRightPix.X);
      if CurrClip.Y < PrevClip.Y then
        begin
          PLo := @CurrClip;
          PHi := @PrevClip;
        end
      else
        begin
          PHi := @CurrClip;
          PLo := @PrevClip;
        end; {else}
      if (PHi^.Y > ViewTopLeftPix.Y)and(PLo^.Y <= ViewTopLeftPix.Y) then
        ClipAginstHorz(PHi, ViewTopLeftPix.Y);
      if (PHi^.Y >= ViewBottomRightPix.Y)and(PLo^.Y < ViewBottomRightPix.Y) then
        ClipAginstHorz(PLo, ViewBottomRightPix.Y);
    end; {else}
  CurrClipOnBitmap := (((CurrClip.X >= ViewTopLeftPix.X) and
                        (CurrClip.X <= ViewBottomRightPix.X))and
                       ((CurrClip.Y <= ViewTopLeftPix.Y) and
                        (CurrClip.Y >= ViewBottomRightPix.Y)));
  PrevClipOnBitmap := (((PrevClip.X >= ViewTopLeftPix.X) and
                        (PrevClip.X <= ViewBottomRightPix.X))and
                       ((PrevClip.Y <= ViewTopLeftPix.Y) and
                        (PrevClip.Y >= ViewBottomRightPix.Y)));
  ClipedLineCrossesBmp := CurrClipOnBitmap and PrevClipOnBitmap;
end; {procedure ClipCurrPrevLine;}

{------------------------------------}

begin {procedure DrawParametricFunc}
  LinePointCount := 0;
  with CurrFuncObj do
    begin
      XLocalFuncCalc := XFuncCalc;
      YLocalFuncCalc := YFuncCalc;
      CurrLoBoundVal := LoBoundVal.FloatValue;
      CurrHiBoundVal := HiBoundVal.FloatValue;
      GraphBitmap.Canvas.Pen.Color := FuncColor;
      GraphBitmap.Canvas.Pen.Width := LineWidth;
      DoFixUpPoint := (LineWidth = 1);
    end; {with}
  if CurrLoBoundVal >= CurrHiBoundVal then
    exit;
  if DoIsTerminated then Exit;
  with CurrPoint do
    begin
      Exists := false;
      OnBmp := false;
      XVal := 0;
      YVal := 0;
    end; {with}
  BoundDiff := CurrHiBoundVal - CurrLoBoundVal;
  CurrSamplesFloat := CurrFuncObj.Samples;
  if DoIsTerminated then Exit;
  for I := 0 to CurrFuncObj.Samples do
    begin
      if DoIsTerminated then Exit;
      PrevPoint := CurrPoint;
      {begin math function evaluation code}
      {Calc current X value and store for evaluation}
      XLocalFuncCalc.FloatVarValues[0] := ((I/CurrSamplesFloat)*BoundDiff)+CurrLoBoundVal;
      CurrPoint.Exists := XLocalFuncCalc.EvaluateMathFunc;
      if DoIsTerminated then Exit;
      if CurrPoint.Exists then
        begin
          {Calc current Y value and store for evaluation}
          YLocalFuncCalc.FloatVarValues[0] := ((I/CurrSamplesFloat)*BoundDiff)+CurrLoBoundVal;
          CurrPoint.Exists := YLocalFuncCalc.EvaluateMathFunc;
        end; {if}
      {end math function evaluation code}
      if DoIsTerminated then Exit;
      if CurrPoint.Exists then
        begin
          try
            with CurrPoint do
              begin
                XVal := (XLocalFuncCalc.FloatResultValue * LocalXViewZoom);
                YVal := (YLocalFuncCalc.FloatResultValue * LocalYViewZoom);
                OnBmp := (((XVal >= ViewTopLeftPix.X) and
                          (XVal <= ViewBottomRightPix.X))and
                         ((YVal <= ViewTopLeftPix.Y) and
                          (YVal >= ViewBottomRightPix.Y)));
              end; {with}
          except
            CurrPoint.Exists := false;
          end; {except}
        end; {if}
      if DoIsTerminated then Exit;
      if not CurrPoint.Exists then
        begin
          GotUnDefinedPoint;
        end
      else
        if not PrevPoint.Exists then
          begin
            if CurrPoint.OnBmp then
              DrawToCurrPoint;
          end
        else
          if CurrPoint.OnBmp and PrevPoint.OnBmp then
            begin
              DrawToCurrPoint;
            end
          else
            begin
              ClipCurrPrevLine;
              if DoIsTerminated then Exit;
              if not ClipedLineCrossesBmp then
                GotUnDefinedPoint
              else
                begin
                  if not PrevPoint.OnBmp then
                    begin
                      GotUnDefinedPoint;
                      if DoIsTerminated then Exit;
                      DrawToClippedPoint(PrevClip);
                      if DoIsTerminated then Exit;
                    end; {if}
                  DrawToClippedPoint(CurrClip);
                  if DoIsTerminated then Exit;
                  if not CurrPoint.OnBmp then
                    GotUnDefinedPoint;
                end; {else}
            end; {else}
      if DoIsTerminated then Exit;
    end; {for}
  if DoIsTerminated then Exit;
  GotUnDefinedPoint;
end; {procedure DrawParametricFunc}

{-----------------------------------------------------------------------}

procedure DrawAllFunctions;
var
  FuncCounter : integer;
begin {procedure DrawAllFunctions;}
  GraphBitmap.Canvas.Pen.Style := psSolid;
  for FuncCounter := 0 to FuncList.Count-1 do
    begin
      if DoIsTerminated then Exit;
      if (TObject(FuncList[FuncCounter]) is TFunc2DObj) then
        Draw2DCartesianFunc(FuncList[FuncCounter])
      else {if (FuncObjList[FuncCounter] is TFuncParametricObj) then}
        DrawParametricFunc(FuncList[FuncCounter]);
      if DoIsTerminated then Exit;
    end; {for}
end; {procedure DrawAllFunctions;}

{-----------------------------------------------------------------------}

begin {procedure TPlotGraph2D.GraphWinPaint;}
  if DoIsTerminated then Exit;
  FillGraphBmpBackGround;
  if DoIsTerminated then Exit;
  InitialCalculations;
  if DoIsTerminated then Exit;
  DrawTheAxisLines;
  if DoIsTerminated then Exit;
  DrawAllFunctions;
end; {procedure TPlotGraph2D.GraphWinPaint;}

{=====================================================================}

procedure TPlotGraph2D.RestartAnimatedGraphPaint;
begin {procedure TPlotGraph2D.RestartAnimatedGraphPaint;}
  GraphWinPaint;
end; {procedure TPlotGraph2D.RestartAnimatedGraphPaint;}

{---------------------------------------------------------------------}

function TPlotGraph2D.StepAnimatedGraphPaint : boolean;
begin {function TPlotGraph2D.StepAnimatedGraphPaint}
  GraphWinPaint;
  result := true;
end; {function TPlotGraph2D.StepAnimatedGraphPaint}

{---------------------------------------------------------------------}

end.
