unit PtGrList;

(*
Copyright 1999-2011 Greg VanMullem
This program is distributed under the terms of the GNU General Public License Version 3

This file is part of MathGV.

MathGV is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

MathGV is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MathGV.  If not, see <http://www.gnu.org/licenses/>.
*)

interface

uses
  Classes, Graphics, PtGraph, FuncObj, SeqList;

type
  TPlotGraphFuncList = class(TPlotGraph)
  private
    { Private declarations }
    FFuncList : TList;
  protected
    { Protected declarations }
    FunctionListDidChange : boolean;
  public
    { Public declarations }
    property FuncList : TList read FFuncList;
    constructor Create;
    destructor Destroy; override;
    procedure ClearFuncList;
    procedure AssignFuncList(SourceFuncList: TList);
  end; {TPlotGraphFuncList}

implementation

{ TPlotGraphFuncList }

{---------------------------------------------------------------------}

procedure TPlotGraphFuncList.AssignFuncList(SourceFuncList: TList);
begin {procedure TPlotGraphFuncList.AssignFuncList}
  if AssignSequenceLists(FFuncList, SourceFuncList) then
    FunctionListDidChange := true;
end; {procedure TPlotGraphFuncList.AssignFuncList}

{---------------------------------------------------------------------}

procedure TPlotGraphFuncList.ClearFuncList;
var
  I : integer;
begin {procedure TPlotGraphFuncList.ClearFuncList;}
  if FFuncList.Count >= 1 then
    begin
      FunctionListDidChange := true;
      for I := 0 to FFuncList.Count -1 do
        if Assigned(FFuncList[I]) then
          TObject(FFuncList[I]).Free;
      FFuncList.Clear;
    end; {if}
end; {procedure TPlotGraphFuncList.ClearFuncList;}

{---------------------------------------------------------------------}

constructor TPlotGraphFuncList.Create;
begin {constructor TPlotGraphFuncList.Create}
  inherited;
  FunctionListDidChange := false;
  FFuncList := TList.Create;
end; {constructor TPlotGraphFuncList.Create}

{---------------------------------------------------------------------}

destructor TPlotGraphFuncList.Destroy;
begin {destructor TPlotGraphFuncList.Destroy;}
  ClearFuncList;
  FFuncList.free;
  inherited;
end; {destructor TPlotGraphFuncList.Destroy;}

{---------------------------------------------------------------------}

end.
