unit PtGrPolr;

(*
Copyright 1999-2011 Greg VanMullem
This program is distributed under the terms of the GNU General Public License Version 3

This file is part of MathGV.

MathGV is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

MathGV is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MathGV.  If not, see <http://www.gnu.org/licenses/>.
*)

interface

uses
  Classes, Graphics,
  Windows, //eventually delete this, use Delphi call instead of API call.
  SysUtils, MathCalc, FloatStr, PtGrList, VwPoObj, SetPoObj, FuncPolr, PtShared;

const
  MaxPolarFuncCount = 252;

type
  TVirtualPix = record
    X,Y : TMathFloat;
  end;
  TBmpPix = record
    X,Y : integer;
  end;
  TAxisLineRec = record
    InView : boolean;
    P1, P2 : TVirtualPix;
  end;
  TAxisLineArray = array[1..MaxPolarAxisCount] of TAxisLineRec;

type
  TPlotGraphPolar = class(TPlotGraphFuncList)
  private
    { Private declarations }
    LinePointArray : TLinePointArray;
    AxisLineArray : TAxisLineArray;
    FViewPolarZoomObj : TViewPolarZoomObj;
    FSettingPolarObj : TSettingPolarObj;
    procedure GraphWinPaint;
  protected
    { Protected declarations }
  public
    { Public declarations }
    property ViewPolarZoomObj : TViewPolarZoomObj read FViewPolarZoomObj;
    property SettingPolarObj : TSettingPolarObj read FSettingPolarObj;
    constructor Create;
    destructor Destroy; override;
    function FullGraphPaint : string; override;
    procedure RestartAnimatedGraphPaint; override;
    function StepAnimatedGraphPaint : boolean; override;
  end;

implementation

{ TPlotGraphPolar }

{---------------------------------------------------------------------}

constructor TPlotGraphPolar.Create;
begin {constructor TPlotGraphPolar.Create}
  inherited;
  FViewPolarZoomObj := TViewPolarZoomObj.Create;
  FSettingPolarObj := TSettingPolarObj.Create;
end; {constructor TPlotGraphPolar.Create}

{---------------------------------------------------------------------}

destructor TPlotGraphPolar.Destroy;
begin {destructor TPlotGraphPolar.Destroy;}
  FViewPolarZoomObj.free;
  FSettingPolarObj.free;
  inherited;
end; {destructor TPlotGraphPolar.Destroy;}

{---------------------------------------------------------------------}

function TPlotGraphPolar.FullGraphPaint : string;
begin {function TPlotGraphPolar.FullGraphPaint}
  Result := '';
  GraphWinPaint;
end; {function TPlotGraphPolar.FullGraphPaint}

{=====================================================================}

procedure TPlotGraphPolar.GraphWinPaint;
const
  MinTickPixDistance = 5;
  TickMLabelDistance = 1;
var
  { All Bmp Variables use the TopLeft corner = (0,0) standard}
  BmpCenter : TBmpPix;
  BmpHeightSub1,
  BmpWidthSub1 : integer;
  { All View Variables use a cartesian type cordinate system
      with the Center at (0,0) if PolarViewRec.ViewCenterX = 0 and
      PolarViewRec.ViewCenterY = 0}
  ViewCenterPix,
  ViewTopLeftPix, ViewTopRightPix,
  ViewBottomLeftPix, ViewBottomRightPix : TVirtualPix;
  { Angles are in Degrees }
  ViewTopLeftAngle, ViewTopRightAngle,
  ViewBottomLeftAngle, ViewBottomRightAngle : TMathFloat;
  GraphCenterInView : boolean;
  TMLableStr : string;
  { Begin Local Variables }
  LocalViewZoom : TMathFloat;
  LocalRLimit : TMathFloat;
  LocalIsRestrict : boolean;

{----------------------------------------------------------------------}

procedure VirtualToBmpPix(const TheVirRec : TVirtualPix;
                          var TheBmpRec : TBmpPix);
begin {procedure VirtualToBmpPix}
  TheBmpRec.X :=Round(TheVirRec.X - ViewCenterPix.X) + BmpCenter.X;
  TheBmpRec.Y :=Round(ViewCenterPix.Y - TheVirRec.Y) + BmpCenter.Y;
end; {procedure VirtualToBmpPix}

{----------------------------------------------------------------------}

function VirFloatXToBmpX(TheVir : TMathFloat) : integer;
begin {function VirFloatXToBmpX}
  Result := Round(TheVir - ViewCenterPix.X) + BmpCenter.X;
end; {function VirFloatXToBmpX}

{----------------------------------------------------------------------}

function VirFloatYToBmpY(TheVir : TMathFloat) : integer;
begin {function VirFloatYToBmpY}
  Result := Round(ViewCenterPix.Y - TheVir) + BmpCenter.Y;
end; {function VirFloatYToBmpY}

{----------------------------------------------------------------------}

function SafeVirFloatXToBmpX(TheVir : TMathFloat) : integer;
var
  TmpI : longint;
begin {function SafeVirFloatXToBmpX}
  TmpI := Round(TheVir - ViewCenterPix.X) + BmpCenter.X;
  if TmpI > 30000 then
    Result := 30000
  else
    begin
      if TmpI < -30000 then
        Result := -30000
      else
        Result := TmpI;
    end; {else}
end; {function SafeVirFloatXToBmpX}

{----------------------------------------------------------------------}

function SafeVirFloatYToBmpY(TheVir : TMathFloat) : integer;
var
  TmpI : longint;
begin {function SafeVirFloatYToBmpY}
  TmpI := Round(-(TheVir - ViewCenterPix.Y))+BmpCenter.Y;
  if TmpI > 30000 then
    Result := 30000
  else
    begin
      if TmpI < -30000 then
        Result := -30000
      else
        Result := TmpI;
    end; {else}
end; {function SafeVirFloatYToBmpY}

{---------------------------------------------------------------------}

function CalcDistanceToPix(const ThePix : TVirtualPix): TMathFloat;
begin {function CalcDistanceToPix}
  with ThePix do
    begin
      Result := sqrt((x*x)+(y*y));
    end; {with}
end; {function CalcDistanceToPix}

{----------------------------------------------------------------------}

function CalcAngleToPix(const ThePix : TVirtualPix): TMathFloat;
begin {function CalcAngleToPix}
  with ThePix do
    begin
      if X = 0 then
        begin
          if Y = 0 then
            Result := 0
          else
            if Y > 0 then
              Result := 90
            else
              Result := 270;
          exit;
        end; {if}
      if Y = 0 then
        begin
          if X >= 0 then
            Result := 0
          else
            Result := 180;
          exit;
        end; {if}
      if (X > 0) and (Y > 0) then {Quadrant 1}
        Result := arctan(Y/X)
      else
        if (X < 0) and (Y > 0) then {Quadrant 2}
          Result := pi - arctan(Y/abs(X))
        else
          if (X < 0) and (Y < 0) then {Quadrant 3}
            Result := pi + arctan(abs(Y)/abs(X))
          else {if (X > 0) and (Y < 0) then} {Quadrant 4}
            Result := (2*pi) - arctan(abs(Y)/X);
       {convert Result from Radians to Degrees}
      Result := Result * (180/pi);
    end; {with}
end; {function CalcAngleToPix}

{----------------------------------------------------------------------}

procedure InitialCalculations;
begin {procedure InitialCalculations}
  if DoIsTerminated then Exit;
  {Begin Set Local Var's}
  LocalViewZoom := ViewPolarZoomObj.ViewZoom;
  LocalIsRestrict := SettingPolarObj.Restrict;
  if LocalIsRestrict then
    LocalRLimit := abs(SettingPolarObj.RestrictAmtVal.FloatValue)*LocalViewZoom
  else
    LocalRLimit := 0;
  {End Set Local Var's}
  BmpHeightSub1 := BMPHeight -1;
  BmpWidthSub1 := BMPWidth -1;
  BmpCenter.X := (BMPWidth div 2);
  BmpCenter.Y := (BMPHeight div 2);
  if DoIsTerminated then Exit;
  with ViewCenterPix do
    begin
      X := ViewPolarZoomObj.ViewCenterX;
      Y := ViewPolarZoomObj.ViewCenterY;
    end; {with}
  with ViewTopLeftPix do
    begin
      X := ViewCenterPix.X - BmpCenter.X;
      Y := ViewCenterPix.Y + BmpCenter.Y;
    end; {with}
  with ViewTopRightPix do
    begin
      X := ViewCenterPix.X + (BmpWidthSub1 - BmpCenter.X);
      Y := ViewTopLeftPix.Y;
    end; {with}
  if DoIsTerminated then Exit;
  with ViewBottomLeftPix do
    begin
      X := ViewTopLeftPix.X;
      Y := ViewCenterPix.Y - (BmpHeightSub1 - BmpCenter.Y);
    end; {with}
  with ViewBottomRightPix do
    begin
      X := ViewTopRightPix.X;
      Y := ViewBottomLeftPix.Y;
    end; {with}
  if ((ViewTopLeftPix.X <= 0)and(ViewTopRightPix.X >= 0))and
     ((ViewTopLeftPix.Y >= 0)and(ViewBottomLeftPix.Y <= 0)) then
    GraphCenterInView := true
  else
    GraphCenterInView := false;
  if DoIsTerminated then Exit;
  ViewTopLeftAngle := CalcAngleToPix(ViewTopLeftPix);
  if DoIsTerminated then Exit;
  ViewTopRightAngle := CalcAngleToPix(ViewTopRightPix);
  if DoIsTerminated then Exit;
  ViewBottomLeftAngle := CalcAngleToPix(ViewBottomLeftPix);
  if DoIsTerminated then Exit;
  ViewBottomRightAngle := CalcAngleToPix(ViewBottomRightPix);
end; {procedure InitialCalculations}

{-----------------------------------------------------------------------}

procedure CalcAxisLines;
var
  I, PointsFound : integer;
  CurrAngle, CurrAngleRadians : TMathFloat;
  CurrLimitPix : TVirtualPix;
  FloatCalcSuccess : boolean;
  Distance1, Distance2 : TMathFloat;
  ReplacePix1, ReplacePix2 : Boolean;
  CenterLeftEdge, CenterRightEdge,
  CenterTopEdge, CenterBottomEdge : boolean;

{------------------------------------}

  procedure FindAndSaveHorzPix(YVal, XVal1Small, XVal2Large : TMathFloat);
  var
    TmpPix : TVirtualPix;
  begin {procedure FindAndSaveHorzPix}
    if (YVal = 0)or(CurrAngle = 0)or(CurrAngle = 180) then
      begin
        if (CurrAngle < 90)or(CurrAngle > 270) then
                       {same as =0 but accounts for rounding errors}
          TmpPix.X := XVal2Large
        else
          TmpPix.X := XVal1Small;
        TmpPix.Y := YVal;
      end
    else
      begin
        TmpPix.Y := YVal;
        TmpPix.X := (YVal*(cos(CurrAngleRadians)/
                                sin(CurrAngleRadians)));
        if TmpPix.X < XVal1Small then
          TmpPix.X := XVal1Small;
        if TmpPix.X > XVal2Large then
          TmpPix.X := XVal2Large;
      end; {else}
    if PointsFound = 0 then
      AxisLineArray[I].P1 := TmpPix
    else
      AxisLineArray[I].P2 := TmpPix;
    inc(PointsFound);
  end; {procedure FindAndSaveHorzPix}

{------------------------------------}

  procedure FindAndSaveVertPix(XVal, YVal1Small, YVal2Large : TMathFloat);
  var
    TmpPix : TVirtualPix;
  begin {procedure FindAndSaveVertPix}
    if (XVal = 0)or(CurrAngle = 90)or(CurrAngle = 270) then
      begin
        if CurrAngle < 180 then
                       {same as =90 but accounts for rounding errors}
          TmpPix.Y := YVal2Large
        else
          TmpPix.Y := YVal1Small;
        TmpPix.X := XVal;
      end
    else
      begin
        TmpPix.X := XVal;
        TmpPix.Y := (XVal*(sin(CurrAngleRadians)/
                                cos(CurrAngleRadians)));
        if TmpPix.Y < YVal1Small then
          TmpPix.Y := YVal1Small;
        if TmpPix.Y > YVal2Large then
          TmpPix.Y := YVal2Large;
      end; {else}
    if PointsFound = 0 then
      AxisLineArray[I].P1 := TmpPix
    else
      AxisLineArray[I].P2 := TmpPix;
    inc(PointsFound);
  end; {procedure FindAndSaveVertPix}

{------------------------------------}

  function IsBetween(A1,A2,M : TMathFloat): boolean;
  var
    BigA, SmallA : TMathFloat;
  begin
    Result := false;
    if A1 >= A2 then
      begin
        BigA := A1;
        SmallA := A2;
      end
    else
      begin
        BigA := A2;
        SmallA := A1;
      end; {if}
    if (SmallA <= 90) and (BigA >= 270) then
      begin
        if (M <= SmallA) or (M >= BigA) then
          Result := true;
      end
    else
      if (M >= SmallA)and(M <= BigA) then
        Result := true
  end;

{------------------------------------}

begin {procedure CalcAxisLines;}
  if DoIsTerminated then Exit;
  if GraphCenterInView then
    begin
      CenterLeftEdge := (ViewTopLeftPix.X = 0);
      CenterRightEdge := (ViewBottomRightPix.X = 0);
      CenterTopEdge := (ViewTopLeftPix.Y = 0);
      CenterBottomEdge := (ViewBottomRightPix.Y = 0);
    end
  else
    begin
      CenterLeftEdge := false;
      CenterRightEdge := false;
      CenterTopEdge := false;
      CenterBottomEdge := false;
    end;
  if DoIsTerminated then Exit;
  for I := 1 to (SettingPolarObj.AxisCount) do
    begin
      if GraphCenterInView then
        begin
          AxisLineArray[I].P1.X := 0;
          AxisLineArray[I].P1.Y := 0;
          PointsFound := 1;
        end
      else
        PointsFound := 0;
      CurrAngle := SettingPolarObj.AxisAngles[I];
      CurrAngleRadians := CurrAngle*(pi/180);
      if DoIsTerminated then Exit;
      if IsBetween(ViewTopLeftAngle, ViewTopRightAngle, CurrAngle)and
         (not CenterTopEdge) then
        FindAndSaveHorzPix(ViewTopLeftPix.Y,
                           ViewTopLeftPix.X,
                           ViewTopRightPix.X);
      if DoIsTerminated then Exit;
      if (PointsFound < 2)and IsBetween(ViewBottomLeftAngle,
          ViewBottomRightAngle, CurrAngle)and(not CenterBottomEdge) then
        FindAndSaveHorzPix(ViewBottomLeftPix.Y,
                           ViewBottomLeftPix.X,
                           ViewBottomRightPix.X);
      if DoIsTerminated then Exit;
      if (PointsFound < 2) and IsBetween(ViewBottomLeftAngle,
          ViewTopLeftAngle, CurrAngle)and(not CenterLeftEdge) then
        FindAndSaveVertPix(ViewBottomLeftPix.X,
                           ViewBottomLeftPix.Y,
                           ViewTopLeftPix.Y);
      if DoIsTerminated then Exit;
      if (PointsFound < 2) and IsBetween(ViewBottomRightAngle,
          ViewTopRightAngle, CurrAngle)and(not CenterRightEdge) then
        FindAndSaveVertPix(ViewBottomRightPix.X,
                           ViewBottomRightPix.Y,
                           ViewTopRightPix.Y);
      if DoIsTerminated then Exit;
      if PointsFound >= 2 then
        begin
          AxisLineArray[I].InView := true;
          if LocalIsRestrict then
            begin
              ReplacePix1 := false;
              ReplacePix2 := false;
              if DoIsTerminated then Exit;
              Distance1 := CalcDistanceToPix(AxisLineArray[I].P1);
              if DoIsTerminated then Exit;
              Distance2 := CalcDistanceToPix(AxisLineArray[I].P2);
              if DoIsTerminated then Exit;
              if Distance1 >= Distance2 then
                begin
                  if (LocalRLimit < Distance2) then
                    AxisLineArray[I].InView := false
                  else
                    if (LocalRLimit < Distance1) then
                      ReplacePix1 := true;
                end
              else
                begin
                  if (LocalRLimit < Distance1) then
                    AxisLineArray[I].InView := false
                  else
                    if (LocalRLimit < Distance2) then
                      ReplacePix2 := true;
                end; {else}
              if DoIsTerminated then Exit;
              if ReplacePix1 or ReplacePix2 then
                begin
                  FloatCalcSuccess := true;
                  try
                    CurrLimitPix.X :=(LocalRLimit*cos(CurrAngleRadians));
                    CurrLimitPix.Y :=(LocalRLimit*sin(CurrAngleRadians));
                  except
                    FloatCalcSuccess := false;
                  end; {try...Except}
                  if FloatCalcSuccess then
                    begin
                      if ReplacePix1 then
                        AxisLineArray[I].P1 := CurrLimitPix
                      else
                        AxisLineArray[I].P2 := CurrLimitPix;
                    end; {if}
                end; {if}
            end; {if}
        end
      else
        AxisLineArray[I].InView := false;
      if DoIsTerminated then Exit;
    end; {for}
end; {procedure CalcAxisLines;}

{-----------------------------------------------------------------------}

procedure DrawAxisLines;
var
  I : integer;
  Draw1, Draw2 : TBmpPix;
begin {procedure DrawAxisLines;}
  if DoIsTerminated then Exit;
  with SettingPolarObj do
    begin
      GraphBitmap.Canvas.Pen.Color := AxisColor;
      GraphBitmap.Canvas.Pen.Width := AxisWidth;
      GraphBitmap.Canvas.Pen.Style := AxisStyle;
    end; {with}
  if DoIsTerminated then Exit;
  for I := 1 to (SettingPolarObj.AxisCount) do
    begin
      if DoIsTerminated then Exit;
      if AxisLineArray[I].InView then
        begin
          VirtualToBmpPix(AxisLineArray[I].P1, Draw1);
          if DoIsTerminated then Exit;
          VirtualToBmpPix(AxisLineArray[I].P2, Draw2);
          if DoIsTerminated then Exit;
          with Draw1 do
            GraphBitmap.Canvas.MoveTo(X,Y);
          if DoIsTerminated then Exit;
          with Draw2 do
            begin
              GraphBitmap.Canvas.LineTo(X,Y);
              if DoIsTerminated then Exit;
              if (SettingPolarObj.AxisStyle = psSolid) then
                GraphBitmap.Canvas.pixels[X,Y] := SettingPolarObj.AxisColor;
              if DoIsTerminated then Exit;
            end; {with}
        end; {if}
    end; {for}
end; {procedure DrawAxisLines;}

{-----------------------------------------------------------------------}

procedure FormatTMLableStr(TheTMCount : TMathFloat);
begin {procedure FormatTMLableStr}
  with SettingPolarObj do
    TMLableStr := FloatCordToFormatStr(TheTMCount*TickMarkIncrements.FloatValue,
                  TMLabelFormat, ShowTMLabelComa,
                  ShowTMLabelPlus, LabelDecimalPlaces);
end; {procedure FormatTMLableStr}

{-----------------------------------------------------------------------}

function DivAndRoundUp(TheNearDist,TheTickMPixInc : TMathFloat):TMathFloat;
var
  DivResult : TMathFloat;
begin {function DivAndRoundUp}
  DivResult := TheNearDist / TheTickMPixInc;
  if (Frac(DivResult) = 0) then
    Result := Int(DivResult)
  else
    Result := Int(DivResult) + 1;
end; {function DivAndRoundUp}

{-----------------------------------------------------------------------}

procedure DrawTickCircles;
  { This is the largest possible Arc
  GraphBitmap.Canvas.Arc(-6676,-6676, 1500,1500, 0,350, 350,0);
    This is the Largest possible Ellipse
  GraphBitmap.Canvas.Ellipse(-6676,-6676, 1500,1500);}
var
  { Distances are in number of pixels }
  ViewTopLeftDist, ViewTopRightDist,
  ViewBottomLeftDist, ViewBottomRightDist : TMathFloat;
  NearestSide, LongestDist : TMathFloat;
  BiggestAngle, SmallestAngle : TMathFloat;
  CurrTickInc : TMathFloat;
  TickMPixInc : TMathFloat;
  TickMCounter : TMathFloat;

{----------------------------------}

 procedure DrawLabelAtCircle;
  var
    TextPosPix : TBmpPix;
    InterSectPix : TVirtualPix;
    CurrAngleRadians : TMathFloat;
    I : integer;
 begin {procedure DrawLabelAtCircle;}
   for I := 1 to (SettingPolarObj.AxisCount) do
     if AxisLineArray[I].InView then
       begin
           {convert To Radians}
         CurrAngleRadians := SettingPolarObj.AxisAngles[I]*(pi/180);
         InterSectPix.X := CurrTickInc * cos(CurrAngleRadians);
         InterSectPix.Y := CurrTickInc * sin(CurrAngleRadians);
         with InterSectPix do
         if (((X >= ViewTopLeftPix.X) and
              (X <= ViewBottomRightPix.X))and
             ((Y <= ViewTopLeftPix.Y) and
              (Y >= ViewBottomRightPix.Y))) then {if point ON BMP view}
           begin
             VirtualToBmpPix(InterSectPix, TextPosPix);
             FormatTMLableStr(TickMCounter);
             with TextPosPix do
               GraphBitmap.Canvas.textOut(X+TickMLabelDistance,
                                     Y+TickMLabelDistance,TMLableStr);
           end; {if}
       end; {if}
 end; {procedure DrawLabelAtCircle;}

 {----------------------------------}

   procedure FindBigAndSmallAngles;
   begin {procedure FindBigAndSmallAngles;}
     {Find Nearest Angle}
     if (ViewTopLeftPix.X <= 0) and (ViewBottomRightPix.X >= 0) then
       begin  {Y Axis Crosses threw bmp}
         if abs(ViewTopLeftPix.Y) <= abs(ViewBottomRightPix.Y) then
           NearestSide := abs(ViewTopLeftPix.Y)
         else
           NearestSide := abs(ViewBottomRightPix.Y);
       end
     else
       if (ViewTopLeftPix.Y >= 0) and (ViewBottomRightPix.Y <= 0) then
         begin  {X Axis Crosses threw bmp}
           if abs(ViewTopLeftPix.X) <= abs(ViewBottomRightPix.X) then
             NearestSide := abs(ViewTopLeftPix.X)
           else
             NearestSide := abs(ViewBottomRightPix.X);
         end
       else
         begin
           NearestSide := ViewTopLeftDist;
           if NearestSide > ViewTopRightDist then
             NearestSide := ViewTopRightDist;
           if NearestSide > ViewBottomLeftDist then
             NearestSide := ViewBottomLeftDist;
           if NearestSide > ViewBottomRightDist then
             NearestSide := ViewBottomRightDist;
         end; {else}
     {Set the Angles}
     if (ViewTopLeftPix.Y >= 0)and(ViewBottomRightPix.Y <= 0)and
        (ViewTopLeftPix.X > 0)then
       begin
         BiggestAngle := ViewBottomLeftAngle;
         SmallestAngle := ViewTopLeftAngle;
       end
     else
       begin
         {Set BiggestAngle}
         BiggestAngle := ViewTopLeftAngle;
         if ViewTopRightAngle > BiggestAngle then
           BiggestAngle := ViewTopRightAngle;
         if ViewBottomLeftAngle > BiggestAngle then
           BiggestAngle := ViewBottomLeftAngle;
         if ViewBottomRightAngle > BiggestAngle then
           BiggestAngle := ViewBottomRightAngle;
         {Set SmallestAngle}
         SmallestAngle := ViewTopLeftAngle;
         if ViewTopRightAngle < SmallestAngle then
           SmallestAngle := ViewTopRightAngle;
         if ViewBottomLeftAngle < SmallestAngle then
           SmallestAngle := ViewBottomLeftAngle;
         if ViewBottomRightAngle < SmallestAngle then
           SmallestAngle := ViewBottomRightAngle;
       end; {else}
     {Convert to Radians}
     BiggestAngle := BiggestAngle*(pi/180);
     SmallestAngle := SmallestAngle*(pi/180);
   end; {procedure FindBigAndSmallAngles;}

{----------------------------------}

  procedure DrawUnCenteredPartCircle;
  var
    AdjacentDivHyp, TheArcCos, IAngle : TMathFloat;
    BigMinusSmallAngle : TMathFloat;
    I, Loop : longint;
    CrossZeroDeg : boolean;
    ArcCosSuccessful : boolean;
    TmpPixX, TmpPixY : integer;

{-------------------------}

  function ArcCos(X : TMathFloat) : TMathFloat;
    begin {ArcCos(x) = ArcTan (-x / sqrt(1 - sqr(x))) + (pi/2)}
      ArcCosSuccessful := true;
      try
        if X = 1 then
          ArcCos := 0
        else
          begin
            if X = (-1) then
              ArcCos := pi
            else
              begin
                if (-1 <= X) and (X <= 1) then
                  ArcCos := ArcTan ((-X) / sqrt(1 - sqr(X))) + (pi/2)
                else
                  begin
                    ArcCosSuccessful := false;
                    {guarantee valid floating point value is returned}
                    ArcCos := 0;
                  end; {else}
              end; {else}
          end; {else}
      except
        on EUnderFlow do ArcCos := 0;
        else
          begin
            ArcCosSuccessful := false;
            ArcCos := 0;
          end; {else}
      end; {except}
    end; {function ArcCos}

{-------------------------}

  begin {procedure DrawUnCenteredPartCircle;}
    {Radius = CurrTickInc}
    if (SmallestAngle < (pi/2))and(BiggestAngle > ((3*pi)/2)) then
      begin
        CrossZeroDeg := true;
        BigMinusSmallAngle := ((2*pi)-BiggestAngle) + SmallestAngle;
      end
    else
      begin
        CrossZeroDeg := false;
        BigMinusSmallAngle := BiggestAngle - SmallestAngle;
      end; {else}
    AdjacentDivHyp := (CurrTickInc - 0.5)/CurrTickInc;
    TheArcCos := ArcCos(AdjacentDivHyp);
    if not ArcCosSuccessful then
      exit; {should never need to exit; Fail Safe}
    if (TheArcCos <= 0) then
      I := 0
    else
      I := trunc(BigMinusSmallAngle/(2*TheArcCos));
    {begin Deal with possible errors}
    if I < 0 then
      I := 0;
    if I > 500 then
      I := 500;
    {end Deal with possible errors}
    GraphBitmap.Canvas.MoveTo(SafeVirFloatXToBmpX(CurrTickInc*cos(SmallestAngle)),
                         SafeVirFloatYToBmpY(CurrTickInc*sin(SmallestAngle)));
    if CrossZeroDeg then
      begin
        for Loop := 1 to I do
          begin
            IAngle := SmallestAngle - (BigMinusSmallAngle*(Loop/(I+1)));
            if IAngle < 0 then
              IAngle := (2*pi) + IAngle;
            GraphBitmap.Canvas.LineTo(SafeVirFloatXToBmpX(CurrTickInc*cos(IAngle)),
                                 SafeVirFloatYToBmpY(CurrTickInc*sin(IAngle)));
          end; {for}
      end
    else
      begin
        for Loop := 1 to I do
          begin
            IAngle := (BigMinusSmallAngle*(Loop/(I+1)))+SmallestAngle;
            GraphBitmap.Canvas.LineTo(SafeVirFloatXToBmpX(CurrTickInc*cos(IAngle)),
                                 SafeVirFloatYToBmpY(CurrTickInc*sin(IAngle)));
          end; {for}
      end; {else}
    TmpPixX := SafeVirFloatXToBmpX(CurrTickInc*cos(BiggestAngle));
    TmpPixY := SafeVirFloatYToBmpY(CurrTickInc*sin(BiggestAngle));
    GraphBitmap.Canvas.LineTo(TmpPixX,TmpPixY);
    if (SettingPolarObj.CircleStyle = psSolid) then
      GraphBitmap.Canvas.pixels[TmpPixX,TmpPixY] := SettingPolarObj.TickMarkColor;
  end; {procedure DrawUnCenteredPartCircle;}

{----------------------------------}

begin {procedure DrawTickCircles;}
  if DoIsTerminated then Exit;
  TickMPixInc :=SettingPolarObj.TickMarkIncrements.FloatValue * LocalViewZoom;
  with SettingPolarObj do
    begin
      GraphBitmap.Canvas.Pen.Color := TickMarkColor;
      GraphBitmap.Canvas.Pen.Width := CircleWidth;
      GraphBitmap.Canvas.Pen.Style := CircleStyle;
    end; {with}
  if DoIsTerminated then Exit;
  ViewTopLeftDist := CalcDistanceToPix(ViewTopLeftPix);
  if DoIsTerminated then Exit;
  ViewTopRightDist := CalcDistanceToPix(ViewTopRightPix);
  if DoIsTerminated then Exit;
  ViewBottomLeftDist := CalcDistanceToPix(ViewBottomLeftPix);
  if DoIsTerminated then Exit;
  ViewBottomRightDist := CalcDistanceToPix(ViewBottomRightPix);
  if DoIsTerminated then Exit;
  LongestDist := ViewTopLeftDist;
  if LongestDist < ViewTopRightDist then
    LongestDist := ViewTopRightDist;
  if LongestDist < ViewBottomLeftDist then
    LongestDist := ViewBottomLeftDist;
  if LongestDist < ViewBottomRightDist then
    LongestDist := ViewBottomRightDist;
  {make sure that LocalRLimit >= LongestDist}
  if LocalIsRestrict then
    begin
      if LocalRLimit < LongestDist then
        LongestDist := LocalRLimit;
    end; {if}
  if DoIsTerminated then Exit;
  if GraphCenterInView then
    begin
      CurrTickInc := TickMPixInc;
      TickMCounter := 1;
      while CurrTickInc <= LongestDist do
        begin
          if DoIsTerminated then Exit;
          GraphBitmap.Canvas.Ellipse(VirFloatXToBmpX(-CurrTickInc),
                                VirFloatYToBmpY(CurrTickInc),
                                VirFloatXToBmpX(CurrTickInc),
                                VirFloatYToBmpY(-CurrTickInc));
          if DoIsTerminated then Exit;
          if SettingPolarObj.LabelTickMarks then
            DrawLabelAtCircle;
          TickMCounter := TickMCounter + 1;
          CurrTickInc := TickMCounter * TickMPixInc;
        end; {while}
    end
  else
    begin
      FindBigAndSmallAngles;
      if DoIsTerminated then Exit;
      if (TickMPixInc < NearestSide)and(TickMPixInc <> 0) then
        begin
          TickMCounter := (DivAndRoundUp(NearestSide,TickMPixInc));
          CurrTickInc := TickMPixInc * TickMCounter;
        end
      else
        begin
          TickMCounter := 1;
          CurrTickInc := TickMPixInc;
        end; {else}
      if DoIsTerminated then Exit;
      while CurrTickInc <= LongestDist do
        begin
          if DoIsTerminated then Exit;
          if CurrTickInc <= 4000 then
            GraphBitmap.Canvas.Ellipse(VirFloatXToBmpX(-CurrTickInc),
                                  VirFloatYToBmpY(CurrTickInc),
                                  VirFloatXToBmpX(CurrTickInc),
                                  VirFloatYToBmpY(-CurrTickInc))
          else
            DrawUnCenteredPartCircle;
          if DoIsTerminated then Exit;
          if SettingPolarObj.LabelTickMarks then
            DrawLabelAtCircle;
          TickMCounter := TickMCounter + 1;
          CurrTickInc := TickMCounter * TickMPixInc;
        end; {while}
    end;
end; {procedure DrawTickCircles;}

{-----------------------------------------------------------------------}

procedure DrawTickDashes;
var
  I : integer;
  DashLength : integer;
  CurrAngleRadians : TMathFloat;
  CurrTickInc, CurrTickX, CurrTickY : TMathFloat;
  CurrLowDash, CurrHiDash : TVirtualPix;
  CurrLowDPix, CurrHiDPix : TBmpPix;
  NearDist, FarDist, TmpDist : TMathFloat;
  NearPix, FarPix : TVirtualPix;
  TickMPixInc : TMathFloat;
  TickMCounter : TMathFloat;

{---------------------------------}

  procedure DrawLabelAtDash;
  var
    TextPosPix : TBmpPix;
  begin {procedure DrawLabelAtDash;}
    FormatTMLableStr(TickMCounter);
    if (CurrLowDash.X = CurrHiDash.X)or
       (CurrLowDPix.X = CurrHiDPix.X) then {Vertical Dash}
      begin
        if CurrLowDPix.Y >= CurrHiDPix.Y then
          TextPosPix := CurrLowDPix
        else
          TextPosPix := CurrHiDPix;
        TextPosPix.Y := TextPosPix.Y + TickMLabelDistance;
      end
    else
    if (CurrLowDash.Y = CurrHiDash.Y)or
       (CurrLowDPix.Y = CurrHiDPix.Y) then {Horizontal Dash}
      begin
        if CurrLowDPix.X >= CurrHiDPix.X then
          TextPosPix := CurrLowDPix
        else
          TextPosPix := CurrHiDPix;
        TextPosPix.X := TextPosPix.X + TickMLabelDistance;
        TextPosPix.Y := TextPosPix.Y -
                        (GraphBitmap.Canvas.textHeight(TMLableStr) div 2);
      end
    else
      begin
        if ((CurrLowDash.Y-CurrHiDash.Y)/
            (CurrLowDash.X-CurrHiDash.X)) > 0 then {positive Slope}
          begin
            if CurrLowDPix.Y <= CurrHiDPix.Y then
              TextPosPix := CurrLowDPix
            else
              TextPosPix := CurrHiDPix;
            TextPosPix.X := TextPosPix.X + TickMLabelDistance;
            TextPosPix.Y := TextPosPix.Y -
                            (GraphBitmap.Canvas.textHeight(TMLableStr)) -
                            TickMLabelDistance;
          end
        else  {Negative Slope}
          begin
            if CurrLowDPix.Y >= CurrHiDPix.Y then
              TextPosPix := CurrLowDPix
            else
              TextPosPix := CurrHiDPix;
            TextPosPix.X := TextPosPix.X + TickMLabelDistance;
            TextPosPix.Y := TextPosPix.Y + TickMLabelDistance;
          end; {else}
      end; {else}
    {Code Here}
    with TextPosPix do
      GraphBitmap.Canvas.textOut(X,Y,TMLableStr);
  end; {procedure DrawLabelAtDash;}

{---------------------------------}

begin {procedure DrawTickDashes;}
  if DoIsTerminated then Exit;
  TickMPixInc :=SettingPolarObj.TickMarkIncrements.FloatValue * LocalViewZoom;
  with SettingPolarObj do
    begin
      GraphBitmap.Canvas.Pen.Color := TickMarkColor;
      GraphBitmap.Canvas.Pen.Width := AxisWidth;
      GraphBitmap.Canvas.Pen.Style := psSolid;
    end; {with}
  DashLength :=  SettingPolarObj.DashLength;
  if DoIsTerminated then Exit;
  for I := 1 to SettingPolarObj.AxisCount do
    begin
      if DoIsTerminated then Exit;
      if AxisLineArray[I].InView then
        begin
          with AxisLineArray[I] do
            begin
              NearDist := CalcDistanceToPix(P1);
              FarDist := CalcDistanceToPix(P2);
              if DoIsTerminated then Exit;
              if FarDist < NearDist then
                begin
                  NearPix := P2;
                  FarPix := P1;
                  TmpDist := NearDist;
                  NearDist := FarDist;
                  FarDist := TmpDist;
                end
              else
                begin
                  NearPix := P1;
                  FarPix := P2;
                end; {else}
            end; {with}
          if (TickMPixInc < NearDist)and(TickMPixInc <> 0) then
            begin
              TickMCounter := (DivAndRoundUp(NearDist,TickMPixInc));
              CurrTickInc := TickMPixInc * TickMCounter;
            end
          else
            begin
              TickMCounter := 1;
              CurrTickInc := TickMPixInc;
            end; {else}
          FarDist := FarDist + 0.0000000001; {account for rounding errors}
          while CurrTickInc <= FarDist do
            begin
              if DoIsTerminated then Exit;
              {convert To Radians}
              CurrAngleRadians := SettingPolarObj.AxisAngles[I]*(pi/180);
              CurrTickX := CurrTickInc*cos(CurrAngleRadians);
              CurrTickY := CurrTickInc*sin(CurrAngleRadians);
              CurrLowDash.X := CurrTickX + DashLength*sin(CurrAngleRadians);
              CurrLowDash.Y := CurrTickY - DashLength*cos(CurrAngleRadians);
              CurrHiDash.X := CurrTickX - DashLength*sin(CurrAngleRadians);
              CurrHiDash.Y := CurrTickY + DashLength*cos(CurrAngleRadians);
              VirtualToBmpPix(CurrLowDash, CurrLowDPix);
              if DoIsTerminated then Exit;
              VirtualToBmpPix(CurrHiDash, CurrHiDPix);
              with CurrLowDPix do
                GraphBitmap.Canvas.MoveTo(X, Y);
              with CurrHiDPix do
                begin
                  GraphBitmap.Canvas.lineTo(X, Y);
                  GraphBitmap.Canvas.pixels[X, Y] := SettingPolarObj.TickMarkColor;
                end; {with}
              if DoIsTerminated then Exit;
              if SettingPolarObj.LabelTickMarks then
                DrawLabelAtDash;
              TickMCounter := TickMCounter + 1;
              CurrTickInc := TickMPixInc * TickMCounter;
            end; {while}
        end; {if}
    end; {for}
end; {procedure DrawTickDashes;}

{-----------------------------------------------------------------------}

procedure DrawAFunction(CurrFuncObj : TFuncPolarObj);
type
  TPolarPoint = record
    XVal, YVal : TMathFloat;
    Exists : boolean;
    OnBmp : boolean;
  end; {TPolarPoint}
  PClipRec = ^TClipRec;
  TClipRec = record
    X, Y : TMathFloat;
  end; {TClipRec}
var
  I : integer;
  CurrLoBoundVal,
  CurrHiBoundVal : TMathFloat;
  BoundDiff, CurrSamplesFloat : TMathFloat;
  CurrPoint, PrevPoint : TPolarPoint;
  CurrClip, PrevClip : TClipRec;
  ClipedLineCrossesBmp : boolean;
  LinePointCount : integer;
  DoFixUpPoint : boolean;
  TmpParamValue : TMathFloat;
  {Local Variables}
  LocalFuncCalc : TMathPolarFunc;

{------------------------------------}

procedure DrawToClippedPoint(const TheClippedPoint : TClipRec);
begin {procedure DrawToClippedPoint}
  inc(LinePointCount);
  with LinePointArray[LinePointCount] do
    begin
      X := VirFloatXToBmpX(TheClippedPoint.X);
      Y := VirFloatYToBmpY(TheClippedPoint.Y);
    end; {with}
  if LinePointCount >= LinePointArraySize then
    begin
      LinePointCount := 1;
      PolyLine(GraphBitmap.Canvas.Handle,
               LinePointArray, LinePointArraySize);
      LinePointArray[1] := LinePointArray[LinePointArraySize];
    end; {if}
end; {procedure DrawToClippedPoint}

{------------------------------------}

procedure DrawToCurrPoint;
begin {procedure DrawToCurrPoint;}
  inc(LinePointCount);
  with LinePointArray[LinePointCount] do
    begin
      X := VirFloatXToBmpX(CurrPoint.XVal);
      Y := VirFloatYToBmpY(CurrPoint.YVal);
    end; {with}
  if LinePointCount >= LinePointArraySize then
    begin
      LinePointCount := 1;
      PolyLine(GraphBitmap.Canvas.Handle,
               LinePointArray, LinePointArraySize);
      LinePointArray[1] := LinePointArray[LinePointArraySize];
    end; {if}
end; {procedure DrawToCurrPoint;}

{------------------------------------}

procedure GotUnDefinedPoint;
begin {procedure GotUnDefinedPoint;}
  if LinePointCount > 0 then
    begin
      if LinePointCount = 1 then
        begin
          if DoFixUpPoint then
            with LinePointArray[1] do
              GraphBitmap.Canvas.Pixels[X, Y]:= CurrFuncObj.FuncColor;
        end
      else
        begin
          PolyLine(GraphBitmap.Canvas.Handle, LinePointArray, LinePointCount);
          if DoFixUpPoint then
            with LinePointArray[LinePointCount] do
              GraphBitmap.Canvas.Pixels[X, Y] := CurrFuncObj.FuncColor;
        end; {else}
      LinePointCount := 0;
    end; {if}
end; {procedure GotUnDefinedPoint;}

{------------------------------------}

procedure ClipCurrPrevLine;
  {This Procedure Assumes that the line has allready been tested}
  {     and that it requires clipping.}
var
  PHi, PLo : PClipRec;
  CurrClipOnBitmap, PrevClipOnBitmap : boolean;

{------------------------}

procedure ClipAginstVert(ClipedPoint : PClipRec;
                         RectXVal: TMathFloat);
var
  MSlope : TMathFloat;
begin
  MSlope := (CurrClip.Y-PrevClip.Y)/(CurrClip.X-PrevClip.X);
  with ClipedPoint^ do
    begin
      Y := (Y + (RectXVal - X)*(MSlope));
      X := RectXVal;
    end; {with}
end; {procedure ClipAginstVert}

{------------------------}

procedure ClipAginstHorz(ClipedPoint : PClipRec;
                         RectYVal: TMathFloat);
var
  MSlope : TMathFloat;
begin
  MSlope := (CurrClip.X-PrevClip.X)/(CurrClip.Y-PrevClip.Y);
  with ClipedPoint^ do
    begin
      X := (X + (RectYVal - Y)*(MSlope));
      Y := RectYVal;
    end; {with}
end; {procedure ClipAginstHorz}

{------------------------}

begin {procedure ClipCurrPrevLine;}
  CurrClip.X := CurrPoint.XVal;
  CurrClip.Y := CurrPoint.YVal;
  PrevClip.X := PrevPoint.XVal;
  PrevClip.Y := PrevPoint.YVal;
  if ((not CurrPoint.OnBmp) and (not PrevPoint.OnBmp)) then
    begin
      if((CurrClip.X < ViewTopLeftPix.X)and
         (PrevClip.X < ViewTopLeftPix.X))or
        ((CurrClip.X > ViewBottomRightPix.X)and
         (PrevClip.X > ViewBottomRightPix.X))or
        ((CurrClip.Y > ViewTopLeftPix.Y)and
         (PrevClip.Y > ViewTopLeftPix.Y))or
        ((CurrClip.Y < ViewBottomRightPix.Y)and
         (PrevClip.Y < ViewBottomRightPix.Y))then
           begin
             ClipedLineCrossesBmp := false;
             exit; {Line does not cross BMP}
           end; {if}
    end; {if}
  if CurrClip.X = PrevClip.X then
    begin
      if CurrClip.Y < PrevClip.Y then
        begin
          PLo := @CurrClip;
          PHi := @PrevClip;
        end
      else
        begin
          PHi := @CurrClip;
          PLo := @PrevClip;
        end; {else}
      if (PHi^.Y > ViewTopLeftPix.Y)and(PLo^.Y <= ViewTopLeftPix.Y) then
        PHi^.Y := ViewTopLeftPix.Y;
      if (PHi^.Y >= ViewBottomRightPix.Y)and(PLo^.Y < ViewBottomRightPix.Y) then
        PLo^.Y := ViewBottomRightPix.Y;
    end
  else
  if CurrClip.Y = PrevClip.Y then
    begin
      if CurrClip.X < PrevClip.X then
        begin {swap them}
          PLo := @CurrClip;
          PHi := @PrevClip;
        end
      else
        begin
          PHi := @CurrClip;
          PLo := @PrevClip;
        end; {else}
      if (PHi^.X >= ViewTopLeftPix.X)and(PLo^.X < ViewTopLeftPix.X) then
        PLo^.X := ViewTopLeftPix.X;
      if (PHi^.X > ViewBottomRightPix.X)and(PLo^.X <= ViewBottomRightPix.X) then
        PHi^.X := ViewBottomRightPix.X;
    end
  else
    begin
      if CurrClip.X < PrevClip.X then
        begin {swap them}
          PLo := @CurrClip;
          PHi := @PrevClip;
        end
      else
        begin
          PHi := @CurrClip;
          PLo := @PrevClip;
        end; {else}
      if (PHi^.X >= ViewTopLeftPix.X)and(PLo^.X < ViewTopLeftPix.X) then
        ClipAginstVert(PLo, ViewTopLeftPix.X);
      if (PHi^.X > ViewBottomRightPix.X)and(PLo^.X <= ViewBottomRightPix.X) then
        ClipAginstVert(PHi, ViewBottomRightPix.X);
      if CurrClip.Y < PrevClip.Y then
        begin
          PLo := @CurrClip;
          PHi := @PrevClip;
        end
      else
        begin
          PHi := @CurrClip;
          PLo := @PrevClip;
        end; {else}
      if (PHi^.Y > ViewTopLeftPix.Y)and(PLo^.Y <= ViewTopLeftPix.Y) then
        ClipAginstHorz(PHi, ViewTopLeftPix.Y);
      if (PHi^.Y >= ViewBottomRightPix.Y)and(PLo^.Y < ViewBottomRightPix.Y) then
        ClipAginstHorz(PLo, ViewBottomRightPix.Y);
    end; {else}
  CurrClipOnBitmap := (((CurrClip.X >= ViewTopLeftPix.X) and
                        (CurrClip.X <= ViewBottomRightPix.X))and
                       ((CurrClip.Y <= ViewTopLeftPix.Y) and
                        (CurrClip.Y >= ViewBottomRightPix.Y)));
  PrevClipOnBitmap := (((PrevClip.X >= ViewTopLeftPix.X) and
                        (PrevClip.X <= ViewBottomRightPix.X))and
                       ((PrevClip.Y <= ViewTopLeftPix.Y) and
                        (PrevClip.Y >= ViewBottomRightPix.Y)));
  ClipedLineCrossesBmp := CurrClipOnBitmap and PrevClipOnBitmap;
end; {procedure ClipCurrPrevLine;}

{------------------------------------}

begin {procedure DrawAFunction}
  LinePointCount := 0;
  with CurrFuncObj do
    begin
      LocalFuncCalc := FuncCalc;
      CurrLoBoundVal := LoBoundVal.FloatValue;
      CurrHiBoundVal := HiBoundVal.FloatValue;
      GraphBitmap.Canvas.Pen.Color := FuncColor;
      GraphBitmap.Canvas.Pen.Width := LineWidth;
      DoFixUpPoint := (LineWidth = 1);
    end; {with}
  if DoIsTerminated then Exit;
  if CurrLoBoundVal >= CurrHiBoundVal then
    exit;
  with CurrPoint do
    begin
      Exists := false;
      OnBmp := false;
      XVal := 0;
      YVal := 0;
    end; {with}
  BoundDiff := CurrHiBoundVal - CurrLoBoundVal;
  CurrSamplesFloat := CurrFuncObj.Samples;
  if DoIsTerminated then Exit;
  for I := 0 to CurrFuncObj.Samples do
    begin
      PrevPoint := CurrPoint;
      {begin math function evaluation code}
      with LocalFuncCalc do
        begin
          {Calc current theta value and store for evaluation}
          TmpParamValue := ((I/CurrSamplesFloat)*BoundDiff)+CurrLoBoundVal;
          FloatVarValues[0] := TmpParamValue;
          FloatVarValues[1] := TmpParamValue;
          CurrPoint.Exists := EvaluateMathFunc;
        end; {with}
      {end math function evaluation code}
      if DoIsTerminated then Exit;
      if CurrPoint.Exists then
        begin
          try
            with CurrPoint do
              begin
                XVal := (LocalFuncCalc.FloatResultValue * cos(TmpParamValue)) *
                        LocalViewZoom;
                YVal := (LocalFuncCalc.FloatResultValue * sin(TmpParamValue)) *
                        LocalViewZoom;
                OnBmp := (((XVal >= ViewTopLeftPix.X) and
                          (XVal <= ViewBottomRightPix.X))and
                         ((YVal <= ViewTopLeftPix.Y) and
                          (YVal >= ViewBottomRightPix.Y)));
              end; {with}
          except
            CurrPoint.Exists := false;
          end; {except}
        end; {if}
      if DoIsTerminated then Exit;
      if not CurrPoint.Exists then
        begin
          GotUnDefinedPoint;
        end
      else
        if not PrevPoint.Exists then
          begin
            if CurrPoint.OnBmp then
              DrawToCurrPoint;
          end
        else
          if CurrPoint.OnBmp and PrevPoint.OnBmp then
            begin
              DrawToCurrPoint;
            end
          else
            begin
              ClipCurrPrevLine;
              if DoIsTerminated then Exit;
              if not ClipedLineCrossesBmp then
                GotUnDefinedPoint
              else
                begin
                  if not PrevPoint.OnBmp then
                    begin
                      GotUnDefinedPoint;
                      if DoIsTerminated then Exit;
                      DrawToClippedPoint(PrevClip);
                    end; {if}
                  if DoIsTerminated then Exit;
                  DrawToClippedPoint(CurrClip);
                  if DoIsTerminated then Exit;
                  if not CurrPoint.OnBmp then
                    GotUnDefinedPoint;
                end; {else}
            end; {else}
      if DoIsTerminated then Exit;
    end; {for}
  if DoIsTerminated then Exit;
  GotUnDefinedPoint;
end; {procedure DrawAFunction}

{-----------------------------------------------------------------------}

procedure DrawAllFunctions;
var
  FuncCounter : integer;
begin {procedure DrawAllFunctions;}
  GraphBitmap.Canvas.Pen.Style := psSolid;
  for FuncCounter := 0 to FuncList.Count-1 do
    begin
      if DoIsTerminated then Exit;
      DrawAFunction(FuncList[FuncCounter]);
      if DoIsTerminated then Exit;
    end; {for}
end; {procedure DrawAllFunctions;}

{---------------------------------------------------------------------}

begin {procedure TPlotGraphPolar.GraphWinPaint;}
  if DoIsTerminated then Exit;
  FillGraphBmpBackGround;
  if DoIsTerminated then Exit;
  if SettingPolarObj.LabelTickMarks then
    GraphBitmap.Canvas.Font.Assign(SettingPolarObj.LabelFont);
  if DoIsTerminated then Exit;
  InitialCalculations;
  if DoIsTerminated then Exit;
  CalcAxisLines;
  if DoIsTerminated then Exit;
  DrawAxisLines;
  if DoIsTerminated then Exit;
  With SettingPolarObj do
  if (TickMarkIncrements.FloatValue > 0)and (MinTickPixDistance <=
     (TickMarkIncrements.FloatValue*LocalViewZoom)) then
    begin
      if SettingPolarObj.TickMarkType = Circles then
        DrawTickCircles;
      if DoIsTerminated then Exit;
      if SettingPolarObj.TickMarkType = Dashes then
        DrawTickDashes;
      if DoIsTerminated then Exit;
    end;
  if DoIsTerminated then Exit;
  DrawAllFunctions;
end; {procedure TPlotGraphPolar.GraphWinPaint;}

{=====================================================================}

procedure TPlotGraphPolar.RestartAnimatedGraphPaint;
begin {procedure TPlotGraphPolar.RestartAnimatedGraphPaint;}
  GraphWinPaint;
end; {procedure TPlotGraphPolar.RestartAnimatedGraphPaint;}

{---------------------------------------------------------------------}

function TPlotGraphPolar.StepAnimatedGraphPaint : boolean;
begin {function TPlotGraphPolar.StepAnimatedGraphPaint}
  GraphWinPaint;
  result := true;
end; {function TPlotGraphPolar.StepAnimatedGraphPaint}

{---------------------------------------------------------------------}

end.
