unit PtImage;

(*
Copyright 1999-2011 Greg VanMullem
This program is distributed under the terms of the GNU General Public License Version 3

This file is part of MathGV.

MathGV is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

MathGV is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MathGV.  If not, see <http://www.gnu.org/licenses/>.
*)

interface

uses
  SysUtils, windows, graphics, forms, classes, controls, extctrls, PtShared,
  PtThread, FileSave, Dialogs, CentrDlg, CentrErr, GrDraw, GrGlobal, Selctdlg, Printers,
  jpeg, Selected, ClipBrd, SetGraph, DrawObjs, CustEdit, LabelDlg, LbGlobal;

type
  TPlotImageBox = class(TScrollBox)
  private
    { Private declarations }
    FOnRunningStart : TNotifyEvent;
    FOnRunningEnd : TNotifyEvent;
    FBitmapAttribRec : TBitmapAttribRec;
    CurrCursor : TCursor;
    IsMouseLeftDown : boolean;
    IsMouseDoubleClick : boolean;
    IsDragLabelMode : boolean;
    CurrSelectNdx : integer;
    CurrSelectPtr : TDrawBaseClass;
    FLastMouseX,
    FLastMouseY : integer;
    RegionOrigin, RegionMovePt : TPoint;
    FOnDoneDrawing : TNotifyEvent;
    FOnTurnDrawToolOff : TNotifyEvent;
    FOnLabelWinStatusChange : TLabelWinStatusChangeEvent;
    FOnDisplayStatusMsg : TNotifyEvent;
    FOnGetToolBarAttrib : TGetToolBarAttribEvent;
    FOnGetDrawingTool : TGetDrawingToolEvent;
    FOnGetResizeLabelsMode : TGetResizeLabelsModeEvent;
    FOnShowLabelPopupMenu : TShowLabelPopupMenuEvent;
    FLabelWindowStatus : TLabelWindowStatus;
    ScreenImage : TImage;
    CustomTextEdit : TCustomTextEdit;
    NoLabelFixupSectionCount : integer;
    RedrawInNoLabelFixupSection : boolean;
    procedure RestartLabelListObjectChanged(HiddenLabelIndex : integer);
    function IsPointOnBmp(TheX, TheY: Integer) : boolean;
    function SaveTextLabel(DeleteWhenEmpty : boolean) : boolean;
    procedure ForceLabelWindowStatusNormal;
    procedure SetLabelWindowStatus(NewLWStatus : TLabelWindowStatus);
    property LabelWindowStatus : TLabelWindowStatus read FLabelWindowStatus
                                                 write SetLabelWindowStatus;
    function GetSettingBmpAttribsObj : TSettingBmpAttribsObj;
    function GetFileName : string;
    function GetFileNameNoPath : string;
    function GetCaptionTitleStringWithBrackets : string;
    function GetHasChangedSinceSave : boolean;
    procedure DrawMouseDown(Button: TMouseButton;
                          Shift: TShiftState; X, Y: Integer);
    procedure DrawMouseMove(Shift: TShiftState; X, Y: Integer);
    procedure DrawMouseUp(Button: TMouseButton;
                         Shift: TShiftState; X, Y: Integer);
    procedure ScreenImageMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure ScreenImageMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure ScreenImageMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure ScreenImageDblClick(Sender: TObject);
    function SetDrawModeNormal(ShouldRestartThread : boolean) : boolean;
    procedure DrawSelectRectangle(TopLeft, BottomRight: TPoint);
    procedure DoTurnDrawToolOffEvent;
    function DoGetToolBarAttribEvent : TToolBarAttribClass;
    function DoGetDrawingToolEvent : TDrawingTool;
    function DoGetResizeLabelsModeEvent : boolean;
    procedure DoShowLabelPopupMenuEvent(ScreenX, ScreenY : integer;
                                        ShowInPlaceEditorItem : boolean);
    procedure ForcePointOnBmp(var TheX, TheY: Integer);
    procedure ShowNormalCursor;
    procedure ShowNewCustomCursor(TheNewCursor : TCursor);
    function ImageToScrollX(X : integer) : integer;
    function ImageToScrollY(Y : integer) : integer;
    procedure StartEditingEvent(Sender: TObject; EditTopLeft : TPoint;
                                EditWidth, EditHeight : integer);
    procedure DoneEditingEvent(Sender: TObject);
    procedure SelectALabel(TheSelectNDX : integer);
    function DoesListHaveLabel : boolean;
    function DoesListHaveFloodFill : boolean;
    procedure DoDisplayStatusMsgEvent;
    procedure BeginNoLabelFixupSection;
    procedure EndNoLabelFixupSection;
    procedure LabelFixupAfterRedraw;
    //assign next event to PlotThread when created in decendant class
    procedure HandleDonePlotingEvent(PlotBitmap: TBitmap);
    //assign next event to PlotThread when created in decendant class
    //this event must return immediately, it can't display a dialog and wait.
    procedure HandleThreadPlotErrorEvent(Sender: TObject; const ImagePlotErrorString : string);
    //assign next event to PlotThread when created in decendant class
    procedure HandleRunningStartEvent(Sender: TObject);
    //assign next event to PlotThread when created in decendant class
    procedure HandleRunningEndEvent(Sender: TObject);
    function GetPlotTypeString : string;
  protected
    { Protected declarations }
    PlotThread : TPlotThread; //must be created and freed in descendant class
    property BmpWidth : integer read FBitmapAttribRec.BmpWidth;
    property BmpHeight : integer read FBitmapAttribRec.BmpHeight;
    property BackgroundColor : TColor read FBitmapAttribRec.BackgroundColor;
    procedure AssignEventsPlotThread(ThePlotThread : TPlotThread); //must be called in descendant class
  published
    { Published declarations }
    property OnDoneDrawing : TNotifyEvent read FOnDoneDrawing
                                          write FOnDoneDrawing;
    property OnTurnDrawToolOff : TNotifyEvent read FOnTurnDrawToolOff
                                              write FOnTurnDrawToolOff;
    property OnLabelWinStatusChange : TLabelWinStatusChangeEvent
             read FOnLabelWinStatusChange write FOnLabelWinStatusChange;
    property OnDisplayStatusMsg : TNotifyEvent
             read FOnDisplayStatusMsg write FOnDisplayStatusMsg;
    property OnGetToolBarAttrib : TGetToolBarAttribEvent read FOnGetToolBarAttrib
                                                         write FOnGetToolBarAttrib;
    property OnGetDrawingTool : TGetDrawingToolEvent read FOnGetDrawingTool
                                                     write FOnGetDrawingTool;
    property OnGetResizeLabelsMode : TGetResizeLabelsModeEvent read FOnGetResizeLabelsMode
                                                               write FOnGetResizeLabelsMode;
    property OnShowLabelPopupMenu : TShowLabelPopupMenuEvent read FOnShowLabelPopupMenu
                                                             write FOnShowLabelPopupMenu;
    property OnRunningStart : TNotifyEvent read FOnRunningStart write FOnRunningStart;
    property OnRunningEnd : TNotifyEvent read FOnRunningEnd write FOnRunningEnd;
  public
    { Public declarations }
    property LastMouseX : integer read FLastMouseX;
    property LastMouseY : integer read FLastMouseY;
    property SettingBmpAttribsObj : TSettingBmpAttribsObj read GetSettingBmpAttribsObj;
    property FileName : string read GetFileName;
    property FileNameNoPath : string read GetFileNameNoPath;
    property CaptionTitleStringWithBrackets : string read GetCaptionTitleStringWithBrackets;
    property PlotTypeString : string read GetPlotTypeString;
    property HasChangedSinceSave : boolean read GetHasChangedSinceSave;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure InitalizeNew(TheColorDepth : TPixelFormat;
                           TheDefaultBmpWidth, TheDefaultBmpHeight : integer);
    function InitalizeFromFile(TheColorDepth : TPixelFormat;
                               const TheFileName : string;
                               ReadGraphFile : TReadGraphFile) : boolean;
    function SaveFile : boolean;
    function SaveFileAs(const TheFileName : string) : boolean;
    function SaveOnExitPrompt : boolean;
    procedure ReStartGraphThread;
    procedure TerminateAllThreads;
    procedure WaitForTermAllThreads;
    procedure RestartFieldObjectChanged;
    procedure CenterTheBitmap;
    procedure ShowSelectRegionDlg;
    procedure ShowLabelListDlg;
    procedure PrintTheBMP(PrintSelected : boolean; PrintScalePercent : integer);
    procedure SaveAsPictureFile(const TheSaveFileStr : string; IsBmpNotJpeg : boolean);
    procedure CopyBMPToClipBoard;
    procedure SetLastMouseToOffBmp;
    function IsRegionSelected : boolean;
    procedure SelectTopLabel;
    procedure SelectNextLabelOrFFill;
    procedure SelectPrevLabelOrFFill;
    procedure MoveSelectedToFront;
    procedure MoveSelectedToBack;
    procedure ShowFloodFillLocations;
    procedure ModifyLabelAttributes;
    procedure InPlaceEditTextLabel;
    procedure DeleteSelectLabel;
    procedure DeleteAllLabels;
    procedure DeleteAllFloodFills;
    procedure DeleteAllLabelsAndFFills;
  end; {TPlotImageBox}

implementation

{ TPlotImageBox }

procedure TPlotImageBox.AssignEventsPlotThread(ThePlotThread: TPlotThread);
begin {procedure TPlotImageBox.AssignEventsPlotThread}
  ThePlotThread.OnDonePloting := HandleDonePlotingEvent;
  ThePlotThread.OnPlotError := HandleThreadPlotErrorEvent;
  ThePlotThread.OnRunningStart := HandleRunningStartEvent;
  ThePlotThread.OnRunningEnd := HandleRunningEndEvent;
end; {procedure TPlotImageBox.AssignEventsPlotThread}

procedure TPlotImageBox.BeginNoLabelFixupSection;
begin {procedure TPlotImageBox.BeginNoLabelFixupSection;}
  inc(NoLabelFixupSectionCount);
  RedrawInNoLabelFixupSection := false;
end; {procedure TPlotImageBox.BeginNoLabelFixupSection;}

procedure TPlotImageBox.CenterTheBitmap;
begin {procedure TPlotImageBox.CenterTheBitmap;}
  HorzScrollBar.Position := (abs(HorzScrollBar.Range-ClientWidth)) div 2;
  VertScrollBar.Position := (abs(VertScrollBar.Range-ClientHeight)) div 2;
end; {procedure TPlotImageBox.CenterTheBitmap;}

procedure TPlotImageBox.CopyBMPToClipBoard;
var
  UseSelected : boolean;
  TmpBmp : TBitMap;
  TmpWidth, TmpHeight : integer;
begin {procedure TPlotImageBox.CopyBMPToClipBoard;}
  TmpBmp := nil;
  if IsRegionSelected then //same as, LabelWindowStatus = lwRegionSelected
    begin
      if PromptUseSelected(true, UseSelected) then
        begin
          if not IsRegionSelected then //same as, LabelWindowStatus <> lwRegionSelected
            begin
              {Default, just incase something weird causes}
              {  LabelWindowStatus to change during prompt}
              SetDrawModeNormal(true);
              Clipboard.Assign(ScreenImage.Picture);
              Exit;
            end; {if}
          TmpWidth := abs(RegionOrigin.X-RegionMovePt.X) + 1;
          TmpHeight := abs(RegionOrigin.Y-RegionMovePt.Y) + 1;
          SetDrawModeNormal(true);
          if UseSelected then
            begin
              try
                TmpBmp := TBitMap.Create;
                TmpBmp.HandleType := bmDIB;
                TmpBmp.PixelFormat := ScreenImage.Picture.bitmap.PixelFormat;
                TmpBmp.Width := TmpWidth;
                TmpBmp.Height := TmpHeight;
                TmpBmp.Canvas.CopyRect(Rect(0, 0, TmpWidth-1, TmpHeight-1),
                                  ScreenImage.Canvas,
                                  Rect(RegionOrigin.X, RegionOrigin.Y,
                                       RegionMovePt.X, RegionMovePt.Y));
                Clipboard.Assign(TmpBmp);
              finally
                TmpBmp.free;
              end; {try..Finally}
            end
          else
            Clipboard.Assign(ScreenImage.Picture);
        end; {if}
    end
  else
    begin
      SetDrawModeNormal(true);
      Clipboard.Assign(ScreenImage.Picture);
    end; {else}
end; {procedure TPlotImageBox.CopyBMPToClipBoard;}

constructor TPlotImageBox.Create(AOwner: TComponent);
begin {constructor TPlotImageBox.Create}
  inherited Create(AOwner);
  FOnRunningStart := nil;
  FOnRunningEnd := nil;
  NoLabelFixupSectionCount := 0;
  RedrawInNoLabelFixupSection := false;
  FBitmapAttribRec.BmpHeight := -1;
  FBitmapAttribRec.BmpWidth := -1;
  FBitmapAttribRec.BackgroundColor := clWhite;
  CurrCursor := crDefault;
  PlotThread := nil;
  FOnDoneDrawing := nil;;
  FOnTurnDrawToolOff := nil;
  FOnLabelWinStatusChange := nil;
  FOnDisplayStatusMsg := nil;
  FOnGetToolBarAttrib := nil;
  FOnGetDrawingTool := nil;
  FOnGetResizeLabelsMode := nil;
  FOnShowLabelPopupMenu := nil;
  ScreenImage := TImage.Create(Self);
  ScreenImage.parent := self;
  ScreenImage.Anchors := [akLeft,akTop];
  ScreenImage.Top := 0;
  ScreenImage.left := 0;
  ScreenImage.AutoSize := true;
  ScreenImage.Picture.Bitmap.HandleType := bmDIB;
  ScreenImage.OnMouseDown := ScreenImageMouseDown;
  ScreenImage.OnMouseMove := ScreenImageMouseMove;
  ScreenImage.OnMouseUp := ScreenImageMouseUp;
  ScreenImage.OnDblClick := ScreenImageDblClick;
  CustomTextEdit := TCustomTextEdit.Create(Self);
  CustomTextEdit.Parent := Self;
  CustomTextEdit.OnStartEditing := StartEditingEvent;
  CustomTextEdit.OnDoneEditing := DoneEditingEvent;
  IsMouseLeftDown := false;
  IsMouseDoubleClick := false;
  IsDragLabelMode := true;
  FLastMouseX := -1; //might want to set to -1000000000
  FLastMouseY := -1;
  CurrSelectNdx := -1;
  CurrSelectPtr := nil;
  RegionOrigin.X := 0;
  RegionOrigin.Y := 0;
  RegionMovePt.X := 0;
  RegionMovePt.Y := 0;
  ForceLabelWindowStatusNormal;
end; {constructor TPlotImageBox.Create}

procedure TPlotImageBox.DeleteAllFloodFills;
var
  I : integer;
  TmpObj : TDrawBaseClass;
begin {procedure TPlotImageBox.DeleteAllFloodFills;}
  if PlotThread.LabelList.Count > 0 then
    if DoesListHaveFloodFill Then
      if mrYes = MessageDlgCenter('Delete all flood fills from this graph?',
                            mtConfirmation,[mbYes,mbCancel],0,nil) then
        begin
          SetDrawModeNormal(false);
          PlotThread.SetHasChangedSinceSaveTrue;
          I := 0;
          while I < PlotThread.LabelList.Count do
            begin
              TmpObj := PlotThread.LabelList.Items[I];
              if TmpObj is TDrawFloodFill then
                begin
                  TmpObj.Free;
                  PlotThread.LabelList.Delete(I);
                end
              else
                inc(I);
            end; {while}
          RestartLabelListObjectChanged(lbHiddenLabelNone);
        end; {if}
end; {procedure TPlotImageBox.DeleteAllFloodFills;}

procedure TPlotImageBox.DeleteAllLabels;
var
  I : integer;
  TmpObj : TDrawBaseClass;
begin {procedure TPlotImageBox.DeleteAllLabels;}
  if PlotThread.LabelList.Count > 0 then
    if DoesListHaveLabel Then
      if mrYes = MessageDlgCenter('Delete all labels from this graph?',
                            mtConfirmation,[mbYes,mbCancel],0,nil) then
        begin
          SetDrawModeNormal(false);
          PlotThread.SetHasChangedSinceSaveTrue;
          I := 0;
          while I < PlotThread.LabelList.Count do
            begin
              TmpObj := PlotThread.LabelList.Items[I];
              if TmpObj is TDrawFloodFill then
                inc(I)
              else
                begin
                  TmpObj.Free;
                  PlotThread.LabelList.Delete(I);
                end; {else}
            end; {while}
          RestartLabelListObjectChanged(lbHiddenLabelNone);
        end; {if}
end; {procedure TPlotImageBox.DeleteAllLabels;}

procedure TPlotImageBox.DeleteAllLabelsAndFFills;
var
  I : integer;
begin {procedure TPlotImageBox.DeleteAllLabelsAndFFills;}
  if PlotThread.LabelList.Count > 0 then
    begin
      if mrYes = MessageDlgCenter('Delete all labels and flood fills from this graph?',
                            mtConfirmation,[mbYes,mbCancel],0,nil) then
        begin
          SetDrawModeNormal(false);
          PlotThread.SetHasChangedSinceSaveTrue;
          for I := 0 to (PlotThread.LabelList.Count-1) do
            TDrawBaseClass(PlotThread.LabelList.Items[I]).Free;
          PlotThread.LabelList.Clear;
          RestartLabelListObjectChanged(lbHiddenLabelNone);
        end; {if}
    end; {if}
end; {procedure TPlotImageBox.DeleteAllLabelsAndFFills;}

procedure TPlotImageBox.DeleteSelectLabel;
begin {procedure TPlotImageBox.DeleteSelectLabel;}
  if LabelWindowStatus = lwLabelSelected then
    begin
      LabelWindowStatus := lwNormal;
      CurrSelectPtr.Free;
      PlotThread.LabelList.delete(CurrSelectNdx);
      PlotThread.SetHasChangedSinceSaveTrue;
      RestartLabelListObjectChanged(lbHiddenLabelNone);
    end; {if}
end; {procedure TPlotImageBox.DeleteSelectLabel;}

destructor TPlotImageBox.Destroy;
begin {destructor TPlotImageBox.Destroy;}
  ScreenImage.free;
  {CustomTextEdit is automaticly destroyed by its parent}
  inherited;
end; {destructor TPlotImageBox.Destroy;}

procedure TPlotImageBox.DoDisplayStatusMsgEvent;
begin {procedure TPlotImageBox.DoDisplayStatusMsgEvent;}
  if Assigned(FOnDisplayStatusMsg) then
    FOnDisplayStatusMsg(Self);
end; {procedure TPlotImageBox.DoDisplayStatusMsgEvent;}

function TPlotImageBox.DoesListHaveFloodFill: boolean;
var
  I : integer;
begin {function TPlotImageBox.DoesListHaveFloodFill}
  Result := false;
  for I := 0 to (PlotThread.LabelList.Count-1) do
    begin
      if TObject(PlotThread.LabelList.Items[I]) is TDrawFloodFill then
        begin
          Result := true;
          exit;
        end; {if}
    end; {for}
end; {function TPlotImageBox.DoesListHaveFloodFill}

function TPlotImageBox.DoesListHaveLabel: boolean;
var
  I : integer;
begin {function TPlotImageBox.DoesListHaveLabel}
  Result := false;
  for I := 0 to (PlotThread.LabelList.Count-1) do
    begin
      if not(TObject(PlotThread.LabelList.Items[I]) is TDrawFloodFill) then
        begin
          Result := true;
          exit;
        end; {if}
    end; {for}
end; {function TPlotImageBox.DoesListHaveLabel}

function TPlotImageBox.DoGetDrawingToolEvent: TDrawingTool;
var
  TmpDrawingTool : TDrawingTool;
begin {function TPlotImageBox.DoGetDrawingToolEvent}
  BeginNoLabelFixupSection;
  try
    TmpDrawingTool := dtOff; //set to a default value
    if Assigned(FOnGetDrawingTool) then
      FOnGetDrawingTool(TmpDrawingTool);
    Result := TmpDrawingTool;
  finally
    EndNoLabelFixupSection;
  end; {try..finally}
end; {function TPlotImageBox.DoGetDrawingToolEvent}

function TPlotImageBox.DoGetResizeLabelsModeEvent: boolean;
begin {function TPlotImageBox.DoGetResizeLabelsModeEvent}
  BeginNoLabelFixupSection;
  try
    Result := false;
    if Assigned(FOnGetResizeLabelsMode) then
      FOnGetResizeLabelsMode(Result);
  finally
    EndNoLabelFixupSection;
  end; {try..finally}
end; {function TPlotImageBox.DoGetResizeLabelsModeEvent}

function TPlotImageBox.DoGetToolBarAttribEvent: TToolBarAttribClass;
begin {function TPlotImageBox.DoGetToolBarAttribEvent}
  BeginNoLabelFixupSection;
  try
    Result := nil;
    if Assigned(FOnGetToolBarAttrib) then
      FOnGetToolBarAttrib(Result);
  finally
    EndNoLabelFixupSection;
  end; {try..finally}
end; {function TPlotImageBox.DoGetToolBarAttribEvent}

procedure TPlotImageBox.DoneEditingEvent(Sender: TObject);
var
  NeedToRedraw : boolean;
begin {procedure TPlotImageBox.DoneEditingEvent}
  if LabelWindowStatus = lwEditingText then
    begin
      LabelWindowStatus := lwNormal;
      PlotThread.SetHasChangedSinceSaveTrue;
      NeedToRedraw :=  ((CurrSelectPtr as TDrawText).LabelTextStr <> '')or
        ((CurrSelectPtr as TDrawText).LabelTextStr <> CustomTextEdit.TextString);
      if CustomTextEdit.TextString <> '' then
        begin
          with (CurrSelectPtr as TDrawText) do
            begin
              DidFieldsChange := true;
              LabelTextStr := CustomTextEdit.TextString;
              DrawToPoint := CalcTextBottomRight(OriginPoint, LabelTextStr,
                  LabelTextFont, ScreenImage.Picture.bitmap, FBitmapAttribRec);
              if ForceTextToStayOnBmp(FBitmapAttribRec) then
                NeedToRedraw := true;
            end; {with}
        end
      else
        begin
          {delete label if empty string}
          CurrSelectPtr.Free;
          PlotThread.LabelList.delete(CurrSelectNdx);
        end; {else}
      if NeedToRedraw then
        RestartLabelListObjectChanged(lbHiddenLabelNone);
    end; {if}
end; {procedure TPlotImageBox.DoneEditingEvent}

procedure TPlotImageBox.SelectALabel(TheSelectNDX: integer);
var
  NeedToRedraw : boolean;
begin {procedure TPlotImageBox.SelectALabel}
  if not ((LabelWindowStatus = lwLabelSelected)and
          (CurrSelectNdx = TheSelectNDX)) then
    begin
      NeedToRedraw := SetDrawModeNormal(false);
      if (TheSelectNDX >= 0) and (TheSelectNDX < PlotThread.LabelList.Count) then
        begin
          LabelWindowStatus := lwLabelSelected;
          CurrSelectNdx := TheSelectNDX;
          CurrSelectPtr := PlotThread.LabelList.Items[TheSelectNDX];
          CurrSelectPtr.ToggleSelectMarks(ScreenImage.Picture.bitmap, FBitmapAttribRec);
        end; {if}
      if NeedToRedraw then
        RestartLabelListObjectChanged(lbHiddenLabelNone);
    end; {if}
end; {procedure TPlotImageBox.SelectALabel}

procedure TPlotImageBox.DoShowLabelPopupMenuEvent(ScreenX,
  ScreenY: integer; ShowInPlaceEditorItem: boolean);
begin {procedure TPlotImageBox.DoShowLabelPopupMenuEvent}
  BeginNoLabelFixupSection;
  try
    if Assigned(FOnShowLabelPopupMenu) then
      FOnShowLabelPopupMenu(ScreenX, ScreenY, ShowInPlaceEditorItem);
  finally
    EndNoLabelFixupSection;
  end; {try..finally}
end; {procedure TPlotImageBox.DoShowLabelPopupMenuEvent}

procedure TPlotImageBox.DoTurnDrawToolOffEvent;
begin {procedure TPlotImageBox.DoTurnDrawToolOffEvent;}
  BeginNoLabelFixupSection;
  try
    if Assigned(FOnTurnDrawToolOff) then
      FOnTurnDrawToolOff(Self);
  finally
    EndNoLabelFixupSection;
  end; {try..finally}
end; {procedure TPlotImageBox.DoTurnDrawToolOffEvent;}

procedure TPlotImageBox.DrawMouseDown(Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);

var
  TmpDrawingTool : TDrawingTool;

{------------------------}

function IsRoomForLabel : boolean;
begin {function IsRoomForLabel}
  Result := true;
  if PlotThread.LabelList.Count >= MaxLabelListObjs then
    begin
      Result := false;
      ErrorDlgCenterShowNoWait('This graph can not accept more than '+
                                IntToStr(MaxLabelListObjs)+' labels.', nil);
    end; {if}
end; {function IsRoomForLabel}

{------------------------}

procedure StartSelectingRegion;
begin {procedure StartSelectingRegion;}
  SetDrawModeNormal(true);
  DoTurnDrawToolOffEvent;
  ForcePointOnBmp(X, Y);
  RegionOrigin.X := X;
  RegionOrigin.Y := Y;
  RegionMovePt := RegionOrigin;
  LabelWindowStatus := lwDrawingRegion;
  DrawSelectRectangle(RegionOrigin, RegionMovePt);
end; {procedure StartSelectingRegion;}

{------------------------}

procedure StartSelectDragResize(ShowDragResizeCursor : boolean);
var
  LowestLabelDist, LowestLabelNdx, CurrDist, I : integer;
  CurrDrawObj : TDrawBaseClass;
  RedrawRequired : boolean;
begin {procedure StartSelectDragResize;}
  RedrawRequired := false;
  if not(LabelWindowStatus in [lwNormal, lwLabelSelected, lwFloodFillsVisible]) then
    begin
      if SetDrawModeNormal(false) then
        RedrawRequired := true;
    end; {if}
  {begin find LowestLabelDist}
  LowestLabelNdx := -1;
  LowestLabelDist := 32000;
  for I := 0 to (PlotThread.LabelList.Count-1) do
    begin
      CurrDrawObj := PlotThread.LabelList.Items[I];
      if (LabelWindowStatus = lwFloodFillsVisible) or
         (not (CurrDrawObj is TDrawFloodFill)) then
        begin
          CurrDist:= abs(CurrDrawObj.DistanceFromPix(X,Y));
          if CurrDist <= LowestLabelDist then
            begin
              LowestLabelNdx := I;
              LowestLabelDist := CurrDist;
            end; {if}
        end; {if}
    end; {for}
  {end find LowestLabelDist}
  {begin find distance from selected flood fill}
  if LabelWindowStatus = lwLabelSelected then
    if CurrSelectPtr is TDrawFloodFill then
      begin
        CurrDist := CurrSelectPtr.DistanceFromPix(X,Y);
        if CurrDist <= LowestLabelDist then
          begin
            LowestLabelNdx := CurrSelectNdx;
            LowestLabelDist := CurrDist;
          end; {if}
      end; {if}
  {end find distance from selected flood fill}
  if LowestLabelDist > MaxSelectDistance then
    begin
      if SetDrawModeNormal(false) then
        RedrawRequired := true;
    end
  else
    begin
      if not ((LabelWindowStatus = lwLabelSelected)and
              (CurrSelectNdx = LowestLabelNdx)) then
        begin
          if SetDrawModeNormal(false) then
            RedrawRequired := true;
          LabelWindowStatus := lwLabelSelected;
          CurrSelectNdx := LowestLabelNdx;
          CurrSelectPtr := PlotThread.LabelList.Items[LowestLabelNdx];
        end; {if}
      if IsDragLabelMode then
        CurrSelectPtr.MouseEvent(X, Y, deDownSelectDrag,
                                 ScreenImage.Picture.bitmap, FBitmapAttribRec)
      else
        CurrSelectPtr.MouseEvent(X, Y, deDownSelectResize,
                                 ScreenImage.Picture.bitmap, FBitmapAttribRec);
      if ShowDragResizeCursor then
        ShowNewCustomCursor(CurrSelectPtr.GetDragResizeCursor);
    end; {else}
  if RedrawRequired then
    RestartLabelListObjectChanged(lbHiddenLabelNone);
end; {procedure StartSelectDragResize;}

{------------------------}

procedure StartDrawFloodFill;
var
  TmpFFill : TDrawFloodFill;
  TmpToolBarAttribClass : TToolBarAttribClass;
  TmpTBBrushColor : TColor;
  RedrawRequired : boolean;
begin {procedure StartDrawFloodFill;}
  PlotThread.SetHasChangedSinceSaveTrue;
  RedrawRequired := SetDrawModeNormal(false);
  TmpToolBarAttribClass := DoGetToolBarAttribEvent;
  if Assigned(TmpToolBarAttribClass) then
    TmpTBBrushColor := TmpToolBarAttribClass.BrushColor
  else
    TmpTBBrushColor := Dflt_ToolBar_BrushColor;
  if IsRoomForLabel then
    begin
      ForcePointOnBmp(X, Y);
      if ScreenImage.Canvas.Pixels[X,Y] <> TmpTBBrushColor then
        begin
          try
            TmpFFill := TDrawFloodFill.Create;
          except
            on EOutOfMemory do
              begin
                MessageDlgErrorOutOfMemoryNoWait; {User Error Message}
                if RedrawRequired then
                  RestartLabelListObjectChanged(lbHiddenLabelNone);
                Exit;
              end; {on EOutOfMemory do}
          end; {try..except}
          if TmpFFill = nil then
            begin
              MessageDlgUnknownErrorNoWait; {User Error Message}
              if RedrawRequired then
                RestartLabelListObjectChanged(lbHiddenLabelNone);
              Exit;
            end; {if}
          TmpFFill.SetAttributes(TmpToolBarAttribClass);
          try
            PlotThread.LabelList.Add(TmpFFill);
          except
            on EOutOfMemory do
              begin
                TmpFFill.free;
                MessageDlgErrorOutOfMemoryNoWait; {User Error Message}
                if RedrawRequired then
                  RestartLabelListObjectChanged(lbHiddenLabelNone);
                Exit;
              end; {on EOutOfMemory do}
          end; {try..except}
          TmpFFill.MouseEvent(X, Y, deDownNew, ScreenImage.Picture.bitmap, FBitmapAttribRec);
          RedrawRequired := true;
        end; {if}
    end; {if}
  if RedrawRequired then
    RestartLabelListObjectChanged(lbHiddenLabelNone);
end; {procedure StartDrawFloodFill;}

{------------------------}

procedure StartDrawLabel;
var
  TmpDrawObj : TDrawBaseClass;
begin { procedure StartDrawLabel;}
  PlotThread.SetHasChangedSinceSaveTrue;
  SetDrawModeNormal(true);
  if IsRoomForLabel then
    begin
      try
        case TmpDrawingTool of
          dtLine:
            TmpDrawObj := TDrawLine.Create;
          dtRectangle:
            TmpDrawObj := TDrawInclosedShape.Create(dtRectangle);
          dtEllipse:
            TmpDrawObj := TDrawInclosedShape.Create(dtEllipse);
          dtRoundRect:
            TmpDrawObj := TDrawInclosedShape.Create(dtRoundRect);
          dtText:
            TmpDrawObj := TDrawText.Create;
          dtFreeHand:
            TmpDrawObj := TDrawFreeHand.Create;
          else
            exit; {an error occured}
        end; {case}
      except
        on EOutOfMemory do
          begin
            MessageDlgErrorOutOfMemoryNoWait; {User Error Message}
            Exit;
          end; {on EOutOfMemory do}
      end; {try..except}
      if TmpDrawObj = nil then
        begin
          MessageDlgUnknownErrorNoWait; {User Error Message}
          Exit;
        end; {if}
      TmpDrawObj.SetAttributes(DoGetToolBarAttribEvent);
      try
        PlotThread.LabelList.Add(TmpDrawObj);
      except
        on EOutOfMemory do
          begin
            TmpDrawObj.free;
            MessageDlgErrorOutOfMemoryNoWait; {User Error Message}
            Exit;
          end; {on EOutOfMemory do}
      end; {try..except}
      LabelWindowStatus := lwDrawingNewLabel;
      CurrSelectNdx := PlotThread.LabelList.Count - 1;
      CurrSelectPtr := TmpDrawObj;
      TmpDrawObj.MouseEvent(X, Y, deDownNew, ScreenImage.Picture.bitmap, FBitmapAttribRec);
    end; {if}
end; {procedure StartDrawLabel;}

{------------------------}

procedure StartAddNewTextLabel;
var
  TmpTextObj : TDrawText;
  TmpToolBarAttribClass : TToolBarAttribClass;
  TmpTBFont : TFont;
begin {procedure StartAddNewTextLabel;}
  PlotThread.SetHasChangedSinceSaveTrue;
  SetDrawModeNormal(true);
  TmpToolBarAttribClass := DoGetToolBarAttribEvent;
  if Assigned(TmpToolBarAttribClass) then
    TmpTBFont := TmpToolBarAttribClass.Font
  else
    TmpTBFont := nil;
  if IsRoomForLabel then
    begin
      try
        TmpTextObj := TDrawText.Create;
      except
        on EOutOfMemory do
          begin
            MessageDlgErrorOutOfMemoryNoWait; {User Error Message}
            Exit;
          end; {on EOutOfMemory do}
      end; {try..except}
      if TmpTextObj = nil then
        begin
          MessageDlgUnknownErrorNoWait; {User Error Message}
          Exit;
        end; {if}
      TmpTextObj.SetFontAttributes(Point(X, Y), '', TmpTBFont,
                                   ScreenImage.Picture.bitmap, FBitmapAttribRec);
      try
        PlotThread.LabelList.Add(TmpTextObj);
      except
        on EOutOfMemory do
          begin
            TmpTextObj.free;
            MessageDlgErrorOutOfMemoryNoWait; {User Error Message}
            Exit;
          end; {on EOutOfMemory do}
      end; {try..except}
      LabelWindowStatus := lwEditingText;
      CurrSelectNdx := PlotThread.LabelList.Count - 1;
      CurrSelectPtr := TmpTextObj;
      CustomTextEdit.ShowTextEdit(Point(X, Y),
          Point(ImageToScrollX(X), ImageToScrollY(Y)), TmpTBFont, '');
      ScrollInView(CustomTextEdit);
    end; {if}
end; {procedure StartAddNewTextLabel;}

{------------------------}

procedure StartInPlaceTextEditor;
begin {procedure StartInPlaceTextEditor;}
  if LabelWindowStatus = lwLabelSelected then
  if CurrSelectPtr is TDrawText then
  if (CurrSelectPtr as TDrawText).DistanceFromPix(X,Y) <= MaxSelectDistance then
    begin
      LabelWindowStatus := lwEditingText;
      PlotThread.SetHasChangedSinceSaveTrue;
      with TDrawText(CurrSelectPtr) do
        begin
          ToggleSelectMarks(ScreenImage.Picture.bitmap, FBitmapAttribRec);
          CustomTextEdit.ShowTextEdit(OriginPoint,
              Point(ImageToScrollX(OriginPoint.X),
                    ImageToScrollY(OriginPoint.Y)),
              LabelTextFont, LabelTextStr);
          ScrollInView(CustomTextEdit);
        end; {with}
    end; {if}
end; {procedure StartInPlaceTextEditor;}

{------------------------}

procedure ShowLabelPopupMenu;
var
  TmpScreenPoint : TPoint;
begin {procedure ShowLabelPopupMenu;}
  if LabelWindowStatus = lwLabelSelected then
    begin
      TmpScreenPoint := ScreenImage.ClientToScreen(Point(X, Y));
      DoShowLabelPopupMenuEvent(TmpScreenPoint.X, TmpScreenPoint.Y,
                                (CurrSelectPtr is TDrawText));
    end; {if}
end; {procedure ShowLabelPopupMenu;}

{------------------------}

begin {procedure TPlotImageBox.DrawMouseDown}
  TmpDrawingTool := DoGetDrawingToolEvent;
  {begin take focus from tool bar combo boxes}
  if LabelWindowStatus = lwEditingText then
    begin
      if CustomTextEdit.TextEditVisible then
        CustomTextEdit.SetFocus;
    end
  else
    begin
      if Screen.ActiveControl <> Self then
        Windows.SetFocus(Handle);
    end; {else}
  {end take focus from tool bar combo boxes}
  if Button = mbLeft then
    begin
      if IsMouseLeftDown then
        begin
          {an error occured}
          SetDrawModeNormal(true);
          ShowNormalCursor;
        end; {if}
      IsMouseLeftDown := true;
      if not IsPointOnBmp(X, Y) then
        SetDrawModeNormal(true) {do not process mouse down events off bmp}
      else
        begin
          if SaveTextLabel(true) then
            RestartLabelListObjectChanged(lbHiddenLabelNone);
          if (Shift = [ssShift,ssLeft])or (TmpDrawingTool = dtSelectRegion) then
            StartSelectingRegion
          else
            begin
              if TmpDrawingTool = dtOff then
                begin
                  if IsMouseDoubleClick then
                    StartInPlaceTextEditor
                  else
                    begin
                      IsDragLabelMode := not ((Shift = [ssCtrl,ssLeft])or
                                          (DoGetResizeLabelsModeEvent));
                      StartSelectDragResize(true);
                    end; {else}
                end
              else
                begin
                  if TmpDrawingTool = dtFloodFill then
                    StartDrawFloodFill
                  else
                    begin
                      if TmpDrawingTool = dtText then
                        StartAddNewTextLabel
                      else
                        StartDrawLabel;
                    end; {else}
                end; {else}
            end; {else}
        end; {else}
    end
  else
    if (Button = mbRight) and (not IsMouseLeftDown) and
       IsPointOnBmp(X, Y) and (TmpDrawingTool = dtOff) then
      begin
        {SetDrawModeNormal; Remarked out to fix a bug.}
        ShowNormalCursor;
        IsDragLabelMode := true;
        StartSelectDragResize(false);
        ShowLabelPopupMenu;
      end; {if}
  IsMouseDoubleClick := false;
end; {procedure TPlotImageBox.DrawMouseDown}

procedure TPlotImageBox.DrawMouseMove(Shift: TShiftState; X, Y: Integer);
begin {procedure TPlotImageBox.DrawMouseMove}
  if IsMouseLeftDown and ((LastMouseX <> X) or (LastMouseY <> Y)) then
    begin
      if not(ssLeft in Shift) then
        begin
          {an error has occured}
          SetDrawModeNormal(true);
          ShowNormalCursor;
          IsMouseLeftDown := false;
          Exit;
        end; {if}
      case LabelWindowStatus of
        {lwNormal: Ignore when in this status.}
        {lwEditingNewText: Ignore when in this status.}
        lwLabelSelected:
          begin
            {begin drag/Resize label}
            LabelWindowStatus := lwDragResizeLabel;
            if IsDragLabelMode then
              CurrSelectPtr.MouseEvent(X, Y, deMoveBeginDrag,
                ScreenImage.Picture.bitmap, FBitmapAttribRec)
            else
              CurrSelectPtr.MouseEvent(X, Y, deMoveBeginResize,
                ScreenImage.Picture.bitmap, FBitmapAttribRec);
            RestartLabelListObjectChanged(CurrSelectNdx);
          end;
        lwDrawingNewLabel:
          CurrSelectPtr.MouseEvent(X, Y, deMoveNew,
            ScreenImage.Picture.bitmap, FBitmapAttribRec);
        lwDragResizeLabel:
          begin
            if IsDragLabelMode then
              CurrSelectPtr.MouseEvent(X, Y, deMoveContinueDrag,
                ScreenImage.Picture.bitmap, FBitmapAttribRec)
            else
              CurrSelectPtr.MouseEvent(X, Y, deMoveContinueResize,
                ScreenImage.Picture.bitmap, FBitmapAttribRec);
          end;
        lwDrawingRegion:
          begin
            {erase at old position}
            DrawSelectRectangle(RegionOrigin, RegionMovePt);
            ForcePointOnBmp(X, Y);
            RegionMovePt.X := X;
            RegionMovePt.Y := Y;
            {draw at new postion}
            DrawSelectRectangle(RegionOrigin, RegionMovePt);
          end;
        lwFloodFillsVisible, lwRegionSelected:
          SetDrawModeNormal(true); {an error has occured if this code executes.}
      end; {case}
    end; {if}
end; {procedure TPlotImageBox.DrawMouseMove}

procedure TPlotImageBox.DrawMouseUp(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
begin {procedure TPlotImageBox.DrawMouseUp}
  if IsMouseLeftDown and (Button = mbLeft) then
    begin
      IsMouseLeftDown := false;
      ShowNormalCursor;
      case LabelWindowStatus of
        {lwNormal: lwLabelSelected: lwFloodFillSelected: Ignore these states.}
        {lwEditingNewText: Ignore when in this status.}
        lwDrawingNewLabel:
          begin
            LabelWindowStatus := lwNormal;
            CurrSelectPtr.MouseEvent(X, Y, deUpNew,
              ScreenImage.Picture.bitmap, FBitmapAttribRec);
            RestartLabelListObjectChanged(lbHiddenLabelNone);
          end;
        lwDragResizeLabel:
          begin
            LabelWindowStatus := lwLabelSelected;
            if IsDragLabelMode then
              CurrSelectPtr.MouseEvent(X, Y, deUpEndDrag,
                ScreenImage.Picture.bitmap, FBitmapAttribRec)
            else
              CurrSelectPtr.MouseEvent(X, Y, deUpEndResize,
                ScreenImage.Picture.bitmap, FBitmapAttribRec);
            RestartLabelListObjectChanged(lbHiddenLabelNone);
          end;
        lwDrawingRegion:
          begin
            LabelWindowStatus := lwRegionSelected;
            {erase at old position}
            DrawSelectRectangle(RegionOrigin, RegionMovePt);
            ForcePointOnBmp(X, Y);
            RegionMovePt.X := X;
            RegionMovePt.Y := Y;
            {draw at new postion}
            FindTopLeftPoint(RegionOrigin, RegionMovePt);
            DrawSelectRectangle(RegionOrigin, RegionMovePt);
          end;
        lwFloodFillsVisible, lwRegionSelected:
          SetDrawModeNormal(true); {an error has occured if this code executes.}
      end; {else}
    end; {if}
end; {procedure TPlotImageBox.DrawMouseUp}

procedure TPlotImageBox.DrawSelectRectangle(TopLeft, BottomRight: TPoint);
begin {procedure TPlotImageBox.DrawSelectRectangle}
  FindTopLeftPoint(TopLeft, BottomRight);
  with ScreenImage.Canvas do
    begin
      Pen.Mode := pmNotXor;
      Brush.Color := clwhite;
      Brush.Style := bsClear;
      Pen.Style := psDot;
      Pen.Color := clBlack;
      Pen.Width := 1;
      Rectangle(TopLeft.X, TopLeft.Y, BottomRight.X, BottomRight.Y);
    end; {with}
end; {procedure TPlotImageBox.DrawSelectRectangle}

procedure TPlotImageBox.EndNoLabelFixupSection;
begin {procedure TPlotImageBox.EndNoLabelFixupSection;}
  dec(NoLabelFixupSectionCount);
  if (NoLabelFixupSectionCount = 0) and RedrawInNoLabelFixupSection then
    begin
      RedrawInNoLabelFixupSection := false;
      LabelFixupAfterRedraw;
    end; {if}
end; {procedure TPlotImageBox.EndNoLabelFixupSection;}

procedure TPlotImageBox.ForceLabelWindowStatusNormal;
begin {procedure TPlotImageBox.ForceLabelWindowStatusNormal;}
  FLabelWindowStatus := lwLabelSelected; //set to tmp value for next line.
  SetLabelWindowStatus(lwNormal);        //force menus to update.
end; {procedure TPlotImageBox.ForceLabelWindowStatusNormal;}

procedure TPlotImageBox.ForcePointOnBmp(var TheX, TheY: Integer);
begin {procedure TPlotImageBox.ForcePointOnBmp}
  if TheX < 0 then
    TheX := 0
  else
    if TheX > (BmpWidth - 1) then
      TheX := (BmpWidth - 1);
  if TheY < 0 then
    TheY := 0
  else
    if TheY > (BmpHeight - 1) then
      TheY := (BmpHeight - 1);
end; {procedure TPlotImageBox.ForcePointOnBmp}

function TPlotImageBox.GetHasChangedSinceSave: boolean;
begin {function TPlotImageBox.GetHasChangedSinceSave}
  Result := PlotThread.HasChangedSinceSave;
end; {function TPlotImageBox.GetHasChangedSinceSave}

function TPlotImageBox.GetCaptionTitleStringWithBrackets: string;
begin {function TPlotImageBox.GetCaptionTitleStringWithBrackets}
  Result := PlotThread.CaptionTitleStringWithBrackets;
end; {function TPlotImageBox.GetCaptionTitleStringWithBrackets}

function TPlotImageBox.GetFileName: string;
begin {function TPlotImageBox.GetFileName}
  Result := PlotThread.FileName;
end; {function TPlotImageBox.GetFileName}

function TPlotImageBox.GetFileNameNoPath: string;
begin {function TPlotImageBox.GetFileNameNoPath}
  Result := PlotThread.FileNameNoPath;
end; {function TPlotImageBox.GetFileNameNoPath}

function TPlotImageBox.GetPlotTypeString: string;
begin {function TPlotImageBox.GetPlotTypeString}
  Result := PlotThread.PlotTypeString;
end; {function TPlotImageBox.GetPlotTypeString}

function TPlotImageBox.GetSettingBmpAttribsObj: TSettingBmpAttribsObj;
begin {function TPlotImageBox.GetSettingBmpAttribsObj}
  Result := PlotThread.SettingBmpAttribsObj;
end; {function TPlotImageBox.GetSettingBmpAttribsObj}

procedure TPlotImageBox.HandleDonePlotingEvent(PlotBitmap: TBitmap);
begin {procedure TPlotImageBox.HandleDonePlotingEvent}
  DoCopyBitmapToImageBuffers(ScreenImage, PlotBitmap);
  {begin fix up labels that are selected or being drawn}
  if NoLabelFixupSectionCount = 0 then //if not in NoLabelFixupSection then
    LabelFixupAfterRedraw;
  RedrawInNoLabelFixupSection := true;
  {end fix up labels that are selected or being drawn}
  if Assigned(FOnDoneDrawing) then
    FOnDoneDrawing(Self);
end; {procedure TPlotImageBox.HandleDonePlotingEvent}

procedure TPlotImageBox.HandleRunningEndEvent(Sender: TObject);
begin {TPlotImageBox.HandleRunningEndEvent}
  if Assigned(FOnRunningEnd) then
    FOnRunningEnd(Self);
end; {TPlotImageBox.HandleRunningEndEvent}

procedure TPlotImageBox.HandleRunningStartEvent(Sender: TObject);
begin {TPlotImageBox.HandleRunningStartEvent}
  if Assigned(FOnRunningStart) then
    FOnRunningStart(Self);
end; {TPlotImageBox.HandleRunningStartEvent}

procedure TPlotImageBox.HandleThreadPlotErrorEvent(Sender: TObject;
  const ImagePlotErrorString: string);
begin {procedure TPlotImageBox.HandleThreadPlotErrorEvent}
  ErrorDlgCenterShowNoWait(ImagePlotErrorString, nil);
end; {procedure TPlotImageBox.HandleThreadPlotErrorEvent}

function TPlotImageBox.ImageToScrollX(X: integer): integer;
begin {function TPlotImageBox.ImageToScrollX}
  Result := X - HorzScrollBar.Position;
end; {function TPlotImageBox.ImageToScrollX}

function TPlotImageBox.ImageToScrollY(Y: integer): integer;
begin {function TPlotImageBox.ImageToScrollY}
  Result := Y - VertScrollBar.Position;
end; {function TPlotImageBox.ImageToScrollY}

function TPlotImageBox.InitalizeFromFile(TheColorDepth: TPixelFormat;
  const TheFileName : string; ReadGraphFile: TReadGraphFile): boolean;
begin {function TPlotImageBox.InitalizeFromFile}
  Assert((TheColorDepth in SupportedBmpColorDepths), 'Invalid picture color depth');
  Result := PlotThread.InitalizeFromFile(TheColorDepth, TheFileName, ReadGraphFile);
  if Result then
    begin
      ScreenImage.Picture.Bitmap.PixelFormat := PlotThread.SettingBmpAttribsObj.BmpColorDepth;
      ScreenImage.Picture.Bitmap.Width := PlotThread.SettingBmpAttribsObj.BmpWidth;
      ScreenImage.Picture.Bitmap.Height := PlotThread.SettingBmpAttribsObj.BmpHeight;
      FBitmapAttribRec.BackgroundColor := PlotThread.SettingBmpAttribsObj.BackgroundColor;
      FBitmapAttribRec.BmpHeight := PlotThread.SettingBmpAttribsObj.BmpHeight;
      FBitmapAttribRec.BmpWidth := PlotThread.SettingBmpAttribsObj.BmpWidth;
      PlotThread.AssignAllObjectsAfterInitFromFile;
      ForceLabelWindowStatusNormal;
    end; {if}
end; {function TPlotImageBox.InitalizeFromFile}

procedure TPlotImageBox.InitalizeNew(TheColorDepth: TPixelFormat;
  TheDefaultBmpWidth, TheDefaultBmpHeight: integer);
begin {procedure TPlotImageBox.InitalizeNew}
  Assert((TheColorDepth in SupportedBmpColorDepths), 'Invalid picture color depth');
  Assert((TheDefaultBmpWidth <= MaxBmpWidth), 'Bitmap width to large');
  Assert((TheDefaultBmpHeight <= MaxBmpHeight), 'Bitmap height to large');
  Assert((TheDefaultBmpWidth >= MinBmpWidth), 'Bitmap width to small');
  Assert((TheDefaultBmpHeight >= MinBmpHeight), 'Bitmap height to small');
  ScreenImage.Picture.Bitmap.PixelFormat := TheColorDepth;
  ScreenImage.Picture.Bitmap.Width := TheDefaultBmpWidth;
  ScreenImage.Picture.Bitmap.Height := TheDefaultBmpHeight;
  FBitmapAttribRec.BackgroundColor := clWhite;
  FBitmapAttribRec.BmpHeight := TheDefaultBmpHeight;
  FBitmapAttribRec.BmpWidth := TheDefaultBmpWidth;
  PlotThread.InitalizeNew(TheColorDepth, TheDefaultBmpWidth, TheDefaultBmpHeight);
  FBitmapAttribRec.BackgroundColor := PlotThread.SettingBmpAttribsObj.BackgroundColor;
  ForceLabelWindowStatusNormal;
end; {procedure TPlotImageBox.InitalizeNew}

procedure TPlotImageBox.InPlaceEditTextLabel;
begin {procedure TPlotImageBox.InPlaceEditTextLabel;}
  if LabelWindowStatus = lwLabelSelected then
  if CurrSelectPtr is TDrawText then
    begin
      LabelWindowStatus := lwEditingText;
      PlotThread.SetHasChangedSinceSaveTrue;
      with TDrawText(CurrSelectPtr) do
        begin
          ToggleSelectMarks(ScreenImage.Picture.bitmap, FBitmapAttribRec);
          CustomTextEdit.ShowTextEdit(OriginPoint,
              Point(ImageToScrollX(OriginPoint.X),
                    ImageToScrollY(OriginPoint.Y)),
              LabelTextFont, LabelTextStr);
          ScrollInView(CustomTextEdit);
        end; {with}
    end; {if}
end; {procedure TPlotImageBox.InPlaceEditTextLabel;}

function TPlotImageBox.IsPointOnBmp(TheX, TheY: Integer): boolean;
begin {function TPlotImageBox.IsPointOnBmp}
  Result := (TheX >= 0) and (TheX < BmpWidth) and
            (TheY >= 0) and (TheY < BmpHeight);
end; {function TPlotImageBox.IsPointOnBmp}

function TPlotImageBox.IsRegionSelected: boolean;
begin {function TPlotImageBox.IsRegionSelected}
  Result := LabelWindowStatus = lwRegionSelected;
end; {function TPlotImageBox.IsRegionSelected}

procedure TPlotImageBox.LabelFixupAfterRedraw;
{---------------------}
procedure NestedDrawFloodFillLocations;
var
  I : integer;
  TmpObj : TDrawBaseClass;
begin {procedure NestedDrawFloodFillLocations;}
  if PlotThread.LabelList.Count > 0 then
    begin
      for I := 0 to (PlotThread.LabelList.Count-1) do
        begin
          TmpObj := PlotThread.LabelList.Items[I];
          if TmpObj is TDrawFloodFill then
            begin
              TDrawFloodFill(TmpObj).ToggleLocationMark(
                  ScreenImage.Picture.bitmap, FBitmapAttribRec);
            end; {if}
        end; {for}
    end; {if}
end; {procedure NestedDrawFloodFillLocations;}
{---------------------}
begin {procedure TPlotImageBox.LabelFixupAfterRedraw;}
  {fix up labels that are selected or being drawn}
  case LabelWindowStatus of
    {lwNormal, lwEditingText: do nothing}
    lwDrawingNewLabel, lwDragResizeLabel:
      CurrSelectPtr.RedrawMoveResizeMode(
          ScreenImage.Picture.bitmap, FBitmapAttribRec);
    lwLabelSelected:
      CurrSelectPtr.RedrawSelectMarks(
          ScreenImage.Picture.bitmap, FBitmapAttribRec);
    lwFloodFillsVisible:
      NestedDrawFloodFillLocations;
    lwDrawingRegion, lwRegionSelected:
      DrawSelectRectangle(RegionOrigin, RegionMovePt);
  end; {case}
end; {procedure TPlotImageBox.LabelFixupAfterRedraw;}

procedure TPlotImageBox.ModifyLabelAttributes;
var
  TmpSelectPtr : TDrawBaseClass;
  TmpSelectNdx : integer;
  NeedToRedraw : boolean;
begin {procedure TPlotImageBox.ModifyLabelAttributes;}
  if LabelWindowStatus = lwLabelSelected then
    begin
      TmpSelectPtr := CurrSelectPtr;
      TmpSelectNdx := CurrSelectNdx;
      SetDrawModeNormal(true);
      if ModifyLabelObj(CurrSelectPtr, FBitmapAttribRec, ScreenImage.Picture.bitmap, nil) then
        begin
          PlotThread.SetHasChangedSinceSaveTrue;
          if CurrSelectPtr is TDrawText then
            if TDrawText(CurrSelectPtr).LabelTextStr = '' then
              begin
                DeleteSelectLabel; //this calls RestartLabelListObjectChanged;
                exit;
              end; {if}
          LabelWindowStatus := lwLabelSelected;
          CurrSelectPtr := TmpSelectPtr;
          CurrSelectNdx := TmpSelectNdx;
          RestartLabelListObjectChanged(lbHiddenLabelNone);
        end
      else
        begin
          {nothing changed so reselect the label}
          SelectALabel(TmpSelectNdx);
        end; {else}
    end
  else
    if LabelWindowStatus = lwEditingText then
      begin
        PlotThread.SetHasChangedSinceSaveTrue;
        TmpSelectPtr := CurrSelectPtr;
        TmpSelectNdx := CurrSelectNdx;
        if SaveTextLabel(false) then // sets LabelWindowStatus := lwNormal;
          RestartLabelListObjectChanged(lbHiddenLabelNone);
        NeedToRedraw := false;
        if ModifyLabelObj(TmpSelectPtr, FBitmapAttribRec, ScreenImage.Picture.bitmap, nil) then
          NeedToRedraw := true;
        if (TmpSelectPtr as TDrawText).LabelTextStr = '' then
          begin
            LabelWindowStatus := lwNormal;
            TmpSelectPtr.Free;
            PlotThread.LabelList.delete(TmpSelectNdx);
          end
        else
          begin
            LabelWindowStatus := lwLabelSelected;
            NeedToRedraw := true;
            CurrSelectPtr := TmpSelectPtr;
            CurrSelectNdx := TmpSelectNdx;
          end; {else}
        if NeedToRedraw then
          RestartLabelListObjectChanged(lbHiddenLabelNone);
      end; {else}
end; {procedure TPlotImageBox.ModifyLabelAttributes;}

procedure TPlotImageBox.MoveSelectedToBack;
begin {procedure TPlotImageBox.MoveSelectedToBack;}
  if LabelWindowStatus = lwLabelSelected then
    begin
      {move label to end of list if not already there}
      if CurrSelectNdx <> 0 then
        begin
          PlotThread.LabelList.Move(CurrSelectNdx, 0);
          CurrSelectNdx := 0;
          RestartLabelListObjectChanged(lbHiddenLabelNone);
        end; {if}
    end; {if}
end; {procedure TPlotImageBox.MoveSelectedToBack;}

procedure TPlotImageBox.MoveSelectedToFront;
begin {procedure TPlotImageBox.MoveSelectedToFront;}
  if LabelWindowStatus = lwLabelSelected then
    begin
      {move label to end of list if not already there}
      if CurrSelectNdx <> (PlotThread.LabelList.Count-1) then
        begin
          PlotThread.LabelList.Move(CurrSelectNdx, (PlotThread.LabelList.Count-1));
          CurrSelectNdx := (PlotThread.LabelList.Count-1);
          RestartLabelListObjectChanged(lbHiddenLabelNone);
        end; {if}
    end; {if}
end; {procedure TPlotImageBox.MoveSelectedToFront;}

procedure TPlotImageBox.PrintTheBMP(PrintSelected: boolean; PrintScalePercent : integer);
var
  pH, pW : integer;
  TmpBitMap : TBitMap;
begin {procedure TPlotImageBox.PrintTheBMP}
  PrintSelected := (IsRegionSelected and PrintSelected);
  {Deal with all 4 posibilities seprately}
  if (PrintScalePercent = 100) and (PrintSelected) then
    begin
      SetDrawModeNormal(true);
      Printer.BeginDoc;
      Printer.Canvas.CopyRect(
         Rect(0, 0, abs(RegionOrigin.X-RegionMovePt.X),
                    abs(RegionOrigin.Y-RegionMovePt.Y)),
         ScreenImage.Canvas,
         Rect(RegionOrigin.X, RegionOrigin.Y,
              RegionMovePt.X, RegionMovePt.Y));
      Printer.EndDoc;
    end
  else
  if (PrintScalePercent = 100) and (not PrintSelected) then
    begin
      SetDrawModeNormal(true);
      Printer.BeginDoc;
      Printer.Canvas.Draw(0,0,ScreenImage.Picture.Graphic);
      Printer.EndDoc;
    end
  else
  if (not(PrintScalePercent = 100)) and (PrintSelected) then
    begin
      SetDrawModeNormal(true);
      TmpBitMap := TBitMap.Create;
      try
        if TmpBitMap = nil then
          begin
            MessageDlgErrorOutOfMemoryNoWait; {User Error Message}
            Exit;
          end; {if}
        try
          TmpBitMap.HandleType := bmDIB;
          TmpBitMap.PixelFormat := ScreenImage.Picture.bitmap.PixelFormat;
          TmpBitMap.Width := abs(RegionOrigin.X-RegionMovePt.X)+1;
          TmpBitMap.Height := abs(RegionOrigin.Y-RegionMovePt.Y)+1;
        except
          on EOutOfMemory do
            begin
              TmpBitMap.free;
              MessageDlgErrorOutOfMemoryNoWait; {User Error Message}
              Exit;
            end; {on EOutOfMemory do}
        end; {try..except}
        TmpBitMap.Canvas.CopyRect(
                  Rect(0, 0, abs(RegionOrigin.X-RegionMovePt.X),
                             abs(RegionOrigin.Y-RegionMovePt.Y)),
                  ScreenImage.Canvas,
                  Rect(RegionOrigin.X, RegionOrigin.Y,
                       RegionMovePt.X, RegionMovePt.Y));
        pH := Round((TmpBitMap.Height - 1) * (PrintScalePercent/100));
        pW := Round((TmpBitMap.Width - 1) * (PrintScalePercent/100));
        Printer.BeginDoc;
        Printer.Canvas.StretchDraw(Rect(0,0,pW,pH),TmpBitMap);
        Printer.EndDoc;
      finally
        TmpBitMap.free;
      end; {try..finally}
    end
  else
  {if (not(CurrPrintScalePercent = 100)) and (not PrintSelected) then}
    begin
      SetDrawModeNormal(true);
      pH := Round((BmpHeight - 1) * (PrintScalePercent/100));
      pW := Round((BmpWidth - 1) * (PrintScalePercent/100));
      Printer.BeginDoc;
      Printer.Canvas.StretchDraw(Rect(0,0,pW,pH),ScreenImage.Picture.Graphic);
      Printer.EndDoc;
    end; {if}
end; {procedure TPlotImageBox.PrintTheBMP}

procedure TPlotImageBox.RestartFieldObjectChanged;
var
  DidBmpShrink : boolean;
  I : integer;
  TmpLabel : TDrawBaseClass;
begin {procedure TPlotImageBox.RestartFieldObjectChanged;}
  PlotThread.SetHasChangedSinceSaveTrue;
  //stop the appropriate thread
  if PlotThread.SettingBmpAttribsObj.DidBackGroundColorChange or
      PlotThread.SettingBmpAttribsObj.DidColorDepthChange or
      PlotThread.SettingBmpAttribsObj.DidWidthHeightChange then
    PlotThread.WaitForTermAllThreads
  else
    PlotThread.WaitForTermGraphThread;
  //assign the fields
  if PlotThread.SettingBmpAttribsObj.DidColorDepthChange then
    ScreenImage.Picture.Bitmap.PixelFormat := PlotThread.SettingBmpAttribsObj.BmpColorDepth;
  if PlotThread.SettingBmpAttribsObj.DidBackGroundColorChange then
    FBitmapAttribRec.BackgroundColor := PlotThread.SettingBmpAttribsObj.BackgroundColor;
  if PlotThread.SettingBmpAttribsObj.DidWidthHeightChange then
    begin
      DidBmpShrink := ((FBitmapAttribRec.BmpWidth > PlotThread.SettingBmpAttribsObj.BmpWidth) or
                       (FBitmapAttribRec.BmpHeight > PlotThread.SettingBmpAttribsObj.BmpHeight));
      //resize bmp
      ScreenImage.Picture.Bitmap.Width := PlotThread.SettingBmpAttribsObj.BmpWidth;
      ScreenImage.Picture.Bitmap.Height := PlotThread.SettingBmpAttribsObj.BmpHeight;
      FBitmapAttribRec.BmpWidth := PlotThread.SettingBmpAttribsObj.BmpWidth;
      FBitmapAttribRec.BmpHeight := PlotThread.SettingBmpAttribsObj.BmpHeight;
      //center bmp
      CenterTheBitmap;
      //delete labels that are off the bmp
      if DidBmpShrink then
        begin
          SetDrawModeNormal(false);
          {Delete Labels and Flood Fills that are not on the new BitMap}
          I := 0;
          while I < PlotThread.LabelList.Count do
            begin
              TmpLabel := PlotThread.LabelList.Items[I];
              if TmpLabel.IsLabelOnBmp(FBitmapAttribRec) then
                inc(I)
              else
                begin
                  TmpLabel.Free;
                  PlotThread.LabelList.Delete(I);
                end; {else}
            end; {while}
          PlotThread.AssignLabelObjects(lbHiddenLabelNone);
        end; {if}
    end; {if}
  PlotThread.AssignFieldObjects;
  PlotThread.ReStartGraphThread;
end; {procedure TPlotImageBox.RestartFieldObjectChanged;}

procedure TPlotImageBox.RestartLabelListObjectChanged(HiddenLabelIndex : integer);
begin {procedure TPlotImageBox.RestartLabelListObjectChanged}
  PlotThread.WaitForTermLabelThread;
  PlotThread.AssignLabelObjects(HiddenLabelIndex);
  PlotThread.ReStartLabelThread;
end; {procedure TPlotImageBox.RestartLabelListObjectChanged}

procedure TPlotImageBox.ReStartGraphThread;
begin {procedure TPlotImageBox.ReStartGraphThread;}
  PlotThread.ReStartGraphThread;
end; {procedure TPlotImageBox.ReStartGraphThread;}

procedure TPlotImageBox.SaveAsPictureFile(const TheSaveFileStr: string;
  IsBmpNotJpeg: boolean);
{---------------------------------}
procedure DoSaveAsJpegFile(TheBitmap : TBitmap);
var
  jp: TJpegImage;
begin {procedure DoSaveAsJpegFile}
  jp := TJpegImage.Create;
  try
    jp.CompressionQuality := 100; //best possible quality.
    jp.GrayScale := False; //use color.
    jp.Performance := jpBestQuality; //may not be used but set just incase.
    jp.PixelFormat := jf24Bit; //use native jpg pixel format.
    jp.ProgressiveEncoding := True; //allways save with ProgressiveEncoding.
    jp.ProgressiveDisplay := False; //may not be used but set just incase.
    jp.Scale := jsFullSize; //may not be used but set just incase.
    jp.Assign(TheBitmap);
    jp.SaveToFile(TheSaveFileStr);
  finally
    jp.Free;
  end;
end; {procedure DoSaveAsJpegFile}
{---------------------------------}
var
  UseSelected : boolean;
  TmpBmp : TBitMap;
  TmpWidth, TmpHeight : integer;
begin {procedure TPlotImageBox.SaveAsPictureFile}
  if IsRegionSelected then
    begin
      if PromptUseSelected(false, UseSelected) then
        begin
          SetDrawModeNormal(true);
          if UseSelected then
            begin
              TmpWidth := abs(RegionOrigin.X-RegionMovePt.X) + 1;
              TmpHeight := abs(RegionOrigin.Y-RegionMovePt.Y) + 1;
              TmpBmp := nil;
              try
                TmpBmp := TBitMap.Create;
                TmpBmp.HandleType := bmDIB;
                TmpBmp.PixelFormat := ScreenImage.Picture.bitmap.PixelFormat;
                TmpBmp.Width := TmpWidth;
                TmpBmp.Height := TmpHeight;
                TmpBmp.Canvas.CopyRect(Rect(0, 0, TmpWidth-1, TmpHeight-1),
                                  ScreenImage.Canvas,
                                  Rect(RegionOrigin.X, RegionOrigin.Y,
                                       RegionMovePt.X, RegionMovePt.Y));
                if IsBmpNotJpeg then
                  TmpBmp.SaveToFile(TheSaveFileStr)
                else
                  DoSaveAsJpegFile(TmpBmp);
              finally
                TmpBmp.free;
              end; {try..Finally}
            end
          else
            begin
              if IsBmpNotJpeg then
                ScreenImage.Picture.SaveToFile(TheSaveFileStr)
              else
                DoSaveAsJpegFile(ScreenImage.Picture.Bitmap);
            end; {else}
        end; {if}
    end
  else
    begin
      SetDrawModeNormal(true);
      if IsBmpNotJpeg then
        ScreenImage.Picture.SaveToFile(TheSaveFileStr)
      else
        DoSaveAsJpegFile(ScreenImage.Picture.Bitmap);
    end; {else}
end; {procedure TPlotImageBox.SaveAsPictureFile}

function TPlotImageBox.SaveFile : boolean;
begin {function TPlotImageBox.SaveFile}
  SetDrawModeNormal(true);
  BeginNoLabelFixupSection;
  try
    Result := PlotThread.SaveFile;
  finally
    EndNoLabelFixupSection;
  end; {try..finally}
end; {function TPlotImageBox.SaveFile}

function TPlotImageBox.SaveFileAs(const TheFileName: string) : boolean;
begin {function TPlotImageBox.SaveFileAs}
  SetDrawModeNormal(true);
  BeginNoLabelFixupSection;
  try
    Result := PlotThread.SaveFileAs(TheFileName);
  finally
    EndNoLabelFixupSection;
  end; {try..finally}
end; {function TPlotImageBox.SaveFileAs}

function TPlotImageBox.SaveOnExitPrompt: boolean;
{Returns True if the graph is required to close}
var
  msgAnswer : integer;
begin {function TPlotImageBox.SaveOnExitPrompt}
  SetDrawModeNormal(true); {force clean up}
  if PlotThread.FileName = '' then
    Result := true
  else
    if (not PlotThread.HasChangedSinceSave) then
      Result := true
    else
      begin
        msgAnswer := MessageDlgCenter('Save changes to graph: ' +
            PlotThread.FileNameNoPath + ' ?', mtConfirmation, mbYesNoCancel, 0, nil);
        if (msgAnswer = mrNo) then
          Result := true
        else
          begin
            if (msgAnswer = mrCancel) then
              Result := false
            else {if (msgAnswer = mrYes or mrOk) then}
              begin
                BeginNoLabelFixupSection;
                try
                  Result := PlotThread.SaveFile;
                finally
                  EndNoLabelFixupSection;
                end; {try..finally}
              end; {else}
          end; {else}
      end; {else}
end; {function TPlotImageBox.SaveOnExitPrompt}

function TPlotImageBox.SaveTextLabel(DeleteWhenEmpty: boolean): boolean;
{Returns true if label changed and graph needs to be redrawn}
begin {function TPlotImageBox.SaveTextLabel}
  Result := false;
  if LabelWindowStatus = lwEditingText then
    begin
      LabelWindowStatus := lwNormal;
      PlotThread.SetHasChangedSinceSaveTrue;
      Result :=  ((CurrSelectPtr as TDrawText).LabelTextStr <> '')or
                  ((CurrSelectPtr as TDrawText).LabelTextStr <>
                   CustomTextEdit.TextString);
      if CustomTextEdit.TextString <> '' then
        begin
          with (CurrSelectPtr as TDrawText) do
            begin
              DidFieldsChange := true;
              LabelTextStr := CustomTextEdit.TextString;
              DrawToPoint := CalcTextBottomRight(OriginPoint, LabelTextStr,
                   LabelTextFont, ScreenImage.Picture.bitmap, FBitmapAttribRec);
              if ForceTextToStayOnBmp(FBitmapAttribRec) then
                Result := true;
            end; {with}
        end
      else
        begin
          if DeleteWhenEmpty then
            begin
              {delete label if empty string}
              CurrSelectPtr.Free;
              PlotThread.LabelList.delete(CurrSelectNdx);
            end; {if}
        end; {else}
      CustomTextEdit.HideTextEdit;
    end; {if}
end; {function TPlotImageBox.SaveTextLabel}

procedure TPlotImageBox.ScreenImageDblClick(Sender: TObject);
begin {procedure TPlotImageBox.ScreenImageDblClick}
  IsMouseDoubleClick := true;
end; {procedure TPlotImageBox.ScreenImageDblClick}

procedure TPlotImageBox.ScreenImageMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin {procedure TPlotImageBox.ScreenImageMouseDown}
  DrawMouseDown(Button, Shift, X, Y);
  FLastMouseX := X;
  FLastMouseY := Y;
end; {procedure TPlotImageBox.ScreenImageMouseDown}

procedure TPlotImageBox.ScreenImageMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
begin {procedure TPlotImageBox.ScreenImageMouseMove}
  DrawMouseMove(Shift, X, Y);
  FLastMouseX := X;
  FLastMouseY := Y;
  DoDisplayStatusMsgEvent;
end; {procedure TPlotImageBox.ScreenImageMouseMove}

procedure TPlotImageBox.ScreenImageMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin {procedure TPlotImageBox.ScreenImageMouseUp}
  DrawMouseUp(Button, Shift, X, Y);
  FLastMouseX := X;
  FLastMouseY := Y;
end; {procedure TPlotImageBox.ScreenImageMouseUp}

procedure TPlotImageBox.SelectNextLabelOrFFill;
begin {procedure TPlotImageBox.SelectNextLabelOrFFill;}
  if LabelWindowStatus = lwLabelSelected then
    begin
      if CurrSelectNdx < (PlotThread.LabelList.Count-1) then
        SelectALabel(CurrSelectNdx + 1)
      else
        SelectALabel(0);
    end; {if}
end; {procedure TPlotImageBox.SelectNextLabelOrFFill;}

procedure TPlotImageBox.SelectPrevLabelOrFFill;
begin {procedure TPlotImageBox.SelectPrevLabelOrFFill;}
  if LabelWindowStatus = lwLabelSelected then
    begin
      if CurrSelectNdx > 0 then
        SelectALabel(CurrSelectNdx - 1)
      else
        SelectALabel(PlotThread.LabelList.Count - 1);
    end; {if}
end; {procedure TPlotImageBox.SelectPrevLabelOrFFill;}

procedure TPlotImageBox.SelectTopLabel;
var
  I : integer;
begin {procedure TPlotImageBox.SelectTopLabel;}
  I := PlotThread.LabelList.Count;
  if I >= 1 then
    SelectALabel(I-1) //calls SetDrawModeNormal(true) before selecting the label and redrawing
  else
    SetDrawModeNormal(true);
end; {procedure TPlotImageBox.SelectTopLabel;}

function TPlotImageBox.SetDrawModeNormal(ShouldRestartThread : boolean) : boolean;
{Returns True if the calling method is required to call RestartLabelListObjectChanged.}
{Will only return True if ShouldRestartThread = False.}
var
  I : Integer;
  TmpObj : TDrawBaseClass;
begin {procedure TPlotImageBox.SetDrawModeNormal}
  Result := false;
  if LabelWindowStatus <> lwNormal then
    begin
      case LabelWindowStatus of
        {lwNormal: do nothing.}
        lwEditingText:
          begin
            if SaveTextLabel(true) then //sets this, LabelWindowStatus := lwNormal;
              begin
                if ShouldRestartThread then
                  RestartLabelListObjectChanged(lbHiddenLabelNone)
                else
                  Result := true;
              end; {if}
          end;
        lwDrawingNewLabel:
          begin
            {Error may have occured}
            LabelWindowStatus := lwNormal;
            PlotThread.SetHasChangedSinceSaveTrue;
            CurrSelectPtr.ErrorSetToNormalMode;
            if CurrSelectPtr is TDrawLabel then
              CurrSelectPtr.MouseEvent(TDrawLabel(CurrSelectPtr).DrawToPoint.X,
                                       TDrawLabel(CurrSelectPtr).DrawToPoint.Y,
                                       deUpNew, ScreenImage.Picture.bitmap,
                                       FBitmapAttribRec);
            if ShouldRestartThread then
              RestartLabelListObjectChanged(lbHiddenLabelNone)
            else
              Result := true;
          end;
        lwDragResizeLabel:
          begin
            {Error may have occured}
            LabelWindowStatus := lwNormal;
            PlotThread.SetHasChangedSinceSaveTrue;
            CurrSelectPtr.ErrorSetToNormalMode;
            if ShouldRestartThread then
              RestartLabelListObjectChanged(lbHiddenLabelNone)
            else
              Result := true;
          end;
        lwLabelSelected:
          begin
            LabelWindowStatus := lwNormal;
            CurrSelectPtr.ToggleSelectMarks(ScreenImage.Picture.bitmap, FBitmapAttribRec);
          end;
        lwFloodFillsVisible:
          begin
            LabelWindowStatus := lwNormal;
            for I := 0 to (PlotThread.LabelList.Count-1) do
              begin
                TmpObj := PlotThread.LabelList.Items[I];
                if TmpObj is TDrawFloodFill then
                  TDrawFloodFill(TmpObj).ToggleLocationMark(
                      ScreenImage.Picture.bitmap, FBitmapAttribRec);
              end; {for}
          end;
        lwDrawingRegion, lwRegionSelected:
          begin
            {erase selected region rectange}
            LabelWindowStatus := lwNormal;
            DrawSelectRectangle(RegionOrigin, RegionMovePt);
          end;
      end; {case}
      ShowNormalCursor;
      LabelWindowStatus := lwNormal;
    end; {if}
end; {procedure TPlotImageBox.SetDrawModeNormal}

procedure TPlotImageBox.SetLabelWindowStatus(
  NewLWStatus: TLabelWindowStatus);
begin {procedure TPlotImageBox.SetLabelWindowStatus}
  BeginNoLabelFixupSection;
  try
    if FLabelWindowStatus <> NewLWStatus then
      begin
        FLabelWindowStatus := NewLWStatus;
        if Assigned(FOnLabelWinStatusChange) then
          FOnLabelWinStatusChange(Self, NewLWStatus);
      end; {if}
  finally
    EndNoLabelFixupSection;
  end; {try..finally}
end; {procedure TPlotImageBox.SetLabelWindowStatus}

procedure TPlotImageBox.SetLastMouseToOffBmp;
begin {procedure TPlotImageBox.SetLastMouseToOffBmp;}
  FLastMouseX := -1;
  FLastMouseY := -1;
end; {procedure TPlotImageBox.SetLastMouseToOffBmp;}

procedure TPlotImageBox.ShowFloodFillLocations;
var
  DidFindFloodFill : boolean;
  I : integer;
  TmpObj : TDrawBaseClass;
begin {procedure TPlotImageBox.ShowFloodFillLocations;}
  SetDrawModeNormal(true);
  if PlotThread.LabelList.Count > 0 then
    begin
      DidFindFloodFill := false;
      for I := 0 to (PlotThread.LabelList.Count-1) do
        begin
          TmpObj := PlotThread.LabelList.Items[I];
          if TmpObj is TDrawFloodFill then
            begin
              TDrawFloodFill(TmpObj).ToggleLocationMark(
                  ScreenImage.Picture.bitmap, FBitmapAttribRec);
              DidFindFloodFill := true;
            end; {if}
        end; {for}
      if DidFindFloodFill then
        LabelWindowStatus := lwFloodFillsVisible;
    end; {if}
end; {procedure TPlotImageBox.ShowFloodFillLocations;}

procedure TPlotImageBox.ShowLabelListDlg;
var
  TmpIsLabelSelected : boolean;
  TmpCurrSelectNdx : integer;
begin {procedure TPlotImageBox.ShowLabelListDlg;}
  TmpIsLabelSelected := (LabelWindowStatus = lwLabelSelected);
  TmpCurrSelectNdx := CurrSelectNdx;
  SetDrawModeNormal(true);
  if DisplayLabelListDlg(PlotThread.LabelList, FBitmapAttribRec, ScreenImage.Picture.bitmap,
                         TmpIsLabelSelected, TmpCurrSelectNdx) then
    begin
      PlotThread.SetHasChangedSinceSaveTrue;
      if TmpIsLabelSelected then
        begin
          LabelWindowStatus := lwLabelSelected;
          CurrSelectNdx := TmpCurrSelectNdx;
          CurrSelectPtr := PlotThread.LabelList.Items[TmpCurrSelectNdx];
        end; {if}
      RestartLabelListObjectChanged(lbHiddenLabelNone);
    end
  else
    if TmpIsLabelSelected then
      SelectALabel(TmpCurrSelectNdx);
end; {procedure TPlotImageBox.ShowLabelListDlg;}

procedure TPlotImageBox.ShowNewCustomCursor(TheNewCursor: TCursor);
begin {procedure TPlotImageBox.ShowNewCustomCursor}
  BeginNoLabelFixupSection;
  try
    if CurrCursor <> TheNewCursor then
      begin
        CurrCursor := TheNewCursor;
        Screen.Cursor := TheNewCursor;
      end; {if}
  finally
    EndNoLabelFixupSection;
  end; {try..finally}
end; {procedure TPlotImageBox.ShowNewCustomCursor}

procedure TPlotImageBox.ShowNormalCursor;
begin {procedure TPlotImageBox.ShowNormalCursor;}
  BeginNoLabelFixupSection;
  try
    if CurrCursor <> crDefault then
      begin
        CurrCursor := crDefault;
        Screen.Cursor := crDefault;
      end; {if}
  finally
    EndNoLabelFixupSection;
  end; {try..finally}
end; {procedure TPlotImageBox.ShowNormalCursor;}

procedure TPlotImageBox.ShowSelectRegionDlg;
var
  NewRegionOrigin, NewRegionMovePt : TPoint;
begin {procedure TPlotImageBox.ShowSelectRegionDlg;}
  NewRegionOrigin := RegionOrigin;
  NewRegionMovePt := RegionMovePt;
  FindTopLeftPoint(NewRegionOrigin, NewRegionMovePt);
  if ModifySelectedRegionDlg((LabelWindowStatus = lwRegionSelected),
              NewRegionOrigin, NewRegionMovePt, BmpWidth, BmpHeight) then
    begin
      SetDrawModeNormal(true);
      DoTurnDrawToolOffEvent;
      LabelWindowStatus := lwRegionSelected;
      FindTopLeftPoint(NewRegionOrigin, NewRegionMovePt);
      RegionOrigin := NewRegionOrigin;
      RegionMovePt := NewRegionMovePt;
      DrawSelectRectangle(RegionOrigin, RegionMovePt);
    end; {if}
end; {procedure TPlotImageBox.ShowSelectRegionDlg;}

procedure TPlotImageBox.StartEditingEvent(Sender: TObject;
  EditTopLeft: TPoint; EditWidth, EditHeight: integer);
var
  TheFillRect : TRect;
begin {procedure TPlotImageBox.StartEditingEvent}
  if LabelWindowStatus = lwEditingText then
    begin
      PlotThread.SetHasChangedSinceSaveTrue;
      if (CurrSelectPtr as TDrawText).LabelTextStr <> '' then
        begin
          with TheFillRect do
            begin
              Top := EditTopLeft.Y;
              Left := EditTopLeft.X;
              Right := Left + EditWidth;
              Bottom := Top + EditHeight;
              if Left < 0 then Left := 0
              else if Left > BmpWidth then Left := BmpWidth;
              if Right < 0 then Right := 0
              else if Right > BmpWidth then Right := BmpWidth;
              if Top < 0 then Top := 0
              else if Top > BmpHeight then Top := BmpHeight;
              if Bottom < 0 then Bottom := 0
              else if Bottom > BmpHeight then Bottom := BmpHeight;
            end; {with}
          ScreenImage.Canvas.Brush.Style := bsSolid;
          ScreenImage.Canvas.Brush.Color := BackgroundColor;
          ScreenImage.Canvas.FillRect(TheFillRect);
          RestartLabelListObjectChanged(CurrSelectNdx);
        end; {if}
    end; {if}
end; {procedure TPlotImageBox.StartEditingEvent}

procedure TPlotImageBox.WaitForTermAllThreads;
begin {procedure TPlotImageBox.WaitForTermAllThreads;}
  PlotThread.WaitForTermAllThreads;
end; {procedure TPlotImageBox.WaitForTermAllThreads;}

procedure TPlotImageBox.TerminateAllThreads;
begin {procedure TPlotImageBox.TerminateAllThreads;}
  PlotThread.TerminateAllThreads;
end; {procedure TPlotImageBox.TerminateAllThreads;}

end.
