unit PtShared;

(*
Copyright 1999-2011 Greg VanMullem
This program is distributed under the terms of the GNU General Public License Version 3

This file is part of MathGV.

MathGV is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

MathGV is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MathGV.  If not, see <http://www.gnu.org/licenses/>.
*)

interface

{Shared declarations for paint units}

uses
  wintypes, Graphics, ExtCtrls;

const
  LinePointArraySize = 128;

type
  TLinePointArray = packed array[1..LinePointArraySize] of TPoint;

type
  TIsTerminatedEvent = procedure(var ShouldTerminate : boolean) of object;
  TDonePlotingEvent = procedure(PlotBitmap : TBitmap) of object;

procedure DoCopyBitmapToImageBuffers(TheDestinationImage : TImage; TheSourceBmp : TBitmap);
procedure DoCopyBitmapBuffers(TheDestinationBmp, TheSourceBmp : TBitmap);

implementation

{---------------------------------------------------------------------}

procedure DoCopyBitmapToImageBuffers(TheDestinationImage : TImage; TheSourceBmp : TBitmap);
begin {procedure DoCopyBitmapToImageBuffers}
(*
  Assert(Assigned(TheSourceBmp), 'TheSourceBmp = nil in DoCopyBitmapBuffers');
  Assert(Assigned(TheDestinationImage), 'TheDestinationBmp = nil in DoCopyBitmapBuffers');
  Assert((not TheSourceBmp.Empty), 'TheSourceBmp is empty in DoCopyBitmapBuffers');
  Assert((not TheDestinationImage.Picture.Bitmap.Empty), 'TheDestinationBmp is empty in DoCopyBitmapBuffers');
  Assert(TheSourceBmp.Width = TheDestinationImage.Width, 'Width not equal in procedure DoCopyBitmapBuffers');
  Assert(TheSourceBmp.Height = TheDestinationImage.Height, 'Width not equal in procedure DoCopyBitmapBuffers');
  Assert(TheSourceBmp.Width > 0, 'Width =< 0 in procedure DoCopyBitmapBuffers');
  Assert(TheSourceBmp.Height > 0, 'Height =< 0 in procedure DoCopyBitmapBuffers');
  Assert(TheSourceBmp.HandleType = bmDIB, 'TheSourceBmp.HandleType <> bmDIB');
  Assert(TheDestinationImage.Picture.Bitmap.HandleType = bmDIB, 'TheDestinationBmp.HandleType <> bmDIB');
  Assert(TheSourceBmp.PixelFormat = TheDestinationImage.Picture.Bitmap.PixelFormat, 'PixelFormat not equal in procedure DoCopyBitmapBuffers');
*)

  TheSourceBmp.Canvas.Lock;
  TheDestinationImage.Picture.Bitmap.Canvas.Lock;
  try
    //copy the bitmap
    TheSourceBmp.Transparent := False;
    TheSourceBmp.Canvas.CopyMode := cmSrcCopy;
    TheDestinationImage.Picture.Bitmap.Canvas.CopyMode := cmSrcCopy;
    TheSourceBmp.Canvas.Brush.Style := bsSolid;
    TheDestinationImage.Picture.Bitmap.Canvas.Draw(0, 0, TheSourceBmp);
  finally
    TheSourceBmp.Canvas.Unlock;
    TheDestinationImage.Picture.Bitmap.Canvas.Unlock;
  end;

end; {procedure DoCopyBitmapToImageBuffers}

{---------------------------------------------------------------------}

procedure DoCopyBitmapBuffers(TheDestinationBmp, TheSourceBmp : TBitmap);
begin {procedure DoCopyBitmapBuffers}
(*
  Assert(Assigned(TheSourceBmp), 'TheSourceBmp = nil in DoCopyBitmapBuffers');
  Assert(Assigned(TheDestinationBmp), 'TheDestinationBmp = nil in DoCopyBitmapBuffers');
  Assert((not TheSourceBmp.Empty), 'TheSourceBmp is empty in DoCopyBitmapBuffers');
  Assert((not TheDestinationBmp.Empty), 'TheDestinationBmp is empty in DoCopyBitmapBuffers');
  Assert(TheSourceBmp.Width = TheDestinationBmp.Width, 'Width not equal in procedure DoCopyBitmapBuffers');
  Assert(TheSourceBmp.Height = TheDestinationBmp.Height, 'Width not equal in procedure DoCopyBitmapBuffers');
  Assert(TheSourceBmp.Width > 0, 'Width =< 0 in procedure DoCopyBitmapBuffers');
  Assert(TheSourceBmp.Height > 0, 'Height =< 0 in procedure DoCopyBitmapBuffers');
  Assert(TheSourceBmp.HandleType = bmDIB, 'TheSourceBmp.HandleType <> bmDIB');
  Assert(TheDestinationBmp.HandleType = bmDIB, 'TheDestinationBmp.HandleType <> bmDIB');
  Assert(TheSourceBmp.PixelFormat = TheDestinationBmp.PixelFormat, 'PixelFormat not equal in procedure DoCopyBitmapBuffers');
*)

  TheSourceBmp.Canvas.Lock;
  TheDestinationBmp.Canvas.Lock;
  try
    //copy the bitmap
    TheSourceBmp.Transparent := False;
    TheDestinationBmp.Transparent := False;
    TheSourceBmp.Canvas.CopyMode := cmSrcCopy;
    TheDestinationBmp.Canvas.CopyMode := cmSrcCopy;
    TheSourceBmp.Canvas.Brush.Style := bsSolid;
    TheDestinationBmp.Canvas.Brush.Style := bsSolid;
    TheDestinationBmp.Canvas.Draw(0, 0, TheSourceBmp);
  finally
    TheSourceBmp.Canvas.Unlock;
    TheDestinationBmp.Canvas.Unlock;
  end;

end; {procedure DoCopyBitmapBuffers}

{---------------------------------------------------------------------}

end.
