unit PtThList;

(*
Copyright 1999-2011 Greg VanMullem
This program is distributed under the terms of the GNU General Public License Version 3

This file is part of MathGV.

MathGV is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

MathGV is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MathGV.  If not, see <http://www.gnu.org/licenses/>.
*)

interface

uses
  Classes, Graphics, PtThGraf, PtGrList, FuncObj;

type
  TPlotThreadFuncList = class(TPlotThreadGraph)
  private
    { Private declarations }
    FFuncList : TList;
  protected
    { Protected declarations }
    function IsColorAlreadyInUse(TheColor: TColor): boolean; virtual; abstract;
    procedure DoAssignFieldObjects; override;
  public
    { Public declarations }
    property FuncList : TList read FFuncList;
    constructor Create;
    destructor Destroy; override;
    procedure AddNewFuncListObj(TheFunctionObj : TBaseFuncObj);
    procedure ClearFuncListObj;
    function IsFreeFuncListObjSlot : boolean; virtual; abstract;
    function GetNextDefaultColor : TColor;
  end; {TPlotThreadFuncList}

implementation

const
  DefaultColorsCount = 15;
  DefaultColorsArray : array[1..DefaultColorsCount] of TColor =
    (clBlue,   clRed,     clGreen, clMaroon, clNavy,
     clOlive,  clPurple,  clTeal,  clGray,   clSilver,
     clLime,   clFuchsia, clAqua,  clBlack,  clWhite);

{---------------------------------------------------------------------}

{ TPlotThreadFuncList }

procedure TPlotThreadFuncList.AddNewFuncListObj(
  TheFunctionObj: TBaseFuncObj);
begin {procedure TPlotThreadFuncList.AddNewFuncListObj}
  FFuncList.Add(TheFunctionObj);
end; {procedure TPlotThreadFuncList.AddNewFuncListObj}

{---------------------------------------------------------------------}

procedure TPlotThreadFuncList.ClearFuncListObj;
var
  I : integer;
begin {procedure TPlotThreadFuncList.ClearFuncListObj;}
  if FFuncList.Count >= 1 then
    for I := 0 to FFuncList.Count -1 do
      if Assigned(FFuncList[I]) then
        TObject(FFuncList[I]).Free;
  FFuncList.Clear;
end; {procedure TPlotThreadFuncList.ClearFuncListObj;}

{---------------------------------------------------------------------}

constructor TPlotThreadFuncList.Create;
begin {constructor TPlotThreadFuncList.Create}
  inherited;
  FFuncList := TList.Create;
end; {constructor TPlotThreadFuncList.Create}

{---------------------------------------------------------------------}

destructor TPlotThreadFuncList.Destroy;
begin {destructor TPlotThreadFuncList.Destroy;}
  ClearFuncListObj;
  FFuncList.free;
  inherited;
end; {destructor TPlotThreadFuncList.Destroy;}

{---------------------------------------------------------------------}

procedure TPlotThreadFuncList.DoAssignFieldObjects;
begin {procedure TPlotThreadFuncList.DoAssignFieldObjects;}
  inherited;
  Assert((PlotGraph is TPlotGraphFuncList), 'Invalid PlotGraph object type in DoAssignFieldObjects');
  (PlotGraph as TPlotGraphFuncList).AssignFuncList(FFuncList);
end; {procedure TPlotThreadFuncList.DoAssignFieldObjects;}

{---------------------------------------------------------------------}

function TPlotThreadFuncList.GetNextDefaultColor : TColor;
var
  I : integer;
begin {function TPlotThreadFuncList.GetNextDefaultColor}
  for I := 1 to DefaultColorsCount do
    begin
      if (not IsColorAlreadyInUse(DefaultColorsArray[I])) then
        begin
          Result := DefaultColorsArray[I];
          exit;
        end; {if}
    end; {for}
  {if all colors are used then}
  Result := DefaultColorsArray[1]; {guarantee valid color is returned}
  I := (FFuncList.Count mod DefaultColorsCount)+1;
  if (I >= 1) and (I <= DefaultColorsCount) then
    Result := DefaultColorsArray[I];
end; {function TPlotThreadFuncList.GetNextDefaultColor}

{---------------------------------------------------------------------}

end.
