unit PtThPolr;

(*
Copyright 1999-2011 Greg VanMullem
This program is distributed under the terms of the GNU General Public License Version 3

This file is part of MathGV.

MathGV is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

MathGV is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MathGV.  If not, see <http://www.gnu.org/licenses/>.
*)

interface

uses
  SysUtils, Classes, Graphics, GrGlobal, PtThList, VwPoObj, SetPoObj, FileSave,
  funcObj, funcPolr, PtGrPolr;

const
  MaxPolarFuncCount = PtGrPolr.MaxPolarFuncCount;

type
  TPlotThreadPolar = class(TPlotThreadFuncList)
  private
    { Private declarations }
    FViewPolarZoomObj : TViewPolarZoomObj;
    FSettingPolarObj : TSettingPolarObj;
  protected
    { Protected declarations }
    function IsColorAlreadyInUse(TheColor : TColor) : boolean; override;
    procedure DoAssignFieldObjects; override;
    function WriteFileToDisk(const FString : string) : boolean; override;
    function GetPlotTypeString : string; override;
  public
    { Public declarations }
    property ViewPolarZoomObj : TViewPolarZoomObj read FViewPolarZoomObj;
    property SettingPolarObj : TSettingPolarObj read FSettingPolarObj;
    constructor Create;
    destructor Destroy; override;
    function IsFreeFuncListObjSlot : boolean; override;
    function InitalizeFromFile(TheColorDepth : TPixelFormat;
                               const TheFileName : string;
                               ReadGraphFile : TReadGraphFile) : boolean; override;
  end; {TPlotThreadPolar}

implementation

{ TPlotThreadPolar }

constructor TPlotThreadPolar.Create;
begin
  inherited;
  PlotGraph := TPlotGraphPolar.Create;
  FViewPolarZoomObj := TViewPolarZoomObj.Create;
  FSettingPolarObj := TSettingPolarObj.Create;
end;

destructor TPlotThreadPolar.Destroy;
begin
  PlotGraph.free;
  FViewPolarZoomObj.free;
  FSettingPolarObj.free;
  inherited;
end;

procedure TPlotThreadPolar.DoAssignFieldObjects;
begin
  if FSettingPolarObj.DidFieldsChange and
    (SettingBmpAttribsObj.BackgroundColor <> FSettingPolarObj.BackColor) then
    begin
      //special fix up for backward file compatibility
      SettingBmpAttribsObj.BackgroundColor := FSettingPolarObj.BackColor;
      SettingBmpAttribsObj.DidBackGroundColorChange := true;
    end; {if}
  if FViewPolarZoomObj.DidFieldsChange then
    begin
      TPlotGraphPolar(PlotGraph).ViewPolarZoomObj.Assign(FViewPolarZoomObj);
      FViewPolarZoomObj.DidFieldsChange := false;
    end; {if}
  if FSettingPolarObj.DidFieldsChange then
    begin
      TPlotGraphPolar(PlotGraph).SettingPolarObj.Assign(FSettingPolarObj);
      FSettingPolarObj.DidFieldsChange := false;
    end; {if}
  inherited;
end;

function TPlotThreadPolar.GetPlotTypeString: string;
begin
  Result := 'Polar';
end;

function TPlotThreadPolar.InitalizeFromFile(TheColorDepth: TPixelFormat;
  const TheFileName : string; ReadGraphFile: TReadGraphFile): boolean;

{-------------------------------}

procedure ReadVer1Fields;
var
  I : integer;
  SaveFuncCount : integer;
  TmpFuncPolarObj : TFuncPolarObj;
begin {procedure ReadVer1Fields;}
  with ReadGraphFile do
    begin
      ReadSettingBmpAttribs(ReadGraphFile);
      if ViewPolarZoomObj.ReadFromDisk(ReadGraphFile) then
        FileMsgViewCenterChanged;
      SettingPolarObj.ReadFromDisk(ReadGraphFile);
      {begin Read objects owned by FuncObjList : TList;}
      SaveFuncCount := ReadSmallint;
      for I := 1 to SaveFuncCount do
        begin
          TmpFuncPolarObj := TFuncPolarObj.Create;
          try
            TmpFuncPolarObj.ReadFromDisk(ReadGraphFile);
            FuncList.Add(TmpFuncPolarObj);
          except
            TmpFuncPolarObj.Free;
            raise;
          end; {try..except}
        end; {for}
      {end Read objects owned by FuncObjList : TList;}
      ReadAncestorDataFromDisk(ReadGraphFile);
    end; {with}
end; {procedure ReadVer1Fields;}

{-------------------------------}

procedure ReadVer2_3Fields;
var
  I : integer;
  SaveFuncCount : integer;
  TmpFuncPolarObj : TFuncPolarObj;
begin {procedure ReadVer2_3Fields;}
  with ReadGraphFile do
    begin
      ReadSettingBmpAttribs(ReadGraphFile);
      if ViewPolarZoomObj.ReadFromDisk(ReadGraphFile) then
        FileMsgViewCenterChanged;
      SettingPolarObj.ReadFromDisk(ReadGraphFile);
      {begin Read objects owned by FuncObjList : TList;}
      SaveFuncCount := ReadLongint;
      for I := 1 to SaveFuncCount do
        begin
          TmpFuncPolarObj := TFuncPolarObj.Create;
          try
            TmpFuncPolarObj.ReadFromDisk(ReadGraphFile);
            FuncList.Add(TmpFuncPolarObj);
          except
            TmpFuncPolarObj.Free;
            raise;
          end; {try..except}
        end; {for}
      {end Read objects owned by FuncObjList : TList;}
      ReadAncestorDataFromDisk(ReadGraphFile);
    end; {with}
end; {procedure ReadVer2_3Fields;}

{-------------------------------}

procedure ReadVer4Fields;
var
  I : integer;
  SaveFuncCount : integer;
  TmpFuncPolarObj : TFuncPolarObj;
begin {procedure ReadVer4Fields;}
  with ReadGraphFile do
    begin
      ReadSettingBmpAttribs(ReadGraphFile);
      ReadSettingGrBGPict(ReadGraphFile);
      if ViewPolarZoomObj.ReadFromDisk(ReadGraphFile) then
        FileMsgViewCenterChanged;
      SettingPolarObj.ReadFromDisk(ReadGraphFile);
      {begin Read objects owned by FuncObjList : TList;}
      SaveFuncCount := ReadLongint;
      for I := 1 to SaveFuncCount do
        begin
          TmpFuncPolarObj := TFuncPolarObj.Create;
          try
            TmpFuncPolarObj.ReadFromDisk(ReadGraphFile);
            FuncList.Add(TmpFuncPolarObj);
          except
            TmpFuncPolarObj.Free;
            raise;
          end; {try..except}
        end; {for}
      {end Read objects owned by FuncObjList : TList;}
      ReadAncestorDataFromDisk(ReadGraphFile);
    end; {with}
end; {procedure ReadVer4Fields;}

{-------------------------------}

begin
  Result := true;
  try
    if ReadGraphFile.FileVersion = 1 then
      ReadVer1Fields
    else
      if ReadGraphFile.FileVersion <= 3 then
        ReadVer2_3Fields
      else
        ReadVer4Fields;
    ReadGraphFile.CloseDataFile;
  except
    Result := false;
    ReadGraphFile.CloseDataFile;
  end; {try..except}
  if Result then
    begin
      SettingBmpAttribsObj.BackgroundColor := SettingPolarObj.BackColor;
      SettingBmpAttribsObj.BmpColorDepth := TheColorDepth;
      FFileName := TheFileName;
    end; {if}
end;

function TPlotThreadPolar.IsColorAlreadyInUse(TheColor: TColor): boolean;
var
  I : integer;
begin
  Result := (TheColor = FSettingPolarObj.BackColor);
  if not Result then
    begin
      for I := 0 to (FuncList.Count-1) do
        begin
          Assert(Assigned(FuncList[I]), 'nil in FuncList in function function TPlotThreadPolar.IsColorAlreadyInUse');
          if (TObject(FuncList[I]) as TFuncPolarObj).FuncColor = TheColor then
            begin
              Result := true;
              exit;
            end; {if}
        end; {for}
    end; {if}
end;

function TPlotThreadPolar.IsFreeFuncListObjSlot: boolean;
begin
  Result := FuncList.Count < MaxPolarFuncCount;
end;

function TPlotThreadPolar.WriteFileToDisk(const FString: string): boolean;
var
  I : integer;
  SaveFuncCount : integer;
  WriteGraphFile : TWriteGraphFile;
begin
  Result := false;
  WriteGraphFile := TWriteGraphFile.Create;
  if not Assigned(WriteGraphFile) then
    Exit;
  try
    with WriteGraphFile do
      if RewriteSaveFile(FString, grType2DPolar, DecimalSeparator) then
        begin
          SaveSettingBmpAttribs(WriteGraphFile);
          SaveSettingGrBGPict(WriteGraphFile);
          ViewPolarZoomObj.SaveToDisk(WriteGraphFile);
          SettingPolarObj.SaveToDisk(WriteGraphFile);
          {begin Save objects owned by FuncObjList : TList;}
          SaveFuncCount := FuncList.Count;
          WriteLongint(SaveFuncCount);
          for I := 1 to SaveFuncCount do
            (TObject(FuncList[I-1]) as TFuncPolarObj).SaveToDisk(WriteGraphFile);
          {end Save objects owned by FuncObjList : TList;}
          SaveAncestorDataToDisk(WriteGraphFile);
          Result := CloseValidFile;
        end; {if}
  except
    Result := False;
  end; {try..except}
  WriteGraphFile.Free;
end;

end.
