unit Set3ddlg;

(*
Copyright 1999-2011 Greg VanMullem
This program is distributed under the terms of the GNU General Public License Version 3

This file is part of MathGV.

MathGV is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

MathGV is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MathGV.  If not, see <http://www.gnu.org/licenses/>.
*)

interface

uses WinTypes, WinProcs, Classes, Graphics, Forms, Controls, Buttons,
  StdCtrls, ExtCtrls, Clsample, ComnDlgs, CentrDlg, dialogs, Set3dObj, DefFont;

type
  TSettings3DDlg = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    HelpBtn: TBitBtn;
    GraphAttribsBox: TGroupBox;
    EBackgroundColor: TButton;
    EAllAxisLineColors: TButton;
    EBackGroundSample: TColorSample;
    XSettingsBox: TGroupBox;
    EXShowAxisLine: TCheckBox;
    EXLabelNeg: TCheckBox;
    EXLabelPos: TCheckBox;
    EXShowPlusSigns: TCheckBox;
    EXShowMinusSigns: TCheckBox;
    XFontButton: TButton;
    XColorButton: TButton;
    XAxisColorSample: TColorSample;
    YSettingsBox: TGroupBox;
    YAxisColorSample: TColorSample;
    EYShowPlusSigns: TCheckBox;
    EYShowMinusSigns: TCheckBox;
    YFontButton: TButton;
    YColorButton: TButton;
    ZSettingsBox: TGroupBox;
    ZAxisColorSample: TColorSample;
    EZShowPlusSigns: TCheckBox;
    EZShowMinusSigns: TCheckBox;
    ZFontButton: TButton;
    ZColorButton: TButton;
    EYShowAxisLine: TCheckBox;
    EZShowAxisLine: TCheckBox;
    EYLabelPos: TCheckBox;
    EYLabelNeg: TCheckBox;
    EZLabelPos: TCheckBox;
    EZLabelNeg: TCheckBox;
    EAllLabelFonts: TButton;
    procedure EBackgroundColorClick(Sender: TObject);
    procedure XFontButtonClick(Sender: TObject);
    procedure YFontButtonClick(Sender: TObject);
    procedure ZFontButtonClick(Sender: TObject);
    procedure XColorButtonClick(Sender: TObject);
    procedure YColorButtonClick(Sender: TObject);
    procedure ZColorButtonClick(Sender: TObject);
    procedure EAllAxisLineColorsClick(Sender: TObject);
    procedure EAllLabelFontsClick(Sender: TObject);
  private
    { Private declarations }
    FXLabelFont,
    FYLabelFont,
    FZLabelFont : TFont;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure GetTheFields(The3DObj : TSetting3DObj);
    procedure SetTheFields(The3DObj : TSetting3DObj);
  end;

function ModifyGraph3DSettings(The3DObj : TSetting3DObj) : boolean;

implementation

{$R *.DFM}

{---------------------------------------------------------------------}

function ModifyGraph3DSettings(The3DObj : TSetting3DObj) : boolean;
var
  Settings3DDlg: TSettings3DDlg;
begin {function ModifyGraph3DSettings}
  Result := false;
  Settings3DDlg := TSettings3DDlg.Create(Application);
  try
    SetDlgDefaultPos(Settings3DDlg, nil);
    Settings3DDlg.SetTheFields(The3DObj);
    if Settings3DDlg.ShowModal = IDOK then
      begin
        Result := true;
        Settings3DDlg.GetTheFields(The3DObj);
        The3DObj.DidFieldsChange := true;
      end { if Result = true then}
  finally
    Settings3DDlg.Free;
  end; {try..finally}
end; {function ModifyGraph3DSettings}

{---------------------------------------------------------------------}

constructor TSettings3DDlg.Create(AOwner: TComponent);
begin {constructor TSettings3DDlg.Create}
  inherited Create(AOwner);
  FXLabelFont := CreateANewTFontClass;
  FYLabelFont := CreateANewTFontClass;
  FZLabelFont := CreateANewTFontClass;
end; {constructor TSettings3DDlg.Create}

{---------------------------------------------------------------------}

destructor TSettings3DDlg.Destroy;
begin {destructor TSettings3DDlg.Destroy;}
  FXLabelFont.Free;
  FYLabelFont.Free;
  FZLabelFont.Free;
  inherited destroy;
end; {destructor TSettings3DDlg.Destroy;}

{---------------------------------------------------------------------}

procedure TSettings3DDlg.GetTheFields(The3DObj : TSetting3DObj);
begin {procedure TSettings3DDlg.GetTheFields}
  with The3DObj do
    begin
      BackGroundColor := EBackGroundSample.ColorValue;
      {X Axis Settings}
      XShowAxisLine := EXShowAxisLine.Checked;
      XAxisColor := XAxisColorSample.ColorValue;
      XLabelPos := EXLabelPos.Checked;
      XLabelNeg := EXLabelNeg.Checked;
      XShowPlusSigns := EXShowPlusSigns.Checked;
      XShowMinusSigns := EXShowMinusSigns.Checked;
      {Y Axis Settings}
      YShowAxisLine := EYShowAxisLine.Checked;
      YAxisColor := YAxisColorSample.ColorValue;
      YLabelPos := EYLabelPos.Checked;
      YLabelNeg := EYLabelNeg.Checked;
      YShowPlusSigns := EYShowPlusSigns.Checked;
      YShowMinusSigns := EYShowMinusSigns.Checked;
      {Z Axis Settings}
      ZShowAxisLine := EZShowAxisLine.Checked;
      ZAxisColor := ZAxisColorSample.ColorValue;
      ZLabelPos := EZLabelPos.Checked;
      ZLabelNeg := EZLabelNeg.Checked;
      ZShowPlusSigns := EZShowPlusSigns.Checked;
      ZShowMinusSigns := EZShowMinusSigns.Checked;
      XLabelFont.Assign(FXLabelFont);
      YLabelFont.Assign(FYLabelFont);
      ZLabelFont.Assign(FZLabelFont);
    end; {with}
end; {procedure TSettings3DDlg.GetTheFields}

{---------------------------------------------------------------------}

procedure TSettings3DDlg.SetTheFields(The3DObj : TSetting3DObj);
begin {procedure TSettings3DDlg.SetTheFields}
  with The3DObj do
    begin
      EBackGroundSample.ColorValue := BackGroundColor;
      {X Axis Settings}
      EXShowAxisLine.Checked := XShowAxisLine;
      XAxisColorSample.ColorValue := XAxisColor;
      EXLabelPos.Checked := XLabelPos;
      EXLabelNeg.Checked := XLabelNeg;
      EXShowPlusSigns.Checked := XShowPlusSigns;
      EXShowMinusSigns.Checked := XShowMinusSigns;
      {Y Axis Settings}
      EYShowAxisLine.Checked := YShowAxisLine;
      YAxisColorSample.ColorValue := YAxisColor;
      EYLabelPos.Checked := YLabelPos;
      EYLabelNeg.Checked := YLabelNeg;
      EYShowPlusSigns.Checked := YShowPlusSigns;
      EYShowMinusSigns.Checked := YShowMinusSigns;
      {Z Axis Settings}
      EZShowAxisLine.Checked := ZShowAxisLine;
      ZAxisColorSample.ColorValue := ZAxisColor;
      EZLabelPos.Checked := ZLabelPos;
      EZLabelNeg.Checked := ZLabelNeg;
      EZShowPlusSigns.Checked := ZShowPlusSigns;
      EZShowMinusSigns.Checked := ZShowMinusSigns;
      FXLabelFont.Assign(XLabelFont);
      FYLabelFont.Assign(YLabelFont);
      FZLabelFont.Assign(ZLabelFont);
    end; {with}
end; {procedure TSettings3DDlg.SetTheFields}

{---------------------------------------------------------------------}

procedure TSettings3DDlg.EBackgroundColorClick(Sender: TObject);
begin
  EBackGroundSample.ColorValue := ReturnNewColorComnDlg(EBackGroundSample.ColorValue, Self);
end;

procedure TSettings3DDlg.XFontButtonClick(Sender: TObject);
begin
  ShowComnFontDlg(FXLabelFont, Self);
end;

procedure TSettings3DDlg.YFontButtonClick(Sender: TObject);
begin
  ShowComnFontDlg(FYLabelFont, Self);
end;

procedure TSettings3DDlg.ZFontButtonClick(Sender: TObject);
begin
  ShowComnFontDlg(FZLabelFont, Self);
end;

procedure TSettings3DDlg.XColorButtonClick(Sender: TObject);
begin
  XAxisColorSample.ColorValue := ReturnNewColorComnDlg(XAxisColorSample.ColorValue, Self);
end;

procedure TSettings3DDlg.YColorButtonClick(Sender: TObject);
begin
  YAxisColorSample.ColorValue := ReturnNewColorComnDlg(YAxisColorSample.ColorValue, Self);
end;

procedure TSettings3DDlg.ZColorButtonClick(Sender: TObject);
begin
  ZAxisColorSample.ColorValue := ReturnNewColorComnDlg(ZAxisColorSample.ColorValue, Self);
end;

procedure TSettings3DDlg.EAllAxisLineColorsClick(Sender: TObject);
var
  TmpColor : TColor;
begin
  TmpColor := XAxisColorSample.ColorValue;
  if ShowComnColorDlg(TmpColor, Self) then
    begin
      XAxisColorSample.ColorValue := TmpColor;
      YAxisColorSample.ColorValue := TmpColor;
      ZAxisColorSample.ColorValue := TmpColor;
    end; {if}
end;

procedure TSettings3DDlg.EAllLabelFontsClick(Sender: TObject);
begin
  if ShowComnFontDlg(FXLabelFont, Self) then
    begin
      FYLabelFont.Assign(FXLabelFont);
      FZLabelFont.Assign(FXLabelFont);
    end; {if}
end;

end.
