unit SeqList;

(*
Copyright 1999-2011 Greg VanMullem
This program is distributed under the terms of the GNU General Public License Version 3

This file is part of MathGV.

MathGV is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

MathGV is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MathGV.  If not, see <http://www.gnu.org/licenses/>.
*)

interface

uses
  Classes;

type
  TSequenceListObj = class
  private
  { Private declarations }
    ListSequenceIndexNumber : integer;
  protected
  { Protected declarations }
    function CreateNewSeqObj : TObject; virtual; abstract;
    procedure AssignSeqObj(Source : TObject); virtual; abstract;
  public
  { Public declarations }
    DidFieldsChange : boolean;
    constructor Create;
  end; {TSequenceListObj = class}

function AssignSequenceLists(DestinationList, SourceList : TList) : boolean;

implementation

{---------------------------------------------------------------------}

function AssignSequenceLists(DestinationList, SourceList : TList) : boolean;
//returns true if the DestinationList was changed
var
  DestListWasChanged : boolean;
  DidMoveOrDelete : boolean;
  AddedToEndCount : integer;

{-----------------------------}

function AreListsDifferent : boolean;
var
  I : integer;
begin {function AreListsDifferent}
  Result := false;
  //test to see if list counts are different
  if SourceList.Count <> DestinationList.Count then
  begin
    Result := true;
    Exit;
  end; {if}
  //test for nil values in lists
  for I := 0 to (SourceList.Count - 1) do
    if not Assigned(SourceList[I]) then
      begin
        Result := true;
        Exit;
      end; {if}
  for I := 0 to (DestinationList.Count - 1) do
    if not Assigned(DestinationList[I]) then
      begin
        Result := true;
        Exit;
      end; {if}
  //test DidFieldsChange flags in SourceList
  for I := 0 to (SourceList.Count - 1) do
    if TSequenceListObj(SourceList[I]).DidFieldsChange then
      begin
        Result := true;
        Exit;
      end; {if}
  //test Sequence Index Numbers
  for I := 0 to (SourceList.Count - 1) do
    if TSequenceListObj(SourceList[I]).ListSequenceIndexNumber <> I then
      begin
        Result := true;
        Exit;
      end; {if}
  for I := 0 to (DestinationList.Count - 1) do
    if TSequenceListObj(DestinationList[I]).ListSequenceIndexNumber <> I then
      begin
        Result := true;
        Exit;
      end; {if}
end; {function AreListsDifferent}

{-----------------------------}

procedure ClearDestinationList;
var
  I : integer;
begin {procedure ClearDestinationList;}
  for I := 0 to (DestinationList.Count - 1) do
    if DestinationList[I] <> nil then
      TObject(DestinationList[I]).free;
  DestinationList.Clear;
end; {procedure ClearDestinationList;}

{-----------------------------}

procedure ClearDestinationIfContainsNil;
var
  I : integer;
  ContainsNil : boolean;
begin {procedure ClearDestinationIfContainsNil;}
  ContainsNil := false;
  for I := 0 to (DestinationList.Count - 1) do
    if DestinationList[I] = nil then
      ContainsNil := true;
  if ContainsNil then
    ClearDestinationList;
end; {procedure ClearDestinationIfContainsNil;}

{-----------------------------}

procedure DeleteSourceListNils;
var
  I : integer;
begin {procedure DeleteSourceListNils;}
  I := 0;
  while I <= (SourceList.Count - 1) do
    begin
      if SourceList[I] = nil then
        SourceList.Delete(I)
      else
        begin
          Assert(TObject(SourceList[I]) is TSequenceListObj, 'List Object is not of type TSequenceListObj');
          inc(I);
        end; {else}
    end; {while}
end; {procedure DeleteSourceListNils;}

{-----------------------------}

procedure SearchForMoveDelOrAdd;
var
  I, TmpSourceCount : integer;
begin {procedure SearchForMoveDelOrAdd;}
  AddedToEndCount := 0;
  for I := (SourceList.Count - 1) downto 0 do
    begin
      if TSequenceListObj(SourceList[I]).ListSequenceIndexNumber = -1 then
        inc(AddedToEndCount)
      else
        break;
    end; {for}
  DidMoveOrDelete := false;
  TmpSourceCount := SourceList.Count - AddedToEndCount;
  if TmpSourceCount <> DestinationList.Count then
    DidMoveOrDelete := true
  else
    begin
      for I := 0 to (TmpSourceCount - 1) do
        if I <> TSequenceListObj(SourceList[I]).ListSequenceIndexNumber then
          begin
            DidMoveOrDelete := true;
            break;
          end; {if}
    end; {else}
end; {procedure SearchForMoveDelOrAdd;}

{-----------------------------}

procedure AddNewObjs;
var
  I : integer;
  TmpObj : TObject;
begin {procedure AddNewObjs;}
  Assert(SourceList.Count > 0, 'Error in AssignSequenceLists, SourceList.Count <= 0');
  Assert(AddedToEndCount > 0, 'Error in AssignSequenceLists, AddedToEndCount <= 0');
  Assert((SourceList.Count - AddedToEndCount) >= 0, 'Error in AssignSequenceLists, list count < 0');
  DestinationList.Capacity := DestinationList.Count + AddedToEndCount;
  for I := (SourceList.Count - AddedToEndCount) to (SourceList.Count - 1) do
    begin
      TmpObj := TSequenceListObj(SourceList[I]).CreateNewSeqObj;
      try
        TSequenceListObj(TmpObj).AssignSeqObj(SourceList[I]);
      except
        TmpObj.free;
        raise;
      end; {try..except}
      TSequenceListObj(SourceList[I]).ListSequenceIndexNumber := DestinationList.Add(TmpObj);
      TSequenceListObj(SourceList[I]).DidFieldsChange := false;
    end; {for}
end; {procedure AddNewObjs;}

{-----------------------------}

procedure MoveDeleteAddObjs;
type
  TTmpReorderArray = array of TSequenceListObj;
var
  I, SeqNdx : integer;
  TmpObj : TObject;
  TmpReorderArray : TTmpReorderArray;
begin {procedure MoveDeleteAddObjs;}
  SetLength(TmpReorderArray, SourceList.Count);
  for I := 0 to Length(TmpReorderArray) - 1 do //initalize array
    TmpReorderArray[I] := nil;
  try
    //move to TmpReorderArray
    for I := 0 to (SourceList.Count - 1) do
      begin
        SeqNdx := TSequenceListObj(SourceList[I]).ListSequenceIndexNumber;
        if (SeqNdx <= -1) or (SeqNdx >= DestinationList.Count) then
          begin
            TmpObj := TSequenceListObj(SourceList[I]).CreateNewSeqObj;
            try
              TSequenceListObj(TmpObj).AssignSeqObj(SourceList[I]);
            except
              TmpObj.free;
              raise;
            end; {try..except}
            TmpReorderArray[I] := TSequenceListObj(TmpObj);
            TSequenceListObj(SourceList[I]).DidFieldsChange := false;
          end
        else
          begin
            if DestinationList[SeqNdx] = nil then
              begin
                TmpObj := TSequenceListObj(SourceList[I]).CreateNewSeqObj;
                try
                  TSequenceListObj(TmpObj).AssignSeqObj(SourceList[I]);
                except
                  TmpObj.free;
                  raise;
                end; {try..except}
                TmpReorderArray[I] := TSequenceListObj(TmpObj);
                TSequenceListObj(SourceList[I]).DidFieldsChange := false;
              end
            else
              TmpReorderArray[I] := DestinationList[SeqNdx];
            DestinationList[SeqNdx] := nil;
          end; {else}
      end; {for}
    //Delete anything left in TmpDestList
    for I := 0 to (DestinationList.Count - 1) do
      if DestinationList[I] <> nil then
        TObject(DestinationList[I]).free;
    //Resize TmpDestList
    if Length(TmpReorderArray) = 0 then
      DestinationList.Clear
    else
      begin
        DestinationList.Count := Length(TmpReorderArray);
        DestinationList.Capacity := DestinationList.Count;
      end; {else}
  except
    //free memory if error occured, errors should almost never happen
    for I := 0 to Length(TmpReorderArray) - 1 do //initalize array
      if TmpReorderArray[I] <> nil then
        TObject(TmpReorderArray[I]).free;
    TmpReorderArray := nil;
    raise;
  end; {try..except}
  //move from TmpReorderArray to TmpDestList
  for I := 0 to (Length(TmpReorderArray) - 1) do
    DestinationList[I] := TmpReorderArray[I];
  //delete array
  TmpReorderArray := nil;
end; {procedure MoveDeleteAddObjs;}

{-----------------------------}

procedure ResetSequenceIndexNumbers;
var
  I : integer;
begin {procedure ResetSequenceIndexNumbers;}
  //reset Sequence Index Numbers
  for I := 0 to (SourceList.Count - 1) do
    TSequenceListObj(SourceList[I]).ListSequenceIndexNumber := I;
  for I := 0 to (DestinationList.Count - 1) do
    TSequenceListObj(DestinationList[I]).ListSequenceIndexNumber := I;
end; {procedure ResetSequenceIndexNumbers;}

{-----------------------------}

procedure AssignObjectListFields;
var
  I : integer;
begin {procedure AssignObjectListFields;}
  Assert((SourceList.Count = DestinationList.Count), 'Error in procedure AssignObjectListFields');
  if SourceList.Count = DestinationList.Count then //fail safe code
    for I := 0 to (SourceList.Count - 1) do
      if TSequenceListObj(SourceList[I]).DidFieldsChange then
        begin
          TSequenceListObj(DestinationList[I]).AssignSeqObj(TSequenceListObj(SourceList[I]));
          TSequenceListObj(SourceList[I]).DidFieldsChange := false;
        end; {if}
end; {procedure AssignObjectListFields;}

{-----------------------------}

begin {procedure AssignSequenceLists}
  Assert(Assigned(DestinationList), 'DestinationList parameter is nil in procedure AssignSequenceLists');
  Assert(Assigned(SourceList), 'SourceList parameter is nil in procedure AssignSequenceLists');
  DestListWasChanged := false;
  try
    if AreListsDifferent then
      begin
        DestListWasChanged := true;
        ClearDestinationIfContainsNil; //fail safe code
        DeleteSourceListNils;          //fail safe code
        if SourceList.Count = 0 then
          ClearDestinationList
        else
          begin
            SearchForMoveDelOrAdd;
            if DidMoveOrDelete then
              MoveDeleteAddObjs
            else
              if AddedToEndCount > 0 then
                AddNewObjs;
            ResetSequenceIndexNumbers;
            AssignObjectListFields;
          end; {else}
      end; {if}
  except
    //dead code remarked out; DestListWasChanged := true;
    ClearDestinationList;
    raise;
  end; {try..except}
  Result := DestListWasChanged;
end; {procedure AssignSequenceLists}

{---------------------------------------------------------------------}

{ TSequenceListObj }

constructor TSequenceListObj.Create;
begin {constructor TSequenceListObj.Create;}
  inherited;
  DidFieldsChange := false;
  ListSequenceIndexNumber := -1;
end; {constructor TSequenceListObj.Create;}

{---------------------------------------------------------------------}

end.
