unit Set2dObj;

(*
Copyright 1999-2011 Greg VanMullem
This program is distributed under the terms of the GNU General Public License Version 3

This file is part of MathGV.

MathGV is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

MathGV is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MathGV.  If not, see <http://www.gnu.org/licenses/>.
*)

interface

uses Graphics, FloatStr, MathCalc, MathFunc, FileSave, DefFont;

type
  TTickMark2DType = (tmNone, tmTickMarks, tmGridMarks, tmGridLines);

  TSetting2DObj = class
    Public
    DidFieldsChange : boolean;
    {X axis values}
    XShowAxisLines : boolean;
    XLimitLoAxisStr : string;
    XLimitLoAxisEmpty : boolean;
    XLimitLoAxisVal : TMathValue;
    XLimitHiAxisStr : string;
    XLimitHiAxisEmpty : boolean;
    XLimitHiAxisVal : TMathValue;
    XAxisColor : TColor;
    XAxisWidth : integer;
    XAxisStyle : TPenStyle;
    XTickMarkType : TTickMark2DType;
    XTickMarkColor : TColor;
    XTickMarkIncStr : string;
    XTickMarkIncVal : TMathValue;
    XTickMarkLength : integer;
    XGridLineStyle : TPenStyle;
    XShowTMLabels : boolean;
    XTMLabelFormat : TFloatNumFormat;
    XShowTMLabelComa : boolean;
    XShowTMLabelPlus : boolean;
    XTMLabelDecPlaces : integer;
    {Y axis values}
    YShowAxisLines : boolean;
    YLimitLoAxisStr : string;
    YLimitLoAxisEmpty : boolean;
    YLimitLoAxisVal : TMathValue;
    YLimitHiAxisStr : string;
    YLimitHiAxisEmpty : boolean;
    YLimitHiAxisVal : TMathValue;
    YAxisColor : TColor;
    YAxisWidth : integer;
    YAxisStyle : TPenStyle;
    YTickMarkType : TTickMark2DType;
    YTickMarkColor : TColor;
    YTickMarkIncStr : string;
    YTickMarkIncVal : TMathValue;
    YTickMarkLength : integer;
    YGridLineStyle : TPenStyle;
    YShowTMLabels : boolean;
    YTMLabelFormat : TFloatNumFormat;
    YShowTMLabelComa : boolean;
    YShowTMLabelPlus : boolean;
    YTMLabelDecPlaces : integer;
    {general values}
    BackColor : TColor;
    {TFont values}
    XLabelFont : TFont;
    YLabelFont : TFont;
    constructor Create;
    destructor Destroy; override;
    procedure SaveToDisk(WriteGraphFile : TWriteGraphFile);
    procedure ReadFromDisk(ReadGraphFile : TReadGraphFile);
    procedure Assign(Source : TSetting2DObj);
  end; {TSetting2DObj = class}

implementation

type
  TDiskV3_Setting2DRec = packed record
    {X axis values}
    XShowAxisLines : boolean;
    XLimitLoAxisEmpty : boolean;
    XLimitLoAxisVal : TPackedMathValue;
    XLimitHiAxisEmpty : boolean;
    XLimitHiAxisVal : TPackedMathValue;
    XAxisColor : TColor;
    XAxisWidth : integer;
    XAxisStyle : TPenStyle;
    XTickMarkType : TTickMark2DType;
    XTickMarkColor : TColor;
    XTickMarkIncVal : TPackedMathValue;
    XTickMarkLength : integer;
    XGridLineStyle : TPenStyle;
    XShowTMLabels : boolean;
    XTMLabelFormat : TFloatNumFormat;
    XShowTMLabelComa : boolean;
    XShowTMLabelPlus : boolean;
    XTMLabelDecPlaces : integer;
    {Y axis values}
    YShowAxisLines : boolean;
    YLimitLoAxisEmpty : boolean;
    YLimitLoAxisVal : TPackedMathValue;
    YLimitHiAxisEmpty : boolean;
    YLimitHiAxisVal : TPackedMathValue;
    YAxisColor : TColor;
    YAxisWidth : integer;
    YAxisStyle : TPenStyle;
    YTickMarkType : TTickMark2DType;
    YTickMarkColor : TColor;
    YTickMarkIncVal : TPackedMathValue;
    YTickMarkLength : integer;
    YGridLineStyle : TPenStyle;
    YShowTMLabels : boolean;
    YTMLabelFormat : TFloatNumFormat;
    YShowTMLabelComa : boolean;
    YShowTMLabelPlus : boolean;
    YTMLabelDecPlaces : integer;
    {general values}
    BackColor : TColor;
  end; {TDiskV3_Setting2DRec = packed record}

  TDiskV2_Setting2DRec = packed record
    {X axis values}
    XShowAxisLines : boolean;
    XLimitLoAxisEmpty : boolean;
    XLimitLoAxisVal : TMathFloat;
    XLimitHiAxisEmpty : boolean;
    XLimitHiAxisVal : TMathFloat;
    XAxisColor : TColor;
    XAxisWidth : integer;
    XAxisStyle : TPenStyle;
    XTickMarkType : TTickMark2DType;
    XTickMarkColor : TColor;
    XTickMarkIncVal : TMathFloat;
    XTickMarkLength : integer;
    XGridLineStyle : TPenStyle;
    XShowTMLabels : boolean;
    XTMLabelFormat : TFloatNumFormat;
    XShowTMLabelComa : boolean;
    XShowTMLabelPlus : boolean;
    XTMLabelDecPlaces : integer;
    {Y axis values}
    YShowAxisLines : boolean;
    YLimitLoAxisEmpty : boolean;
    YLimitLoAxisVal : TMathFloat;
    YLimitHiAxisEmpty : boolean;
    YLimitHiAxisVal : TMathFloat;
    YAxisColor : TColor;
    YAxisWidth : integer;
    YAxisStyle : TPenStyle;
    YTickMarkType : TTickMark2DType;
    YTickMarkColor : TColor;
    YTickMarkIncVal : TMathFloat;
    YTickMarkLength : integer;
    YGridLineStyle : TPenStyle;
    YShowTMLabels : boolean;
    YTMLabelFormat : TFloatNumFormat;
    YShowTMLabelComa : boolean;
    YShowTMLabelPlus : boolean;
    YTMLabelDecPlaces : integer;
    {general values}
    BackColor : TColor;
  end; {TDiskV2_Setting2DRec = packed record}

  TDiskV1_Setting2DRec = packed record
    ShowAxisLines : boolean;
    AxisColor : TColor;
    AxisWidth : smallint;
    AxisStyle : TPenStyle;
    BackColor : TColor;
    ShowTickMarks : boolean;
    TickMarkIncStr : string[50];
    TickMarkIncrements : Extended;
    TickMarkColor : TColor;
    TickMarkLength : smallint;
    LabelTickMarks : boolean;
    LabelDecimalPlaces : smallint;
  end; {TDiskV1_Setting2DRec = packed record}

{---------------------------------------------------------------------}

{ TSetting2DObj }

constructor TSetting2DObj.Create;
begin {constructor TSetting2DObj.Create;}
  inherited Create;
  DidFieldsChange := false;
  {X axis values}
  XShowAxisLines := true;
  XLimitLoAxisStr := '';
  XLimitLoAxisEmpty := true;
  InitMathValue(XLimitLoAxisVal);
  XLimitHiAxisStr := '';
  XLimitHiAxisEmpty := true;
  InitMathValue(XLimitHiAxisVal);
  XAxisColor := clBlack;
  XAxisWidth := 1;
  XAxisStyle := psSolid;
  XTickMarkType := tmTickMarks;
  XTickMarkColor := clBlack;;
  XTickMarkIncStr := '1';
  XTickMarkIncVal := LongintToMathValue(1);
  XTickMarkLength := 5;
  XGridLineStyle := psDot;
  XShowTMLabels := true;
  XTMLabelFormat := lfMaxDecimal;
  XShowTMLabelComa := true;
  XShowTMLabelPlus := false;
  XTMLabelDecPlaces := GetFloatCordDecPlacesDefault(lfMaxDecimal);
  {Y axis values}
  YShowAxisLines := true;
  YLimitLoAxisStr := '';
  YLimitLoAxisEmpty := true;
  InitMathValue(YLimitLoAxisVal);
  YLimitHiAxisStr := '';
  YLimitHiAxisEmpty := true;
  InitMathValue(YLimitHiAxisVal);
  YAxisColor := clBlack;
  YAxisWidth := 1;
  YAxisStyle := psSolid;
  YTickMarkType := tmTickMarks;
  YTickMarkColor := clBlack;;
  YTickMarkIncStr := '1';
  YTickMarkIncVal := LongintToMathValue(1);
  YTickMarkLength := 5;
  YGridLineStyle := psDot;
  YShowTMLabels := true;
  YTMLabelFormat := lfMaxDecimal;
  YShowTMLabelComa := true;
  YShowTMLabelPlus := false;
  YTMLabelDecPlaces := GetFloatCordDecPlacesDefault(lfMaxDecimal);
  {general values}
  BackColor := clWhite;
  {TFont values}
  XLabelFont := CreateANewTFontClass;
  YLabelFont := CreateANewTFontClass;
end; {constructor TSetting2DObj.Create;}

{---------------------------------------------------------------------}

destructor TSetting2DObj.Destroy;
begin {destructor TSetting2DObj.Destroy;}
  XLabelFont.Free;
  YLabelFont.Free;
  inherited Destroy;
end; {destructor TSetting2DObj.Destroy;}

{---------------------------------------------------------------------}

procedure TSetting2DObj.SaveToDisk(WriteGraphFile : TWriteGraphFile);
var
  DiskRec : TDiskV3_Setting2DRec;
begin {procedure TSetting2DObj.SaveToDisk}
  {X axis values}
  DiskRec.XShowAxisLines := XShowAxisLines;
  DiskRec.XLimitLoAxisEmpty := XLimitLoAxisEmpty;
  DiskRec.XLimitLoAxisVal := PackMathValue(XLimitLoAxisVal);
  DiskRec.XLimitHiAxisEmpty := XLimitHiAxisEmpty;
  DiskRec.XLimitHiAxisVal := PackMathValue(XLimitHiAxisVal);
  DiskRec.XAxisColor := XAxisColor;
  DiskRec.XAxisWidth := XAxisWidth;
  DiskRec.XAxisStyle := XAxisStyle;
  DiskRec.XTickMarkType := XTickMarkType;
  DiskRec.XTickMarkColor := XTickMarkColor;
  DiskRec.XTickMarkIncVal := PackMathValue(XTickMarkIncVal);
  DiskRec.XTickMarkLength := XTickMarkLength;
  DiskRec.XGridLineStyle := XGridLineStyle;
  DiskRec.XShowTMLabels := XShowTMLabels;
  DiskRec.XTMLabelFormat := XTMLabelFormat;
  DiskRec.XShowTMLabelComa := XShowTMLabelComa;
  DiskRec.XShowTMLabelPlus := XShowTMLabelPlus;
  DiskRec.XTMLabelDecPlaces := XTMLabelDecPlaces;
  {Y axis values}
  DiskRec.YShowAxisLines := YShowAxisLines;
  DiskRec.YLimitLoAxisEmpty := YLimitLoAxisEmpty;
  DiskRec.YLimitLoAxisVal := PackMathValue(YLimitLoAxisVal);
  DiskRec.YLimitHiAxisEmpty := YLimitHiAxisEmpty;
  DiskRec.YLimitHiAxisVal := PackMathValue(YLimitHiAxisVal);
  DiskRec.YAxisColor := YAxisColor;
  DiskRec.YAxisWidth := YAxisWidth;
  DiskRec.YAxisStyle := YAxisStyle;
  DiskRec.YTickMarkType := YTickMarkType;
  DiskRec.YTickMarkColor := YTickMarkColor;
  DiskRec.YTickMarkIncVal := PackMathValue(YTickMarkIncVal);
  DiskRec.YTickMarkLength := YTickMarkLength;
  DiskRec.YGridLineStyle := YGridLineStyle;
  DiskRec.YShowTMLabels := YShowTMLabels;
  DiskRec.YTMLabelFormat := YTMLabelFormat;
  DiskRec.YShowTMLabelComa := YShowTMLabelComa;
  DiskRec.YShowTMLabelPlus := YShowTMLabelPlus;
  DiskRec.YTMLabelDecPlaces := YTMLabelDecPlaces;
  {general values}
  DiskRec.BackColor := BackColor;
  with WriteGraphFile do
    begin
      WriteRecord(DiskRec, sizeof(DiskRec));
      WriteMathString(XLimitLoAxisStr);
      WriteMathString(XLimitHiAxisStr);
      WriteMathString(XTickMarkIncStr);
      WriteMathString(YLimitLoAxisStr);
      WriteMathString(YLimitHiAxisStr);
      WriteMathString(YTickMarkIncStr);
      {TFont values}
      WriteFont(XLabelFont);
      WriteFont(YLabelFont);
    end; {with}
end; {procedure TSetting2DObj.SaveToDisk}

{---------------------------------------------------------------------}

procedure TSetting2DObj.ReadFromDisk(ReadGraphFile : TReadGraphFile);

{----------------------------------}

procedure ReadVer1Fields;
var
  DiskRec : TDiskV1_Setting2DRec;
begin {procedure ReadVer1Fields;}
  ReadGraphFile.ReadRecord(DiskRec, sizeof(DiskRec));
  {X axis values}
  XShowAxisLines := DiskRec.ShowAxisLines;
  XAxisColor := DiskRec.AxisColor;
  XAxisWidth := DiskRec.AxisWidth;
  XAxisStyle := DiskRec.AxisStyle;
  if DiskRec.ShowTickMarks then
    XTickMarkType := tmTickMarks
  else
    XTickMarkType := tmNone;
  XTickMarkIncStr := DiskRec.TickMarkIncStr;
  XTickMarkIncVal := FloatToMathValue(DiskRec.TickMarkIncrements);
  XTickMarkColor := DiskRec.TickMarkColor;
  XTickMarkLength := DiskRec.TickMarkLength;
  XShowTMLabels := DiskRec.LabelTickMarks;
  XTMLabelDecPlaces := DiskRec.LabelDecimalPlaces;
  {Y axis values}
  YShowAxisLines := DiskRec.ShowAxisLines;
  YAxisColor := DiskRec.AxisColor;
  YAxisWidth := DiskRec.AxisWidth;
  YAxisStyle := DiskRec.AxisStyle;
  if DiskRec.ShowTickMarks then
    YTickMarkType := tmTickMarks
  else
    YTickMarkType := tmNone;
  YTickMarkIncStr := DiskRec.TickMarkIncStr;
  YTickMarkIncVal := FloatToMathValue(DiskRec.TickMarkIncrements);
  YTickMarkColor := DiskRec.TickMarkColor;
  YTickMarkLength := DiskRec.TickMarkLength;
  YShowTMLabels := DiskRec.LabelTickMarks;
  YTMLabelDecPlaces := DiskRec.LabelDecimalPlaces;
  {general values}
  BackColor := DiskRec.BackColor;
  {TFont values}
  ReadGraphFile.Read16BitFont(XLabelFont);
  YLabelFont.Assign(XLabelFont);
  {fix and calc math strings}
  with ReadGraphFile do
    begin
      FixCalcMathConstString(XTickMarkIncStr, XTickMarkIncVal);
      FixCalcMathConstString(YTickMarkIncStr, YTickMarkIncVal);
    end; {with}
end; {procedure ReadVer1Fields;}

{----------------------------------}

procedure ReadVer2Fields;
var
  DiskRec : TDiskV2_Setting2DRec;
begin {procedure ReadVer2Fields;}
  ReadGraphFile.ReadRecord(DiskRec, sizeof(DiskRec));
  {X axis values}
  XShowAxisLines := DiskRec.XShowAxisLines;
  XLimitLoAxisEmpty := DiskRec.XLimitLoAxisEmpty;
  XLimitLoAxisVal := FloatToMathValue(DiskRec.XLimitLoAxisVal);
  XLimitHiAxisEmpty := DiskRec.XLimitHiAxisEmpty;
  XLimitHiAxisVal := FloatToMathValue(DiskRec.XLimitHiAxisVal);
  XAxisColor := DiskRec.XAxisColor;
  XAxisWidth := DiskRec.XAxisWidth;
  XAxisStyle := DiskRec.XAxisStyle;
  XTickMarkType := DiskRec.XTickMarkType;
  XTickMarkColor := DiskRec.XTickMarkColor;
  XTickMarkIncVal := FloatToMathValue(DiskRec.XTickMarkIncVal);
  XTickMarkLength := DiskRec.XTickMarkLength;
  XGridLineStyle := DiskRec.XGridLineStyle;
  XShowTMLabels := DiskRec.XShowTMLabels;
  XTMLabelFormat := DiskRec.XTMLabelFormat;
  XShowTMLabelComa := DiskRec.XShowTMLabelComa;
  XShowTMLabelPlus := DiskRec.XShowTMLabelPlus;
  XTMLabelDecPlaces := DiskRec.XTMLabelDecPlaces;
  {Y axis values}
  YShowAxisLines := DiskRec.YShowAxisLines;
  YLimitLoAxisEmpty := DiskRec.YLimitLoAxisEmpty;
  YLimitLoAxisVal := FloatToMathValue(DiskRec.YLimitLoAxisVal);
  YLimitHiAxisEmpty := DiskRec.YLimitHiAxisEmpty;
  YLimitHiAxisVal := FloatToMathValue(DiskRec.YLimitHiAxisVal);
  YAxisColor := DiskRec.YAxisColor;
  YAxisWidth := DiskRec.YAxisWidth;
  YAxisStyle := DiskRec.YAxisStyle;
  YTickMarkType := DiskRec.YTickMarkType;
  YTickMarkColor := DiskRec.YTickMarkColor;
  YTickMarkIncVal := FloatToMathValue(DiskRec.YTickMarkIncVal);
  YTickMarkLength := DiskRec.YTickMarkLength;
  YGridLineStyle := DiskRec.YGridLineStyle;
  YShowTMLabels := DiskRec.YShowTMLabels;
  YTMLabelFormat := DiskRec.YTMLabelFormat;
  YShowTMLabelComa := DiskRec.YShowTMLabelComa;
  YShowTMLabelPlus := DiskRec.YShowTMLabelPlus;
  YTMLabelDecPlaces := DiskRec.YTMLabelDecPlaces;
  {general values}
  BackColor := DiskRec.BackColor;
  with ReadGraphFile do
    begin
      //read X
      ReadMathConstString(XLimitLoAxisStr, XLimitLoAxisVal, XLimitLoAxisEmpty);
      ReadMathConstString(XLimitHiAxisStr, XLimitHiAxisVal, XLimitHiAxisEmpty);
      ReadMathConstString(XTickMarkIncStr, XTickMarkIncVal, False);
      //read Y
      ReadMathConstString(YLimitLoAxisStr, YLimitLoAxisVal, YLimitLoAxisEmpty);
      ReadMathConstString(YLimitHiAxisStr, YLimitHiAxisVal, YLimitHiAxisEmpty);
      ReadMathConstString(YTickMarkIncStr, YTickMarkIncVal, False);
      {TFont values}
      Read32BitFont(XLabelFont);
      Read32BitFont(YLabelFont);
    end; {with}
end; {procedure ReadVer2Fields;}

{----------------------------------}

procedure ReadVer3Fields;
var
  DiskRec : TDiskV3_Setting2DRec;
begin {procedure ReadVer3Fields;}
  ReadGraphFile.ReadRecord(DiskRec, sizeof(DiskRec));
  {X axis values}
  XShowAxisLines := DiskRec.XShowAxisLines;
  XLimitLoAxisEmpty := DiskRec.XLimitLoAxisEmpty;
  XLimitLoAxisVal := UnPackMathValue(DiskRec.XLimitLoAxisVal);
  XLimitHiAxisEmpty := DiskRec.XLimitHiAxisEmpty;
  XLimitHiAxisVal := UnPackMathValue(DiskRec.XLimitHiAxisVal);
  XAxisColor := DiskRec.XAxisColor;
  XAxisWidth := DiskRec.XAxisWidth;
  XAxisStyle := DiskRec.XAxisStyle;
  XTickMarkType := DiskRec.XTickMarkType;
  XTickMarkColor := DiskRec.XTickMarkColor;
  XTickMarkIncVal := UnPackMathValue(DiskRec.XTickMarkIncVal);
  XTickMarkLength := DiskRec.XTickMarkLength;
  XGridLineStyle := DiskRec.XGridLineStyle;
  XShowTMLabels := DiskRec.XShowTMLabels;
  XTMLabelFormat := DiskRec.XTMLabelFormat;
  XShowTMLabelComa := DiskRec.XShowTMLabelComa;
  XShowTMLabelPlus := DiskRec.XShowTMLabelPlus;
  XTMLabelDecPlaces := DiskRec.XTMLabelDecPlaces;
  {Y axis values}
  YShowAxisLines := DiskRec.YShowAxisLines;
  YLimitLoAxisEmpty := DiskRec.YLimitLoAxisEmpty;
  YLimitLoAxisVal := UnPackMathValue(DiskRec.YLimitLoAxisVal);
  YLimitHiAxisEmpty := DiskRec.YLimitHiAxisEmpty;
  YLimitHiAxisVal := UnPackMathValue(DiskRec.YLimitHiAxisVal);
  YAxisColor := DiskRec.YAxisColor;
  YAxisWidth := DiskRec.YAxisWidth;
  YAxisStyle := DiskRec.YAxisStyle;
  YTickMarkType := DiskRec.YTickMarkType;
  YTickMarkColor := DiskRec.YTickMarkColor;
  YTickMarkIncVal := UnPackMathValue(DiskRec.YTickMarkIncVal);
  YTickMarkLength := DiskRec.YTickMarkLength;
  YGridLineStyle := DiskRec.YGridLineStyle;
  YShowTMLabels := DiskRec.YShowTMLabels;
  YTMLabelFormat := DiskRec.YTMLabelFormat;
  YShowTMLabelComa := DiskRec.YShowTMLabelComa;
  YShowTMLabelPlus := DiskRec.YShowTMLabelPlus;
  YTMLabelDecPlaces := DiskRec.YTMLabelDecPlaces;
  {general values}
  BackColor := DiskRec.BackColor;
  with ReadGraphFile do
    begin
      //read X
      ReadMathConstString(XLimitLoAxisStr, XLimitLoAxisVal, XLimitLoAxisEmpty);
      ReadMathConstString(XLimitHiAxisStr, XLimitHiAxisVal, XLimitHiAxisEmpty);
      ReadMathConstString(XTickMarkIncStr, XTickMarkIncVal, False);
      //read Y
      ReadMathConstString(YLimitLoAxisStr, YLimitLoAxisVal, YLimitLoAxisEmpty);
      ReadMathConstString(YLimitHiAxisStr, YLimitHiAxisVal, YLimitHiAxisEmpty);
      ReadMathConstString(YTickMarkIncStr, YTickMarkIncVal, False);
      {TFont values}
      Read32BitFont(XLabelFont);
      Read32BitFont(YLabelFont);
    end; {with}
end; {procedure ReadVer3Fields;}

{----------------------------------}

begin {procedure TSetting2DObj.ReadFromDisk}
  DidFieldsChange := true;
  if ReadGraphFile.FileVersion = 1 then
    ReadVer1Fields
  else
    if ReadGraphFile.FileVersion = 2 then
      ReadVer2Fields
    else {if ReadGraphFile.FileVersion >= 3 then}
      ReadVer3Fields;
end; {procedure TSetting2DObj.ReadFromDisk}

{---------------------------------------------------------------------}

procedure TSetting2DObj.Assign(Source: TSetting2DObj);
begin {procedure TSetting2DObj.Assign}
  {X axis values}
  XShowAxisLines := Source.XShowAxisLines;
  XLimitLoAxisStr := Source.XLimitLoAxisStr;
  XLimitLoAxisEmpty := Source.XLimitLoAxisEmpty;
  XLimitLoAxisVal := Source.XLimitLoAxisVal;
  XLimitHiAxisStr := Source.XLimitHiAxisStr;
  XLimitHiAxisEmpty := Source.XLimitHiAxisEmpty;
  XLimitHiAxisVal := Source.XLimitHiAxisVal;
  XAxisColor := Source.XAxisColor;
  XAxisWidth := Source.XAxisWidth;
  XAxisStyle := Source.XAxisStyle;
  XTickMarkType := Source.XTickMarkType;
  XTickMarkColor := Source.XTickMarkColor;
  XTickMarkIncStr := Source.XTickMarkIncStr;
  XTickMarkIncVal := Source.XTickMarkIncVal;
  XTickMarkLength := Source.XTickMarkLength;
  XGridLineStyle := Source.XGridLineStyle;
  XShowTMLabels := Source.XShowTMLabels;
  XTMLabelFormat := Source.XTMLabelFormat;
  XShowTMLabelComa := Source.XShowTMLabelComa;
  XShowTMLabelPlus := Source.XShowTMLabelPlus;
  XTMLabelDecPlaces := Source.XTMLabelDecPlaces;
  {Y axis values}
  YShowAxisLines := Source.YShowAxisLines;
  YLimitLoAxisStr := Source.YLimitLoAxisStr;
  YLimitLoAxisEmpty := Source.YLimitLoAxisEmpty;
  YLimitLoAxisVal := Source.YLimitLoAxisVal;
  YLimitHiAxisStr := Source.YLimitHiAxisStr;
  YLimitHiAxisEmpty := Source.YLimitHiAxisEmpty;
  YLimitHiAxisVal := Source.YLimitHiAxisVal;
  YAxisColor := Source.YAxisColor;
  YAxisWidth := Source.YAxisWidth;
  YAxisStyle := Source.YAxisStyle;
  YTickMarkType := Source.YTickMarkType;
  YTickMarkColor := Source.YTickMarkColor;
  YTickMarkIncStr := Source.YTickMarkIncStr;
  YTickMarkIncVal := Source.YTickMarkIncVal;
  YTickMarkLength := Source.YTickMarkLength;
  YGridLineStyle := Source.YGridLineStyle;
  YShowTMLabels := Source.YShowTMLabels;
  YTMLabelFormat := Source.YTMLabelFormat;
  YShowTMLabelComa := Source.YShowTMLabelComa;
  YShowTMLabelPlus := Source.YShowTMLabelPlus;
  YTMLabelDecPlaces := Source.YTMLabelDecPlaces;
  {general values}
  BackColor := Source.BackColor;
  {TFont values}
  XLabelFont.Assign(Source.XLabelFont);
  YLabelFont.Assign(Source.YLabelFont);
  {Make Memory Unique}
  {X axis values}
  UniqueString(XLimitLoAxisStr);
  UniqueString(XLimitHiAxisStr);
  UniqueString(XTickMarkIncStr);
  {Y axis values}
  UniqueString(YLimitLoAxisStr);
  UniqueString(YLimitHiAxisStr);
  UniqueString(YTickMarkIncStr);
end; {procedure TSetting2DObj.Assign}

{---------------------------------------------------------------------}

end.
