unit Set3dObj;

(*
Copyright 1999-2011 Greg VanMullem
This program is distributed under the terms of the GNU General Public License Version 3

This file is part of MathGV.

MathGV is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

MathGV is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MathGV.  If not, see <http://www.gnu.org/licenses/>.
*)

interface

uses FileSave, Graphics, DefFont;

type
  TSetting3DObj = class
    public
    DidFieldsChange : boolean;
    BackGroundColor : TColor;
    {X Axis Settings}
    XShowAxisLine : boolean;
    XAxisColor : TColor;
    XLabelPos : boolean;
    XLabelNeg : boolean;
    XShowPlusSigns : boolean;
    XShowMinusSigns : boolean;
    XLabelFont : TFont;
    {Y Axis Settings}
    YShowAxisLine : boolean;
    YAxisColor : TColor;
    YLabelPos : boolean;
    YLabelNeg : boolean;
    YShowPlusSigns : boolean;
    YShowMinusSigns : boolean;
    YLabelFont : TFont;
    {Z Axis Settings}
    ZShowAxisLine : boolean;
    ZAxisColor : TColor;
    ZLabelPos : boolean;
    ZLabelNeg : boolean;
    ZShowPlusSigns : boolean;
    ZShowMinusSigns : boolean;
    ZLabelFont : TFont;
    constructor Create;
    destructor Destroy; override;
    procedure SaveToDisk(WriteGraphFile : TWriteGraphFile);
    procedure ReadFromDisk(ReadGraphFile : TReadGraphFile);
    procedure Assign(Source : TSetting3DObj);
  end; {TSetting3DObj}

implementation

type
  TDiskV2_Setting3DRec = packed record
    BackGroundColor : TColor;
    {X Axis Settings}
    XShowAxisLine : boolean;
    XAxisColor : TColor;
    XLabelPos : boolean;
    XLabelNeg : boolean;
    XShowPlusSigns : boolean;
    XShowMinusSigns : boolean;
    {Y Axis Settings}
    YShowAxisLine : boolean;
    YAxisColor : TColor;
    YLabelPos : boolean;
    YLabelNeg : boolean;
    YShowPlusSigns : boolean;
    YShowMinusSigns : boolean;
    {Z Axis Settings}
    ZShowAxisLine : boolean;
    ZAxisColor : TColor;
    ZLabelPos : boolean;
    ZLabelNeg : boolean;
    ZShowPlusSigns : boolean;
    ZShowMinusSigns : boolean;
  end; {TDiskV2_Setting3DRec = packed record}

  TDiskV1_Setting3DRec = packed record
    BackGroundColor : TColor;
    ShowXLine,
    ShowYLine,
    ShowZLine : boolean;
    AxisLinesColor : TColor;
    LabelPosX,
    LabelNegX,
    LabelPosY,
    LabelNegY,
    LabelPosZ,
    LabelNegZ : boolean;
    ShowPlusSigns,
    ShowMinusSigns : boolean;
  end; {TDiskV1_Setting3DRec = packed record}

{---------------------------------------------------------------------}

{ TSetting3DObj }

constructor TSetting3DObj.Create;
begin {constructor TSetting3DObj.Create;}
  inherited Create;
  DidFieldsChange := false;
  BackGroundColor := clwhite;
  {X Axis Settings}
  XShowAxisLine := true;
  XAxisColor := clBlack;
  XLabelPos := true;
  XLabelNeg := true;
  XShowPlusSigns := true;
  XShowMinusSigns := true;
  {Y Axis Settings}
  YShowAxisLine := true;
  YAxisColor := clBlack;
  YLabelPos := true;
  YLabelNeg := true;
  YShowPlusSigns := true;
  YShowMinusSigns := true;
  {Z Axis Settings}
  ZShowAxisLine := true;
  ZAxisColor := clBlack;
  ZLabelPos := true;
  ZLabelNeg := true;
  ZShowPlusSigns := true;
  ZShowMinusSigns := true;
  {initalize TFont classes}
  XLabelFont := CreateANewTFontClass;
  YLabelFont := CreateANewTFontClass;
  ZLabelFont := CreateANewTFontClass;
end; {constructor TSetting3DObj.Create;}

{---------------------------------------------------------------------}

destructor TSetting3DObj.Destroy;
begin {destructor TSetting3DObj.Destroy;}
  XLabelFont.Free;
  YLabelFont.Free;
  ZLabelFont.Free;
  inherited Destroy;
end; {destructor TSetting3DObj.Destroy;}

{---------------------------------------------------------------------}

procedure TSetting3DObj.SaveToDisk(WriteGraphFile: TWriteGraphFile);
var
  DiskRec : TDiskV2_Setting3DRec;
begin {procedure TSetting3DObj.SaveToDisk}
  DiskRec.BackGroundColor := BackGroundColor;
  {X Axis Settings}
  DiskRec.XShowAxisLine := XShowAxisLine;
  DiskRec.XAxisColor := XAxisColor;
  DiskRec.XLabelPos := XLabelPos;
  DiskRec.XLabelNeg := XLabelNeg;
  DiskRec.XShowPlusSigns := XShowPlusSigns;
  DiskRec.XShowMinusSigns := XShowMinusSigns;
  {Y Axis Settings}
  DiskRec.YShowAxisLine := YShowAxisLine;
  DiskRec.YAxisColor := YAxisColor;
  DiskRec.YLabelPos := YLabelPos;
  DiskRec.YLabelNeg := YLabelNeg;
  DiskRec.YShowPlusSigns := YShowPlusSigns;
  DiskRec.YShowMinusSigns := YShowMinusSigns;
  {Z Axis Settings}
  DiskRec.ZShowAxisLine := ZShowAxisLine;
  DiskRec.ZAxisColor := ZAxisColor;
  DiskRec.ZLabelPos := ZLabelPos;
  DiskRec.ZLabelNeg := ZLabelNeg;
  DiskRec.ZShowPlusSigns := ZShowPlusSigns;
  DiskRec.ZShowMinusSigns := ZShowMinusSigns;
  with WriteGraphFile do
    begin
      WriteRecord(DiskRec, sizeof(DiskRec));
      WriteFont(XLabelFont);
      WriteFont(YLabelFont);
      WriteFont(ZLabelFont);
    end; {with}
end; {procedure TSetting3DObj.SaveToDisk}

{---------------------------------------------------------------------}

procedure TSetting3DObj.ReadFromDisk(ReadGraphFile: TReadGraphFile);

{-------------------------------------}

procedure ReadVer1Fields;
var
  DiskRec : TDiskV1_Setting3DRec;
begin {procedure ReadVer1Fields;}
  ReadGraphFile.ReadRecord(DiskRec, sizeof(DiskRec));
  BackGroundColor := DiskRec.BackGroundColor;
  {X Axis Settings}
  XShowAxisLine := DiskRec.ShowXLine;
  XAxisColor := DiskRec.AxisLinesColor;
  XLabelPos := DiskRec.LabelPosX;
  XLabelNeg := DiskRec.LabelNegX;
  XShowPlusSigns := DiskRec.ShowPlusSigns;
  XShowMinusSigns := DiskRec.ShowMinusSigns;
  {Y Axis Settings}
  YShowAxisLine := DiskRec.ShowYLine;
  YAxisColor := DiskRec.AxisLinesColor;
  YLabelPos := DiskRec.LabelPosY;
  YLabelNeg := DiskRec.LabelNegY;
  YShowPlusSigns := DiskRec.ShowPlusSigns;
  YShowMinusSigns := DiskRec.ShowMinusSigns;
  {Z Axis Settings}
  ZShowAxisLine := DiskRec.ShowZLine;
  ZAxisColor := DiskRec.AxisLinesColor;
  ZLabelPos := DiskRec.LabelPosZ;
  ZLabelNeg := DiskRec.LabelNegZ;
  ZShowPlusSigns := DiskRec.ShowPlusSigns;
  ZShowMinusSigns := DiskRec.ShowMinusSigns;
  {read fonts}
  ReadGraphFile.Read16BitFont(XLabelFont);
  YLabelFont.Assign(XLabelFont);
  ZLabelFont.Assign(XLabelFont);
end; {procedure ReadVer1Fields;}

{-------------------------------------}

procedure ReadVer2Fields;
var
  DiskRec : TDiskV2_Setting3DRec;
begin {procedure ReadVer2Fields;}
  ReadGraphFile.ReadRecord(DiskRec, sizeof(DiskRec));
  BackGroundColor := DiskRec.BackGroundColor;
  {X Axis Settings}
  XShowAxisLine := DiskRec.XShowAxisLine;
  XAxisColor := DiskRec.XAxisColor;
  XLabelPos := DiskRec.XLabelPos;
  XLabelNeg := DiskRec.XLabelNeg;
  XShowPlusSigns := DiskRec.XShowPlusSigns;
  XShowMinusSigns := DiskRec.XShowMinusSigns;
  {Y Axis Settings}
  YShowAxisLine := DiskRec.YShowAxisLine;
  YAxisColor := DiskRec.YAxisColor;
  YLabelPos := DiskRec.YLabelPos;
  YLabelNeg := DiskRec.YLabelNeg;
  YShowPlusSigns := DiskRec.YShowPlusSigns;
  YShowMinusSigns := DiskRec.YShowMinusSigns;
  {Z Axis Settings}
  ZShowAxisLine := DiskRec.ZShowAxisLine;
  ZAxisColor := DiskRec.ZAxisColor;
  ZLabelPos := DiskRec.ZLabelPos;
  ZLabelNeg := DiskRec.ZLabelNeg;
  ZShowPlusSigns := DiskRec.ZShowPlusSigns;
  ZShowMinusSigns := DiskRec.ZShowMinusSigns;
  with ReadGraphFile do
    begin
      Read32BitFont(XLabelFont);
      Read32BitFont(YLabelFont);
      Read32BitFont(ZLabelFont);
    end; {with}
end; {procedure ReadVer2Fields;}

{-------------------------------------}

begin {procedure TSetting3DObj.ReadFromDisk}
  DidFieldsChange := true;
  if ReadGraphFile.FileVersion = 1 then
    ReadVer1Fields
  else {if (FileVersion >= 2) then}
    ReadVer2Fields;
end; {procedure TSetting3DObj.ReadFromDisk}

{---------------------------------------------------------------------}

procedure TSetting3DObj.Assign(Source: TSetting3DObj);
begin {procedure TSetting3DObj.Assign}
  BackGroundColor := Source.BackGroundColor;
  {X Axis Settings}
  XShowAxisLine := Source.XShowAxisLine;
  XAxisColor := Source.XAxisColor;
  XLabelPos := Source.XLabelPos;
  XLabelNeg := Source.XLabelNeg;
  XShowPlusSigns := Source.XShowPlusSigns;
  XShowMinusSigns := Source.XShowMinusSigns;
  XLabelFont.Assign(Source.XLabelFont);
  {Y Axis Settings}
  YShowAxisLine := Source.YShowAxisLine;
  YAxisColor := Source.YAxisColor;
  YLabelPos := Source.YLabelPos;
  YLabelNeg := Source.YLabelNeg;
  YShowPlusSigns := Source.YShowPlusSigns;
  YShowMinusSigns := Source.YShowMinusSigns;
  YLabelFont.Assign(Source.YLabelFont);
  {Z Axis Settings}
  ZShowAxisLine := Source.ZShowAxisLine;
  ZAxisColor := Source.ZAxisColor;
  ZLabelPos := Source.ZLabelPos;
  ZLabelNeg := Source.ZLabelNeg;
  ZShowPlusSigns := Source.ZShowPlusSigns;
  ZShowMinusSigns := Source.ZShowMinusSigns;
  ZLabelFont.Assign(Source.ZLabelFont);
end; {procedure TSetting3DObj.Assign}

{---------------------------------------------------------------------}

end.
