unit SetPoDlg;

(*
Copyright 1999-2011 Greg VanMullem
This program is distributed under the terms of the GNU General Public License Version 3

This file is part of MathGV.

MathGV is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

MathGV is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MathGV.  If not, see <http://www.gnu.org/licenses/>.
*)

interface

uses sysutils, WinTypes, WinProcs, Classes, Graphics, Forms, Controls, Buttons,
     StdCtrls, ExtCtrls, Clsample, Spin, ComnDlgs, CentrDlg, Dialogs,
     MathCalc, MathFunc, FloatStr, EditMath, SetPoObj, DefFont;

type
  TSettingsPolarDlg = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    HelpBtn: TBitBtn;
    BackColorBox: TGroupBox;
    BackColorButton: TButton;
    EBackColorSample: TColorSample;
    AxisBox: TGroupBox;
    EAxisList: TMemo;
    Label1: TLabel;
    AxisColorButton: TButton;
    EAxisColorSample: TColorSample;
    Label2: TLabel;
    EAxisWidth: TScrollBar;
    EAxisWidthLabel: TLabel;
    AxisStyleBox: TGroupBox;
    ASolidPen: TSpeedButton;
    ADashPen: TSpeedButton;
    ADotPen: TSpeedButton;
    ADashDotPen: TSpeedButton;
    ADashDotDotPen: TSpeedButton;
    TickMarkBox: TGroupBox;
    Label3: TLabel;
    TickMarkColorButton: TButton;
    ETickMarkColorSample: TColorSample;
    TickTypeBox: TGroupBox;
    EDashes: TRadioButton;
    ECircles: TRadioButton;
    LabelBox: TGroupBox;
    ELabelTickMarks: TCheckBox;
    TextFieldTMLabelDP: TLabel;
    ELabelDecimalPlaces: TSpinEdit;
    LabelFontButton: TButton;
    SizeTypeLabel: TLabel;
    ETickMarkSize: TScrollBar;
    ETMSizeLabel: TLabel;
    CricleStyleBox: TGroupBox;
    CSolidPen: TSpeedButton;
    CDashPen: TSpeedButton;
    CDotPen: TSpeedButton;
    CDashDotPen: TSpeedButton;
    CDashDotDotPen: TSpeedButton;
    Button1: TButton;
    ENoTicks: TRadioButton;
    RaidusBox1: TGroupBox;
    ERestrict: TCheckBox;
    ETickMarkIncrements: TEditMathConst;
    ERestrictAmt: TEditMathConst;
    EShowTMLabelComa: TCheckBox;
    ETMLabelFormat: TComboBox;
    EShowTMLabelPlus: TCheckBox;
    procedure EAxisWidthChange(Sender: TObject);
    procedure ETickMarkSizeChange(Sender: TObject);
    procedure EDashesExit(Sender: TObject);
    procedure EDashesEnter(Sender: TObject);
    procedure EDashesClick(Sender: TObject);
    procedure EDashesKeyPress(Sender: TObject; var Key: Char);
    procedure AxisColorButtonClick(Sender: TObject);
    procedure TickMarkColorButtonClick(Sender: TObject);
    procedure BackColorButtonClick(Sender: TObject);
    procedure LabelFontButtonClick(Sender: TObject);
    procedure EAxisListExit(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure ETMLabelFormatChange(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
  private
    { Private declarations }
    FLabelFont : TFont;
    FAxisCount : integer;
    FAxisAngles : TPolarAxisAngles;
    OriginalAxisCount : integer;
    OriginalAxisAngles : PPolarAxisAngles;
    FTickMarkType : TTickMarkType;
    FDashLength : integer;
    FCircleWidth : integer;
    procedure UpdateSLabel;
    procedure SortAndDelDupeAngles;
    function GetVerifyEditLines : boolean;
    procedure EnableLabelProperties;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SetTheFields(ThePolarObj : TSettingPolarObj);
    procedure GetTheFields(ThePolarObj : TSettingPolarObj);
  end;

function ModifyGraphPolarSettings(ThePolarObj : TSettingPolarObj) : boolean;

implementation

uses AxisList;

{$R *.DFM}

{---------------------------------------------------------------------}

function ModifyGraphPolarSettings(ThePolarObj : TSettingPolarObj) : boolean;
var
  SettingsPolarDlg: TSettingsPolarDlg;
begin {function ModifyGraphPolarSettings}
  Result := false;
  SettingsPolarDlg := TSettingsPolarDlg.Create(Application);
  try
    SetDlgDefaultPos(SettingsPolarDlg, nil);
    SettingsPolarDlg.SetTheFields(ThePolarObj);
    if SettingsPolarDlg.ShowModal = IDOK then
      begin
        Result := true;
        SettingsPolarDlg.GetTheFields(ThePolarObj);
        ThePolarObj.DidFieldsChange := true;
      end { if Result = true then}
  finally
    SettingsPolarDlg.Free;
  end; {try..finally}
end; {function ModifyGraphPolarSettings}

{---------------------------------------------------------------------}

constructor TSettingsPolarDlg.Create(AOwner: TComponent);
var
  I : integer;
begin {constructor TSettingsPolarDlg.Create}
  inherited Create(AOwner);
  FLabelFont := CreateANewTFontClass;
  for I := 1 to NumLabelFormatCount do
    ETMLabelFormat.Items.Append(NumLabelFormatStrArray[I]);
end; {constructor TSettingsPolarDlg.Create}

{---------------------------------------------------------------------}

destructor TSettingsPolarDlg.Destroy;
begin {destructor TSettingsPolarDlg.Destroy;}
  FLabelFont.Free;
  inherited destroy;
end; {destructor TSettingsPolarDlg.Destroy;}

{---------------------------------------------------------------------}

procedure TSettingsPolarDlg.UpdateSLabel;
begin {procedure TSettingsPolarDlg.UpdateSLabel;}
  if FTickMarkType = Dashes then
    FDashLength := ETickMarkSize.Position
  else
    if FTickMarkType = Circles then
      FCircleWidth := ETickMarkSize.Position;
  if EDashes.Checked then
    begin
      SizeTypeLabel.Caption := SizeDashStr;
      ETickMarkSize.Position := FDashLength;
      FTickMarkType := Dashes;
    end
  else
    if ECircles.Checked then
      begin
        SizeTypeLabel.Caption := SizeCircleStr;
        ETickMarkSize.Position := FCircleWidth;
        FTickMarkType := Circles;
      end
    else
      begin
        SizeTypeLabel.Caption := SizeNoTicksStr;
        ETickMarkSize.Position := 1;
        FTickMarkType := NoTicks;
      end;
end; {procedure TSettingsPolarDlg.UpdateSLabel;}

{---------------------------------------------------------------------}

procedure TSettingsPolarDlg.SortAndDelDupeAngles;
{Bubble sort AxisAngles Array}
var
  DoneSort : boolean;
  I : integer;
  Swap : TMathFloat;

{-----------------------------}

function SearchDupe : integer;
var
  Count : integer;
begin {function SearchDupe}
  Result := -1;
  for Count := 1 to (FAxisCount-1) do
    begin
      if FAxisAngles[Count] = FAxisAngles[Count+1] then
        begin
          Result := Count;
          exit;
        end; {if}
    end; {for}
end; {function SearchDupe}

{-----------------------------}

procedure DeleteDupe(Ndx : integer);
var
  Count : integer;
begin {function DeleteDupe}
  for Count := Ndx to (FAxisCount-1) do
    begin
      FAxisAngles[Count] := FAxisAngles[Count+1];
    end; {for}
  dec(FAxisCount);
end; {function DeleteDupe}

{-----------------------------}

begin {procedure TSettingsPolarDlg.SortAndDelDupeAngles;}
  if FAxisCount >= 2 then
    begin
      {Bubble Sort}
      repeat
        DoneSort := true;
        for I := 1 to (FAxisCount-1) do
          begin
            if FAxisAngles[I] > FAxisAngles[I+1] then
              begin
                Swap := FAxisAngles[I];
                FAxisAngles[I] := FAxisAngles[I+1];
                FAxisAngles[I+1] := Swap;
                DoneSort := false;
              end;
          end; {for}
        until DoneSort;
      {Remove Dupes}
      I := SearchDupe;
      while (FAxisCount >= 2) and (I <> -1) do
        begin
          DeleteDupe(I);
          I := SearchDupe;
        end; {while}
    end; {if}
end; {procedure TSettingsPolarDlg.SortAndDelDupeAngles;}

{---------------------------------------------------------------------}

procedure TSettingsPolarDlg.SetTheFields(ThePolarObj : TSettingPolarObj);
var
  I : integer;
begin {procedure TSettingsPolarDlg.SetTheFields}
  with ThePolarObj do
  begin
    EBackColorSample.ColorValue := BackColor;
    OriginalAxisCount := AxisCount;
    OriginalAxisAngles := @AxisAngles;
    FAxisCount := AxisCount;
    FAxisAngles := AxisAngles;
    EAxisList.Lines.clear;
    for I := 1 to AxisCount do
      EAxisList.Lines.Add(PositiveRotToStr(AxisAngles[I]));
    EAxisColorSample.ColorValue := AxisColor;
    EAxisWidth.Position := AxisWidth;
    case AxisStyle of
      psSolid : ASolidPen.Down := true;
      psDash : ADashPen.Down := true;
      psDot : ADotPen.Down := true;
      psDashDot : ADashDotPen.Down := true;
      psDashDotDot : ADashDotDotPen.Down := true;
    end; {case}
    ETickMarkIncrements.text := TickMarkIncStr;
    FTickMarkType := TickMarkType;
    if TickMarkType = Dashes then
      begin
        ETickMarkSize.Position := DashLength;
        SizeTypeLabel.Caption := SizeDashStr;
        EDashes.Checked := true;
      end
    else
      if TickMarkType = Circles then
        begin
          ETickMarkSize.Position := CircleWidth;
          SizeTypeLabel.Caption := SizeCircleStr;
          ECircles.Checked := true;
        end
      else
        begin
          ETickMarkSize.Position := 1;
          SizeTypeLabel.Caption := SizeNoTicksStr;
          ENoTicks.Checked := true;
        end; {else}
    FDashLength := DashLength;
    FCircleWidth := CircleWidth;
    ETickMarkColorSample.ColorValue := TickMarkColor;
    case CircleStyle of
      psSolid : CSolidPen.Down := true;
      psDash : CDashPen.Down := true;
      psDot : CDotPen.Down := true;
      psDashDot : CDashDotPen.Down := true;
      psDashDotDot : CDashDotDotPen.Down := true;
    end; {case}
    ELabelTickMarks.Checked := LabelTickMarks;
    ETMLabelFormat.ItemIndex := TFloatNumFormatToCombo(TMLabelFormat);
    EShowTMLabelComa.Checked := ShowTMLabelComa;
    EShowTMLabelPlus.Checked := ShowTMLabelPlus;
    ELabelDecimalPlaces.Value := LabelDecimalPlaces;
    ERestrict.checked := Restrict;
    ERestrictAmt.text := RestrictAmtStr;
    FLabelFont.Assign(LabelFont);
  end; {with}
  EnableLabelProperties;
end; {procedure TSettingsPolarDlg.SetTheFields}

{---------------------------------------------------------------------}

procedure TSettingsPolarDlg.GetTheFields(ThePolarObj : TSettingPolarObj);
begin {procedure TSettingsPolarDlg.GetTheFields}
  with ThePolarObj do
  begin
    BackColor := EBackColorSample.ColorValue;
    AxisCount := FAxisCount;
    AxisAngles := FAxisAngles;
    AxisColor := EAxisColorSample.ColorValue;
    AxisWidth := EAxisWidth.Position;
    if ASolidPen.Down = true then AxisStyle := psSolid
    else if ADashPen.Down = true then AxisStyle := psDash
    else if ADotPen.Down = true then AxisStyle := psDot
    else if ADashDotPen.Down = true then AxisStyle := psDashDot
    else if ADashDotDotPen.Down = true then AxisStyle := psDashDotDot
    else {catch any errors} AxisStyle := psSolid;
    TickMarkIncStr := ETickMarkIncrements.text;
    TickMarkIncrements := ETickMarkIncrements.LastMathValue;
    if EDashes.Checked then
      begin
        TickMarkType := Dashes;
        FDashLength := ETickMarkSize.Position;
      end
    else
      if ECircles.Checked then
        begin
          TickMarkType := Circles;
          FCircleWidth := ETickMarkSize.Position;
        end
      else
        TickMarkType := NoTicks;
    DashLength := FDashLength;
    CircleWidth := FCircleWidth;
    TickMarkColor := ETickMarkColorSample.ColorValue;
    if CSolidPen.Down = true then CircleStyle := psSolid
    else if CDashPen.Down = true then CircleStyle := psDash
    else if CDotPen.Down = true then CircleStyle := psDot
    else if CDashDotPen.Down = true then CircleStyle := psDashDot
    else if CDashDotDotPen.Down = true then CircleStyle := psDashDotDot
    else {catch any errors} CircleStyle := psSolid;
    LabelTickMarks := ELabelTickMarks.Checked;
    TMLabelFormat := ComboToTFloatNumFormat(ETMLabelFormat.ItemIndex);
    ShowTMLabelComa := EShowTMLabelComa.Checked;
    ShowTMLabelPlus := EShowTMLabelPlus.Checked;
    LabelDecimalPlaces := ELabelDecimalPlaces.Value;
    Restrict := ERestrict.checked;
    RestrictAmtStr := ERestrictAmt.text;
    RestrictAmtVal := ERestrictAmt.LastMathValue;
    LabelFont.Assign(FLabelFont);
  end; {with}
end; {procedure TSettingsPolarDlg.GetTheFields}

{---------------------------------------------------------------------}

function TSettingsPolarDlg.GetVerifyEditLines : boolean;
begin {function TSettingsPolarDlg.GetVerifyEditLines}
  Result := false;
  if ETickMarkIncrements.ValidateMathStr then
    if ERestrictAmt.ValidateMathStr then
      Result := true;
end; {function TSettingsPolarDlg.GetVerifyEditLines}

{---------------------------------------------------------------------}

procedure TSettingsPolarDlg.EnableLabelProperties;
var
  ComaCharEnabled : boolean;
  PlusSignCharEnabled : boolean;
  DecimalPlacesLabelText : string;
begin {procedure TSettingsPolarDlg.EnableLabelProperties;}
  GetFloatCordDispOptions(ComboToTFloatNumFormat(ETMLabelFormat.ItemIndex),
              ComaCharEnabled, PlusSignCharEnabled, DecimalPlacesLabelText);
  EShowTMLabelComa.Enabled := ComaCharEnabled;
  EShowTMLabelPlus.Enabled := PlusSignCharEnabled;
  TextFieldTMLabelDP.Caption := DecimalPlacesLabelText;
end; {procedure TSettingsPolarDlg.EnableLabelProperties;}

{---------------------------------------------------------------------}

procedure TSettingsPolarDlg.EAxisWidthChange(Sender: TObject);
begin
  EAxisWidthLabel.caption := IntToStr(EAxisWidth.Position);
end;

procedure TSettingsPolarDlg.ETickMarkSizeChange(Sender: TObject);
begin
  ETMSizeLabel.caption := IntToStr(ETickMarkSize.Position);
end;

procedure TSettingsPolarDlg.EDashesExit(Sender: TObject);
begin
  UpdateSLabel;
end;

procedure TSettingsPolarDlg.EDashesEnter(Sender: TObject);
begin
  UpdateSLabel;
end;

procedure TSettingsPolarDlg.EDashesClick(Sender: TObject);
begin
  UpdateSLabel;
end;

procedure TSettingsPolarDlg.EDashesKeyPress(Sender: TObject;
  var Key: Char);
begin
  UpdateSLabel;
end;

procedure TSettingsPolarDlg.AxisColorButtonClick(Sender: TObject);
begin
  EAxisColorSample.ColorValue := ReturnNewColorComnDlg(EAxisColorSample.ColorValue, Self);
end;

procedure TSettingsPolarDlg.TickMarkColorButtonClick(Sender: TObject);
begin
  ETickMarkColorSample.ColorValue := ReturnNewColorComnDlg(ETickMarkColorSample.ColorValue, Self);
end;

procedure TSettingsPolarDlg.BackColorButtonClick(Sender: TObject);
begin
  EBackColorSample.ColorValue := ReturnNewColorComnDlg(EBackColorSample.ColorValue, Self);
end;

procedure TSettingsPolarDlg.LabelFontButtonClick(Sender: TObject);
begin
  ShowComnFontDlg(FLabelFont, Self)
end;

{---------------------------------------------------------------------}

procedure TSettingsPolarDlg.EAxisListExit(Sender: TObject);
var
  I, MemoCount, TheAngleCount : integer;
  TmpVal : TMathFloat;
  DidError, DidReset : boolean;

{-----------------------------}

  procedure ConfirmReset;
  begin {procedure ConfirmReset;}
    if mrOk = MessageDlgCenter(
             'This will reset the axis list to original values.',
              mtConfirmation, [mbOK,mbCancel], 0, Self) then
      begin
        DidReset := true;
        FAxisCount := OriginalAxisCount;
        TheAngleCount := FAxisCount;
        FAxisAngles := OriginalAxisAngles^;
      end; {if}
  end; {procedure ConfirmReset;}

{-----------------------------}

begin {procedure TSettingsPolarDlg.EAxisListExit}
  DidError := false;
  DidReset := false;
  I := 0;
  TheAngleCount := 0;
  MemoCount := EAxisList.Lines.Count;
  while (I < MemoCount) and (TheAngleCount < MaxPolarAxisCount) and
        (not DidError) do
    begin
      if not StrToPositiveRot(EAxisList.Lines[I],TmpVal) then
        begin
          if EAxisList.Lines[I] <> '' then
            begin
              DidError := true;
              if mrAbort = MessageDlgCenter(
                 'Invalid numeric format in axis angle #'+IntToStr(I),
                  mtError, [mbOK,mbAbort], 0, Self) then
                ConfirmReset;
              EAxisList.SetFocus;
            end; {if}
        end
      else
        begin
          inc(TheAngleCount);
          FAxisAngles[TheAngleCount] := TmpVal;
        end; {else}
      inc(I);
    end; {while}
  if not(DidError and (not DidReset)) then
    begin
      FAxisCount := TheAngleCount;
      SortAndDelDupeAngles;
      EAxisList.Lines.Clear;
      for I := 1 to FAxisCount do
         EAxisList.Lines.Add(PositiveRotToStr(FAxisAngles[I]));
    end; {if}
end; {procedure TSettingsPolarDlg.EAxisListExit}

{---------------------------------------------------------------------}

procedure TSettingsPolarDlg.Button1Click(Sender: TObject);
var
  I : integer;
begin
  if ShowStandardAxisListDlg(FAxisCount, FAxisAngles, self) then
    begin
      EAxisList.Lines.Clear;
      for I := 1 to FAxisCount do
         EAxisList.Lines.Add(PositiveRotToStr(FAxisAngles[I]));
    end; {if}
end;

procedure TSettingsPolarDlg.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  CanClose := true;
  if ModalResult = IDOK then
    begin
      CanClose := GetVerifyEditLines;
    end; {if}
end;

procedure TSettingsPolarDlg.ETMLabelFormatChange(Sender: TObject);
begin
  EnableLabelProperties;
  ELabelDecimalPlaces.Value := GetFloatCordDecPlacesDefault(
                     ComboToTFloatNumFormat(ETMLabelFormat.ItemIndex));
end;

procedure TSettingsPolarDlg.FormKeyPress(Sender: TObject; var Key: Char);
begin
  FormKeyPressSpinEditClose(self, Key);
end;

end.
