unit ValueLst;

(*
Copyright 1999-2011 Greg VanMullem
This program is distributed under the terms of the GNU General Public License Version 3

This file is part of MathGV.

MathGV is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

MathGV is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MathGV.  If not, see <http://www.gnu.org/licenses/>.
*)

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, CentrDlg, func2D, MathCalc, MathFunc;

type
  TSpec2DValuesRowDlg = class(TForm)
    OKBtn: TBitBtn;
    HelpBtn: TBitBtn;
    Label1: TLabel;
    ValueListMemo: TMemo;
    FormatBox: TGroupBox;
    EDecimal: TRadioButton;
    EFraction: TRadioButton;
    EInteger: TRadioButton;
    WasTruncatedLabel: TLabel;
    procedure EFormatTypeClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    TheObj : TFunc2DSpecialItem;
    SpecialValueFunc : TMathFunction;
    function CalcValueList : boolean;
  end;

{var
  Spec2DValuesRowDlg: TSpec2DValuesRowDlg;}

procedure ShowSpec2DValuesRowDlg(TheObj : TFunc2DSpecialItem;
            SpecialValueFunc : TMathFunction; TheCenterOnForm : TForm);

implementation

{$R *.DFM}

{---------------------------------------------------------------------}

procedure ShowSpec2DValuesRowDlg(TheObj : TFunc2DSpecialItem;
            SpecialValueFunc : TMathFunction; TheCenterOnForm : TForm);
var
  Spec2DValuesRowDlg : TSpec2DValuesRowDlg;
begin {procedure ShowSpec2DValuesRowDlg}
  Spec2DValuesRowDlg := TSpec2DValuesRowDlg.Create(Application);
  try
    SetDlgDefaultPos(Spec2DValuesRowDlg, TheCenterOnForm);
    Spec2DValuesRowDlg.TheObj := TheObj;
    Spec2DValuesRowDlg.SpecialValueFunc := SpecialValueFunc;
    if Spec2DValuesRowDlg.CalcValueList then
      Spec2DValuesRowDlg.ShowModal;
  finally
    Spec2DValuesRowDlg.Free;
  end; {try..finally}
end; {procedure ShowSpec2DValuesRowDlg}

{---------------------------------------------------------------------}

{ TSpec2DValuesRowDlg }

function TSpec2DValuesRowDlg.CalcValueList : boolean;
var
  CurrPointRec : TMathValue;
  EDecimalIsChecked, EFractionIsChecked : boolean;
  S : string;

{------------------------------}

procedure AddCurrPointToArray;
var
  TmpStr : string;
begin {procedure AddCurrPointToArray}
  if S = '' then
    TmpStr := ''
  else
    TmpStr := ListSeparator + ' ';
  if EDecimalIsChecked then
    TmpStr := TmpStr + FloatToStr(CurrPointRec.FloatValue)
  else
    if EFractionIsChecked then
      begin
        if CurrPointRec.ValueType = vtFraction then
          TmpStr := TmpStr + IntToStr(CurrPointRec.Numerator) + '/' + IntToStr(CurrPointRec.Denominator)
        else
          TmpStr := TmpStr + FloatToStr(CurrPointRec.FloatValue);
      end
    else //if EInteger.Checked then
      begin
        if CurrPointRec.ValueType = vtFraction then
          begin
            TmpStr := TmpStr + IntToStr(CurrPointRec.Numerator);
            if CurrPointRec.Denominator <> 1 then
              TmpStr := TmpStr + '/' + IntToStr(CurrPointRec.Denominator)
          end
        else
          TmpStr := TmpStr + FloatToStr(CurrPointRec.FloatValue);
      end; {else}
  S := S + TmpStr;
end; {procedure AddCurrPointToArray}

{------------------------------}

procedure CalcSingleValue;
begin {procedure CalcSingleValue;}
  CurrPointRec := TheObj.FirstVal;
  AddCurrPointToArray;
end; {procedure CalcSingleValue;}

{------------------------------}

procedure CalcSeriesValues;
var
  Steps, Points : integer;
  Diff, Curr : TMathValue;

{--------------}

procedure DoAddCurrValue;
begin {procedure DoAddCurrValue}
  CurrPointRec := Curr;
  if TheObj.ItemType = itFuncSeries then
    begin
      SpecialValueFunc.MathVarValues[0] := Curr;
      if SpecialValueFunc.EvaluateMathFunc then
        CurrPointRec := SpecialValueFunc.MathResultValue;
    end; {if}
  AddCurrPointToArray;
end; {procedure DoAddCurrValue}

{--------------}

begin {procedure CalcSeriesValues;}
  Assert((TheObj.FirstVal.ValueType <> vtEmpty) and (TheObj.SecondVal.ValueType <> vtEmpty) and
         (TheObj.EndVal.ValueType <> vtEmpty), 'Empty series value');
  with TheObj do
    begin
      //parse and verify the function
      if ItemType = itFuncSeries then
        begin
          SpecialValueFunc.FuncString := TheObj.FunctionStr;
          if not SpecialValueFunc.VerifyMathFunc then
            begin
              Result := false;
              Exit;
            end; {if}
        end; {if}
      //add first, second and end values
      Curr := FirstVal;
      DoAddCurrValue;
      Curr := SecondVal;
      DoAddCurrValue;
      if ((FirstVal.FloatValue < SecondVal.FloatValue) and (SecondVal.FloatValue < EndVal.FloatValue)) then
        begin
          Points := 3;
          //Diff := abs(SecondVal - FirstVal);
          Diff := AbsFuncMathValue(SubMathValue(SecondVal, FirstVal));
          Steps := 1;
          //Curr := SecondVal.FloatValue + (Diff * Steps);
          Curr := AddMathValue(SecondVal, MulMathValue(Diff, LongintToMathValue(Steps)));
          while Curr.FloatValue < EndVal.FloatValue do
            begin
              inc(Points);
              if Points > (MaxPointsPerSeries + 32) then
                Break; //point count to large
              DoAddCurrValue;
              inc(Steps);
              //Curr := SecondVal.FloatValue + (Diff * Steps);
              Curr := AddMathValue(SecondVal, MulMathValue(Diff, LongintToMathValue(Steps)));
            end; {while}
        end
      else
        if ((FirstVal.FloatValue > SecondVal.FloatValue) and (SecondVal.FloatValue > EndVal.FloatValue)) then
          begin
            Points := 3;
            //Diff := abs(FirstVal.FloatValue - SecondVal.FloatValue);
            Diff := AbsFuncMathValue(SubMathValue(FirstVal, SecondVal));
            Steps := 1;
            //Curr := SecondVal.FloatValue - (Diff * Steps);
            Curr := SubMathValue(SecondVal, MulMathValue(Diff, LongintToMathValue(Steps)));
            while Curr.FloatValue > EndVal.FloatValue do
              begin
                inc(Points);
                if Points > (MaxPointsPerSeries + 32) then
                  Break; //point count to large
                DoAddCurrValue;
                inc(Steps);
                //Curr := SecondVal.FloatValue - (Diff * Steps);
                Curr := SubMathValue(SecondVal, MulMathValue(Diff, LongintToMathValue(Steps)));
              end; {while}
          end;
      Curr := EndVal;
      DoAddCurrValue;
    end; {with}
end; {procedure CalcSeriesValues;}

{------------------------------}

begin {function TSpec2DValuesRowDlg.CalcValueList}
  try
    ValueListMemo.Clear;
    EDecimalIsChecked := EDecimal.Checked;
    EFractionIsChecked := EFraction.Checked;
    Result := true;
    S := '';
    //add points
    if TheObj.ItemType = itSingleValue then
      CalcSingleValue
    else {itSeries, itFuncSeries}
      CalcSeriesValues;
  except
    Result := false;
  end; {try..except}
  if Result then
    begin
      ValueListMemo.Text := S;
      if (Length(ValueListMemo.Text) <> Length(S)) then
        WasTruncatedLabel.Visible := True
      else
        WasTruncatedLabel.Visible := False;
      while (Length(ValueListMemo.Text) <> Length(S)) and (Length(S) > 5000) do
        begin
          if Length(S) > 100000 then
            SetLength(S, (Length(S) - 1000))
          else
            SetLength(S, (Length(S) - 100));
          ValueListMemo.Text := S;
        end; {while}
    end; {if}
end; {function TSpec2DValuesRowDlg.CalcValueList}

{---------------------------------------------------------------------}

procedure TSpec2DValuesRowDlg.EFormatTypeClick(Sender: TObject);
begin
  CalcValueList;
end;

end.
