unit vw2dDlg;

(*
Copyright 1999-2011 Greg VanMullem
This program is distributed under the terms of the GNU General Public License Version 3

This file is part of MathGV.

MathGV is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

MathGV is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MathGV.  If not, see <http://www.gnu.org/licenses/>.
*)

interface

uses WinTypes, WinProcs, Classes, Graphics, Forms, Controls, Buttons,
  StdCtrls, Dialogs, Spin, sysutils, MathCalc, MathFunc, FloatStr,
  editmath, CentrDlg, Vw2dObj;

type
  TChange2DViewDlg = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    HelpBtn: TBitBtn;
    GraphBox: TGroupBox;
    Label6: TLabel;
    Label7: TLabel;
    PixelsBox: TGroupBox;
    Label8: TLabel;
    Label9: TLabel;
    EPixelsX: TSpinEdit;
    EPixelsY: TSpinEdit;
    ZoomBox: TGroupBox;
    Label1: TLabel;
    Label2: TLabel;
    Label5: TLabel;
    EXViewZoom: TEditMathConst;
    EYViewZoom: TEditMathConst;
    ECartesianX: TEditMathConst;
    ECartesianY: TEditMathConst;
    Label3: TLabel;
    Label4: TLabel;
    procedure ECartesianXExit(Sender: TObject);
    procedure ECartesianYExit(Sender: TObject);
    procedure EPixelsXExit(Sender: TObject);
    procedure EPixelsYExit(Sender: TObject);
    procedure EXViewZoomExit(Sender: TObject);
    procedure EYViewZoomExit(Sender: TObject);
  private
    { Private declarations }
    FCartesianX,
    FCartesianY : TMathFloat;
    FXViewZoom : TMathFloat;
    FYViewZoom : TMathFloat;
    procedure ConvPixelToCartesian(PX, PY : TMathFloat;
                                   var CX, CY : TMathFloat);
    procedure ConvCartesianToPixel(CX, CY : TMathFloat;
                                   var PX, PY : longint);
    procedure UpDateEditLines;
  public
    { Public declarations }
    procedure SetTheFields(TheView2DZoomObj : TView2DZoomObj);
    procedure GetTheFields(TheView2DZoomObj : TView2DZoomObj);
  end;

function ModifyView2D(TheView2DZoomObj : TView2DZoomObj): boolean;

implementation

{$R *.DFM}

{---------------------------------------------------------------------}

function ModifyView2D(TheView2DZoomObj : TView2DZoomObj) : boolean;
var
  Change2DViewDlg: TChange2DViewDlg;
begin {function ModifyView2D}
  Change2DViewDlg := TChange2DViewDlg.Create(Application);
  try
    SetDlgDefaultPos(Change2DViewDlg, nil);
    Change2DViewDlg.SetTheFields(TheView2DZoomObj);
    Result := (Change2DViewDlg.ShowModal = IDOK);
    if Result then
      begin
        Change2DViewDlg.GetTheFields(TheView2DZoomObj);
        TheView2DZoomObj.DidFieldsChange := true;
      end { if Result = true then}
  finally
    Change2DViewDlg.Free;
  end; {try..finally}
end; {function ModifyView2D}

{---------------------------------------------------------------------}

procedure TChange2DViewDlg.ConvPixelToCartesian(PX, PY : TMathFloat;
                               var CX, CY : TMathFloat);
begin {procedure TChange2DViewDlg.ConvPixelToCartesian}
  CX := PX / FXViewZoom;
  CY := PY / FYViewZoom;
end; {procedure TChange2DViewDlg.ConvPixelToCartesian}

{---------------------------------------------------------------------}

procedure TChange2DViewDlg.ConvCartesianToPixel(CX, CY : TMathFloat;
                                                var PX, PY : longint);
var
  TmpPX, TmpPY : TMathFloat;
begin {TChange2DViewDlg.ConvCartesianToPixel}
  try
    TmpPX := CX * FXViewZoom;
  except
    on EOverFlow do
       begin
         if CX > 0 then
           TmpPX := Max2DViewCenter
         else
           TmpPX := Min2DViewCenter;
       end;
    else
      TmpPX := 0;
  end; {try..Except}
  try
    TmpPY := CY * FYViewZoom;
  except
    on EOverFlow do
       begin
         if CY > 0 then
           TmpPY := Max2DViewCenter
         else
           TmpPY := Min2DViewCenter;
       end;
    else
      TmpPY := 0;
  end; {try..Except}
  if TmpPX > Max2DViewCenter then
    TmpPX := Max2DViewCenter
  else
    if TmpPX < Min2DViewCenter then
      TmpPX := Min2DViewCenter;
  if TmpPY > Max2DViewCenter then
    TmpPY := Max2DViewCenter
  else
    if TmpPY < Min2DViewCenter then
      TmpPY := Min2DViewCenter;
  PX := Round(TmpPX);
  PY := Round(TmpPY);
end; {TChange2DViewDlg.ConvCartesianToPixel}

{---------------------------------------------------------------------}

procedure TChange2DViewDlg.UpDateEditLines;
begin {procedure TChange2DViewDlg.UpDateEditLines}
  ConvPixelToCartesian(EPixelsX.Value, EPixelsY.Value,
                       FCartesianX, FCartesianY);
  ECartesianX.text := FloatCordToStrF(FCartesianX);
  ECartesianY.text := FloatCordToStrF(FCartesianY);
end; {procedure TChange2DViewDlg.UpDateEditLines}

{---------------------------------------------------------------------}

procedure TChange2DViewDlg.SetTheFields(TheView2DZoomObj : TView2DZoomObj);
begin {procedure TChange2DViewDlg.SetTheFields}
  with TheView2DZoomObj do
    begin
      FXViewZoom := XViewZoom;
      EXViewZoom.text := FloatCordToStrF(FXViewZoom);
      FYViewZoom := YViewZoom;
      EYViewZoom.text := FloatCordToStrF(FYViewZoom);
      EPixelsX.Value := trunc(ViewCenterX);
      EPixelsY.Value := trunc(ViewCenterY);
      UpDateEditLines;
    end; {with}
end; {procedure TChange2DViewDlg.SetTheFields}

{---------------------------------------------------------------------}

procedure TChange2DViewDlg.GetTheFields(TheView2DZoomObj : TView2DZoomObj);
begin {procedure TChange2DViewDlg.GetTheFields}
  with TheView2DZoomObj do
    begin
      XViewZoom := FXViewZoom;
      YViewZoom := FYViewZoom;
      ViewCenterX := EPixelsX.Value;
      ViewCenterY := EPixelsY.Value;
    end; {with}
end; {procedure TChange2DViewDlg.GetTheFields}

{---------------------------------------------------------------------}

procedure TChange2DViewDlg.ECartesianXExit(Sender: TObject);
var
  TmpPixelX, TmpPixelY : longint;
begin
  if ECartesianX.ValidateMathStr then
    begin
      ConvCartesianToPixel(ECartesianX.LastValue, FCartesianY,
                           TmpPixelX, TmpPixelY);
      EPixelsX.Value := TmpPixelX;
      EPixelsY.Value := TmpPixelY;
      UpDateEditLines;
    end; {if}
end;

{---------------------------------------------------------------------}

procedure TChange2DViewDlg.ECartesianYExit(Sender: TObject);
var
  TmpPixelX, TmpPixelY : longint;
begin
  if ECartesianY.ValidateMathStr then
    begin
      ConvCartesianToPixel(FCartesianX, ECartesianY.LastValue,
                           TmpPixelX, TmpPixelY);
      EPixelsX.Value := TmpPixelX;
      EPixelsY.Value := TmpPixelY;
      UpDateEditLines;
    end; {if}
end;

{---------------------------------------------------------------------}

procedure TChange2DViewDlg.EPixelsXExit(Sender: TObject);
begin
  if EPixelsX.Value > Max2DViewCenter then
     EPixelsX.Value := Max2DViewCenter
  else
    if EPixelsX.Value < Min2DViewCenter then
       EPixelsX.Value := Min2DViewCenter;
  UpDateEditLines;
end;

{---------------------------------------------------------------------}

procedure TChange2DViewDlg.EPixelsYExit(Sender: TObject);
begin
  if EPixelsY.Value > Max2DViewCenter then
     EPixelsY.Value := Max2DViewCenter
  else
    if EPixelsY.Value < Min2DViewCenter then
       EPixelsY.Value := Min2DViewCenter;
  UpDateEditLines;
end;

{---------------------------------------------------------------------}

procedure TChange2DViewDlg.EXViewZoomExit(Sender: TObject);
var
  TmpF : TMathFloat;
  TheMr : integer;
begin
  if EXViewZoom.ValidateMathStr then
    begin
      TmpF := EXViewZoom.LastValue;
      if (TmpF <= Max2DViewZoom)and(TmpF >= Min2DViewZoom) then
        begin
          FXViewZoom := TmpF;
          UpDateEditLines;
        end
      else
        begin
          TheMr := MessageDlgCenter('Error: X Axis View Zoom must be <= '+
                     Max2DViewZoomStr+' and >= '+Min2DViewZoomStr,
                     mtError, [mbOK,mbCancel,mbIgnore], 0, Self);
          if TheMr = mrCancel then
            EXViewZoom.text := FloatCordToStrF(FXViewZoom)
          else
            if TheMr = mrIgnore then
              begin
                if TmpF >= 1 then
                  FXViewZoom := Max2DViewZoom
                else
                  FXViewZoom := Min2DViewZoom;
                EXViewZoom.text := FloatCordToStrF(FXViewZoom);
                UpDateEditLines;
              end;
          EXViewZoom.SetFocus;
        end; {else}
    end; {if}
end;

{---------------------------------------------------------------------}

procedure TChange2DViewDlg.EYViewZoomExit(Sender: TObject);
var
  TmpF : TMathFloat;
  TheMr : integer;
begin
  if EYViewZoom.ValidateMathStr then
    begin
      TmpF := EYViewZoom.LastValue;
      if (TmpF <= Max2DViewZoom)and(TmpF >= Min2DViewZoom) then
        begin
          FYViewZoom := TmpF;
          UpDateEditLines;
        end
      else
        begin
          TheMr := MessageDlgCenter('Error: Y Axis View Zoom must be <= '+
                     Max2DViewZoomStr+' and >= '+Min2DViewZoomStr,
                     mtError, [mbOK,mbCancel,mbIgnore], 0, Self);
          if TheMr = mrCancel then
            EYViewZoom.text := FloatCordToStrF(FYViewZoom)
          else
            if TheMr = mrIgnore then
              begin
                if TmpF >= 1 then
                  FYViewZoom := Max2DViewZoom
                else
                  FYViewZoom := Min2DViewZoom;
                EYViewZoom.text := FloatCordToStrF(FYViewZoom);
                UpDateEditLines;
              end;
          EYViewZoom.SetFocus;
        end; {else}
    end; {if}
end;

{---------------------------------------------------------------------}

end.
