unit Vw2dObj;

(*
Copyright 1999-2011 Greg VanMullem
This program is distributed under the terms of the GNU General Public License Version 3

This file is part of MathGV.

MathGV is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

MathGV is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MathGV.  If not, see <http://www.gnu.org/licenses/>.
*)

interface

uses MathCalc, MathFunc, FileSave;

const
  Default2DViewZoom = 35;
  Max2DViewZoom = 10000; {10,000}
  Min2DViewZoom = 0.0001;
  Max2DViewZoomStr = '10,000';
  Min2DViewZoomStr = '0.0001';
  Max2DViewCenter =  100000; {+100,000} //Was +1,000,000 in version 1
  Min2DViewCenter = -100000; {-100,000} //Was -1,000,000 in version 1
  Max2DViewCenterStr =  '100,000';
  Min2DViewCenterStr = '-100,000';

type
  TView2DZoomObj = class
    Public
    DidFieldsChange : boolean;
    XViewZoom,
    YViewZoom : TMathFloat;
    ViewCenterX,
    ViewCenterY : TMathFloat;
    constructor Create;
    procedure SaveToDisk(WriteGraphFile : TWriteGraphFile);
    function ReadFromDisk(ReadGraphFile : TReadGraphFile) : boolean;
    procedure Assign(Source : TView2DZoomObj);
  end; {TView2DZoomObj}

implementation

type
  TDiskV2_View2DZoomRec = packed record
    XViewZoom,
    YViewZoom : TMathFloat;
    ViewCenterX,
    ViewCenterY : TMathFloat;
  end; {TDiskV2_View2DZoomRec = packed record}

  TDiskV1_View2DZoomRec = packed record
    ViewZoom : Extended;
    ViewCenterX,
    ViewCenterY : Extended;
  end; {TDiskV1_View2DZoomRec = packed record}

{ TView2DZoomObj }

{---------------------------------------------------------------------}

procedure TView2DZoomObj.Assign(Source: TView2DZoomObj);
begin {procedure TView2DZoomObj.Assign}
  XViewZoom := Source.XViewZoom;
  YViewZoom := Source.YViewZoom;
  ViewCenterX := Source.ViewCenterX;
  ViewCenterY := Source.ViewCenterY;
end; {procedure TView2DZoomObj.Assign}

{---------------------------------------------------------------------}

constructor TView2DZoomObj.Create;
begin {constructor TView2DZoomObj.Create;}
  inherited Create;
  DidFieldsChange := false;
  XViewZoom := Default2DViewZoom;
  YViewZoom := Default2DViewZoom;
  ViewCenterX := 0;
  ViewCenterY := 0;
end; {constructor TView2DZoomObj.Create;}

{---------------------------------------------------------------------}

function TView2DZoomObj.ReadFromDisk(ReadGraphFile: TReadGraphFile) : boolean;
{returns True if view center had to be adjusted}

{-------------------------------------}

procedure ReadVer1Fields;
var
  DiskRec : TDiskV1_View2DZoomRec;
begin {procedure ReadVer1Fields;}
  ReadGraphFile.ReadRecord(DiskRec, sizeof(DiskRec));
  XViewZoom := DiskRec.ViewZoom;
  YViewZoom := DiskRec.ViewZoom;
  if DiskRec.ViewCenterX > Max2DViewCenter then
   ViewCenterX := Max2DViewCenter
  else
   if DiskRec.ViewCenterX < Min2DViewCenter then
     ViewCenterX := Min2DViewCenter
   else
     ViewCenterX := DiskRec.ViewCenterX;
  if DiskRec.ViewCenterY > Max2DViewCenter then
   ViewCenterY := Max2DViewCenter
  else
   if DiskRec.ViewCenterY < Min2DViewCenter then
     ViewCenterY := Min2DViewCenter
   else
     ViewCenterY := DiskRec.ViewCenterY;
  if (ViewCenterX <> DiskRec.ViewCenterX) or
    (ViewCenterY <> DiskRec.ViewCenterY) then
    Result := true;
end; {procedure ReadVer1Fields;}

{-------------------------------------}

procedure ReadVer2Fields;
var
  DiskRec : TDiskV2_View2DZoomRec;
begin {procedure ReadVer2Fields;}
  ReadGraphFile.ReadRecord(DiskRec, sizeof(DiskRec));
  XViewZoom := DiskRec.XViewZoom;
  YViewZoom := DiskRec.YViewZoom;
  ViewCenterX := DiskRec.ViewCenterX;
  ViewCenterY := DiskRec.ViewCenterY;
end; {procedure ReadVer2Fields;}

{-------------------------------------}

begin {function TView2DZoomObj.ReadFromDisk}
  DidFieldsChange := true;
  Result := false;
  if ReadGraphFile.FileVersion = 1 then
    ReadVer1Fields
  else {if (FileVersion >= 2) then}
    ReadVer2Fields;
end; {function TView2DZoomObj.ReadFromDisk}

{---------------------------------------------------------------------}

procedure TView2DZoomObj.SaveToDisk(WriteGraphFile: TWriteGraphFile);
var
  DiskRec : TDiskV2_View2DZoomRec;
begin {procedure TView2DZoomObj.SaveToDisk}
  DiskRec.XViewZoom := XViewZoom;
  DiskRec.YViewZoom := YViewZoom;
  DiskRec.ViewCenterX := ViewCenterX;
  DiskRec.ViewCenterY := ViewCenterY;
  WriteGraphFile.WriteRecord(DiskRec, sizeof(DiskRec));
end; {procedure TView2DZoomObj.SaveToDisk}

{---------------------------------------------------------------------}

end.
