unit Vw3dObj;

(*
Copyright 1999-2011 Greg VanMullem
This program is distributed under the terms of the GNU General Public License Version 3

This file is part of MathGV.

MathGV is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

MathGV is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MathGV.  If not, see <http://www.gnu.org/licenses/>.
*)

interface

uses MathCalc, MathFunc, FileSave;

const
  Max3DViewZoom = 1000;
  Min3DViewZoom = 1;

type
  TView3DZoomObj = class
    Public
    DidFieldsChange : boolean;
    RotateX,
    RotateY,
    RotateZ : TMathFloat;
    ViewZoom : integer;
    constructor Create;
    procedure SaveToDisk(WriteGraphFile : TWriteGraphFile);
    procedure ReadFromDisk(ReadGraphFile : TReadGraphFile);
    procedure Assign(Source : TView3DZoomObj);
  end; {TView3DZoomObj}

implementation

type
  TDiskV2_View3DRec = packed record
    RotateX,
    RotateY,
    RotateZ : TMathFloat;
    ViewZoom : integer;
  end; {TDiskV2_View3DRec = packed record}

  TDiskV1_View3DRec = packed record
    RotateX,
    RotateY,
    RotateZ : TMathFloat;
    ViewZoom : smallint;
  end; {TDiskV1_View3DRec = packed record}

{ TView3DZoomObj }

{---------------------------------------------------------------------}

procedure TView3DZoomObj.Assign(Source: TView3DZoomObj);
begin {procedure TView3DZoomObj.Assign}
  RotateX := Source.RotateX;
  RotateY := Source.RotateY;
  RotateZ := Source.RotateZ;
  ViewZoom := Source.ViewZoom;
end; {procedure TView3DZoomObj.Assign}

{---------------------------------------------------------------------}

constructor TView3DZoomObj.Create;
begin {constructor TView3DZoomObj.Create;}
  inherited Create;
  DidFieldsChange := false;
  RotateX := 0;
  RotateY := 0;
  RotateZ := 0;
  ViewZoom := 20;
end; {constructor TView3DZoomObj.Create;}

{---------------------------------------------------------------------}

procedure TView3DZoomObj.ReadFromDisk(ReadGraphFile: TReadGraphFile);

{-------------------------------------}

procedure ReadVer1Fields;
var
  DiskRec : TDiskV1_View3DRec;
begin {procedure ReadVer1Fields;}
  ReadGraphFile.ReadRecord(DiskRec, sizeof(DiskRec));
  RotateX := DiskRec.RotateX;
  RotateY := DiskRec.RotateY;
  RotateZ := DiskRec.RotateZ;
  ViewZoom := DiskRec.ViewZoom;
end; {procedure ReadVer1Fields;}

{-------------------------------------}

procedure ReadVer2Fields;
var
  DiskRec : TDiskV2_View3DRec;
begin {procedure ReadVer2Fields;}
  ReadGraphFile.ReadRecord(DiskRec, sizeof(DiskRec));
  RotateX := DiskRec.RotateX;
  RotateY := DiskRec.RotateY;
  RotateZ := DiskRec.RotateZ;
  ViewZoom := DiskRec.ViewZoom;
end; {procedure ReadVer2Fields;}

{-------------------------------------}

begin {procedure TView3DZoomObj.ReadFromDisk}
  DidFieldsChange := true;
  if ReadGraphFile.FileVersion = 1 then
    ReadVer1Fields
  else {if (FileVersion >= 2) then}
    ReadVer2Fields;
end; {procedure TView3DZoomObj.ReadFromDisk}

{---------------------------------------------------------------------}

procedure TView3DZoomObj.SaveToDisk(WriteGraphFile: TWriteGraphFile);
var
  DiskRec : TDiskV2_View3DRec;
begin {procedure TView3DZoomObj.SaveToDisk}
  DiskRec.RotateX := RotateX;
  DiskRec.RotateY := RotateY;
  DiskRec.RotateZ := RotateZ;
  DiskRec.ViewZoom := ViewZoom;
  WriteGraphFile.WriteRecord(DiskRec, sizeof(DiskRec));
end; {procedure TView3DZoomObj.SaveToDisk}

{---------------------------------------------------------------------}

end.
