unit VwPoDlg;

(*
Copyright 1999-2011 Greg VanMullem
This program is distributed under the terms of the GNU General Public License Version 3

This file is part of MathGV.

MathGV is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

MathGV is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MathGV.  If not, see <http://www.gnu.org/licenses/>.
*)

interface

uses WinTypes, WinProcs, Classes, Graphics, Forms, Controls, Buttons,
  StdCtrls, ExtCtrls, Spin, sysutils, dialogs, CentrDlg,
  FloatStr, editmath, GrGlobal, MathCalc, MathFunc, VwPoObj;

type
  TChangePolarViewDlg = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    HelpBtn: TBitBtn;
    ZoomBox: TGroupBox;
    Label1: TLabel;
    Label2: TLabel;
    GroupBox1: TGroupBox;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    GroupBox2: TGroupBox;
    Label6: TLabel;
    Label7: TLabel;
    GroupBox3: TGroupBox;
    Label8: TLabel;
    Label9: TLabel;
    EPixelsX: TSpinEdit;
    EPixelsY: TSpinEdit;
    ThetaBox: TGroupBox;
    EDegrees: TRadioButton;
    ERadians: TRadioButton;
    EViewZoom: TEditMathConst;
    EPolarR: TEditMathConst;
    EPolarTheta: TEditMathConst;
    ECartesianX: TEditMathConst;
    ECartesianY: TEditMathConst;
    procedure EViewZoomExit(Sender: TObject);
    procedure EPolarRExit(Sender: TObject);
    procedure EPolarThetaExit(Sender: TObject);
    procedure ECartesianXExit(Sender: TObject);
    procedure ECartesianYExit(Sender: TObject);
    procedure EPixelsXExit(Sender: TObject);
    procedure ERadDegClick(Sender: TObject);
    procedure EPixelsYExit(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
    FPolarR,
    FPolarTheta,
    FCartesianX,
    FCartesianY : TMathFloat;
    FViewZoom : TMathFloat;
    FThetaRadiansType : boolean;
    procedure ConvPixelToCartesian(PX, PY : TMathFloat;
                                   var CX, CY : TMathFloat);
    procedure ConvCartesianToPixel(CX, CY : TMathFloat;
                                   var PX, PY : longint);
    procedure UpDateEditLines;
  public
    { Public declarations }
    procedure SetTheFields(ViewPolarZoomObj : TViewPolarZoomObj);
    procedure GetTheFields(ViewPolarZoomObj : TViewPolarZoomObj);
  end;

function ModifyViewPolar(ViewPolarZoomObj : TViewPolarZoomObj): boolean;

implementation

{$R *.DFM}

{---------------------------------------------------------------------}

function ModifyViewPolar(ViewPolarZoomObj : TViewPolarZoomObj) : boolean;
var
  ChangePolarViewDlg: TChangePolarViewDlg;
begin {function ModifyViewPolar}
  ChangePolarViewDlg := TChangePolarViewDlg.Create(Application);
  try
    SetDlgDefaultPos(ChangePolarViewDlg, nil);
    ChangePolarViewDlg.SetTheFields(ViewPolarZoomObj);
    Result := (ChangePolarViewDlg.ShowModal = IDOK);
    if Result then
      begin
        ChangePolarViewDlg.GetTheFields(ViewPolarZoomObj);
        ViewPolarZoomObj.DidFieldsChange := true;
      end { if Result = true then}
  finally
    ChangePolarViewDlg.Free;
  end; {try..finally}
end; {function ModifyViewPolar}

{---------------------------------------------------------------------}

procedure TChangePolarViewDlg.ConvPixelToCartesian(PX, PY : TMathFloat;
                                                   var CX, CY : TMathFloat);
begin {procedure TChangePolarViewDlg.ConvPixelToCartesian}
  CX := PX / FViewZoom;
  CY := PY / FViewZoom;
end; {procedure TChangePolarViewDlg.ConvPixelToCartesian}

{---------------------------------------------------------------------}

procedure TChangePolarViewDlg.ConvCartesianToPixel(CX, CY : TMathFloat;
                                                   var PX, PY : longint);
var
  TmpPX, TmpPY : TMathFloat;
begin {TChangePolarViewDlg.ConvCartesianToPixel}
  try
    TmpPX := CX * FViewZoom;
  except
    on EOverFlow do
       begin
         if CX > 0 then
           TmpPX := MaxPolarViewCenter
         else
           TmpPX := MinPolarViewCenter;
       end;
    else
      TmpPX := 0;
  end; {try..Except}
  try
    TmpPY := CY * FViewZoom;
  except
    on EOverFlow do
       begin
         if CY > 0 then
           TmpPY := MaxPolarViewCenter
         else
           TmpPY := MinPolarViewCenter;
       end;
    else
      TmpPY := 0;
  end; {try..Except}
  if TmpPX > MaxPolarViewCenter then
    TmpPX := MaxPolarViewCenter
  else
    if TmpPX < MinPolarViewCenter then
      TmpPX := MinPolarViewCenter;
  if TmpPY > MaxPolarViewCenter then
    TmpPY := MaxPolarViewCenter
  else
    if TmpPY < MinPolarViewCenter then
      TmpPY := MinPolarViewCenter;
  PX := Round(TmpPX);
  PY := Round(TmpPY);
end; {TChangePolarViewDlg.ConvCartesianToPixel}

{---------------------------------------------------------------------}

procedure TChangePolarViewDlg.UpDateEditLines;
begin {procedure TChangePolarViewDlg.UpDateEditLines}
  ConvPixelToCartesian(EPixelsX.Value, EPixelsY.Value,
                       FCartesianX, FCartesianY);
  ECartesianX.text := FloatCordToStrF(FCartesianX);
  ECartesianY.text := FloatCordToStrF(FCartesianY);
  ConvCartesianToPolar(FCartesianX, FCartesianY,
                       FPolarR, FPolarTheta);
  EPolarR.text := FloatCordToStrF(FPolarR);
  if FThetaRadiansType then
    EPolarTheta.text := FloatCordToStrF(FPolarTheta)
  else
    EPolarTheta.text := FloatCordToStrF(ConvRadToDeg(FPolarTheta));
end; {procedure TChangePolarViewDlg.UpDateEditLines}

{---------------------------------------------------------------------}

procedure TChangePolarViewDlg.SetTheFields(ViewPolarZoomObj : TViewPolarZoomObj);
begin {procedure TChangePolarViewDlg.SetTheFields}
  with ViewPolarZoomObj do
    begin
      FThetaRadiansType := ThetaRadiansType;
      if FThetaRadiansType then
        ERadians.Checked := true
      else
        EDegrees.Checked := true;
      FViewZoom := ViewZoom;
      EViewZoom.text := FloatCordToStrF(FViewZoom);
      EPixelsX.Value := trunc(ViewCenterX);
      EPixelsY.Value := trunc(ViewCenterY);
      UpDateEditLines;
    end; {with}
end; {procedure TChangePolarViewDlg.SetTheFields}

{---------------------------------------------------------------------}

procedure TChangePolarViewDlg.GetTheFields(ViewPolarZoomObj : TViewPolarZoomObj);
begin {procedure TChangePolarViewDlg.GetTheFields}
  with ViewPolarZoomObj do
    begin
      ViewZoom := FViewZoom;
      ViewCenterX := EPixelsX.Value;
      ViewCenterY := EPixelsY.Value;
      ThetaRadiansType := FThetaRadiansType;
    end; {with}
end; {procedure TChangePolarViewDlg.GetTheFields}

{----------------------------------------------------------------------}

procedure TChangePolarViewDlg.EViewZoomExit(Sender: TObject);
var
  TheMr : integer;
begin
  if EViewZoom.ValidateMathStr then
    begin
      if (EViewZoom.LastValue <= MaxPolarViewZoom) and
         (EViewZoom.LastValue >= MinPolarViewZoom) then
        begin
          FViewZoom := EViewZoom.LastValue;
          UpDateEditLines;
        end
      else
        begin
          TheMr := MessageDlgCenter('Error View Zoom must be <= '+
                     MaxPolarViewZoomStr+' and >= '+MinPolarViewZoomStr,
                     mtError, [mbOK,mbCancel,mbIgnore], 0, Self);
          if TheMr = mrCancel then
            EViewZoom.text := FloatCordToStrF(FViewZoom)
          else
            if TheMr = mrIgnore then
              begin
                if EViewZoom.LastValue >= 1 then
                  FViewZoom := MaxPolarViewZoom
                else
                  FViewZoom := MinPolarViewZoom;
                EViewZoom.text := FloatCordToStrF(FViewZoom);
                UpDateEditLines;
              end;
          EViewZoom.SetFocus;
        end; {else}
    end; {if}
end;

{---------------------------------------------------------------------}

procedure TChangePolarViewDlg.EPolarRExit(Sender: TObject);
var
  TmpCartesianX, TmpCartesianY : TMathFloat;
  TmpPixelX, TmpPixelY : longint;
begin
  if EPolarR.ValidateMathStr then
    begin
      ConvPolarToCartesian(EPolarR.LastValue, FPolarTheta,
                           TmpCartesianX, TmpCartesianY);
      ConvCartesianToPixel(TmpCartesianX, TmpCartesianY,
                           TmpPixelX, TmpPixelY);
      EPixelsX.Value := TmpPixelX;
      EPixelsY.Value := TmpPixelY;
      UpDateEditLines;
    end; {if}
end;

{---------------------------------------------------------------------}

procedure TChangePolarViewDlg.EPolarThetaExit(Sender: TObject);
var
  TmpPolarTheta,
  TmpCartesianX, TmpCartesianY : TMathFloat;
  TmpPixelX, TmpPixelY : longint;
  NumRevolutions : TMathFloat;
begin
  if EPolarTheta.ValidateMathStr then
    begin
      if FThetaRadiansType then
        TmpPolarTheta := EPolarTheta.LastValue
      else
        TmpPolarTheta := ConvDegToRad(EPolarTheta.LastValue);
      {BEGIN force 0 <= Tmp.PolarTheta < 2*pi}
      try
        NumRevolutions := int(TmpPolarTheta/(2*pi));
        TmpPolarTheta := TmpPolarTheta - ((2*pi)*NumRevolutions);
        if TmpPolarTheta < 0 then
          TmpPolarTheta := (2*pi)-abs(TmpPolarTheta);
      except
        TmpPolarTheta := 0;
      end; {except}
      {END; force 0 <= Tmp.PolarTheta < 2*pi}
      ConvPolarToCartesian(FPolarR, TmpPolarTheta,
                           TmpCartesianX, TmpCartesianY);
      ConvCartesianToPixel(TmpCartesianX, TmpCartesianY,
                           TmpPixelX, TmpPixelY);
      EPixelsX.Value := TmpPixelX;
      EPixelsY.Value := TmpPixelY;
      UpDateEditLines;
    end; {if}
end;

{---------------------------------------------------------------------}

procedure TChangePolarViewDlg.ECartesianXExit(Sender: TObject);
var
  TmpPixelX, TmpPixelY : longint;
begin
  if ECartesianX.ValidateMathStr then
    begin
      ConvCartesianToPixel(ECartesianX.LastValue, FCartesianY,
                           TmpPixelX, TmpPixelY);
      EPixelsX.Value := TmpPixelX;
      EPixelsY.Value := TmpPixelY;
      UpDateEditLines;
    end; {if}
end;

{---------------------------------------------------------------------}

procedure TChangePolarViewDlg.ECartesianYExit(Sender: TObject);
var
  TmpPixelX, TmpPixelY : longint;
begin
  if ECartesianY.ValidateMathStr then
    begin
      ConvCartesianToPixel(FCartesianX, ECartesianY.LastValue,
                           TmpPixelX, TmpPixelY);
      EPixelsX.Value := TmpPixelX;
      EPixelsY.Value := TmpPixelY;
      UpDateEditLines;
    end; {if}
end;

{---------------------------------------------------------------------}

procedure TChangePolarViewDlg.EPixelsXExit(Sender: TObject);
begin
  if EPixelsX.Value > MaxPolarViewCenter then
     EPixelsX.Value := MaxPolarViewCenter
  else
    if EPixelsX.Value < MinPolarViewCenter then
       EPixelsX.Value := MinPolarViewCenter;
  UpDateEditLines;
end;

{---------------------------------------------------------------------}

procedure TChangePolarViewDlg.EPixelsYExit(Sender: TObject);
begin
  if EPixelsY.Value > MaxPolarViewCenter then
     EPixelsY.Value := MaxPolarViewCenter
  else
    if EPixelsY.Value < MinPolarViewCenter then
       EPixelsY.Value := MinPolarViewCenter;
  UpDateEditLines;
end;

{---------------------------------------------------------------------}

procedure TChangePolarViewDlg.ERadDegClick(Sender: TObject);
begin
  if ERadians.Checked then
    begin
      FThetaRadiansType := true;
      EPolarTheta.text := FloatCordToStrF(FPolarTheta);
    end
  else
    begin
      FThetaRadiansType := false;
      EPolarTheta.text := FloatCordToStrF(ConvRadToDeg(FPolarTheta));
    end; {else}
end;

{---------------------------------------------------------------------}

procedure TChangePolarViewDlg.FormCreate(Sender: TObject);
begin
  FThetaRadiansType := false;
end;

{---------------------------------------------------------------------}

end.
