unit VwPoObj;

(*
Copyright 1999-2011 Greg VanMullem
This program is distributed under the terms of the GNU General Public License Version 3

This file is part of MathGV.

MathGV is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

MathGV is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MathGV.  If not, see <http://www.gnu.org/licenses/>.
*)

interface

uses MathCalc, MathFunc, FileSave;

const
  DefaultPolarViewZoom = 35;
  MaxPolarViewZoom = 10000; {10,000}
  MinPolarViewZoom = 0.0001;
  MaxPolarViewZoomStr = '10,000';
  MinPolarViewZoomStr = '0.0001';
  MaxPolarViewCenter =  100000; {+100,000} //Was +1,000,000 in version 1
  MinPolarViewCenter = -100000; {-100,000} //Was -1,000,000 in version 1
  MaxPolarViewCenterStr =  '100,000';
  MinPolarViewCenterStr = '-100,000';

type
  TViewPolarZoomObj = class
    Public
    DidFieldsChange : boolean;
    ViewZoom : TMathFloat;
    ViewCenterX,
    ViewCenterY : TMathFloat;
    ThetaRadiansType : boolean;
    constructor Create;
    procedure SaveToDisk(WriteGraphFile : TWriteGraphFile);
    function ReadFromDisk(ReadGraphFile : TReadGraphFile) : boolean;
    procedure Assign(Source : TViewPolarZoomObj);
  end; {TViewPolarZoomObj}

implementation

type
  TDiskV4_ViewPolarZoomRec = packed record
    ViewZoom : TMathFloat;
    ViewCenterX,
    ViewCenterY : TMathFloat;
    ThetaRadiansType : boolean;
  end; {TDiskV4_ViewPolarZoomRec = packed record}

  TDiskV1_ViewPolarZoomRec = packed record
    ViewZoom : TMathFloat;
    ViewCenterX,
    ViewCenterY : TMathFloat;
  end; {TDiskV1_ViewPolarZoomRec = packed record}

{ TViewPolarZoomObj }

{---------------------------------------------------------------------}

procedure TViewPolarZoomObj.Assign(Source: TViewPolarZoomObj);
begin {procedure TViewPolarZoomObj.Assign}
  ViewZoom := Source.ViewZoom;
  ViewCenterX := Source.ViewCenterX;
  ViewCenterY := Source.ViewCenterY;
  ThetaRadiansType := Source.ThetaRadiansType;
end; {procedure TViewPolarZoomObj.Assign}

{---------------------------------------------------------------------}

constructor TViewPolarZoomObj.Create;
begin {constructor TViewPolarZoomObj.Create;}
  inherited Create;
  DidFieldsChange := false;
  ViewZoom := DefaultPolarViewZoom;
  ViewCenterX := 0;
  ViewCenterY := 0;
  ThetaRadiansType := false;
end; {constructor TViewPolarZoomObj.Create;}

{---------------------------------------------------------------------}

function TViewPolarZoomObj.ReadFromDisk(ReadGraphFile: TReadGraphFile): boolean;
{returns True if view center had to be adjusted}

{-------------------------------------}

procedure ReadVer1Fields;
var
  DiskRec : TDiskV1_ViewPolarZoomRec;
  CenterDidChange : boolean;
begin {procedure ReadVer1Fields;}
  ReadGraphFile.ReadRecord(DiskRec, sizeof(DiskRec));
  ViewZoom := DiskRec.ViewZoom;
  ViewCenterX := DiskRec.ViewCenterX;
  ViewCenterY := DiskRec.ViewCenterY;
  //adjust if necessary
  CenterDidChange := false;
  if ViewCenterX > MaxPolarViewCenter then
    begin
      CenterDidChange := true;
      ViewCenterX := MaxPolarViewCenter;
    end
  else
    if ViewCenterX < MinPolarViewCenter then
      begin
        CenterDidChange := true;
        ViewCenterX := MinPolarViewCenter;
      end; {else}
  if ViewCenterY > MaxPolarViewCenter then
    begin
      CenterDidChange := true;
      ViewCenterY := MaxPolarViewCenter;
    end
  else
    if ViewCenterY < MinPolarViewCenter then
      begin
        CenterDidChange := true;
        ViewCenterY := MinPolarViewCenter;
      end; {else}
  if CenterDidChange then
    Result := true;
end; {procedure ReadVer1Fields;}

{-------------------------------------}

procedure ReadVer2Fields;
var
  DiskRec : TDiskV1_ViewPolarZoomRec;
begin {procedure ReadVer2Fields;}
  ReadGraphFile.ReadRecord(DiskRec, sizeof(DiskRec));
  ViewZoom := DiskRec.ViewZoom;
  ViewCenterX := DiskRec.ViewCenterX;
  ViewCenterY := DiskRec.ViewCenterY;
end; {procedure ReadVer2Fields;}

{-------------------------------------}

procedure ReadVer4Fields;
var
  DiskRec : TDiskV4_ViewPolarZoomRec;
begin {procedure ReadVer4Fields;}
  ReadGraphFile.ReadRecord(DiskRec, sizeof(DiskRec));
  ViewZoom := DiskRec.ViewZoom;
  ViewCenterX := DiskRec.ViewCenterX;
  ViewCenterY := DiskRec.ViewCenterY;
  ThetaRadiansType := DiskRec.ThetaRadiansType;
end; {procedure ReadVer4Fields;}

{-------------------------------------}

begin {function TViewPolarZoomObj.ReadFromDisk}
  DidFieldsChange := true;
  Result := false;
  if ReadGraphFile.FileVersion = 1 then
    ReadVer1Fields
  else
    if ReadGraphFile.FileVersion <= 3 then
      ReadVer2Fields
    else {if (FileVersion >= 4) then}
      ReadVer4Fields;
end; {function TViewPolarZoomObj.ReadFromDisk}

{---------------------------------------------------------------------}

procedure TViewPolarZoomObj.SaveToDisk(WriteGraphFile: TWriteGraphFile);
var
  DiskRec : TDiskV4_ViewPolarZoomRec;
begin {procedure TViewPolarZoomObj.SaveToDisk}
  DiskRec.ViewZoom := ViewZoom;
  DiskRec.ViewCenterX := ViewCenterX;
  DiskRec.ViewCenterY := ViewCenterY;
  DiskRec.ThetaRadiansType := ThetaRadiansType;
  WriteGraphFile.WriteRecord(DiskRec, sizeof(DiskRec));
end; {procedure TViewPolarZoomObj.SaveToDisk}

{---------------------------------------------------------------------}

end.
