unit W32_asm;

(*
Copyright 1999-2011 Greg VanMullem
This program is distributed under the terms of the GNU General Public License Version 3

This file is part of MathGV.

MathGV is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

MathGV is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MathGV.  If not, see <http://www.gnu.org/licenses/>.
*)

interface

procedure AsmRotateOneArray(PixelArray : pointer; NumRec : longint;
             ViewZoom, HalfBmpHeight, HalfBmpWidth,
             CosRadX, CosRadY, CosRadZ,
             SinRadX, SinRadY, SinRadZ : longint);

(* --- obsolite procedures ---
procedure AsmCalcArrayMiddleZ(LineArray : pointer;
                              ElementCount : longint);
procedure AsmSortOneLineArray(LineArray : pointer;
                              NumRec : longint; Gap : longint);
procedure AsmSortLines(LinePtrArray : pointer;
                       NumRec : longint; Gap : longint);
*)

implementation

{---------------------------------------------------------------------}

procedure AsmRotateOneArray(PixelArray : pointer; NumRec : longint;
             ViewZoom, HalfBmpHeight, HalfBmpWidth,
             CosRadX, CosRadY, CosRadZ,
             SinRadX, SinRadY, SinRadZ : longint);
{
StaticX     EQU DWORD PTR [esi]
StaticY     EQU DWORD PTR [esi+4]
StaticZ     EQU DWORD PTR [esi+8]
RotateXPlot EQU DWORD PTR [esi+12]
RotateYPlot EQU DWORD PTR [esi+16]
RotateZ     EQU DWORD PTR [esi+20]
AsmRotateOneArray  PROC FAR PixelArray:DWORD, NumRec:WORD, \
                   ViewZoom:DWORD, HalfBmpHeight:DWORD, HalfBmpWidth:DWORD,\
                   CosRadX:DWORD, CosRadY:DWORD, CosRadZ:DWORD, \
                   SinRadX:DWORD, SinRadY:DWORD, SinRadZ:DWORD
                   PUBLIC AsmRotateOneArray
                   LOCAL NewX:DWORD, NewY:DWORD, NewZ:DWORD = LocalSpace
}
label
  MainLoopTop, JmpToTop, DontJmpTop;
var
  NewX, NewY, NewZ : longint;
begin {procedure AsmRotateOneArray}
  asm
   pushad
   //setup variables for loop
   mov esi, PixelArray    //esi := PixelArray
   mov ecx, NumRec       //execute loop NumRec times
   MainLoopTop:
     //Rotate about X Axis
       //NewX := X
       //NewY := cos(theta)*Y - sin(theta)*Z
         //cos(theta)*Y
           mov eax, [esi+4]   //StaticY     EQU DWORD PTR [esi+4]
           imul CosRadX       //edx:eax := cos(theta)*Y
           shrd eax, edx, 16  //dump lower 16 bits leaving 32 result in eax
           mov ebx, eax       //store result in ebx
         //sin(theta)*Z
           mov eax, [esi+8]   //StaticZ     EQU DWORD PTR [esi+8]
           imul SinRadX       //edx:eax := sin(theta)*Z
           shrd eax, edx, 16  //dump lower 16 bits leaving 32 result in eax
         //Do "-"
           sub ebx, eax       //ebx := cos(theta)*Y - sin(theta)*Z
           mov NewY, ebx      //store NewY
       //NewZ := sin(theta)*Y + cos(theta)*Z
         //sin(theta)*Y
           mov eax, [esi+4]   //StaticY     EQU DWORD PTR [esi+4]
           imul SinRadX       //edx:eax := sin(theta)*Y
           shrd eax, edx, 16  //dump lower 16 bits leaving 32 result in eax
           mov ebx, eax       //store result in ebx
         //cos(theta)*Z
           mov eax, [esi+8]   //StaticZ     EQU DWORD PTR [esi+8]
           imul CosRadX       //edx:eax := cos(theta)*Z
           shrd eax, edx, 16  //dump lower 16 bits leaving 32 result in eax
         //Do "+"
           add ebx, eax       //ebx := sin(theta)*Y + cos(theta)*Z
           mov NewZ, ebx      //store NewZ
     //Rotate about Y Axis
       //NewY := Y
       //NewX := cos(theta)*X + sin(theta)*Z
         //cos(theta)*X
           mov eax, [esi]   //StaticX     EQU DWORD PTR [esi]
           imul CosRadY       //edx:eax := cos(theta)*X
           shrd eax, edx, 16  //dump lower 16 bits leaving 32 result in eax
           mov ebx, eax       //store result in ebx
         //sin(theta)*Z
           mov eax, NewZ
           imul SinRadY       //edx:eax := sin(theta)*Z
           shrd eax, edx, 16  //dump lower 16 bits leaving 32 result in eax
         //Do "+"
           add ebx, eax       //ebx := cos(theta)*X + sin(theta)*Z
           mov NewX, ebx      //store NewX
       //NewZ := cos(theta)*Z - sin(theta)*X
         //cos(theta)*Z
           mov eax, NewZ
           imul CosRadY       //edx:eax := cos(theta)*Z
           shrd eax, edx, 16  //dump lower 16 bits leaving 32 result in eax
           mov ebx, eax       //store result in ebx
         //sin(theta)*X
           mov eax, [esi]   //StaticX     EQU DWORD PTR [esi]
           imul SinRadY       //edx:eax := sin(theta)*X
           shrd eax, edx, 16  //dump lower 16 bits leaving 32 result in eax
         //Do "-"
           sub ebx, eax       //ebx := cos(theta)*Z - sin(theta)*X
               //RotateZ     EQU DWORD PTR [esi+20]
           mov [esi+20], ebx   //store final Z Rotation value
     //Rotate about Z Axis
       //NewZ := Z
       //NewX := cos(theta)*X - sin(theta)*Y
         //cos(theta)*X
           mov eax, NewX
           imul CosRadZ       //edx:eax := cos(theta)*X
           shrd eax, edx, 16  //dump lower 16 bits leaving 32 result in eax
           mov ebx, eax       //store result in ebx
         //sin(theta)*Y
           mov eax, NewY
           imul SinRadZ       //edx:eax := sin(theta)*Y
           shrd eax, edx, 16  //dump lower 16 bits leaving 32 result in eax
         //Do "-"
           sub ebx, eax       //ebx := cos(theta)*X - sin(theta)*Y
           xchg NewX, ebx     //store NewX, and load 'old'NewX into ebx
                              //  for next calculation
       //NewY := sin(theta)*X + cos(theta)*Y
          //sin(theta)*X
           mov eax, SinRadZ   //handle next imul backwards
           imul ebx           //edx:eax := X*sin(theta),
                              //  'old'NewX already in ebx
           shrd eax, edx, 16  //dump lower 16 bits leaving 32 result in eax
           mov ebx, eax       //store result in ebx
         //cos(theta)*Y
           mov eax, NewY
           imul CosRadZ       //edx:eax := Cos(theta)*Y
           shrd eax, edx, 16  //dump lower 16 bits leaving 32 result in eax
         //Do "+"
           add eax, ebx       //NewY = eax := sin(theta)*X + cos(theta)*Y
                              //   NewY is stored in eax for next calculation
     //Do ViewZoom and HalfBmpHeight, HalfBmpWidth
       //calc and store Rotate.YPlot
         //Rotate.YPlot := ((-1)*(NewY*ViewZoom))+HalfBmpHeight
           imul ViewZoom      //edx:eax := NewY(eax) * ViewZoom
           shrd eax, edx, 16  //dump lower 16 bits leaving 32 result in eax
           neg eax                  //eax := eax * (-1)
           add eax, HalfBmpHeight   //eax := eax + HalfBmpHeight
               //RotateYPlot EQU DWORD PTR [esi+16]
           mov [esi+16], eax     //store RotateYPlot
       //calc and store Rotate.XPlot
         //Rotate.XPlot := (NewX*ViewZoom)+HalfBmpWidth
           mov eax, NewX
           imul ViewZoom      //edx:eax := NewX(eax) * ViewZoom
           shrd eax, edx, 16  //dump lower 16 bits leaving 32 result in eax
           add eax, HalfBmpWidth    //eax := eax + HalfBmpWidth
               //RotateXPlot EQU DWORD PTR [esi+12]
           mov [esi+12], eax     //store RotateXPlot
     //advance to next record
       add esi, 24  //move esi to next record (24 = SizeOf(TPixRec))
   loop JmpToTop
     jmp DontJmpTop
   JmpToTop:
     jmp MainLoopTop
   DontJmpTop:
   popad
  end; {asm}
end; {procedure AsmRotateOneArray}

{---------------------------------------------------------------------}
(*
procedure AsmCalcArrayMiddleZ(LineArray : pointer;
                              ElementCount : longint);
label
  LoopTop1;
begin {procedure AsmCalcArrayMiddleZ}
  asm
    push eax
    push ebx
    push ecx
    push edx
    mov ebx, LineArray
    mov ecx, ElementCount
    LoopTop1:
      //buggy mov eax, [[ebx+4]+20]    //eax := TLineRec.P1^.Rotate.Z
      //buggy add eax, [[ebx+8]+20]    //eax := eax + TLineRec.P2^.Rotate.Z
      mov edx, [ebx+4]         //edx := TLineRec.P1
      mov eax, [edx+20]        //eax := TLineRec.P1^.Rotate.Z
      mov edx, [ebx+8]         //edx := TLineRec.P2
      add eax, [edx+20]        //eax := eax + TLineRec.P2^.Rotate.Z
      sar eax, 1               //eax := eax / 2
      mov [ebx], eax           //TLineRec.MiddleZ := eax
      add ebx, 13              //point to next TLineRec.MiddleZ
    loop LoopTop1
    pop edx
    pop ecx
    pop ebx
    pop eax
  end; {asm}
end; {procedure AsmCalcArrayMiddleZ}

{---------------------------------------------------------------------}

procedure AsmSortOneLineArray(LineArray : pointer;
                              NumRec : longint; Gap : longint);
label
  RepeatTop1, LoopTop1, AroundEq1, AroundSwap1;
begin {procedure AsmSortOneLineArray}
  asm
    pushad
    RepeatTop1:
      mov edi, LineArray    //load I pointer
      mov esi, edi          //load pointer to first element in array
      mov eax, 13           //eax := sizeof(TLineRec)
      mul Gap               //edx:eax := Gap * eax,sizeof(TLineRec)
      add esi, eax          //point esi to [I + Gap] array element
      mov dl, 0             //dl := Exchange := false
      mov ecx, NumRec       //ecx := NumRec
      sub ecx, Gap          //ecx := NumRec - Gap
      LoopTop1:
         mov eax, [edi]     //eax := LineArray[I].MiddleZ
         mov ebx, [esi]     //ebx := LineArray[I+Gap].MiddleZ
         //test and swap, set Exchange if Swaped
         cmp eax, ebx
         jl AroundSwap1          //if eax < ebx then don't swap
           jne AroundEq1         //if eax = ebx then do extra check
             mov dh, [edi+12]    //dh := LineArray[I].FuncNum
             cmp dh, [esi+12]    //cmp dh, LineArray[I+Gap].FuncNum
             jle AroundSwap1   //don't swap if La[I].Fn <= La[I+Gap].Fn
           AroundEq1:
             //perform swap
             mov [edi], ebx  //swap Allready loaded MiddleZ's
             mov [esi], eax
             mov eax, [edi+4]  //swap next 4 bytes
             xchg eax, [esi+4]
             mov [edi+4], eax
             mov eax, [edi+8]  //swap next 4 bytes
             xchg eax, [esi+8]
             mov [edi+8], eax
             mov al, [edi+12]  //swap last byte
             xchg al, [esi+12]
             mov [edi+12], al
             mov dl, 1             //dl := Exchange := true
         AroundSwap1:
         add edi, 13      //inc I pointer// 13 = sizeof(TLineRec)
         add esi, 13      //inc I+Gap pointer// 13 = sizeof(TLineRec)
      loop LoopTop1
      cmp dl, 0             //dl = Exchange
      jne RepeatTop1        //if Exchange <> false then repeat..until
    //until not(Exchange)
      shr Gap, 1            //Gap := Gap div 2
      cmp Gap, 0
      jne RepeatTop1        //if Gap < 1 then repeat..until
    //until (Gap < 1)
    popad
  end; {asm}
end; {procedure AsmSortOneLineArray}

{---------------------------------------------------------------------}

procedure AsmSortLines(LinePtrArray : pointer;
                       NumRec : longint; Gap : longint);
var
  ILinePtrOffSet, ILineEnd, IpGapLinePtrOffSet, IpGapLineEnd : longint;
label
  RepeatTop2, LoopTop2, AroundEq2, AroundSwap2,
  AroundNextISeg2, AroundDI2,
  AroundNextIpGapSeg2, AroundSI2;
begin {procedure AsmSortLines}
  asm
    pushad
    RepeatTop2:
      //calc and store var's needed to traverse line arrays useing
      //LineI := addr(LinePtrArray[(I div TLineArraySize)]^
      //                          [(I mod TLineArraySize)])//
        mov ebx, LinePtrArray   //ebx := LinePtrArray
        mov ILinePtrOffSet, ebx //store offset of LinePtrArray[0]
        mov edi, [ebx]          //edi := addr(LinePtrArray[0]^[0])
        mov eax, edi
        add eax, 65507 //eax := addr(LinePtrArray[0]^[TLineArraySize-1])//
                       //eax := offset of last element in LinePtrArray[0]^
                       //  65507 = (5040-1)*13
                       //  TLineArraySize = 5040//
                       //  TLinePtrArraySize = 13//
        mov ILineEnd, eax //store offset of last element in LineArray[0]^
      //calc and store var's needed to traverse line arrays useing
      //LineIPlusGap := addr(LinePtrArray[((I + Gap) div TLineArraySize)]^
      //                                 [((I + Gap) mod TLineArraySize)])//
        xor edx, edx  //edx := 0
        mov eax, Gap  //eax := Gap
        mov ecx, 5040 //ecx := TLineArraySize = 5040//
        div ecx       //Result,eax Remainder,edx := 0:Gap / TLineArraySize
                      //  eax := ((I + Gap) div TLineArraySize)//
                      //  edx := ((I + Gap) mod TLineArraySize)//
        shl eax, 2    //Result,eax := Result * 4// (4 byte pointers)
        //ebx := addr(LinePtrArray[((I + Gap) div TLineArraySize)])
          add ebx, eax
        mov IpGapLinePtrOffSet, ebx   //store offset
        //esi := addr(LinePtrArray[((I + Gap) div TLineArraySize)]^[0])
          mov esi, [ebx]
        mov eax, esi
        //eax := addr(LinePtrArray[((I + Gap) div TLineArraySize)]^
        //                       [TLineArraySize-1])
        //calc offset of last element in LineArray[I+Gap]^
          add eax, 65507  //65507 = (5040-1)*13
                          //TLineArraySize = 5040// TLinePtrArraySize = 13//
        mov IpGapLineEnd, eax //store offset
        mov eax, 13           //eax := sizeof(TLineRec)
        mul edx               //edx:eax := Remainder,edx * sizeof(TLineRec)
        add esi, eax          //point esi to [I + Gap] array element
      //setup var's needed for loop
        mov dl, 0             //dl := Exchange := false
        mov ecx, NumRec       //ecx := NumRec
        sub ecx, Gap          //ecx := NumRec - Gap
      LoopTop2:
         mov eax, [edi]   //eax := LineArray[I].MiddleZ
         mov ebx, [esi]   //ebx := LineArray[I+Gap].MiddleZ
         //test and swap, set Exchange if Swaped
         cmp eax, ebx
         jl AroundSwap2          //if eax < ebx then don't swap
           jne AroundEq2         //if eax = ebx then do extra check
             mov dh, [edi+12]    //dh := LineArray[I].FuncNum
             cmp dh, [esi+12]    //cmp dh, LineArray[I+Gap].FuncNum
             jle AroundSwap2   //don't swap if La[I].Fn <= La[I+Gap].Fn
           AroundEq2:
             //perform swap
             mov [edi], ebx      //swap Allready loaded MiddleZ's
             mov [esi], eax
             mov eax, [edi+4]    //swap next 4 bytes
             xchg eax, [esi+4]
             mov [edi+4], eax
             mov eax, [edi+8]    //swap next 4 bytes
             xchg eax, [esi+8]
             mov [edi+8], eax
             mov al, [edi+12]    //swap last byte
             xchg al, [esi+12]
             mov [edi+12], al
             mov dl, 1             //dl := Exchange := true
         AroundSwap2:
         //advance es:di to next lineRec
           cmp edi, ILineEnd
           jne AroundNextISeg2
             //advance edi to next segment (line array)
             add ILinePtrOffset, 4  //next pointer in array
             mov ebx, ILinePtrOffset
             mov edi, [ebx]       //edi := addr(LinePtrArray[next]^[0])
             //ax := offset of last element in LinePtrArray[next]^
               mov eax, edi
               add eax, 65507 //65507 = (5040-1)*13
                              //  TLineArraySize = 5040;
                              //  TLinePtrArraySize = 13;
             //store offset of last element in LineArray[next]^
               mov ILineEnd, eax
             jmp AroundDI2
           AroundNextISeg2:
             add edi, 13    //inc I pointer// 13 = sizeof(TLineRec)
           AroundDI2:
         //advance esi to next lineRec
           cmp esi, IpGapLineEnd
           jne AroundNextIpGapSeg2
             //advance esi to next segment (line array)
             add IpGapLinePtrOffset, 4  //next pointer in array
             mov ebx, IpGapLinePtrOffset
             mov esi, [ebx]      //esi := addr(LinePtrArray[next]^[0])
             //ax := offset of last element in LinePtrArray[next]^
               mov eax, esi
               add eax, 65507 //65507 = (5040-1)*13
                              //  TLineArraySize = 5040;
                              //  TLinePtrArraySize = 13;
             //store offset of last element in LineArray[next]^
               mov IpGapLineEnd, eax
             jmp AroundSI2
           AroundNextIpGapSeg2:
             add esi, 13    //inc I+Gap pointer// 13 = sizeof(TLineRec)
           AroundSI2:
      loop LoopTop2
      cmp dl, 0             //dl = Exchange
      jne RepeatTop2        //if Exchange <> false then repeat..until
    //until not(Exchange)
      shr Gap, 1            //Gap := Gap div 2
      cmp Gap, 0
      jne RepeatTop2        //if Gap < 1 then repeat..until
    //until (Gap < 1)
    popad
  end; {asm}
end; {procedure AsmSortLines}
*)
{---------------------------------------------------------------------}

end.
