unit WinBase;

(*
Copyright 1999-2011 Greg VanMullem
This program is distributed under the terms of the GNU General Public License Version 3

This file is part of MathGV.

MathGV is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

MathGV is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MathGV.  If not, see <http://www.gnu.org/licenses/>.
*)

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, PtImage, FileSave, GrDraw, BmpSizer, ExtCtrls, GrGlobal;

type
  TWindowBase = class(TForm)
    GraphWinMainMenu: TMainMenu;
    Label1: TMenuItem;
    SelectTopLabel1: TMenuItem;
    SelectNext1: TMenuItem;
    SelectPrevious1: TMenuItem;
    NN1: TMenuItem;
    SendSelectedtoFront1: TMenuItem;
    SendSelectedtoBack1: TMenuItem;
    NN2: TMenuItem;
    ShowFloodFillLocations1: TMenuItem;
    EditSelected1: TMenuItem;
    InPlaceTextLabelEditor1: TMenuItem;
    LabelListDialog1: TMenuItem;
    NN3: TMenuItem;
    DeleteSelected1: TMenuItem;
    DeleteAllLabels1: TMenuItem;
    DeleteAllFloodFills1: TMenuItem;
    DeleteAllLabelsAndFloodFills1: TMenuItem;
    LabelPopup: TPopupMenu;
    SendToFront1: TMenuItem;
    SendToBack1: TMenuItem;
    LabelPopN1: TMenuItem;
    EditLabel1: TMenuItem;
    InPlaceEditor1: TMenuItem;
    LabelPopN2: TMenuItem;
    Delete1: TMenuItem;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormCreate(Sender: TObject);
    procedure SelectTopLabel1Click(Sender: TObject);
    procedure SelectNext1Click(Sender: TObject);
    procedure SelectPrevious1Click(Sender: TObject);
    procedure ShowFloodFillLocations1Click(Sender: TObject);
    procedure SendSelectedtoFront1Click(Sender: TObject);
    procedure SendSelectedtoBack1Click(Sender: TObject);
    procedure DeleteSelected1Click(Sender: TObject);
    procedure DeleteAllLabels1Click(Sender: TObject);
    procedure DeleteAllFloodFills1Click(Sender: TObject);
    procedure DeleteAllLabelsAndFloodFills1Click(Sender: TObject);
    procedure LabelListDialog1Click(Sender: TObject);
    procedure InPlaceTextLabelEditor1Click(Sender: TObject);
    procedure EditSelected1Click(Sender: TObject);
  private
    { Private declarations }
    FPlotImageBox : TPlotImageBox;
    procedure WinFormActivate(Sender: TObject);
    procedure HandleDoneDrawing(Sender: TObject);
    procedure HandleDisplayStatusMsg(Sender: TObject);
    procedure HandleTurnDrawToolOff(Sender: TObject);
    procedure HandleLabelWinStatusChange(Sender: TObject; LabelWindowStatus : TLabelWindowStatus);
    procedure HandleGetToolBarAttribEvent(out TheToolBarAttribClass : TToolBarAttribClass);
    procedure HandleGetDrawingToolEvent(out TheDrawingTool : TDrawingTool);
    procedure HandleGetResizeLabelsModeEvent(out TheResizeLabelsMode : boolean);
    procedure HandleShowLabelPopupMenuEvent(ScreenX, ScreenY : integer;
                                            ShowInPlaceEditorItem : boolean);
    procedure HandleRunningEndEvent(Sender: TObject);
    procedure HandleRunningStartEvent(Sender: TObject);
    function GetFileName : string;
    function GetFileNameNoPath : string;
    function GetCaptionTitleStringWithBrackets : string;
    function GetPlotTypeString : string;
  protected
    { Protected declarations }
    procedure SetPlotImageBox(ThePlotImageBox : TPlotImageBox); //must be called by ancestor
    procedure SendWinActivateMessage; virtual; abstract;
  public
    { Public declarations }
    property FileName : string read GetFileName;
    property FileNameNoPath : string read GetFileNameNoPath;
    property CaptionTitleStringWithBrackets : string read GetCaptionTitleStringWithBrackets;
    property PlotTypeString : string read GetPlotTypeString;
    procedure InitalizeNew(TheColorDepth : TPixelFormat;
                           TheDefaultBmpWidth, TheDefaultBmpHeight : integer);
    function InitalizeFromFile(TheColorDepth : TPixelFormat;
                           const TheFileName : string;
                           ReadGraphFile : TReadGraphFile) : boolean;
    procedure ReStartGraphThread;
    procedure TerminateAllThreads;
    procedure WaitForTermAllThreads;
    procedure DisplayStatusMessage; virtual; abstract;
    procedure CenterTheBitmap;
    function SaveFile : boolean;
    function SaveFileAs(const TheFileName : string) : boolean;
    procedure PrintTheBMP(PrintSelected : boolean; PrintScalePercent : integer);
    procedure SaveAsPictureFile(const TheSaveFileStr : string; IsBmpNotJpeg : boolean);
    procedure ReSizeTheBMP;
    function IsRegionSelected : boolean;
    procedure CopyBMPToClipBoard;
    procedure ShowSelectRegionDlg;
  end; {TWindowBase}

{var
  WindowBase: TWindowBase;}

implementation

{$R *.DFM}

uses
  fsmain;

const
  RunningCaptionStr = ' - (Running)';

procedure TWindowBase.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Action := caFree;
end;

procedure TWindowBase.HandleDoneDrawing(Sender: TObject);
begin
  DisplayStatusMessage;
  Update;
end;

procedure TWindowBase.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  CanClose := FPlotImageBox.SaveOnExitPrompt;
  if CanClose then
    begin
      FPlotImageBox.WaitForTermAllThreads;
      MainForm.StatusLineObj.BlankStatusLine;
      MainForm.DecWinCount;
      MainForm.GraphDeactivateMessage;
      MainForm.AddFileNameToList(FPlotImageBox.FileName);
    end; {if}
end;

procedure TWindowBase.HandleDisplayStatusMsg(Sender: TObject);
begin
  DisplayStatusMessage;
end;

procedure TWindowBase.FormCreate(Sender: TObject);
begin
  FPlotImageBox := nil;
  MainForm.IncWinCount;
  SendWinActivateMessage;
  OnActivate := WinFormActivate;
end;

function TWindowBase.GetCaptionTitleStringWithBrackets: string;
begin
  Result := FPlotImageBox.CaptionTitleStringWithBrackets;
end;

function TWindowBase.GetFileName : string;
begin
  Result := FPlotImageBox.FileName;
end;

function TWindowBase.GetFileNameNoPath: string;
begin
  Result := FPlotImageBox.FileNameNoPath;
end;

function TWindowBase.GetPlotTypeString: string;
begin
  Result := FPlotImageBox.PlotTypeString;
end;

procedure TWindowBase.WinFormActivate(Sender: TObject);
begin
  SendWinActivateMessage;
  FPlotImageBox.SetLastMouseToOffBmp;
  DisplayStatusMessage;
end;

procedure TWindowBase.SetPlotImageBox(ThePlotImageBox: TPlotImageBox);
begin
  FPlotImageBox := ThePlotImageBox;
  FPlotImageBox.OnDoneDrawing := HandleDoneDrawing;
  FPlotImageBox.OnDisplayStatusMsg := HandleDisplayStatusMsg;
  FPlotImageBox.OnTurnDrawToolOff := HandleTurnDrawToolOff;
  FPlotImageBox.OnLabelWinStatusChange := HandleLabelWinStatusChange;
  FPlotImageBox.OnGetToolBarAttrib := HandleGetToolBarAttribEvent;
  FPlotImageBox.OnGetDrawingTool := HandleGetDrawingToolEvent;
  FPlotImageBox.OnGetResizeLabelsMode := HandleGetResizeLabelsModeEvent;
  FPlotImageBox.OnShowLabelPopupMenu := HandleShowLabelPopupMenuEvent;
  FPlotImageBox.OnRunningEnd := HandleRunningEndEvent;
  FPlotImageBox.OnRunningStart := HandleRunningStartEvent;
  FPlotImageBox.parent := self;
  FPlotImageBox.Top := 0;
  FPlotImageBox.Left := 0;
  FPlotImageBox.Align := alClient;
end;

procedure TWindowBase.CenterTheBitmap;
begin
  FPlotImageBox.CenterTheBitmap;
end;

function TWindowBase.InitalizeFromFile(TheColorDepth: TPixelFormat;
  const TheFileName: string; ReadGraphFile: TReadGraphFile): boolean;
begin
  Result := FPlotImageBox.InitalizeFromFile(TheColorDepth, TheFileName, ReadGraphFile);
  Caption := FPlotImageBox.CaptionTitleStringWithBrackets;
end;

procedure TWindowBase.InitalizeNew(TheColorDepth: TPixelFormat;
  TheDefaultBmpWidth, TheDefaultBmpHeight: integer);
begin
  FPlotImageBox.InitalizeNew(TheColorDepth, TheDefaultBmpWidth, TheDefaultBmpHeight);
  Caption := FPlotImageBox.CaptionTitleStringWithBrackets;
end;

procedure TWindowBase.ReStartGraphThread;
begin
  FPlotImageBox.ReStartGraphThread;
end;

procedure TWindowBase.WaitForTermAllThreads;
begin
  FPlotImageBox.WaitForTermAllThreads;
end;

function TWindowBase.SaveFile: boolean;
begin
  Result := FPlotImageBox.SaveFile;
  Caption := FPlotImageBox.CaptionTitleStringWithBrackets;
end;

function TWindowBase.SaveFileAs(const TheFileName: string): boolean;
begin
  Result := FPlotImageBox.SaveFileAs(TheFileName);
  Caption := FPlotImageBox.CaptionTitleStringWithBrackets;
end;

procedure TWindowBase.PrintTheBMP(PrintSelected: boolean;
  PrintScalePercent: integer);
begin
  FPlotImageBox.PrintTheBMP(PrintSelected, PrintScalePercent);
end;

procedure TWindowBase.SaveAsPictureFile(const TheSaveFileStr: string;
  IsBmpNotJpeg: boolean);
begin
  FPlotImageBox.SaveAsPictureFile(TheSaveFileStr, IsBmpNotJpeg);
end;

function TWindowBase.IsRegionSelected: boolean;
begin
  Result := FPlotImageBox.IsRegionSelected;
end;

procedure TWindowBase.HandleTurnDrawToolOff(Sender: TObject);
begin
  MainForm.TurnDrawingToolOff;
end;

procedure TWindowBase.HandleLabelWinStatusChange(Sender: TObject;
  LabelWindowStatus: TLabelWindowStatus);
begin
  if LabelWindowStatus = lwLabelSelected then
    begin
      InPlaceTextLabelEditor1.Enabled := true;
      SelectTopLabel1.Enabled := true;
      SelectNext1.Enabled := true;
      SelectPrevious1.Enabled := true;
      ShowFloodFillLocations1.Enabled := true;
      SendSelectedtoFront1.Enabled := true;
      SendSelectedtoBack1.Enabled := true;
      EditSelected1.Enabled := true;
      LabelListDialog1.Enabled := true;
      DeleteSelected1.Enabled := true;
      DeleteAllLabels1.Enabled := true;
      DeleteAllFloodFills1.Enabled := true;
      DeleteAllLabelsAndFloodFills1.Enabled := true;
    end
  else
    begin
      InPlaceTextLabelEditor1.Enabled := false;
      case LabelWindowStatus of
        lwNormal, lwFloodFillsVisible, lwRegionSelected:
          begin
            SelectTopLabel1.Enabled := true;
            SelectNext1.Enabled := false;
            SelectPrevious1.Enabled := false;
            ShowFloodFillLocations1.Enabled := true;
            SendSelectedtoFront1.Enabled := false;
            SendSelectedtoBack1.Enabled := false;
            EditSelected1.Enabled := false;
            LabelListDialog1.Enabled := true;
            DeleteSelected1.Enabled := false;
            DeleteAllLabels1.Enabled := true;
            DeleteAllFloodFills1.Enabled := true;
            DeleteAllLabelsAndFloodFills1.Enabled := true;
          end;
        lwEditingText:
          begin
            SelectTopLabel1.Enabled := false;
            SelectNext1.Enabled := false;
            SelectPrevious1.Enabled := false;
            ShowFloodFillLocations1.Enabled := false;
            SendSelectedtoFront1.Enabled := false;
            SendSelectedtoBack1.Enabled := false;
            EditSelected1.Enabled := true;
            LabelListDialog1.Enabled := false;
            DeleteSelected1.Enabled := false;
            DeleteAllLabels1.Enabled := false;
            DeleteAllFloodFills1.Enabled := false;
            DeleteAllLabelsAndFloodFills1.Enabled := false;
          end;
        lwDrawingNewLabel, lwDragResizeLabel, lwDrawingRegion:
          begin
            SelectTopLabel1.Enabled := false;
            SelectNext1.Enabled := false;
            SelectPrevious1.Enabled := false;
            ShowFloodFillLocations1.Enabled := false;
            SendSelectedtoFront1.Enabled := false;
            SendSelectedtoBack1.Enabled := false;
            EditSelected1.Enabled := false;
            LabelListDialog1.Enabled := false;
            DeleteSelected1.Enabled := false;
            DeleteAllLabels1.Enabled := false;
            DeleteAllFloodFills1.Enabled := false;
            DeleteAllLabelsAndFloodFills1.Enabled := false;
          end;
      end; {case}
    end; {else}
end;

procedure TWindowBase.ReSizeTheBMP;
begin {procedure TFuncGraphWindow.ReSizeTheBMP;}
  if GetPictureSizeDlg(FPlotImageBox.SettingBmpAttribsObj) then
    FPlotImageBox.RestartFieldObjectChanged;
end; {procedure TFuncGraphWindow.ReSizeTheBMP;}

procedure TWindowBase.HandleGetToolBarAttribEvent(
  out TheToolBarAttribClass: TToolBarAttribClass);
begin
  TheToolBarAttribClass := MainForm.ToolBarAttribClass;
end;

procedure TWindowBase.HandleGetDrawingToolEvent(
  out TheDrawingTool: TDrawingTool);
begin
  TheDrawingTool := MainForm.DrawingTool;
end;

procedure TWindowBase.HandleShowLabelPopupMenuEvent(ScreenX,
  ScreenY: integer; ShowInPlaceEditorItem: boolean);
begin
  InPlaceEditor1.Visible := ShowInPlaceEditorItem;
  LabelPopup.Popup(ScreenX, ScreenY);
end;

procedure TWindowBase.HandleGetResizeLabelsModeEvent(
  out TheResizeLabelsMode: boolean);
begin
  TheResizeLabelsMode := MainForm.ResizeLabelsMode;
end;

procedure TWindowBase.SelectTopLabel1Click(Sender: TObject);
begin
  FPlotImageBox.SelectTopLabel;
end;

procedure TWindowBase.SelectNext1Click(Sender: TObject);
begin
  FPlotImageBox.SelectNextLabelOrFFill;
end;

procedure TWindowBase.SelectPrevious1Click(Sender: TObject);
begin
  FPlotImageBox.SelectPrevLabelOrFFill;
end;

procedure TWindowBase.ShowFloodFillLocations1Click(Sender: TObject);
begin
  FPlotImageBox.ShowFloodFillLocations;
end;

procedure TWindowBase.ShowSelectRegionDlg;
begin
  FPlotImageBox.ShowSelectRegionDlg;
end;

procedure TWindowBase.SendSelectedtoFront1Click(Sender: TObject);
begin
  FPlotImageBox.MoveSelectedToFront;
end;

procedure TWindowBase.SendSelectedtoBack1Click(Sender: TObject);
begin
  FPlotImageBox.MoveSelectedToBack;
end;

procedure TWindowBase.DeleteSelected1Click(Sender: TObject);
begin
  FPlotImageBox.DeleteSelectLabel;
end;

procedure TWindowBase.DeleteAllLabels1Click(Sender: TObject);
begin
  FPlotImageBox.DeleteAllLabels;
end;

procedure TWindowBase.DeleteAllFloodFills1Click(Sender: TObject);
begin
  FPlotImageBox.DeleteAllFloodFills;
end;

procedure TWindowBase.DeleteAllLabelsAndFloodFills1Click(Sender: TObject);
begin
  FPlotImageBox.DeleteAllLabelsAndFFills;
end;

procedure TWindowBase.LabelListDialog1Click(Sender: TObject);
begin
  FPlotImageBox.ShowLabelListDlg;
end;

procedure TWindowBase.InPlaceTextLabelEditor1Click(Sender: TObject);
begin
  FPlotImageBox.InPlaceEditTextLabel;
end;

procedure TWindowBase.EditSelected1Click(Sender: TObject);
begin
  FPlotImageBox.ModifyLabelAttributes;
end;

procedure TWindowBase.CopyBMPToClipBoard;
begin
  FPlotImageBox.CopyBMPToClipBoard;
end;

procedure TWindowBase.TerminateAllThreads;
begin
  FPlotImageBox.TerminateAllThreads;
end;

procedure TWindowBase.HandleRunningEndEvent(Sender: TObject);
var
  CapStr : string;
begin
  CapStr := Caption;
  if DoesStringEndWith(CapStr, RunningCaptionStr) then
  begin
    //delete from end of string
    SetLength(CapStr, (Length(CapStr) - Length(RunningCaptionStr)));
    Caption := CapStr;
  end;
end;

procedure TWindowBase.HandleRunningStartEvent(Sender: TObject);
var
  CapStr : string;
begin
  CapStr := Caption;
  if not DoesStringEndWith(CapStr, RunningCaptionStr) then
    Caption := CapStr + RunningCaptionStr;
end;

end.
