unit Wind2D;

(*
Copyright 1999-2011 Greg VanMullem
This program is distributed under the terms of the GNU General Public License Version 3

This file is part of MathGV.

MathGV is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

MathGV is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MathGV.  If not, see <http://www.gnu.org/licenses/>.
*)

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  WinGraph, Menus, Imag2D, Set2dDlg, MathCalc, Vw2DDlg, Vw2DObj, Func2D,
  FuncPara, FListDlg, CentrDlg, DgGlobal;

type
  TWindow2D = class(TWindowGraph)
    Function1: TMenuItem;
    New2DFunction1: TMenuItem;
    NewParametricFunction1: TMenuItem;
    N1: TMenuItem;
    EditFunctionList1: TMenuItem;
    View1: TMenuItem;
    ChangeView1: TMenuItem;
    N2: TMenuItem;
    ZoomIn1: TMenuItem;
    ZoomOut1: TMenuItem;
    N3: TMenuItem;
    ZoomXAxisIn1: TMenuItem;
    ZoomXAxisOut1: TMenuItem;
    ZoomYAxisIn1: TMenuItem;
    ZoomYAxisOut1: TMenuItem;
    N4: TMenuItem;
    ScrollViewUp1: TMenuItem;
    ScrollViewDown1: TMenuItem;
    ScrollViewLeft1: TMenuItem;
    ScrollViewRight1: TMenuItem;
    N7: TMenuItem;
    PageViewUp1: TMenuItem;
    PageViewDown1: TMenuItem;
    PageViewLeft1: TMenuItem;
    PageViewRight1: TMenuItem;
    N8: TMenuItem;
    SetXZoomEqualToY1: TMenuItem;
    SetYZoomEqualToX1: TMenuItem;
    CenterFunctionView1: TMenuItem;
    GraphSettings2: TMenuItem;
    N9: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure ZoomIn1Click(Sender: TObject);
    procedure ZoomOut1Click(Sender: TObject);
    procedure CenterFunctionView1Click(Sender: TObject);
    procedure PageViewUp1Click(Sender: TObject);
    procedure PageViewDown1Click(Sender: TObject);
    procedure PageViewLeft1Click(Sender: TObject);
    procedure PageViewRight1Click(Sender: TObject);
    procedure ScrollViewUp1Click(Sender: TObject);
    procedure ScrollViewDown1Click(Sender: TObject);
    procedure ScrollViewLeft1Click(Sender: TObject);
    procedure ScrollViewRight1Click(Sender: TObject);
    procedure ZoomXAxisIn1Click(Sender: TObject);
    procedure ZoomXAxisOut1Click(Sender: TObject);
    procedure ZoomYAxisIn1Click(Sender: TObject);
    procedure ZoomYAxisOut1Click(Sender: TObject);
    procedure SetXZoomEqualToY1Click(Sender: TObject);
    procedure SetYZoomEqualToX1Click(Sender: TObject);
    procedure ChangeView1Click(Sender: TObject);
    procedure New2DFunction1Click(Sender: TObject);
    procedure NewParametricFunction1Click(Sender: TObject);
    procedure EditFunctionList1Click(Sender: TObject);
    procedure GraphSettings2Click(Sender: TObject);
  private
    { Private declarations }
    FImage2D : TImage2D;
  protected
    { Protected declarations }
    procedure SendWinActivateMessage; override;
  public
    { Public declarations }
    procedure ZoomIn; override;
    procedure ZoomInXAxis;
    procedure ZoomInYAxis;
    procedure ZoomOut; override;
    procedure CenterFunctionView; override;
    procedure ZoomOutXAxis;
    procedure ZoomOutYAxis;
    procedure PageViewUp;
    procedure PageViewDown;
    procedure PageViewLeft;
    procedure PageViewRight;
    procedure ScrollViewUp;
    procedure ScrollViewDown;
    procedure ScrollViewLeft;
    procedure ScrollViewRight;
    procedure SetXZoomEqualToY;
    procedure SetYZoomEqualToX;
    procedure ChangeView;
    function ChangeGraphSettings : boolean;
    function Add2DFunction : boolean;
    function AddParametricFunction : boolean;
    function ModifyFunctionList : boolean;
    procedure DisplayStatusMessage; override;
  end; {TWindow2D}

{var
  Window2D: TWindow2D;}

implementation

{$R *.DFM}

uses
  FSMain;

procedure TWindow2D.CenterFunctionView;
begin {procedure TWindow2D.CenterFunctionView;}
  with FImage2D.View2DZoomObj do
    begin
      if (ViewCenterX <> 0) or (ViewCenterY <> 0) then
        begin
          ViewCenterX := 0;
          ViewCenterY := 0;
          DidFieldsChange := true;
          FImage2D.RestartFieldObjectChanged;
        end; {if}
    end; {with}
end; {procedure TWindow2D.CenterFunctionView;}

procedure TWindow2D.FormCreate(Sender: TObject);
begin
  inherited;
  FImage2D := TImage2D.Create(Self);
  SetPlotImageBox(FImage2D);
end;

procedure TWindow2D.ZoomIn;
var
  DidZoomXChange, DidZoomYChange : boolean;
  TempXZoom, TempYZoom : TMathFloat;
begin {procedure TWindow2D.ZoomIn;}
  with FImage2D do
    begin
      TempXZoom := View2DZoomObj.XViewZoom;
      TempYZoom := View2DZoomObj.YViewZoom;
      DidZoomXChange := MainForm.HotKey2DObj.Calc2DZoomIn(TempXZoom);
      DidZoomYChange := MainForm.HotKey2DObj.Calc2DZoomIn(TempYZoom);
      if DidZoomXChange or DidZoomYChange then
        begin
          View2DZoomObj.DidFieldsChange := true;
          if (TempXZoom = TempYZoom)or(TempXZoom >= Max2DViewZoom)or
             (TempYZoom >= Max2DViewZoom) then
            begin
              View2DZoomObj.XViewZoom := TempXZoom;
              View2DZoomObj.YViewZoom := TempYZoom;
            end
          else
            begin
              if (abs(TempXZoom - View2DZoomObj.XViewZoom) <=
                  abs(TempYZoom - View2DZoomObj.YViewZoom)) then
                begin
                  View2DZoomObj.YViewZoom := View2DZoomObj.YViewZoom +
                                         abs(TempXZoom - View2DZoomObj.XViewZoom);
                  if View2DZoomObj.YViewZoom > Max2DViewZoom then
                    View2DZoomObj.YViewZoom := Max2DViewZoom
                  else
                    if View2DZoomObj.YViewZoom < Min2DViewZoom then
                      View2DZoomObj.YViewZoom := Min2DViewZoom;
                  View2DZoomObj.XViewZoom := TempXZoom;
                end
              else
                begin
                  View2DZoomObj.XViewZoom := View2DZoomObj.XViewZoom +
                                         abs(TempYZoom - View2DZoomObj.YViewZoom);
                  if View2DZoomObj.XViewZoom > Max2DViewZoom then
                    View2DZoomObj.XViewZoom := Max2DViewZoom
                  else
                    if View2DZoomObj.XViewZoom < Min2DViewZoom then
                      View2DZoomObj.XViewZoom := Min2DViewZoom;
                  View2DZoomObj.YViewZoom := TempYZoom;
                end; {else}
            end; {else}
          RestartFieldObjectChanged;
        end; {if}
    end; {with}
end; {procedure TWindow2D.ZoomIn;}

procedure TWindow2D.ZoomOut;
var
  DidZoomXChange, DidZoomYChange : boolean;
  TempXZoom, TempYZoom : TMathFloat;
begin {procedure TWindow2D.ZoomOut;}
  with FImage2D do
    begin
      TempXZoom := View2DZoomObj.XViewZoom;
      TempYZoom := View2DZoomObj.YViewZoom;
      DidZoomXChange := MainForm.HotKey2DObj.Calc2DZoomOut(TempXZoom);
      DidZoomYChange := MainForm.HotKey2DObj.Calc2DZoomOut(TempYZoom);
      if DidZoomXChange or DidZoomYChange then
        begin
          View2DZoomObj.DidFieldsChange := true;
          if (TempXZoom = TempYZoom)or(TempXZoom <= Min2DViewZoom)or
             (TempYZoom <= Min2DViewZoom) then
            begin
              View2DZoomObj.XViewZoom := TempXZoom;
              View2DZoomObj.YViewZoom := TempYZoom;
            end
          else
            begin
              if (abs(TempXZoom - View2DZoomObj.XViewZoom) <=
                  abs(TempYZoom - View2DZoomObj.YViewZoom)) then
                begin
                  View2DZoomObj.YViewZoom := View2DZoomObj.YViewZoom -
                                         abs(TempXZoom - View2DZoomObj.XViewZoom);
                  if View2DZoomObj.YViewZoom > Max2DViewZoom then
                    View2DZoomObj.YViewZoom := Max2DViewZoom
                  else
                    if View2DZoomObj.YViewZoom < Min2DViewZoom then
                      View2DZoomObj.YViewZoom := Min2DViewZoom;
                  View2DZoomObj.XViewZoom := TempXZoom;
                end
              else
                begin
                  View2DZoomObj.XViewZoom := View2DZoomObj.XViewZoom -
                                         abs(TempYZoom - View2DZoomObj.YViewZoom);
                  if View2DZoomObj.XViewZoom > Max2DViewZoom then
                    View2DZoomObj.XViewZoom := Max2DViewZoom
                  else
                    if View2DZoomObj.XViewZoom < Min2DViewZoom then
                      View2DZoomObj.XViewZoom := Min2DViewZoom;
                  View2DZoomObj.YViewZoom := TempYZoom;
                end; {else}
            end; {else}
          RestartFieldObjectChanged;
        end; {if}
    end; {with}
end; {procedure TWindow2D.ZoomOut;}

procedure TWindow2D.ZoomIn1Click(Sender: TObject);
begin
  ZoomIn;
end;

procedure TWindow2D.ZoomOut1Click(Sender: TObject);
begin
  ZoomOut;
end;

procedure TWindow2D.CenterFunctionView1Click(Sender: TObject);
begin
  CenterFunctionView;
end;

procedure TWindow2D.PageViewDown;
var
  Tmp : TMathFloat;
begin {procedure TWindow2D.PageViewDown;}
  with FImage2D do
    begin
      Tmp := View2DZoomObj.ViewCenterY-(SettingBmpAttribsObj.BMPHeight div 2);
      if Tmp > Max2DViewCenter then
        Tmp := Max2DViewCenter;
      if Tmp < Min2DViewCenter then
        Tmp := Min2DViewCenter;
      if View2DZoomObj.ViewCenterY <> Tmp then
        begin
          View2DZoomObj.ViewCenterY := Tmp;
          View2DZoomObj.DidFieldsChange := true;
          RestartFieldObjectChanged;
        end; {if}
    end; {with}
end; {procedure TWindow2D.PageViewDown;}

procedure TWindow2D.PageViewLeft;
var
  Tmp : TMathFloat;
begin {procedure TWindow2D.PageViewLeft;}
  with FImage2D do
    begin
      Tmp := View2DZoomObj.ViewCenterX-(SettingBmpAttribsObj.BMPWidth div 2);
      if Tmp > Max2DViewCenter then
        Tmp := Max2DViewCenter;
      if Tmp < Min2DViewCenter then
        Tmp := Min2DViewCenter;
      if View2DZoomObj.ViewCenterX <> Tmp then
        begin
          View2DZoomObj.ViewCenterX := Tmp;
          View2DZoomObj.DidFieldsChange := true;
          RestartFieldObjectChanged;
        end; {if}
    end; {with}
end; {procedure TWindow2D.PageViewLeft;}

procedure TWindow2D.PageViewRight;
var
  Tmp : TMathFloat;
begin {procedure TWindow2D.PageViewRight;}
  with FImage2D do
    begin
      Tmp := View2DZoomObj.ViewCenterX+(SettingBmpAttribsObj.BMPWidth div 2);
      if Tmp > Max2DViewCenter then
        Tmp := Max2DViewCenter;
      if Tmp < Min2DViewCenter then
        Tmp := Min2DViewCenter;
      if View2DZoomObj.ViewCenterX <> Tmp then
        begin
          View2DZoomObj.ViewCenterX := Tmp;
          View2DZoomObj.DidFieldsChange := true;
          RestartFieldObjectChanged;
        end; {if}
    end; {with}
end; {procedure TWindow2D.PageViewRight;}

procedure TWindow2D.PageViewUp;
var
  Tmp : TMathFloat;
begin {procedure TWindow2D.PageViewUp;}
  with FImage2D do
    begin
      Tmp := View2DZoomObj.ViewCenterY+(SettingBmpAttribsObj.BMPHeight div 2);
      if Tmp > Max2DViewCenter then
        Tmp := Max2DViewCenter;
      if Tmp < Min2DViewCenter then
        Tmp := Min2DViewCenter;
      if View2DZoomObj.ViewCenterY <> Tmp then
        begin
          View2DZoomObj.ViewCenterY := Tmp;
          View2DZoomObj.DidFieldsChange := true;
          RestartFieldObjectChanged;
        end; {if}
    end; {with}
end; {procedure TWindow2D.PageViewUp;}

procedure TWindow2D.ScrollViewDown;
var
  Tmp : TMathFloat;
begin {procedure TWindow2D.ScrollViewDown;}
  with FImage2D do
    begin
      Tmp := View2DZoomObj.ViewCenterY - MainForm.HotKey2DObj.GraphScrollViewAmount;
      if Tmp > Max2DViewCenter then
        Tmp := Max2DViewCenter;
      if Tmp < Min2DViewCenter then
        Tmp := Min2DViewCenter;
      if View2DZoomObj.ViewCenterY <> Tmp then
        begin
          View2DZoomObj.ViewCenterY := Tmp;
          View2DZoomObj.DidFieldsChange := true;
          RestartFieldObjectChanged;
        end; {if}
    end; {with}
end; {procedure TWindow2D.ScrollViewDown;}

procedure TWindow2D.ScrollViewLeft;
var
  Tmp : TMathFloat;
begin {procedure TWindow2D.ScrollViewLeft;}
  with FImage2D do
    begin
      Tmp := View2DZoomObj.ViewCenterX - MainForm.HotKey2DObj.GraphScrollViewAmount;
      if Tmp > Max2DViewCenter then
        Tmp := Max2DViewCenter;
      if Tmp < Min2DViewCenter then
        Tmp := Min2DViewCenter;
      if View2DZoomObj.ViewCenterX <> Tmp then
        begin
          View2DZoomObj.ViewCenterX := Tmp;
          View2DZoomObj.DidFieldsChange := true;
          RestartFieldObjectChanged;
        end; {if}
    end; {with}
end; {procedure TWindow2D.ScrollViewLeft;}

procedure TWindow2D.ScrollViewRight;
var
  Tmp : TMathFloat;
begin {procedure TWindow2D.ScrollViewRight;}
  with FImage2D do
    begin
      Tmp := View2DZoomObj.ViewCenterX + MainForm.HotKey2DObj.GraphScrollViewAmount;
      if Tmp > Max2DViewCenter then
        Tmp := Max2DViewCenter;
      if Tmp < Min2DViewCenter then
        Tmp := Min2DViewCenter;
      if View2DZoomObj.ViewCenterX <> Tmp then
        begin
          View2DZoomObj.ViewCenterX := Tmp;
          View2DZoomObj.DidFieldsChange := true;
          RestartFieldObjectChanged;
        end; {if}
    end; {with}
end; {procedure TWindow2D.ScrollViewRight;}

procedure TWindow2D.ScrollViewUp;
var
  Tmp : TMathFloat;
begin {procedure TWindow2D.ScrollViewUp;}
  with FImage2D do
    begin
      Tmp := View2DZoomObj.ViewCenterY + MainForm.HotKey2DObj.GraphScrollViewAmount;
      if Tmp > Max2DViewCenter then
        Tmp := Max2DViewCenter;
      if Tmp < Min2DViewCenter then
        Tmp := Min2DViewCenter;
      if View2DZoomObj.ViewCenterY <> Tmp then
        begin
          View2DZoomObj.ViewCenterY := Tmp;
          View2DZoomObj.DidFieldsChange := true;
          RestartFieldObjectChanged;
        end; {if}
    end; {with}
end; {procedure TWindow2D.ScrollViewUp;}

procedure TWindow2D.ZoomInXAxis;
begin {procedure TWindow2D.ZoomInXAxis;}
  with FImage2D do
    if MainForm.HotKey2DObj.Calc2DZoomIn(View2DZoomObj.XViewZoom) then
      begin
        View2DZoomObj.DidFieldsChange := true;
        RestartFieldObjectChanged;
      end; {if}
end; {procedure TWindow2D.ZoomInXAxis;}

procedure TWindow2D.ZoomInYAxis;
begin {procedure TWindow2D.ZoomInYAxis;}
  with FImage2D do
    if MainForm.HotKey2DObj.Calc2DZoomIn(View2DZoomObj.YViewZoom) then
      begin
        View2DZoomObj.DidFieldsChange := true;
        RestartFieldObjectChanged;
      end; {if}
end; {procedure TWindow2D.ZoomInYAxis;}

procedure TWindow2D.ZoomOutXAxis;
begin {procedure TWindow2D.ZoomOutXAxis;}
  with FImage2D do
    if MainForm.HotKey2DObj.Calc2DZoomOut(View2DZoomObj.XViewZoom) then
      begin
        View2DZoomObj.DidFieldsChange := true;
        RestartFieldObjectChanged;
      end; {if}
end; {procedure TWindow2D.ZoomOutXAxis;}

procedure TWindow2D.ZoomOutYAxis;
begin {procedure TWindow2D.ZoomOutYAxis;}
  with FImage2D do
    if MainForm.HotKey2DObj.Calc2DZoomOut(View2DZoomObj.YViewZoom) then
      begin
        View2DZoomObj.DidFieldsChange := true;
        RestartFieldObjectChanged;
      end; {if}
end; {procedure TWindow2D.ZoomOutYAxis;}

procedure TWindow2D.PageViewUp1Click(Sender: TObject);
begin
  PageViewUp;
end;

procedure TWindow2D.PageViewDown1Click(Sender: TObject);
begin
  PageViewDown;
end;

procedure TWindow2D.PageViewLeft1Click(Sender: TObject);
begin
  PageViewLeft;
end;

procedure TWindow2D.PageViewRight1Click(Sender: TObject);
begin
  PageViewRight;
end;

procedure TWindow2D.ScrollViewUp1Click(Sender: TObject);
begin
  ScrollViewUp;
end;

procedure TWindow2D.ScrollViewDown1Click(Sender: TObject);
begin
  ScrollViewDown;
end;

procedure TWindow2D.ScrollViewLeft1Click(Sender: TObject);
begin
  ScrollViewLeft;
end;

procedure TWindow2D.ScrollViewRight1Click(Sender: TObject);
begin
  ScrollViewRight;
end;

procedure TWindow2D.ZoomXAxisIn1Click(Sender: TObject);
begin
  ZoomInXAxis;
end;

procedure TWindow2D.ZoomXAxisOut1Click(Sender: TObject);
begin
  ZoomOutXAxis;
end;

procedure TWindow2D.ZoomYAxisIn1Click(Sender: TObject);
begin
  ZoomInYAxis;
end;

procedure TWindow2D.ZoomYAxisOut1Click(Sender: TObject);
begin
  ZoomOutYAxis;
end;

procedure TWindow2D.SetXZoomEqualToY1Click(Sender: TObject);
begin
  SetXZoomEqualToY;
end;

procedure TWindow2D.SetYZoomEqualToX1Click(Sender: TObject);
begin
  SetYZoomEqualToX;
end;

procedure TWindow2D.ChangeView1Click(Sender: TObject);
begin
  ChangeView;
end;

function TWindow2D.Add2DFunction: boolean;
var
  Func2DObj : TFunc2DObj;
begin
  Result := false;
  if FImage2D.IsFreeFuncListObjSlot then
    begin
      Func2DObj := TFunc2DObj.Create;
      if NewUserFunctionObj(Func2DObj, FImage2D.GetNextDefaultColor, nil) then
        begin
          FImage2D.AddNewFuncListObj(Func2DObj);
          Result := true;
        end
      else
        Func2DObj.Free;
    end
  else
    begin
      MessageDlgOverMaxFuncCount(Max2DFuncCount);
    end; {else}
end;

function TWindow2D.AddParametricFunction: boolean;
var
  FuncParametricObj : TFuncParametricObj;
begin
  Result := false;
  if FImage2D.IsFreeFuncListObjSlot then
    begin
      FuncParametricObj := TFuncParametricObj.Create;
      if NewUserFunctionObj(FuncParametricObj, FImage2D.GetNextDefaultColor, nil) then
        begin
          FImage2D.AddNewFuncListObj(FuncParametricObj);
          Result := true;
        end
      else
        FuncParametricObj.Free;
    end
  else
    begin
      MessageDlgOverMaxFuncCount(Max2DFuncCount);
    end; {else}
end;

function TWindow2D.ModifyFunctionList: boolean;
var
  DidModify,
  DidDelete,
  DidReOrder : boolean;
begin
  DisplayFuncListDlg(FImage2D.FuncList, DidModify, DidDelete, DidReOrder);
  Result := (DidModify or DidDelete or DidReOrder);
end;

procedure TWindow2D.New2DFunction1Click(Sender: TObject);
begin
  if Add2DFunction then
    FImage2D.RestartFieldObjectChanged;
end;

procedure TWindow2D.NewParametricFunction1Click(Sender: TObject);
begin
  if AddParametricFunction then
    FImage2D.RestartFieldObjectChanged;
end;

procedure TWindow2D.EditFunctionList1Click(Sender: TObject);
begin
  if ModifyFunctionList then
    FImage2D.RestartFieldObjectChanged;
end;

procedure TWindow2D.DisplayStatusMessage;
begin
  with FImage2D do
    MainForm.StatusLineObj.UpDate2DStatusLine(View2DZoomObj,
      SettingBmpAttribsObj.BmpWidth, SettingBmpAttribsObj.BmpHeight,
      LastMouseX, LastMouseY);
end;

procedure TWindow2D.SendWinActivateMessage;
begin
  MainForm.Graph2DActivateMessage;
end;

procedure TWindow2D.GraphSettings2Click(Sender: TObject);
begin
  if ChangeGraphSettings then
    FImage2D.RestartFieldObjectChanged;
end;

procedure TWindow2D.ChangeView;
begin
  with FImage2D do
    if ModifyView2D(View2DZoomObj) then
      RestartFieldObjectChanged;
end;

procedure TWindow2D.SetXZoomEqualToY;
begin
  with FImage2D do
    if View2DZoomObj.XViewZoom <> View2DZoomObj.YViewZoom then
      begin
        View2DZoomObj.XViewZoom := View2DZoomObj.YViewZoom;
        View2DZoomObj.DidFieldsChange := true;
        RestartFieldObjectChanged;
      end; {if}
end;

procedure TWindow2D.SetYZoomEqualToX;
begin
  with FImage2D do
    if View2DZoomObj.XViewZoom <> View2DZoomObj.YViewZoom then
      begin
        View2DZoomObj.YViewZoom := View2DZoomObj.XViewZoom;
        View2DZoomObj.DidFieldsChange := true;
        RestartFieldObjectChanged;
      end; {if}
end;

function TWindow2D.ChangeGraphSettings: boolean;
begin
  Result := ModifyGraph2DSettings(FImage2D.Setting2DObj);
end;

end.
