unit Wind3D;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, WinGraph, Menus, Imag3D, WinBase, Vw3dDlg, Set3ddlg, Flistdlg,
  func3d, DgGlobal, CentrDlg;

type
  TWindow3D = class(TWindowGraph)
    Function1: TMenuItem;
    GraphSettings2: TMenuItem;
    N1: TMenuItem;
    New2DRotationXY1: TMenuItem;
    New3DFunctionXYZ1: TMenuItem;
    N2: TMenuItem;
    EditFuctionList1: TMenuItem;
    View1: TMenuItem;
    ChangeView1: TMenuItem;
    N3: TMenuItem;
    ZoomIn1: TMenuItem;
    ZoomOut1: TMenuItem;
    N4: TMenuItem;
    RotateNegX1: TMenuItem;
    RotatePosX1: TMenuItem;
    RotateNegY1: TMenuItem;
    RotatePosY1: TMenuItem;
    RotatePosZ1: TMenuItem;
    RotateNegZ1: TMenuItem;
    N5: TMenuItem;
    RotateAllAxistoZero1: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure GraphSettings2Click(Sender: TObject);
    procedure New2DRotationXY1Click(Sender: TObject);
    procedure New3DFunctionXYZ1Click(Sender: TObject);
    procedure EditFuctionList1Click(Sender: TObject);
    procedure ChangeView1Click(Sender: TObject);
    procedure ZoomIn1Click(Sender: TObject);
    procedure ZoomOut1Click(Sender: TObject);
    procedure RotateNegX1Click(Sender: TObject);
    procedure RotatePosX1Click(Sender: TObject);
    procedure RotateNegY1Click(Sender: TObject);
    procedure RotatePosY1Click(Sender: TObject);
    procedure RotatePosZ1Click(Sender: TObject);
    procedure RotateNegZ1Click(Sender: TObject);
    procedure RotateAllAxistoZero1Click(Sender: TObject);
  private
    { Private declarations }
    FImage3D : TImage3D;
  protected
    { Protected declarations }
    procedure SendWinActivateMessage; override;
  public
    { Public declarations }
    procedure ZoomIn; override;
    procedure ZoomOut; override;
    procedure RotatePosX;
    procedure RotateNegX;
    procedure RotatePosY;
    procedure RotateNegY;
    procedure RotatePosZ;
    procedure RotateNegZ;
    procedure CenterFunctionView; override;
    procedure ChangeView;
    function ChangeGraphSettings : boolean;
    function AddRotFunction : boolean;
    function AddC3DFunction : boolean;
    function ModifyFunctionList : boolean;
    procedure DisplayStatusMessage; override;
  end;

{var
  Window3D: TWindow3D;}

implementation

{$R *.dfm}

uses
  FSMain;

{ TWindow3D }

procedure TWindow3D.SendWinActivateMessage;
begin
  MainForm.Graph3DActivateMessage;
end;

procedure TWindow3D.FormCreate(Sender: TObject);
begin
  inherited;
  FImage3D := TImage3D.Create(Self);
  SetPlotImageBox(FImage3D);
end;

function TWindow3D.AddC3DFunction: boolean;
var
  Func3DObj : TFunc3DObj;
begin
  Result := false;
  if FImage3D.IsFreeFuncListObjSlot then
    begin
      Func3DObj := TFunc3DObj.Create;
      if NewUserFunctionObj(Func3DObj, FImage3D.GetNextDefaultColor, nil) then
        begin
          FImage3D.AddNewFuncListObj(Func3DObj);
          Result := true;
        end
      else
        Func3DObj.Free;
    end
  else
    begin
      MessageDlgOverMaxFuncCount(Max3DFuncCount);
    end; {else}
end;

function TWindow3D.AddRotFunction: boolean;
var
  FuncRotateObj : TFuncRotateObj;
begin
  Result := false;
  if FImage3D.IsFreeFuncListObjSlot then
    begin
      FuncRotateObj := TFuncRotateObj.Create;
      if NewUserFunctionObj(FuncRotateObj, FImage3D.GetNextDefaultColor, nil) then
        begin
          FImage3D.AddNewFuncListObj(FuncRotateObj);
          Result := true;
        end
      else
        FuncRotateObj.Free;
    end
  else
    begin
      MessageDlgOverMaxFuncCount(Max3DFuncCount);
    end; {else}
end;

procedure TWindow3D.CenterFunctionView;
begin
  with FImage3D.View3DZoomObj do
    begin
      if (RotateX <> 0) or (RotateY <> 0) or (RotateZ <> 0) then
        begin
          RotateX := 0;
          RotateY := 0;
          RotateZ := 0;
          DidFieldsChange := true;
          FImage3D.RestartFieldObjectChanged;
        end; {if}
    end; {with}
end;

function TWindow3D.ChangeGraphSettings: boolean;
begin
  Result := ModifyGraph3DSettings(FImage3D.Setting3DObj);
end;

procedure TWindow3D.ChangeView;
begin
  with FImage3D do
    if ModifyView3D(View3DZoomObj) then
      RestartFieldObjectChanged;
end;

procedure TWindow3D.DisplayStatusMessage;
begin
  with FImage3D do
    MainForm.StatusLineObj.UpDate3DStatusLine(View3DZoomObj,
              SettingBmpAttribsObj.BmpWidth, SettingBmpAttribsObj.BmpHeight,
              LastMouseX, LastMouseY);
end;

function TWindow3D.ModifyFunctionList: boolean;
var
  DidModify,
  DidDelete,
  DidReOrder : boolean;
begin
  DisplayFuncListDlg(FImage3D.FuncList, DidModify, DidDelete, DidReOrder);
  Result := (DidModify or DidDelete or DidReOrder);
end;

procedure TWindow3D.RotateNegX;
begin
  with FImage3D do
    begin
      View3DZoomObj.RotateX := View3DZoomObj.RotateX - MainForm.HotKey3DObj.Rotation3DIncAmt;
      if View3DZoomObj.RotateX >= 360 then
        View3DZoomObj.RotateX := View3DZoomObj.RotateX - 360;
      if View3DZoomObj.RotateX <= -360 then
        View3DZoomObj.RotateX := View3DZoomObj.RotateX + 360;
      View3DZoomObj.DidFieldsChange := true;
      RestartFieldObjectChanged;
    end; {with}
end;

procedure TWindow3D.RotateNegY;
begin
  with FImage3D do
    begin
      View3DZoomObj.RotateY := View3DZoomObj.RotateY - MainForm.HotKey3DObj.Rotation3DIncAmt;
      if View3DZoomObj.RotateY >= 360 then
        View3DZoomObj.RotateY := View3DZoomObj.RotateY - 360;
      if View3DZoomObj.RotateY <= -360 then
        View3DZoomObj.RotateY := View3DZoomObj.RotateY + 360;
      View3DZoomObj.DidFieldsChange := true;
      RestartFieldObjectChanged;
    end; {with}
end;

procedure TWindow3D.RotateNegZ;
begin
  with FImage3D do
    begin
      View3DZoomObj.RotateZ := View3DZoomObj.RotateZ - MainForm.HotKey3DObj.Rotation3DIncAmt;
      if View3DZoomObj.RotateZ >= 360 then
        View3DZoomObj.RotateZ := View3DZoomObj.RotateZ - 360;
      if View3DZoomObj.RotateZ <= -360 then
        View3DZoomObj.RotateZ := View3DZoomObj.RotateZ + 360;
      View3DZoomObj.DidFieldsChange := true;
      RestartFieldObjectChanged;
    end; {with}
end;

procedure TWindow3D.RotatePosX;
begin
  with FImage3D do
    begin
      View3DZoomObj.RotateX := View3DZoomObj.RotateX + MainForm.HotKey3DObj.Rotation3DIncAmt;
      if View3DZoomObj.RotateX >= 360 then
        View3DZoomObj.RotateX := View3DZoomObj.RotateX - 360;
      if View3DZoomObj.RotateX <= -360 then
        View3DZoomObj.RotateX := View3DZoomObj.RotateX + 360;
      View3DZoomObj.DidFieldsChange := true;
      RestartFieldObjectChanged;
    end; {with}
end;

procedure TWindow3D.RotatePosY;
begin
  with FImage3D do
    begin
      View3DZoomObj.RotateY := View3DZoomObj.RotateY + MainForm.HotKey3DObj.Rotation3DIncAmt;
      if View3DZoomObj.RotateY >= 360 then
        View3DZoomObj.RotateY := View3DZoomObj.RotateY - 360;
      if View3DZoomObj.RotateY <= -360 then
        View3DZoomObj.RotateY := View3DZoomObj.RotateY + 360;
      View3DZoomObj.DidFieldsChange := true;
      RestartFieldObjectChanged;
    end; {with}
end;

procedure TWindow3D.RotatePosZ;
begin
  with FImage3D do
    begin
      View3DZoomObj.RotateZ := View3DZoomObj.RotateZ + MainForm.HotKey3DObj.Rotation3DIncAmt;
      if View3DZoomObj.RotateZ >= 360 then
        View3DZoomObj.RotateZ := View3DZoomObj.RotateZ - 360;
      if View3DZoomObj.RotateZ <= -360 then
        View3DZoomObj.RotateZ := View3DZoomObj.RotateZ + 360;
      View3DZoomObj.DidFieldsChange := true;
      RestartFieldObjectChanged;
    end; {with}
end;

procedure TWindow3D.ZoomIn;
begin
  with FImage3D do
    if MainForm.HotKey3DObj.Calc3DZoomIn(View3DZoomObj.ViewZoom) then
      begin
        View3DZoomObj.DidFieldsChange := true;
        RestartFieldObjectChanged;
      end; {if}
end;

procedure TWindow3D.ZoomOut;
begin
  with FImage3D do
    if MainForm.HotKey3DObj.Calc3DZoomOut(View3DZoomObj.ViewZoom) then
      begin
        View3DZoomObj.DidFieldsChange := true;
        RestartFieldObjectChanged;
      end; {if}
end;

procedure TWindow3D.GraphSettings2Click(Sender: TObject);
begin
  if ChangeGraphSettings then
    FImage3D.RestartFieldObjectChanged;
end;

procedure TWindow3D.New2DRotationXY1Click(Sender: TObject);
begin
  if AddRotFunction then
    FImage3D.RestartFieldObjectChanged;
end;

procedure TWindow3D.New3DFunctionXYZ1Click(Sender: TObject);
begin
  if AddC3DFunction then
    FImage3D.RestartFieldObjectChanged;
end;

procedure TWindow3D.EditFuctionList1Click(Sender: TObject);
begin
  if ModifyFunctionList then
    FImage3D.RestartFieldObjectChanged;
end;

procedure TWindow3D.ChangeView1Click(Sender: TObject);
begin
  ChangeView;
end;

procedure TWindow3D.ZoomIn1Click(Sender: TObject);
begin
  ZoomIn;
end;

procedure TWindow3D.ZoomOut1Click(Sender: TObject);
begin
  ZoomOut;
end;

procedure TWindow3D.RotateNegX1Click(Sender: TObject);
begin
  RotateNegX;
end;

procedure TWindow3D.RotatePosX1Click(Sender: TObject);
begin
  RotatePosX;
end;

procedure TWindow3D.RotateNegY1Click(Sender: TObject);
begin
  RotateNegY;
end;

procedure TWindow3D.RotatePosY1Click(Sender: TObject);
begin
  RotatePosY;
end;

procedure TWindow3D.RotatePosZ1Click(Sender: TObject);
begin
  RotatePosZ;
end;

procedure TWindow3D.RotateNegZ1Click(Sender: TObject);
begin
  RotateNegZ;
end;

procedure TWindow3D.RotateAllAxistoZero1Click(Sender: TObject);
begin
  CenterFunctionView;
end;

end.
