unit addpara;

(*
Copyright 1999-2011 Greg VanMullem
This program is distributed under the terms of the GNU General Public License Version 3

This file is part of MathGV.

MathGV is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

MathGV is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MathGV.  If not, see <http://www.gnu.org/licenses/>.
*)

interface

uses SysUtils, WinTypes, WinProcs, Classes, Graphics, dialogs,
     Forms, Controls, Buttons, StdCtrls, ExtCtrls, Clsample, Spin,
     MathCalc, MathFunc, FuncObj, FuncPara, ComnDlgs, CentrDlg, editmath;

type
  TAddParametricFuncDlg = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    HelpBtn: TBitBtn;
    FuncStrBox: TGroupBox;
    Label1: TLabel;
    EXUserFuncStr: TEdit;
    Label3: TLabel;
    GroupBox1: TGroupBox;
    ColorButton1: TButton;
    EFuncColorSample: TColorSample;
    Label4: TLabel;
    ESamples: TSpinEdit;
    ELineWidth: TScrollBar;
    Label5: TLabel;
    LineWidthLabel: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    ELoBoundStr: TEditMathConst;
    EHiBoundStr: TEditMathConst;
    EYUserFuncStr: TEdit;
    Label2: TLabel;
    Label6: TLabel;
    procedure ELineWidthChange(Sender: TObject);
    procedure ColorButton1Click(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormShow(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
  private
    { Private declarations }
    FXFuncCalc,
    FYFuncCalc : TMathParametricFunc;
    function VerifyTheFields : boolean;
  public
    { Public declarations }
    IsGetNewFunction : boolean;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SetTheFields(TheFuncParametricObj : TFuncParametricObj);
    procedure GetTheFields(TheFuncParametricObj : TFuncParametricObj);
  end;

function GetNewParametricDlg(TheFuncParametricObj : TFuncParametricObj; TheCenterOnForm : TForm) : boolean;
function ModifyParametricDlg(TheFuncParametricObj : TFuncParametricObj; TheCenterOnForm : TForm) : boolean;

implementation

{$R *.DFM}

{---------------------------------------------------------------------}

function GetNewParametricDlg(TheFuncParametricObj : TFuncParametricObj; TheCenterOnForm : TForm) : boolean;
var
  AddParametricFuncDlg: TAddParametricFuncDlg;
begin {function GetNewParametricDlg}
  Result := false;
  AddParametricFuncDlg := TAddParametricFuncDlg.Create(Application);
  try
    SetDlgDefaultPos(AddParametricFuncDlg, TheCenterOnForm);
    AddParametricFuncDlg.IsGetNewFunction := true;
    AddParametricFuncDlg.Caption := 'Add Parametric Function';
    AddParametricFuncDlg.SetTheFields(TheFuncParametricObj);
    if AddParametricFuncDlg.ShowModal = IDOK then
      begin
        Result := true;
        AddParametricFuncDlg.GetTheFields(TheFuncParametricObj);
        TheFuncParametricObj.DidFieldsChange := true;
      end { if Result = true then}
  finally
    AddParametricFuncDlg.Free;
  end; {try...Finally}
end; {function GetNewParametricDlg}

{---------------------------------------------------------------------}

function ModifyParametricDlg(TheFuncParametricObj : TFuncParametricObj; TheCenterOnForm : TForm) : boolean;
var
  AddParametricFuncDlg: TAddParametricFuncDlg;
begin {function ModifyParametricDlg}
  Result := false;
  AddParametricFuncDlg := TAddParametricFuncDlg.Create(Application);
  try
    SetDlgDefaultPos(AddParametricFuncDlg, TheCenterOnForm);
    AddParametricFuncDlg.IsGetNewFunction := false;
    AddParametricFuncDlg.Caption := 'Modify Parametric Function';
    AddParametricFuncDlg.SetTheFields(TheFuncParametricObj);
    if AddParametricFuncDlg.ShowModal = IDOK then
      begin
        Result := true;
        AddParametricFuncDlg.GetTheFields(TheFuncParametricObj);
        TheFuncParametricObj.DidFieldsChange := true;
      end { if Result = true then}
  finally
    AddParametricFuncDlg.Free;
  end; {try...Finally}
end; {function ModifyParametricDlg}

{----------------------------------------------------------------------}

constructor TAddParametricFuncDlg.Create(AOwner: TComponent);
begin {constructor TAddParametricFuncDlg.Create}
  inherited Create(AOwner);
  FXFuncCalc := TMathParametricFunc.Create;
  FYFuncCalc := TMathParametricFunc.Create;
  { Add/sub 0.0001 to compensate for float comparison errors }
  with ELoBoundStr do
    begin
      emBoundsCheck := [CheckLoBound,LoBoundCanEqual,
                        CheckHiBound,HiBoundCanEqual];
      emLoBoundVal := (MinParametricDomain-0.0001);
      emLoBoundStr := MinParametricDomainStr;
      emHiBoundVal := (MaxParametricDomain+0.0001);
      emHiBoundStr := MaxParametricDomainStr;
    end; {with}
  with EHiBoundStr do
    begin
      emBoundsCheck := [CheckLoBound,LoBoundCanEqual,
                        CheckHiBound,HiBoundCanEqual];
      emLoBoundVal := (MinParametricDomain-0.0001);
      emLoBoundStr := MinParametricDomainStr;
      emHiBoundVal := (MaxParametricDomain+0.0001);
      emHiBoundStr := MaxParametricDomainStr;
    end; {with}
end; {constructor TAddParametricFuncDlg.Create}

{----------------------------------------------------------------------}

destructor TAddParametricFuncDlg.Destroy;
begin {destructor TAddParametricFuncDlg.Destroy;}
  FXFuncCalc.Free;
  FYFuncCalc.Free;
  inherited Destroy;
end; {destructor TAddParametricFuncDlg.Destroy;}

{----------------------------------------------------------------------}

function TAddParametricFuncDlg.VerifyTheFields : boolean;
begin {function TAddParametricFuncDlg.VerifyTheFields}
  Result := false;
  if ELoBoundStr.ValidateMathStr then
    if EHiBoundStr.ValidateMathStr then
      begin
        if ELoBoundStr.LastValue > EHiBoundStr.LastValue then
          begin
            MessageDlgCenter('Lower Bound must be <= Upper Bound.',
                       mtError, [mbOK], 0, Self);
            ELoBoundStr.SetFocus;
          end
        else
          begin
            //validate X function
            FXFuncCalc.FuncString := EXUserFuncStr.Text;
            if not FXFuncCalc.VerifyMathFunc then
              begin
                ShowFuncErrorMsg(FXFuncCalc.GetLastErrorString,
                                 '"X=" Function Definition', self);
                EXUserFuncStr.SetFocus;
                Exit;
              end; {if}
            //validate Y function
            FYFuncCalc.FuncString := EYUserFuncStr.Text;
            Result := FYFuncCalc.VerifyMathFunc;
            if not Result then
              begin
                ShowFuncErrorMsg(FYFuncCalc.GetLastErrorString,
                                 '"Y=" Function Definition', self);
                EYUserFuncStr.SetFocus;
                //Exit;
              end; {if}
          end; {else}
      end; {if}
end; {function TAddParametricFuncDlg.VerifyTheFields}

{---------------------------------------------------------------------}

procedure TAddParametricFuncDlg.SetTheFields(TheFuncParametricObj : TFuncParametricObj);
begin {procedure TAddParametricFuncDlg.SetTheFields}
  with TheFuncParametricObj do
    begin
      FXFuncCalc.Assign(XFuncCalc);
      FYFuncCalc.Assign(YFuncCalc);
      EXUserFuncStr.Text := XFuncCalc.FuncString;
      EYUserFuncStr.Text := YFuncCalc.FuncString;
      ELoBoundStr.text := LoBoundStr;
      EHiBoundStr.text := HiBoundStr;
      ESamples.Value := Samples;
      EFuncColorSample.ColorValue := FuncColor;
      ELineWidth.Position := LineWidth;
      LineWidthLabel.Caption := IntToStr(LineWidth);
    end; {with}
end; {procedure TAddParametricFuncDlg.SetTheFields}

{---------------------------------------------------------------------}

procedure TAddParametricFuncDlg.GetTheFields(TheFuncParametricObj : TFuncParametricObj);
begin {procedure TAddParametricFuncDlg.GetTheFields}
  with TheFuncParametricObj do
    begin
      XFuncCalc.Assign(FXFuncCalc);
      YFuncCalc.Assign(FYFuncCalc);
      LoBoundStr := ELoBoundStr.text;
      HiBoundStr := EHiBoundStr.text;
      LoBoundVal := ELoBoundStr.LastMathValue;
      HiBoundVal := EHiBoundStr.LastMathValue;
      Samples := ESamples.Value;
      FuncColor := EFuncColorSample.ColorValue;
      LineWidth := ELineWidth.Position;
    end; {with}
end; {procedure TAddParametricFuncDlg.GetTheFields}

{---------------------------------------------------------------------}

procedure TAddParametricFuncDlg.ELineWidthChange(Sender: TObject);
begin
  LineWidthLabel.Caption := IntToStr(ELineWidth.Position);
end;

procedure TAddParametricFuncDlg.ColorButton1Click(Sender: TObject);
begin
  EFuncColorSample.ColorValue := ReturnNewColorComnDlg(EFuncColorSample.ColorValue, Self);
end;

procedure TAddParametricFuncDlg.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  if ModalResult = idOK then
    begin
      CanClose := VerifyTheFields;
    end
  else
    CanClose := true;
end;

procedure TAddParametricFuncDlg.FormShow(Sender: TObject);
begin
  if IsGetNewFunction then
    EXUserFuncStr.SetFocus;
end;

procedure TAddParametricFuncDlg.FormKeyPress(Sender: TObject; var Key: Char);
begin
  FormKeyPressSpinEditClose(Self, Key);
end;

end.
