unit dfilldlg;

(*
Copyright 1999-2011 Greg VanMullem
This program is distributed under the terms of the GNU General Public License Version 3

This file is part of MathGV.

MathGV is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

MathGV is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MathGV.  If not, see <http://www.gnu.org/licenses/>.
*)

interface

uses WinTypes, WinProcs, Classes, Graphics, Forms, Controls, Buttons,
  StdCtrls, Spin, Dialogs, ExtCtrls, Clsample, ComnDlgs, CentrDlg, GrDraw, DrawObjs;

type
  TModifyDrawFloodFillDlg = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    HelpBtn: TBitBtn;
    BrushGroupBox: TGroupBox;
    SolidBrush: TSpeedButton;
    HorizontalBrush: TSpeedButton;
    VerticalBrush: TSpeedButton;
    ForwardDiagonalBrush: TSpeedButton;
    BrushColorSample: TColorSample;
    BackwardDiagonalBrush: TSpeedButton;
    CrossBrush: TSpeedButton;
    DiagonalCrossBrush: TSpeedButton;
    LocPointGroupBox: TGroupBox;
    EPointX: TSpinEdit;
    EPointY: TSpinEdit;
    Label1: TLabel;
    Label2: TLabel;
    BrushColorButton: TBitBtn;
    procedure BrushColorButtonClick(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
  private
    { Private declarations }
  public
    { Public declarations }
    procedure SetTheFields(TheDrawObj : TDrawFloodFill;
                           const TheBmpAttribRec : TBitmapAttribRec);
    procedure GetTheFields(TheDrawObj : TDrawFloodFill);
  end;

function ModifyFloodFillAttribDlg(TheDrawObj : TDrawFloodFill;
         const TheBmpAttribRec : TBitmapAttribRec; TheCenterOnForm : TForm) : boolean;

implementation

{$R *.DFM}

function ModifyFloodFillAttribDlg(TheDrawObj : TDrawFloodFill;
         const TheBmpAttribRec : TBitmapAttribRec; TheCenterOnForm : TForm) : boolean;
var
  ModifyDrawFloodFillDlg : TModifyDrawFloodFillDlg;
begin {function ModifyFloodFillAttribDlg}
  Result := false;
  ModifyDrawFloodFillDlg := TModifyDrawFloodFillDlg.Create(Application);
  try
    SetDlgDefaultPos(ModifyDrawFloodFillDlg, TheCenterOnForm);
    ModifyDrawFloodFillDlg.SetTheFields(TheDrawObj, TheBmpAttribRec);
    if ModifyDrawFloodFillDlg.ShowModal = IDOK then
      begin
        Result := true;
        ModifyDrawFloodFillDlg.GetTheFields(TheDrawObj);
        TheDrawObj.DidFieldsChange := true;
      end; {if}
  finally
    ModifyDrawFloodFillDlg.free;
  end; {try..finally}
end; {function ModifyFloodFillAttribDlg}

procedure TModifyDrawFloodFillDlg.SetTheFields(TheDrawObj: TDrawFloodFill;
                                      const TheBmpAttribRec : TBitmapAttribRec);
begin
  with TheDrawObj do
    begin
      EPointX.MaxValue := TheBmpAttribRec.BmpWidth - 1;
      EPointY.MaxValue := TheBmpAttribRec.BmpHeight - 1;
      EPointX.Value := DrawPoint.X;
      EPointY.Value := DrawPoint.Y;
      BrushColorSample.ColorValue := LabelBrushColor;
      case LabelBrushStyle of
        bsClear, bsSolid: SolidBrush.Down := true;
        bsHorizontal: HorizontalBrush.Down := true;
        bsVertical: VerticalBrush.Down := true;
        bsFDiagonal: ForwardDiagonalBrush.Down := true;
        bsBDiagonal: BackwardDiagonalBrush.Down := true;
        bsCross: CrossBrush.Down := true;
        bsDiagCross: DiagonalCrossBrush.Down := true;
      end; {case}
    end; {with}
end;

procedure TModifyDrawFloodFillDlg.GetTheFields(TheDrawObj: TDrawFloodFill);
begin
  with TheDrawObj do
    begin
      DrawPoint.X := EPointX.Value;
      DrawPoint.Y := EPointY.Value;
      LabelBrushColor := BrushColorSample.ColorValue;
      if SolidBrush.Down then LabelBrushStyle := bsSolid
      else if HorizontalBrush.Down then LabelBrushStyle := bsHorizontal
      else if VerticalBrush.Down then LabelBrushStyle := bsVertical
      else if ForwardDiagonalBrush.Down then LabelBrushStyle := bsFDiagonal
      else if BackwardDiagonalBrush.Down then LabelBrushStyle := bsBDiagonal
      else if CrossBrush.Down then LabelBrushStyle := bsCross
      else if DiagonalCrossBrush.Down then LabelBrushStyle := bsDiagCross;
    end; {with}
end;

procedure TModifyDrawFloodFillDlg.BrushColorButtonClick(Sender: TObject);
begin
  BrushColorSample.ColorValue := ReturnNewColorComnDlg(BrushColorSample.ColorValue, Self);
end;

procedure TModifyDrawFloodFillDlg.FormKeyPress(Sender: TObject;
  var Key: Char);
begin
  FormKeyPressSpinEditClose(Self, Key);
end;

end.
