unit dfreedlg;

(*
Copyright 1999-2011 Greg VanMullem
This program is distributed under the terms of the GNU General Public License Version 3

This file is part of MathGV.

MathGV is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

MathGV is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MathGV.  If not, see <http://www.gnu.org/licenses/>.
*)

interface

uses
  WinTypes, WinProcs, Classes, Graphics, Forms, Controls, Buttons,
  StdCtrls, Spin, Dialogs, ExtCtrls, SysUtils, Clsample, ComnDlgs,
  CentrDlg, DgDraw, GrDraw, DrawObjs;

type
  TModifyDrawFreeHandDlg = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    HelpBtn: TBitBtn;
    PenGroupBox: TGroupBox;
    SolidPen: TSpeedButton;
    DashPen: TSpeedButton;
    DotPen: TSpeedButton;
    DashDotPen: TSpeedButton;
    DashDotDotPen: TSpeedButton;
    PenColorSample: TColorSample;
    BrushGroupBox: TGroupBox;
    BrushColorSample: TColorSample;
    LabelPenWidthScroll: TScrollBar;
    LabelPenSize: TLabel;
    OriginGroupBox: TGroupBox;
    EOriginX: TSpinEdit;
    EOriginY: TSpinEdit;
    Label1: TLabel;
    Label2: TLabel;
    BrushColorButton: TBitBtn;
    PenColorButton: TBitBtn;
    ClearBrush: TSpeedButton;
    SolidBrush: TSpeedButton;
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure LabelPenWidthScrollChange(Sender: TObject);
    procedure PenColorButtonClick(Sender: TObject);
    procedure BrushColorButtonClick(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
  private
    { Private declarations }
    FDrawObj : TDrawFreeHand;
    FBmpAttribRec : TBitmapAttribRec;
  public
    { Public declarations }
    procedure SetTheFields(TheDrawObj : TDrawFreeHand;
                           const TheBmpAttribRec : TBitmapAttribRec);
    procedure GetTheFields(TheDrawObj : TDrawFreeHand);
  end;

function ModifyFreeHandAttribDlg(TheDrawObj : TDrawFreeHand;
           const TheBmpAttribRec : TBitmapAttribRec; TheCenterOnForm : TForm) : boolean;

implementation

{$R *.DFM}

function ModifyFreeHandAttribDlg(TheDrawObj : TDrawFreeHand;
           const TheBmpAttribRec : TBitmapAttribRec; TheCenterOnForm : TForm) : boolean;
var
  ModifyDrawFreeHandDlg : TModifyDrawFreeHandDlg;
begin {function ModifyFreeHandAttribDlg}
  Result := false;
  ModifyDrawFreeHandDlg := TModifyDrawFreeHandDlg.Create(Application);
  try
    SetDlgDefaultPos(ModifyDrawFreeHandDlg, TheCenterOnForm);
    ModifyDrawFreeHandDlg.SetTheFields(TheDrawObj, TheBmpAttribRec);
    if ModifyDrawFreeHandDlg.ShowModal = IDOK then
      begin
        Result := true;
        ModifyDrawFreeHandDlg.GetTheFields(TheDrawObj);
        TheDrawObj.DidFieldsChange := true;
      end; {if}
  finally
    ModifyDrawFreeHandDlg.free;
  end; {try..finally}
end; {function ModifyFreeHandAttribDlg}

procedure TModifyDrawFreeHandDlg.SetTheFields(TheDrawObj: TDrawFreeHand;
                                   const TheBmpAttribRec : TBitmapAttribRec);
begin
  FDrawObj := TheDrawObj;
  FBmpAttribRec := TheBmpAttribRec;
  with TheDrawObj do
    begin
      {Set Origin Cordinate}
      EOriginX.Value := OriginPoint.X;
      EOriginY.Value := OriginPoint.Y;
      {Set PenColor}
      PenColorSample.ColorValue := LabelPenColor;
      {Set PenStyle}
      case LabelPenStyle of
        psSolid: SolidPen.Down := true;
        psDash: DashPen.Down := true;
        psDot: DotPen.Down := true;
        psDashDot: DashDotPen.Down := true;
        psDashDotDot: DashDotDotPen.Down := true;
      end; {case}
      {Set PenWidth}
      LabelPenWidthScroll.Position := LabelPenWidth;
      {Set PenSize}
      LabelPenSize.Caption := IntToStr(LabelPenWidthScroll.Position);
      {Set BrushColor}
      BrushColorSample.ColorValue := LabelBrushColor;
      {Set BrushStyle}
      if LabelBrushStyle = bsSolid then
        SolidBrush.Down := true
      else
        ClearBrush.Down := true;
    end; {with}
end;

procedure TModifyDrawFreeHandDlg.GetTheFields(TheDrawObj: TDrawFreeHand);
begin
  with TheDrawObj do
    begin
      {Get Cordinates}
      DrawToPoint.X := DrawToPoint.X + (EOriginX.Value - OriginPoint.X);
      DrawToPoint.Y := DrawToPoint.Y + (EOriginY.Value - OriginPoint.Y);
      OriginPoint.X := EOriginX.Value;
      OriginPoint.Y := EOriginY.Value;
      {Get PenColor}
      LabelPenColor := PenColorSample.ColorValue;
      {Get PenStyle}
      if SolidPen.Down then LabelPenStyle := psSolid
      else if DashPen.Down then LabelPenStyle := psDash
      else if DotPen.Down then LabelPenStyle := psDot
      else if DashDotPen.Down then LabelPenStyle := psDashDot
      else if DashDotDotPen.Down then LabelPenStyle := psDashDotDot;
      {Get PenWidth}
      LabelPenWidth := LabelPenWidthScroll.Position;
      {Get BrushColor}
      LabelBrushColor := BrushColorSample.ColorValue;
      {Get BrushStyle}
      if SolidBrush.Down then
        LabelBrushStyle := bsSolid
      else
        LabelBrushStyle := bsClear;
    end; {with}
end;

procedure TModifyDrawFreeHandDlg.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  CanClose := true;
  if ModalResult = IDOK then
    begin
      with FDrawObj do
        CanClose := FDrawObj.AreCordinatesOnBmp(
          Point(EOriginX.Value, EOriginY.Value),
          Point((DrawToPoint.X + (EOriginX.Value - OriginPoint.X)),
                (DrawToPoint.Y + (EOriginY.Value - OriginPoint.Y))),
          FBmpAttribRec);
      if not CanClose then
        ShowErrorDlgLabelNotOnBmp;
    end; {if}
end;

procedure TModifyDrawFreeHandDlg.LabelPenWidthScrollChange(
  Sender: TObject);
begin
  LabelPenSize.Caption := IntToStr(LabelPenWidthScroll.Position);
end;

procedure TModifyDrawFreeHandDlg.PenColorButtonClick(Sender: TObject);
begin
  PenColorSample.ColorValue := ReturnNewColorComnDlg(PenColorSample.ColorValue, Self);
end;

procedure TModifyDrawFreeHandDlg.BrushColorButtonClick(Sender: TObject);
begin
  BrushColorSample.ColorValue := ReturnNewColorComnDlg(BrushColorSample.ColorValue, Self);
end;

procedure TModifyDrawFreeHandDlg.FormKeyPress(Sender: TObject;
  var Key: Char);
begin
  FormKeyPressSpinEditClose(Self, Key);
end;

end.
