unit dlinedlg;

(*
Copyright 1999-2011 Greg VanMullem
This program is distributed under the terms of the GNU General Public License Version 3

This file is part of MathGV.

MathGV is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

MathGV is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MathGV.  If not, see <http://www.gnu.org/licenses/>.
*)

interface

uses WinTypes, WinProcs, Classes, Graphics, Forms, Controls, Buttons,
  StdCtrls, Spin, Dialogs, ExtCtrls, SysUtils, Clsample, ComnDlgs, CentrDlg,
  DgDraw, GrDraw, DrawObjs;

type
  TModifyDrawLineDlg = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    HelpBtn: TBitBtn;
    PenGroupBox: TGroupBox;
    SolidPen: TSpeedButton;
    DashPen: TSpeedButton;
    DotPen: TSpeedButton;
    DashDotPen: TSpeedButton;
    DashDotDotPen: TSpeedButton;
    PenColorSample: TColorSample;
    BrushGroupBox: TGroupBox;
    BrushColorSample: TColorSample;
    LabelPenWidthScroll: TScrollBar;
    LabelPenSize: TLabel;
    OriginGroupBox: TGroupBox;
    EOriginPointX: TSpinEdit;
    EOriginPointY: TSpinEdit;
    Label1: TLabel;
    Label2: TLabel;
    EndPointGroupBox: TGroupBox;
    Label3: TLabel;
    Label4: TLabel;
    EEndPointX: TSpinEdit;
    EEndPointY: TSpinEdit;
    PenColorButton: TBitBtn;
    BrushColorButton: TBitBtn;
    ClearBrush: TSpeedButton;
    SolidBrush: TSpeedButton;
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure LabelPenWidthScrollChange(Sender: TObject);
    procedure PenColorButtonClick(Sender: TObject);
    procedure BrushColorButtonClick(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
  private
    { Private declarations }
    FDrawObj : TDrawLine;
    FBmpAttribRec : TBitmapAttribRec;
  public
    { Public declarations }
    procedure SetTheFields(TheDrawObj : TDrawLine;
                           const TheBmpAttribRec : TBitmapAttribRec);
    procedure GetTheFields(TheDrawObj : TDrawLine);
  end;

function ModifyLineAttribDlg(TheDrawObj : TDrawLine;
           const TheBmpAttribRec : TBitmapAttribRec; TheCenterOnForm : TForm) : boolean;

implementation

{$R *.DFM}

function ModifyLineAttribDlg(TheDrawObj : TDrawLine;
           const TheBmpAttribRec : TBitmapAttribRec; TheCenterOnForm : TForm) : boolean;
var
  ModifyDrawLineDlg : TModifyDrawLineDlg;
begin {function ModifyLineAttribDlg}
  Result := false;
  ModifyDrawLineDlg := TModifyDrawLineDlg.Create(Application);
  try
    SetDlgDefaultPos(ModifyDrawLineDlg, TheCenterOnForm);
    ModifyDrawLineDlg.SetTheFields(TheDrawObj, TheBmpAttribRec);
    if ModifyDrawLineDlg.ShowModal = IDOK then
      begin
        Result := true;
        ModifyDrawLineDlg.GetTheFields(TheDrawObj);
        TheDrawObj.DidFieldsChange := true;
      end; {if}
  finally
    ModifyDrawLineDlg.free;
  end; {try..finally}
end; {function ModifyLineAttribDlg}

procedure TModifyDrawLineDlg.SetTheFields(TheDrawObj: TDrawLine;
                                          const TheBmpAttribRec : TBitmapAttribRec);
begin
  FDrawObj := TheDrawObj;
  FBmpAttribRec := TheBmpAttribRec;
  with TheDrawObj do
    begin
      {Begin Set Min and Max Point Values}
      EOriginPointX.MinValue := -100;
      EOriginPointX.MaxValue := FindLabelDlgMaxValue(OriginPoint.X, TheBmpAttribRec.BmpWidth);
      EOriginPointY.MinValue := -100;
      EOriginPointY.MaxValue := FindLabelDlgMaxValue(OriginPoint.Y, TheBmpAttribRec.BmpHeight);
      EEndPointX.MinValue := -100;
      EEndPointX.MaxValue := FindLabelDlgMaxValue(DrawToPoint.X, TheBmpAttribRec.BmpWidth);
      EEndPointY.MinValue := -100;
      EEndPointY.MaxValue := FindLabelDlgMaxValue(DrawToPoint.Y, TheBmpAttribRec.BmpHeight);
      {End Set Min and Max Point Values}
      EOriginPointX.Value := OriginPoint.X;
      EOriginPointY.Value := OriginPoint.Y;
      EEndPointX.Value := DrawToPoint.X;
      EEndPointY.Value := DrawToPoint.Y;
      PenColorSample.ColorValue := LabelPenColor;
      case LabelPenStyle of
        psSolid: SolidPen.Down := true;
        psDash: DashPen.Down := true;
        psDot: DotPen.Down := true;
        psDashDot: DashDotPen.Down := true;
        psDashDotDot: DashDotDotPen.Down := true;
      end; {case}
      LabelPenWidthScroll.Position := LabelPenWidth;
      LabelPenSize.Caption := IntToStr(LabelPenWidthScroll.Position);
      BrushColorSample.ColorValue := LabelBrushColor;
      {Set BrushStyle}
      if LabelBrushStyle = bsSolid then
        SolidBrush.Down := true
      else
        ClearBrush.Down := true;
    end; {with}
end;

procedure TModifyDrawLineDlg.GetTheFields(TheDrawObj: TDrawLine);
begin
  with TheDrawObj do
    begin
      OriginPoint.X := EOriginPointX.Value;
      OriginPoint.Y := EOriginPointY.Value;
      DrawToPoint.X := EEndPointX.Value;
      DrawToPoint.Y := EEndPointY.Value;
      LabelPenColor := PenColorSample.ColorValue;
      if SolidPen.Down then LabelPenStyle := psSolid
      else if DashPen.Down then LabelPenStyle := psDash
      else if DotPen.Down then LabelPenStyle := psDot
      else if DashDotPen.Down then LabelPenStyle := psDashDot
      else if DashDotDotPen.Down then LabelPenStyle := psDashDotDot;
      LabelPenWidth := LabelPenWidthScroll.Position;
      LabelBrushColor := BrushColorSample.ColorValue;
      {Get BrushStyle}
      if SolidBrush.Down then
        LabelBrushStyle := bsSolid
      else
        LabelBrushStyle := bsClear;
    end; {with}
end;

procedure TModifyDrawLineDlg.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  CanClose := true;
  if ModalResult = IDOK then
    begin
      CanClose := FDrawObj.AreCordinatesOnBmp(
                         Point(EOriginPointX.Value, EOriginPointY.Value),
                         Point(EEndPointX.Value, EEndPointY.Value),
                         FBmpAttribRec);
      if not CanClose then
        ShowErrorDlgLabelNotOnBmp;
    end; {if}
end;

procedure TModifyDrawLineDlg.LabelPenWidthScrollChange(Sender: TObject);
begin
  LabelPenSize.Caption := IntToStr(LabelPenWidthScroll.Position);
end;

procedure TModifyDrawLineDlg.PenColorButtonClick(Sender: TObject);
begin
  PenColorSample.ColorValue := ReturnNewColorComnDlg(PenColorSample.ColorValue, Self);
end;

procedure TModifyDrawLineDlg.BrushColorButtonClick(Sender: TObject);
begin
  BrushColorSample.ColorValue := ReturnNewColorComnDlg(BrushColorSample.ColorValue, Self);
end;

procedure TModifyDrawLineDlg.FormKeyPress(Sender: TObject; var Key: Char);
begin
  FormKeyPressSpinEditClose(Self, Key);
end;

end.
