unit drectdlg;

(*
Copyright 1999-2011 Greg VanMullem
This program is distributed under the terms of the GNU General Public License Version 3

This file is part of MathGV.

MathGV is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

MathGV is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MathGV.  If not, see <http://www.gnu.org/licenses/>.
*)

interface

uses WinTypes, WinProcs, Classes, Graphics, Forms, Controls, Buttons,
  StdCtrls, Spin, Dialogs, SysUtils, ExtCtrls, Clsample, GrDraw, DgDraw,
  DrawObjs, ComnDlgs, CentrDlg;

type
  TModifyDrawInclosedDlg = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    HelpBtn: TBitBtn;
    PenGroupBox: TGroupBox;
    SolidPen: TSpeedButton;
    DashPen: TSpeedButton;
    DotPen: TSpeedButton;
    DashDotPen: TSpeedButton;
    DashDotDotPen: TSpeedButton;
    PenColorSample: TColorSample;
    BrushGroupBox: TGroupBox;
    ClearBrush: TSpeedButton;
    SolidBrush: TSpeedButton;
    HorizontalBrush: TSpeedButton;
    VerticalBrush: TSpeedButton;
    ForwardDiagonalBrush: TSpeedButton;
    BrushColorSample: TColorSample;
    ClearPen: TSpeedButton;
    BackwardDiagonalBrush: TSpeedButton;
    CrossBrush: TSpeedButton;
    DiagonalCrossBrush: TSpeedButton;
    LabelPenWidthScroll: TScrollBar;
    LabelPenSize: TLabel;
    TopLeftGroupBox: TGroupBox;
    ETopLeftX: TSpinEdit;
    ETopLeftY: TSpinEdit;
    Label1: TLabel;
    Label2: TLabel;
    BottomRightGroupBox: TGroupBox;
    Label3: TLabel;
    Label4: TLabel;
    EBottomRightX: TSpinEdit;
    EBottomRightY: TSpinEdit;
    ShapeGroupBox: TGroupBox;
    RectangleButton: TSpeedButton;
    EllipseButton: TSpeedButton;
    RoundRectButton: TSpeedButton;
    PenColorButton: TBitBtn;
    BrushColorButton: TBitBtn;
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure LabelPenWidthScrollChange(Sender: TObject);
    procedure PenColorButtonClick(Sender: TObject);
    procedure BrushColorButtonClick(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
  private
    { Private declarations }
    FDrawObj : TDrawInclosedShape;
    FBmpAttribRec : TBitmapAttribRec;
  public
    { Public declarations }
    procedure SetTheFields(TheDrawObj : TDrawInclosedShape;
                           const TheBmpAttribRec : TBitmapAttribRec);
    procedure GetTheFields(TheDrawObj : TDrawInclosedShape);
  end;

function ModifyInclosedAttribDlg(TheDrawObj : TDrawInclosedShape;
           const TheBmpAttribRec : TBitmapAttribRec; TheCenterOnForm : TForm) : boolean;

implementation

{$R *.DFM}

function ModifyInclosedAttribDlg(TheDrawObj : TDrawInclosedShape;
           const TheBmpAttribRec : TBitmapAttribRec; TheCenterOnForm : TForm) : boolean;
var
  ModifyDrawInclosedDlg : TModifyDrawInclosedDlg;
begin {function ModifyInclosedAttribDlg}
  Result := false;
  ModifyDrawInclosedDlg := TModifyDrawInclosedDlg.Create(Application);
  try
    SetDlgDefaultPos(ModifyDrawInclosedDlg, TheCenterOnForm);
    ModifyDrawInclosedDlg.SetTheFields(TheDrawObj, TheBmpAttribRec);
    if ModifyDrawInclosedDlg.ShowModal = IDOK then
      begin
        Result := true;
        ModifyDrawInclosedDlg.GetTheFields(TheDrawObj);
        TheDrawObj.DidFieldsChange := true;
      end; {if}
  finally
    ModifyDrawInclosedDlg.free;
  end; {try..finally}
end; {function ModifyInclosedAttribDlg}

procedure TModifyDrawInclosedDlg.SetTheFields(TheDrawObj: TDrawInclosedShape;
                                      const TheBmpAttribRec : TBitmapAttribRec);
var
  TmpTopLeft, TmpBottomRight : TPoint;
begin
  FDrawObj := TheDrawObj;
  FBmpAttribRec := TheBmpAttribRec;
  with TheDrawObj do
    begin
      {Begin Set Min and Max Point Values}
      ETopLeftX.MinValue := -100;
      ETopLeftX.MaxValue := FindLabelDlgMaxValue(OriginPoint.X,
                                                 TheBmpAttribRec.BmpWidth);
      ETopLeftY.MinValue := -100;
      ETopLeftY.MaxValue := FindLabelDlgMaxValue(OriginPoint.Y,
                                                 TheBmpAttribRec.BmpHeight);
      EBottomRightX.MinValue := -100;
      EBottomRightX.MaxValue := FindLabelDlgMaxValue(DrawToPoint.X,
                                                     TheBmpAttribRec.BmpWidth);
      EBottomRightY.MinValue := -100;
      EBottomRightY.MaxValue := FindLabelDlgMaxValue(DrawToPoint.Y,
                                                     TheBmpAttribRec.BmpHeight);
      {End Set Min and Max Point Values}
      {Set Cordinates}
      TmpTopLeft := OriginPoint;
      TmpBottomRight := DrawToPoint;
      FindTopLeftPoint(TmpTopLeft, TmpBottomRight);
      ETopLeftX.Value := TmpTopLeft.X;
      ETopLeftY.Value := TmpTopLeft.Y;
      EBottomRightX.Value := TmpBottomRight.X;
      EBottomRightY.Value := TmpBottomRight.Y;
      {Set PenColor}
      PenColorSample.ColorValue := LabelPenColor;
      {Set PenStyle}
      case LabelPenStyle of
        psClear: ClearPen.Down := true;
        psSolid: SolidPen.Down := true;
        psDash: DashPen.Down := true;
        psDot: DotPen.Down := true;
        psDashDot: DashDotPen.Down := true;
        psDashDotDot: DashDotDotPen.Down := true;
      end; {case}
      {Set PenWidth}
      LabelPenWidthScroll.Position := LabelPenWidth;
      {Set PenSize}
      LabelPenSize.Caption := IntToStr(LabelPenWidthScroll.Position);
      {Set BrushColor}
      BrushColorSample.ColorValue := LabelBrushColor;
      {Set BrushStyle}
      case LabelBrushStyle of
        bsClear: ClearBrush.Down := true;
        bsSolid: SolidBrush.Down := true;
        bsHorizontal: HorizontalBrush.Down := true;
        bsVertical: VerticalBrush.Down := true;
        bsFDiagonal: ForwardDiagonalBrush.Down := true;
        bsBDiagonal: BackwardDiagonalBrush.Down := true;
        bsCross: CrossBrush.Down := true;
        bsDiagCross: DiagonalCrossBrush.Down := true;
      end; {case}
      {Set DrawingTool}
      case DrawingTool of
        dtRectangle: RectangleButton.Down := true;
        dtEllipse: EllipseButton.Down := true;
        dtRoundRect: RoundRectButton.Down := true;
      end; {case}
    end; {with}
end;

procedure TModifyDrawInclosedDlg.GetTheFields(TheDrawObj: TDrawInclosedShape);
begin
  with TheDrawObj do
    begin
      {Get Cordinates}
      OriginPoint.X := ETopLeftX.Value;
      OriginPoint.Y := ETopLeftY.Value;
      DrawToPoint.X := EBottomRightX.Value;
      DrawToPoint.Y := EBottomRightY.Value;
      {Get PenColor}
      LabelPenColor := PenColorSample.ColorValue;
      {Get PenStyle}
      if ClearPen.Down then LabelPenStyle := psClear
      else if SolidPen.Down then LabelPenStyle := psSolid
      else if DashPen.Down then LabelPenStyle := psDash
      else if DotPen.Down then LabelPenStyle := psDot
      else if DashDotPen.Down then LabelPenStyle := psDashDot
      else if DashDotDotPen.Down then LabelPenStyle := psDashDotDot;
      {Get PenWidth}
      LabelPenWidth := LabelPenWidthScroll.Position;
      {Get BrushColor}
      LabelBrushColor := BrushColorSample.ColorValue;
      {Get BrushStyle}
      if ClearBrush.Down then LabelBrushStyle := bsClear
      else if SolidBrush.Down then LabelBrushStyle := bsSolid
      else if HorizontalBrush.Down then LabelBrushStyle := bsHorizontal
      else if VerticalBrush.Down then LabelBrushStyle := bsVertical
      else if ForwardDiagonalBrush.Down then LabelBrushStyle := bsFDiagonal
      else if BackwardDiagonalBrush.Down then LabelBrushStyle := bsBDiagonal
      else if CrossBrush.Down then LabelBrushStyle := bsCross
      else if DiagonalCrossBrush.Down then LabelBrushStyle := bsDiagCross;
      {Get Shape}
      if RectangleButton.Down then DrawingTool := dtRectangle
      else if EllipseButton.Down then DrawingTool := dtEllipse
      else if RoundRectButton.Down then DrawingTool := dtRoundRect;
    end; {with}
end;

procedure TModifyDrawInclosedDlg.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  CanClose := true;
  if ModalResult = IDOK then
    begin
      if ETopLeftX.Value > EBottomRightX.Value then
        begin
          CanClose := false;
          MessageDlgCenter('Top Left X must be <= Bottom Right X.',mtError,[mbok],0,Self);
        end
      else
        if ETopLeftY.Value > EBottomRightY.Value then
          begin
            CanClose := false;
            MessageDlgCenter('Top Left Y must be <= Bottom Right Y.',mtError,[mbok],0,Self);
          end
        else
          begin
            CanClose := FDrawObj.AreCordinatesOnBmp(
                         Point(ETopLeftX.Value, ETopLeftY.Value),
                         Point(EBottomRightX.Value, EBottomRightY.Value),
                         FBmpAttribRec);
            if not CanClose then
              ShowErrorDlgLabelNotOnBmp;
          end; {else}
    end; {if}
end;

procedure TModifyDrawInclosedDlg.LabelPenWidthScrollChange(
  Sender: TObject);
begin
  LabelPenSize.Caption := IntToStr(LabelPenWidthScroll.Position);
end;

procedure TModifyDrawInclosedDlg.PenColorButtonClick(Sender: TObject);
begin
  PenColorSample.ColorValue := ReturnNewColorComnDlg(PenColorSample.ColorValue, Self);
end;

procedure TModifyDrawInclosedDlg.BrushColorButtonClick(Sender: TObject);
begin
  BrushColorSample.ColorValue := ReturnNewColorComnDlg(BrushColorSample.ColorValue, Self);
end;

procedure TModifyDrawInclosedDlg.FormKeyPress(Sender: TObject;
  var Key: Char);
begin
  FormKeyPressSpinEditClose(Self, Key);
end;

end.
