unit dtextdlg;

(*
Copyright 1999-2011 Greg VanMullem
This program is distributed under the terms of the GNU General Public License Version 3

This file is part of MathGV.

MathGV is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

MathGV is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MathGV.  If not, see <http://www.gnu.org/licenses/>.
*)

interface

uses WinTypes, WinProcs, Classes, Graphics, Forms, Controls, Buttons,
  StdCtrls, ExtCtrls, Spin, SysUtils, GrDraw, DgDraw, DrawObjs, Dialogs, DefFont,
  DgGlobal, ComnDlgs, CentrDlg;

type
  TModifyDrawTextDlg = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    HelpBtn: TBitBtn;
    TopLeftGroupBox: TGroupBox;
    Label2: TLabel;
    Label3: TLabel;
    ETopLeftX: TSpinEdit;
    ETopLeftY: TSpinEdit;
    LabelAttribBox: TGroupBox;
    ETextString: TEdit;
    Label4: TLabel;
    EFontCombo: TComboBox;
    EFontSizeCombo: TComboBox;
    EBoldFontButton: TSpeedButton;
    EItalicFontButton: TSpeedButton;
    EUnderlineFontButton: TSpeedButton;
    Label1: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    FontDlgButton: TButton;
    ScrollBox1: TScrollBox;
    TextSampleLabel: TLabel;
    Label7: TLabel;
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure EFontComboChange(Sender: TObject);
    procedure FontDlgButtonClick(Sender: TObject);
    procedure EBoldFontButtonClick(Sender: TObject);
    procedure EItalicFontButtonClick(Sender: TObject);
    procedure EUnderlineFontButtonClick(Sender: TObject);
    procedure EFontSizeComboExit(Sender: TObject);
    procedure ETextStringChange(Sender: TObject);
    procedure EFontSizeComboChange(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
  private
    { Private declarations }
    FDrawObj : TDrawText;
    FBitmap : TBitmap;
    FBmpAttribRec : TBitmapAttribRec;
    FLabelFont : TFont;
    procedure UpdateTextSample;
  public
    { Public declarations }
    VertResolu : Integer;
    procedure SetTheFields(TheDrawObj : TDrawText; TheBitmap : TBitmap;
                           const TheBmpAttribRec : TBitmapAttribRec);
    procedure GetTheFields(TheDrawObj : TDrawText);
    procedure UpdateFontFields;
    procedure UpdateFontSizesCombo;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  end;

function ModifyTextAttribDlg(TheDrawObj : TDrawText; TheBitmap : TBitmap;
           const TheBmpAttribRec : TBitmapAttribRec; TheCenterOnForm : TForm) : boolean;

implementation

{$R *.DFM}

function ModifyTextAttribDlg(TheDrawObj : TDrawText; TheBitmap : TBitmap;
           const TheBmpAttribRec : TBitmapAttribRec; TheCenterOnForm : TForm) : boolean;
var
  ModifyDrawTextDlg : TModifyDrawTextDlg;
begin {function ModifyTextAttribDlg}
  Result := false;
  ModifyDrawTextDlg := TModifyDrawTextDlg.Create(Application);
  try
    SetDlgDefaultPos(ModifyDrawTextDlg, TheCenterOnForm);
    ModifyDrawTextDlg.SetTheFields(TheDrawObj, TheBitmap, TheBmpAttribRec);
    if ModifyDrawTextDlg.ShowModal = IDOK then
      begin
        Result := true;
        ModifyDrawTextDlg.GetTheFields(TheDrawObj);
        TheDrawObj.DidFieldsChange := true;
      end; {if}
  finally
    ModifyDrawTextDlg.free;
  end; {try..finally}
end; {function ModifyTextAttribDlg}

{ TModifyDrawTextDlg }

function EnumFontFamProc(var LogFont: TLogFont; var TextMetric:
  TTextMetric; FontType: Integer;
  TF: TModifyDrawTextDlg): Integer; Export; StdCall;
var
  S : string;
begin
  if FontType and TRUETYPE_FONTTYPE > 0 then
    Result := 0 {stop enumerating}
  else
    begin
      Result := 1;
      {calc points from TextMetric.tmHeight, per Microsoft}
      S := Format('%.2d', [Round((TextMetric.tmHeight -
        TextMetric.tmInternalLeading)*72 / TF.VertResolu)]);
      with TF.EFontSizeCombo.Items do
        if IndexOf(S) = -1 then
          Add(S);
    end;
end;

procedure TModifyDrawTextDlg.GetTheFields(TheDrawObj: TDrawText);
begin
  with TheDrawObj do
    begin
      LabelTextStr := ETextString.Text;
      LabelTextFont.Assign(FLabelFont);
      OriginPoint.X := ETopLeftX.Value;
      OriginPoint.Y := ETopLeftY.Value;
      DrawToPoint := CalcTextBottomRight(OriginPoint, LabelTextStr,
                                     LabelTextFont, FBitmap, FBmpAttribRec);
    end; {with}
end;

procedure TModifyDrawTextDlg.SetTheFields(TheDrawObj: TDrawText;
            TheBitmap : TBitmap; const TheBmpAttribRec : TBitmapAttribRec);
begin
  FDrawObj := TheDrawObj;
  FBitmap := TheBitmap;
  FBmpAttribRec := TheBmpAttribRec;
  with TheDrawObj do
    begin
      ETopLeftX.Value := OriginPoint.X;
      ETopLeftY.Value := OriginPoint.Y;
      ETextString.Text := LabelTextStr;
      FLabelFont.Assign(LabelTextFont);
    end; {with}
  UpdateFontFields;
end;

procedure TModifyDrawTextDlg.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
var
  TmpTopLeftPoint : TPoint;
begin
  CanClose := true;
  if ModalResult = IDOK then
    begin
      with TmpTopLeftPoint do
        begin
          X := ETopLeftX.Value;
          Y := ETopLeftY.Value;
        end; {with}
      CanClose := FDrawObj.AreCordinatesOnBmp(TmpTopLeftPoint,
          FDrawObj.CalcTextBottomRight(TmpTopLeftPoint, ETextString.Text,
                                       FLabelFont, FBitmap, FBmpAttribRec),
                                              FBmpAttribRec);
      if not CanClose then
        ShowErrorDlgLabelNotOnBmp;
    end; {if}
end;

constructor TModifyDrawTextDlg.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ETextString.MaxLength := MaxTextLabelLength;
  FLabelFont := CreateANewTFontClass;
  EFontCombo.Items := Screen.Fonts;
  VertResolu := GetDeviceCaps(Canvas.Handle, LOGPIXELSY);
end;

destructor TModifyDrawTextDlg.Destroy;
begin
  FLabelFont.Free;
  inherited Destroy;
end;

procedure TModifyDrawTextDlg.EFontComboChange(Sender: TObject);
begin
  if (EFontCombo.Text <> FLabelFont.Name)and(EFontCombo.Text <> '') then
    begin
      FLabelFont.Name := EFontCombo.Text;
      UpdateFontSizesCombo;
      UpdateTextSample;
    end; {if}
end;

procedure TModifyDrawTextDlg.UpdateFontFields;
begin
  with FLabelFont do
    begin
      EFontCombo.ItemIndex := EFontCombo.Items.IndexOf(Name);
      UpdateFontSizesCombo;
      EBoldFontButton.Down := (fsBold in Style);
      EItalicFontButton.Down := (fsItalic in Style);
      EUnderlineFontButton.Down := (fsUnderline in Style);
    end; {with}
  UpdateTextSample;
end;

procedure TModifyDrawTextDlg.UpdateFontSizesCombo;
var
  TmpFontNameStr : string;
begin
  TmpFontNameStr := FLabelFont.Name;
  EFontSizeCombo.Clear;
  EnumFontFamilies(Canvas.Handle, PChar(TmpFontNameStr),
                   @EnumFontFamProc, LongInt(Self));
  if EFontSizeCombo.Items.Count = 0 then {is TrueType}
    EFontSizeCombo.Items.SetText(TrueTypeFontSizes);
  EFontSizeCombo.Text := IntToStr(FLabelFont.Size);
end;

procedure TModifyDrawTextDlg.FontDlgButtonClick(Sender: TObject);
begin
  if ShowComnFontDlg(FLabelFont, Self) then
    UpdateFontFields;
end;

procedure TModifyDrawTextDlg.EBoldFontButtonClick(Sender: TObject);
begin
  if EBoldFontButton.Down then
    FLabelFont.Style := FLabelFont.Style + [fsBold]
  else
    FLabelFont.Style := FLabelFont.Style - [fsBold];
  UpdateTextSample;
end;

procedure TModifyDrawTextDlg.EItalicFontButtonClick(Sender: TObject);
begin
  if EItalicFontButton.Down then
    FLabelFont.Style := FLabelFont.Style + [fsItalic]
  else
    FLabelFont.Style := FLabelFont.Style - [fsItalic];
  UpdateTextSample;
end;

procedure TModifyDrawTextDlg.EUnderlineFontButtonClick(Sender: TObject);
begin
  if EUnderlineFontButton.Down then
    FLabelFont.Style := FLabelFont.Style + [fsUnderline]
  else
    FLabelFont.Style := FLabelFont.Style - [fsUnderline];
  UpdateTextSample;
end;

procedure TModifyDrawTextDlg.EFontSizeComboExit(Sender: TObject);
var
  NewSize : integer;
begin
  try
    NewSize := StrToInt(EFontSizeCombo.Text);
  except
    NewSize := FLabelFont.Size;
  end; {try..except}
  if NewSize > 9999 then
    NewSize := 9999;
  if NewSize < -9999 then
    NewSize := -9999;
  FLabelFont.Size := NewSize;
  EFontSizeCombo.Text := IntToStr(FLabelFont.Size);
  UpdateTextSample;
end;

procedure TModifyDrawTextDlg.EFontSizeComboChange(Sender: TObject);
var
  NewSize : integer;
begin
  try
    NewSize := StrToInt(EFontSizeCombo.Text);
  except
    NewSize := FLabelFont.Size;
  end; {try..except}
  if NewSize > 9999 then
    NewSize := 9999;
  if NewSize < -9999 then
    NewSize := -9999;
  FLabelFont.Size := NewSize;
  UpdateTextSample;
end;

procedure TModifyDrawTextDlg.UpdateTextSample;
begin
  if ETextString.Text = '' then
    TextSampleLabel.Caption := 'Sample Text '
  else
    TextSampleLabel.Caption := ETextString.Text + ' ';
  TextSampleLabel.Font.Assign(FLabelFont);
end;

procedure TModifyDrawTextDlg.ETextStringChange(Sender: TObject);
begin
  UpdateTextSample;
end;

procedure TModifyDrawTextDlg.FormKeyPress(Sender: TObject; var Key: Char);
begin
  FormKeyPressSpinEditClose(Self, Key);
end;

end.
