unit func3d;

(*
Copyright 1999-2011 Greg VanMullem
This program is distributed under the terms of the GNU General Public License Version 3

This file is part of MathGV.

MathGV is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

MathGV is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MathGV.  If not, see <http://www.gnu.org/licenses/>.
*)

interface

uses SysUtils, Graphics, MathCalc, MathFunc, FuncObj, fg3DLine, GrGlobal, FileSave;

const
  f3D_FileType_Rotate = 1;
  f3D_FileType_3D = 2;

type
  TGraph3DFuncType = (ft_Rotate, ft_3DX, ft_3DY, ft_3DZ);

  TMath3DFunc = class(TMathGraphFunc)
  private
    { Private declarations }
    FGraph3DFuncType: TGraph3DFuncType;
    procedure SetGraph3DFuncType(Value : TGraph3DFuncType);
  public
    { Public declarations }
    property Graph3DFuncType : TGraph3DFuncType read FGraph3DFuncType
                                                write SetGraph3DFuncType;
    constructor Create;
    procedure Assign(SourceFunc : TMathFunction); override;
    procedure SaveToDisk(WriteGraphFile : TWriteGraphFile); override;
    procedure ReadFromDisk(ReadGraphFile : TReadGraphFile); override;
  end; {TMath3DFunc = class(TMathGraphFunc)}

  TMathRotateFunc = class(TMathGraphFunc)
  private
    { Private declarations }
  public
    { Public declarations }
    constructor Create;
  end; {TMathRotateFunc = class(TMathGraphFunc)}

  TBaseFunc3DObj = class(TBaseFuncObj)
    private
    { Private declarations }
    protected
    { Protected declarations }
    public
    { Public declarations }
    function AddLinesToList(TheFuncNum : integer;
                            LineObj : TLineObj) : TLineListError; virtual; abstract;
    procedure SaveToDisk(WriteGraphFile : TWriteGraphFile); virtual; abstract;
    procedure ReadFromDisk(ReadGraphFile : TReadGraphFile); virtual; abstract;
  end; {TBaseFunc3DObj = class(TBaseFuncObj)}

  TRotateAxis = (Rot_XAxis, Rot_YAxis, Rot_Line);

  TFuncRotateObj = class(TBaseFunc3DObj)
  protected
    function GetListFuncStr: string; override;
    function CreateNewSeqObj : TObject; override;
    procedure AssignSeqObj(Source : TObject); override;
  public
    { Public declarations }
    FuncCalc : TMathRotateFunc;
    BeginXStr,
    EndXStr : string;
    BeginXVal,
    EndXVal : TMathValue;
    RotAxis : TRotateAxis;
    LineP1XStr,
    LineP1YStr,
    LineP2XStr,
    LineP2YStr : string;
    LineP1XVal,
    LineP1YVal,
    LineP2XVal,
    LineP2YVal : TMathValue;
    Circles,
    CircleDetail,
    Lines,
    LineDetail : integer;
    FuncColor : TColor;
    RoundLargeValues : boolean;
    constructor Create;
    destructor Destroy; override;
    function AddLinesToList(TheFuncNum : integer;
                            LineObj : TLineObj) : TLineListError; override;
    procedure SaveToDisk(WriteGraphFile : TWriteGraphFile); override;
    procedure ReadFromDisk(ReadGraphFile : TReadGraphFile); override;
    procedure Assign(Source : TFuncRotateObj);
  end; {TFuncRotateObj = class(TBaseFunc3DObj)}

  TFunc3DObj = class(TBaseFunc3DObj)
  protected
    function GetListFuncStr: string; override;
    function CreateNewSeqObj : TObject; override;
    procedure AssignSeqObj(Source : TObject); override;
  public
    { Public declarations }
    FuncCalc : TMath3DFunc;
    FuncColor : TColor;
    {Var1}
    BeginVar1Str,
    EndVar1Str : string;
    BeginVar1Val,
    EndVar1Val : TMathValue;
    Var1Lines,
    Var1Detail : integer;
    {Var2}
    BeginVar2Str,
    EndVar2Str : string;
    BeginVar2Val,
    EndVar2Val : TMathValue;
    Var2Lines,
    Var2Detail : integer;
    RoundLargeValues : boolean;
    constructor Create;
    destructor Destroy; override;
    function AddLinesToList(TheFuncNum : integer;
                            LineObj : TLineObj) : TLineListError; override;
    procedure SaveToDisk(WriteGraphFile : TWriteGraphFile); override;
    procedure ReadFromDisk(ReadGraphFile : TReadGraphFile); override;
    procedure Assign(Source : TFunc3DObj);
  end; {TFunc3DObj = class(TBaseFunc3DObj)}

implementation

type
  TDiskV1_FuncRotateRec = packed record
    JunkBytes : packed array[1..4] of byte;
    BeginXStr,
    EndXStr : string[50];
    BeginXVal,
    EndXVal : TMathFloat;
    RotAxis : byte;
    LineP1XStr,
    LineP1YStr,
    LineP2XStr,
    LineP2YStr : string[50];
    LineP1XVal,
    LineP1YVal,
    LineP2XVal,
    LineP2YVal : TMathFloat;
    Circles,
    CircleDetail,
    Lines,
    LineDetail : smallint;
    FuncColor : TColor;
    RoundLargeValues : boolean;
  end; {TDiskV1_FuncRotateRec = packed record}

  TDiskV2_FuncRotateRec = packed record
    BeginXVal,
    EndXVal : TMathFloat;
    RotAxis : TRotateAxis;
    LineP1XVal,
    LineP1YVal,
    LineP2XVal,
    LineP2YVal : TMathFloat;
    Circles,
    CircleDetail,
    Lines,
    LineDetail : integer;
    FuncColor : TColor;
    RoundLargeValues : boolean;
  end; {TDiskV2_FuncRotateRec = packed record}

  TDiskV3_FuncRotateRec = packed record
    BeginXVal,
    EndXVal : TPackedMathValue;
    RotAxis : TRotateAxis;
    LineP1XVal,
    LineP1YVal,
    LineP2XVal,
    LineP2YVal : TPackedMathValue;
    Circles,
    CircleDetail,
    Lines,
    LineDetail : integer;
    FuncColor : TColor;
    RoundLargeValues : boolean;
  end; {TDiskV3_FuncRotateRec = packed record}

  TDiskV1_Func3DRec = packed record
    DependantVar : byte; {'Y=f(x,z)=' etc}
    JunkBytes : packed array[1..4] of byte;
    FuncColor : TColor;
    {Var1}
    BeginVar1Str,
    EndVar1Str : string[50];
    BeginVar1Val,
    EndVar1Val : TMathFloat;
    Var1Lines,
    Var1Detail : smallint;
    {Var2}
    BeginVar2Str,
    EndVar2Str : string[50];
    BeginVar2Val,
    EndVar2Val : TMathFloat;
    Var2Lines,
    Var2Detail : smallint;
    RoundLargeValues : boolean;
  end; {TDiskV1_Func3DRec = packed record}

  TDiskV2_Func3DRec = packed record
    FuncColor : TColor;
    {Var1}
    BeginVar1Val,
    EndVar1Val : TMathFloat;
    Var1Lines,
    Var1Detail : integer;
    {Var2}
    BeginVar2Val,
    EndVar2Val : TMathFloat;
    Var2Lines,
    Var2Detail : integer;
    RoundLargeValues : boolean;
  end; {TDiskV2_Func3DRec = packed record}

  TDiskV3_Func3DRec = packed record
    FuncColor : TColor;
    {Var1}
    BeginVar1Val,
    EndVar1Val : TPackedMathValue;
    Var1Lines,
    Var1Detail : integer;
    {Var2}
    BeginVar2Val,
    EndVar2Val : TPackedMathValue;
    Var2Lines,
    Var2Detail : integer;
    RoundLargeValues : boolean;
  end; {TDiskV3_Func3DRec = packed record}

const
  Axis3D_FileType_X = 1;
  Axis3D_FileType_Y = 2;
  Axis3D_FileType_Z = 3;

{-----------------------------------------------------------------------}

procedure TMath3DFunc.SetGraph3DFuncType(Value : TGraph3DFuncType);
begin {procedure TMath3DFunc.SetGraph3DFuncType}
  VarCount := 2;
  if Value <> FGraph3DFuncType then
    case Value of
      ft_3DX:
        begin
          FGraph3DFuncType := ft_3DX;
          VarStrings[0] := 'Y';
          VarStrings[1] := 'Z';
        end;
      ft_3DZ:
        begin
          FGraph3DFuncType := ft_3DZ;
          VarStrings[0] := 'X';
          VarStrings[1] := 'Y';
        end;
      else {ft_3DY:}
        begin
          FGraph3DFuncType := ft_3DY;
          VarStrings[0] := 'X';
          VarStrings[1] := 'Z';
        end;
    end; {case}
end; {procedure TMath3DFunc.SetGraph3DFuncType}

{-----------------------------------------------------------------------}

constructor TMath3DFunc.Create;
begin {constructor TMath3DFunc.Create;}
  inherited Create;
  FGraph3DFuncType := ft_3DZ;
  VarCount := 2;
  VarStrings[0] := 'X';
  VarStrings[1] := 'Y';
end; {constructor TMath3DFunc.Create;}

{----------------------------------------------------------------------}

procedure TMath3DFunc.Assign(SourceFunc : TMathFunction);
begin {procedure TMath3DFunc.Assign}
  inherited Assign(SourceFunc);
  if SourceFunc is TMath3DFunc then
    FGraph3DFuncType := TMath3DFunc(SourceFunc).Graph3DFuncType;
end; {procedure TMath3DFunc.Assign}

{----------------------------------------------------------------------}

procedure TMath3DFunc.SaveToDisk(WriteGraphFile : TWriteGraphFile);
var
  TmpInt : longint;
begin {procedure TMath3DFunc.SaveToDisk}
  case Graph3DFuncType of
    ft_3DX: TmpInt := Axis3D_FileType_X;
    ft_3DY: TmpInt := Axis3D_FileType_Y;
    ft_3DZ: TmpInt := Axis3D_FileType_Z;
    else
      TmpInt := Axis3D_FileType_Y;
  end; {case}
  WriteGraphFile.WriteLongint(TmpInt);
  inherited SaveToDisk(WriteGraphFile);
end; {procedure TMath3DFunc.SaveToDisk}

{----------------------------------------------------------------------}

procedure TMath3DFunc.ReadFromDisk(ReadGraphFile : TReadGraphFile);
var
  TmpInt : longint;
begin {procedure TMath3DFunc.ReadFromDisk}
  if ReadGraphFile.FileVersion <> 1 then
    begin
      TmpInt := ReadGraphFile.ReadLongint;
      case TmpInt of
        Axis3D_FileType_X: Graph3DFuncType := ft_3DX;
        Axis3D_FileType_Y: Graph3DFuncType := ft_3DY;
        Axis3D_FileType_Z: Graph3DFuncType := ft_3DZ;
      end; {case}
    end; {if}
  inherited ReadFromDisk(ReadGraphFile);
end; {procedure TMath3DFunc.ReadFromDisk}

{----------------------------------------------------------------------}

constructor TMathRotateFunc.Create;
begin {constructor TMathRotateFunc.Create;}
  inherited Create;
  VarCount := 1;
  VarStrings[0] := 'X';
end; {constructor TMathRotateFunc.Create;}

{----------------------------------------------------------------------}

function TFuncRotateObj.GetListFuncStr: string;
begin {function TFuncRotateObj.GetListFuncStr}
  GetListFuncStr := 'Y=f(x)=' + FuncCalc.FuncString;
end; {function TFuncRotateObj.GetListFuncStr}

{-----------------------------------------------------------------------}

constructor TFuncRotateObj.Create;
begin {constructor TFuncRotateObj.Create;}
  inherited Create;
  FuncCalc := TMathRotateFunc.Create;
  BeginXStr := '0';
  EndXStr := '1';
  BeginXVal := LongintToMathValue(0);
  EndXVal := LongintToMathValue(1);
  RotAxis := Rot_XAxis;
  LineP1XStr := '0';
  LineP1YStr := '0';
  LineP2XStr := '1';
  LineP2YStr := '1';
  LineP1XVal := LongintToMathValue(0);
  LineP1YVal := LongintToMathValue(0);
  LineP2XVal := LongintToMathValue(1);
  LineP2YVal := LongintToMathValue(1);
  Circles := 12;
  CircleDetail := 6;
  Lines := 12;
  LineDetail := 6;
  FuncColor := clblue;
  RoundLargeValues := false;
end; {constructor TFuncRotateObj.Create;}

{-----------------------------------------------------------------------}

destructor TFuncRotateObj.Destroy;
begin {destructor TFuncRotateObj.Destroy;}
  FuncCalc.Free;
  inherited Destroy;
end; {destructor TFuncRotateObj.Destroy;}

{-----------------------------------------------------------------------}

function TFuncRotateObj.AddLinesToList(TheFuncNum : integer;
                                       LineObj : TLineObj) : TLineListError;
  {these routines simulate an array of function points
     to make calculations and traversing easier }
type
  TBuild3DRec = record
    PointRec : TPointRec;
    IsDefined : boolean;
    ListPointer : Pointer;
  end; {TBuild3DRec}
  PBuild3DArray = ^TBuild3DArray;
    {dummy type used for casting}
  TBuild3DArray = array[1..3800] of TBuild3DRec;

  TColRec = record
    XFloat,
    YFloat : TMathFloat;
    IsDefined : boolean;
  end; {TColRec}
  PColCordArray = ^TColCordArray;
    {dummy type used for casting}
    {had to change this when TMathFloat = extended}
  TColCordArray = array[1..3120] of TColRec;

var
  BuildLine1, BuildLine2, SwapLine : PBuild3DArray;
  ColCordArray : PColCordArray;
  BuildLineSize : integer;
  RowCount, ColCount : integer;
  RowDetail, ColDetail : integer;
  ColBegin, ColEnd : TMathFloat;
  CurrRow, CurrCol : integer;
  RowCountDetail, ColCountDetail : integer;
  CurrBuild1, CurrBuild2 : TBuild3DRec;
  CurrHighestRow : integer;
  LineListError : TLineListError;
  rl_TransX, rl_TransY,
  rl_Sin, rl_SinNeg, rl_Cos, rl_CosNeg : TMathFloat;
  SinTheta, CosTheta : TMathFloat;
  LocalRotAxis : TRotateAxis;

{-----------------------------------------------------}

function CalcY(InXCord : TMathFloat; var IsDefined : boolean) : TMathFloat;
begin {function CalcY}
  {begin calculate the point}
  with FuncCalc do
    begin
      FloatVarValues[0] := InXCord;
      IsDefined := EvaluateMathFunc;
      Result := FloatResultValue;
    end; {with}
  {end calculate the point}
  if IsDefined then
    if abs(Result) > Max3DDomain then
     begin
       if RoundLargeValues then
         begin
           if Result > Max3DDomain then
             Result := Max3DDomain
           else
             Result := Min3DDomain;
         end
       else
         IsDefined := false;
     end; {if}
end; {function CalcY}

{-----------------------------------------------------}

function Initalizations : boolean;
var
  I : integer;
  ColEndMinusColBegin : TMathFloat;
  TmpColRec : TColRec;
  JunkRaidus, AngleToPoint : TMathFloat;
begin {function Initalizations;}
  LocalRotAxis := RotAxis;
  if RotAxis = Rot_Line then
    begin
      if ((LineP1XVal.FloatValue = LineP2XVal.FloatValue)and(LineP1YVal.FloatValue = LineP2YVal.FloatValue)) then
        LocalRotAxis := Rot_XAxis
      else
        begin
          rl_TransX := (-1)*LineP1XVal.FloatValue;
          rl_TransY := (-1)*LineP1YVal.FloatValue;
          ConvCartesianToPolar((LineP2XVal.FloatValue + rl_TransX),
              (LineP2YVal.FloatValue + rl_TransY), JunkRaidus, AngleToPoint);
          rl_Sin := sin(AngleToPoint);
          rl_SinNeg := sin(-AngleToPoint);
          rl_Cos := cos(AngleToPoint);
          rl_CosNeg := cos(-AngleToPoint);
        end; {else}
    end; {if}
  Result := true;
  if (Lines >= 2) and (Circles >= 2) then
    begin
      {the fallowing code assumes atleast 2
           lines in each axis direction}
      RowCount := ((Lines)*CircleDetail)+Lines;
      ColCount := ((Circles-1)*LineDetail)+Circles;
      RowDetail := CircleDetail;
      ColDetail := LineDetail;
    end {if}
  else
    begin
      {cover all 7 special case}
      {Case #1 if (Lines = 0)and(Circles = 1) then}
      {Case #2 if (Lines = 0)and(Circles >= 2) then}
      if (Lines = 0)and(Circles >= 1) then {cover cases #1,2}
        begin
          if CircleDetail <= 0 then
            RowCount := -1 {Make Nothing happen}
          else
            RowCount := CircleDetail + 1;
          ColCount := Circles;
          RowDetail := 32767;
          ColDetail := -1;
        end {if}
      else
      {Case #3 if (Lines = 1)and(Circles = 0) then}
      {Case #6 if (Lines >= 2)and(Circles = 0) then}
      if (Lines >= 1)and(Circles = 0) then {cover cases #3,6}
        begin
          RowCount := Lines;
          ColCount := LineDetail + 2;
          RowDetail := -1;
          ColDetail := 32767;
        end {if}
      else
      {Case #4}
      if (Lines = 1)and(Circles = 1) then
        begin
          RowCount := CircleDetail + 1;
          ColCount := LineDetail + 2;
          RowDetail := 10000;
          ColDetail := 10000;
        end {if}
      else
      {Case #5}
      if (Lines = 1)and(Circles >= 2) then
        begin
          RowCount := CircleDetail + 1;
          ColCount := ((Circles-1)*LineDetail)+Circles;
          RowDetail := 10000;
          ColDetail := LineDetail;
        end {if}
      else
      {Case #7}
      if (Lines >= 2)and(Circles = 1) then
        begin
          RowCount := ((Lines)*CircleDetail)+Lines;
          ColCount := LineDetail + 2;
          RowDetail := CircleDetail;
          ColDetail := 10000;
        end {if}
      else
        begin {all case's should have been covered}
          Result := false; {this code should never execute}
          exit;
        end; {else}
    end; {else}
  ColBegin := BeginXVal.FloatValue;
  ColEnd := EndXVal.FloatValue;
  BuildLineSize := ColCount*sizeof(TBuild3DRec);
  try
    GetMem(BuildLine1,BuildLineSize);
  except
    on EOutOfMemory do Result := false;
  end; {except}
  if Result then
    begin
      try
        GetMem(BuildLine2,BuildLineSize);
      except
        on EOutOfMemory do
          begin
            FreeMem(BuildLine1,BuildLineSize);
            Result := false;
          end; {on}
      end; {except}
    end; {if}
  if Result then
    begin
      try
        GetMem(ColCordArray,ColCount*sizeof(TColRec));
      except
        on EOutOfMemory do
          begin
            FreeMem(BuildLine2,BuildLineSize);
            FreeMem(BuildLine1,BuildLineSize);
            Result := false;
          end; {on}
      end; {except}
    end; {if}
  if Result then
    begin
      ColEndMinusColBegin := ColEnd-ColBegin;
      if ColCount >= 1 then
        begin
          TmpColRec.XFloat := ColBegin;
          TmpColRec.YFloat := CalcY(TmpColRec.XFloat,TmpColRec.IsDefined);
          ColCordArray^[1] := TmpColRec;
        end; {if}
      if ColCount >= 2 then
        begin
          TmpColRec.XFloat := ColEnd;
          TmpColRec.YFloat := CalcY(TmpColRec.XFloat,TmpColRec.IsDefined);
          ColCordArray^[ColCount] := TmpColRec;
        end; {if}
      if ColCount >= 3 then
       for I := 2 to ColCount-1 do
         begin
           TmpColRec.XFloat :=
              (((I-1)/(ColCount-1))*(ColEndMinusColBegin))+ColBegin;
           TmpColRec.YFloat := CalcY(TmpColRec.XFloat,TmpColRec.IsDefined);
           ColCordArray^[I] := TmpColRec;
         end; {for}
    end; {if}
  CurrHighestRow := 0;
end; {function Initalizations;}

{--------------------------------------------------}

procedure RotatePoint(var Dest : TBuild3DRec;const Source : TColRec);
var
  XLocal, YLocal : TMathFloat;
  HoldCordX : TMathFloat;
begin {procedure RotatePoint}
  Dest.IsDefined := Source.IsDefined;
  if not Dest.IsDefined then
    exit;
  {Dest.PointRec.Z := 0;} {Z is allways Intially zero}
  try
    case LocalRotAxis of
      Rot_XAxis : begin {Rot_XAxis}
                    Dest.PointRec.X := trunc(Source.XFloat*FixedMul);
                    Dest.PointRec.Z :=
                         trunc((Source.YFloat*SinTheta)*FixedMul);
                    Dest.PointRec.Y :=
                         trunc((Source.YFloat*CosTheta)*FixedMul);
                  end;  {Rot_XAxis}
      Rot_YAxis : begin {Rot_YAxis}
                    Dest.PointRec.Y := trunc(Source.YFloat*FixedMul);
                    Dest.PointRec.Z :=
                         trunc((Source.XFloat*SinTheta)*FixedMul);
                    Dest.PointRec.X :=
                         trunc((Source.XFloat*CosTheta)*FixedMul);
                  end;  {Rot_YAxis}
      Rot_Line  :
                  begin {Rot_Line}
                    {translate to origin}
                    XLocal := Source.XFloat + rl_TransX;
                    YLocal := Source.YFloat + rl_TransY;
                    {Rotate around Z Axis too X Axis}
                    HoldCordX := XLocal;
                    XLocal := XLocal*rl_CosNeg - YLocal*rl_SinNeg;
                    YLocal := HoldCordX*rl_SinNeg + YLocal*rl_CosNeg;
                    {Rotate around X Axis}
                    Dest.PointRec.Z := trunc((YLocal*SinTheta)*FixedMul);
                    YLocal := YLocal*cosTheta;
                    {Undo: Rotate around Z Axis too X Axis}
                    HoldCordX := XLocal;
                    XLocal := XLocal*rl_Cos - YLocal*rl_Sin;
                    YLocal := HoldCordX*rl_Sin + YLocal*rl_Cos;
                    {Undo: translate to origin}
                    Dest.PointRec.X := trunc((XLocal - rl_TransX)*FixedMul);
                    Dest.PointRec.Y := trunc((YLocal - rl_TransY)*FixedMul);
                  end;  {Rot_Line}
    end; {case}
  except
    Dest.IsDefined := false;
  end; {try..except}
end; {procedure RotatePoint}

{--------------------------------------------------}

procedure PointArray(Row, Col : integer;
                     var TheBuild3DRec : TBuild3DRec);
var
  I : integer;
  RowCord : TMathFloat;
  TmpBuild3DRec : TBuild3DRec;
begin {procedure PointArray}
  if Row > CurrHighestRow then
    begin
      inc(CurrHighestRow);
      {Swap the Row lines}
      SwapLine := BuildLine1;
      BuildLine1 := BuildLine2;
      BuildLine2 := SwapLine;
      {calc a new row}
      if (Row <= 1)or (Row > RowCount) then
        begin
          for I := 1 to ColCount do
            begin
              TmpBuild3DRec.ListPointer := nil;
              TmpBuild3DRec.IsDefined := ColCordArray^[I].IsDefined;
              if TmpBuild3DRec.IsDefined then
                with TmpBuild3DRec do
                  begin
                    PointRec.X := trunc(ColCordArray^[I].XFloat*FixedMul);
                    PointRec.Y := trunc(ColCordArray^[I].YFloat*FixedMul);
                    PointRec.Z := 0;
                  end; {with}
              BuildLine2^[I] := TmpBuild3DRec;
            end; {for}
        end
      else
        begin
          RowCord :=(2*pi)*((Row-1)/(RowCount));
          SinTheta := sin(RowCord);
          CosTheta := cos(RowCord);
          for I := 1 to ColCount do
            begin
              RotatePoint(TmpBuild3DRec,ColCordArray^[I]);
              TmpBuild3DRec.ListPointer := nil;
              BuildLine2^[I] := TmpBuild3DRec;
            end; {for}
        end; {else}
    end; {if}
  if Row = CurrHighestRow then
    TheBuild3DRec := BuildLine2^[Col]
  else
    TheBuild3DRec := BuildLine1^[Col];
end; {procedure PointArray}

{-----------------------------------------------------}

procedure CleanUpAndDispose;
begin {procedure CleanUpAndDispose;}
  FreeMem(ColCordArray,ColCount*sizeof(TColRec));
  FreeMem(BuildLine2,BuildLineSize);
  FreeMem(BuildLine1,BuildLineSize);
end; {procedure CleanUpAndDispose;}

{-----------------------------------------------------}

begin {procedure TFuncRotateObj.AddLinesToList}
  Result := ll_NoError;
  LineObj.RegisterColor(TheFuncNum, FuncColor);
  LineObj.MarkBeginFunc;
  if not((Lines <= 0) and (Circles <= 0)) then
    begin
      if not Initalizations then
        begin
          Result := ll_NoMem;
          Exit;
        end; {if}
      LineListError := ll_NoError;
      RowCountDetail := 20000; {set to huge number}
      for CurrRow := 1 to RowCount+1 do
        begin
          if RowCountDetail < RowDetail then
            inc(RowCountDetail)
          else
            RowCountDetail := 0;
          ColCountDetail := 20000; {set to huge number}
          for CurrCol := 1 to ColCount do
             begin
               if ColCountDetail < ColDetail then
                 inc(ColCountDetail)
               else
                 ColCountDetail := 0;
               PointArray(CurrRow, CurrCol, CurrBuild1);
               if CurrBuild1.IsDefined then
                 begin
                   if (CurrCol <> 1)and(RowCountDetail = 0)and
                      (CurrRow < (RowCount+1)) then
                     begin
                       PointArray(CurrRow, CurrCol-1, CurrBuild2);
                       if CurrBuild2.IsDefined then
                         LineListError := LineObj.AddItem_Points(
                             TheFuncNum,CurrBuild1.PointRec,
                             CurrBuild2.PointRec,
                             BuildLine2^[CurrCol].ListPointer,
                             BuildLine2^[CurrCol-1].ListPointer);
                     end;
                   if (CurrRow <> 1)and(ColCountDetail = 0) then
                     begin
                       PointArray(CurrRow-1, CurrCol, CurrBuild2);
                       if CurrBuild2.IsDefined then
                         LineListError := LineObj.AddItem_Points(
                           TheFuncNum,CurrBuild1.PointRec,
                           CurrBuild2.PointRec,
                           BuildLine2^[CurrCol].ListPointer,
                           BuildLine1^[CurrCol].ListPointer);
                     end;
                 end; {if}
             end; {for}
          if LineListError <> ll_NoError then
            begin
              Result := LineListError;
              CleanUpAndDispose;
              Exit;
            end; {if}
        end; {for}
      CleanUpAndDispose;
    end; {if}
end; {procedure TFuncRotateObj.AddLinesToList}

{----------------------------------------------------------------------}

procedure TFuncRotateObj.SaveToDisk(WriteGraphFile : TWriteGraphFile);
var
  DiskRec : TDiskV3_FuncRotateRec;
begin {procedure TFuncRotateObj.SaveToDisk}
  WriteGraphFile.WriteLongint(f3D_FileType_Rotate);
  DiskRec.BeginXVal := PackMathValue(BeginXVal);
  DiskRec.EndXVal := PackMathValue(EndXVal);
  DiskRec.RotAxis := RotAxis;
  DiskRec.LineP1XVal := PackMathValue(LineP1XVal);
  DiskRec.LineP1YVal := PackMathValue(LineP1YVal);
  DiskRec.LineP2XVal := PackMathValue(LineP2XVal);
  DiskRec.LineP2YVal := PackMathValue(LineP2YVal);
  DiskRec.Circles := Circles;
  DiskRec.CircleDetail := CircleDetail;
  DiskRec.Lines := Lines;
  DiskRec.LineDetail := LineDetail;
  DiskRec.FuncColor := FuncColor;
  DiskRec.RoundLargeValues := RoundLargeValues;
  with WriteGraphFile do
    begin
      WriteRecord(DiskRec, sizeof(DiskRec));
      WriteMathString(BeginXStr);
      WriteMathString(EndXStr);
      WriteMathString(LineP1XStr);
      WriteMathString(LineP1YStr);
      WriteMathString(LineP2XStr);
      WriteMathString(LineP2YStr);
    end; {with}
  FuncCalc.SaveToDisk(WriteGraphFile);
end; {procedure TFuncRotateObj.SaveToDisk}

{----------------------------------------------------------------------}

procedure TFuncRotateObj.ReadFromDisk(ReadGraphFile : TReadGraphFile);

{---------------------------------}

procedure ReadVer1Fields;
const
  V1_Rot_XAxis = 1;
  V1_Rot_YAxis = 2;
  V1_Rot_Line  = 3;
var
  DiskRec : TDiskV1_FuncRotateRec;
begin {procedure ReadVer1Fields;}
  ReadGraphFile.ReadRecord(DiskRec, sizeof(DiskRec));
  BeginXStr := DiskRec.BeginXStr;
  EndXStr := DiskRec.EndXStr;
  BeginXVal := FloatToMathValue(DiskRec.BeginXVal);
  EndXVal := FloatToMathValue(DiskRec.EndXVal);
  case DiskRec.RotAxis of
    V1_Rot_XAxis: RotAxis := Rot_XAxis;
    V1_Rot_YAxis: RotAxis := Rot_YAxis;
    V1_Rot_Line: RotAxis := Rot_Line;
  end; {case}
  LineP1XStr := DiskRec.LineP1XStr;
  LineP1YStr := DiskRec.LineP1YStr;
  LineP2XStr := DiskRec.LineP2XStr;
  LineP2YStr := DiskRec.LineP2YStr;
  LineP1XVal := FloatToMathValue(DiskRec.LineP1XVal);
  LineP1YVal := FloatToMathValue(DiskRec.LineP1YVal);
  LineP2XVal := FloatToMathValue(DiskRec.LineP2XVal);
  LineP2YVal := FloatToMathValue(DiskRec.LineP2YVal);
  Circles := DiskRec.Circles;
  CircleDetail := DiskRec.CircleDetail;
  Lines := DiskRec.Lines;
  LineDetail := DiskRec.LineDetail;
  FuncColor := DiskRec.FuncColor;
  RoundLargeValues := DiskRec.RoundLargeValues;
  FuncCalc.ReadFromDisk(ReadGraphFile);
  {fix and calc math strings}
  with ReadGraphFile do
    begin
      FixCalcMathConstString(BeginXStr, BeginXVal);
      FixCalcMathConstString(EndXStr, EndXVal);
      FixCalcMathConstString(LineP1XStr, LineP1XVal);
      FixCalcMathConstString(LineP1YStr, LineP1YVal);
      FixCalcMathConstString(LineP2XStr, LineP2XVal);
      FixCalcMathConstString(LineP2YStr, LineP2YVal);
    end; {with}
end; {procedure ReadVer1Fields;}

{---------------------------------}

procedure ReadVer2Fields;
var
  DiskRec : TDiskV2_FuncRotateRec;
begin {procedure ReadVer2Fields;}
  ReadGraphFile.ReadRecord(DiskRec, sizeof(DiskRec));
  BeginXVal := FloatToMathValue(DiskRec.BeginXVal);
  EndXVal := FloatToMathValue(DiskRec.EndXVal);
  RotAxis := DiskRec.RotAxis;
  LineP1XVal := FloatToMathValue(DiskRec.LineP1XVal);
  LineP1YVal := FloatToMathValue(DiskRec.LineP1YVal);
  LineP2XVal := FloatToMathValue(DiskRec.LineP2XVal);
  LineP2YVal := FloatToMathValue(DiskRec.LineP2YVal);
  Circles := DiskRec.Circles;
  CircleDetail := DiskRec.CircleDetail;
  Lines := DiskRec.Lines;
  LineDetail := DiskRec.LineDetail;
  FuncColor := DiskRec.FuncColor;
  RoundLargeValues := DiskRec.RoundLargeValues;
  with ReadGraphFile do
    begin
      ReadMathConstString(BeginXStr, BeginXVal, False);
      ReadMathConstString(EndXStr, EndXVal, False);
      ReadMathConstString(LineP1XStr, LineP1XVal, False);
      ReadMathConstString(LineP1YStr, LineP1YVal, False);
      ReadMathConstString(LineP2XStr, LineP2XVal, False);
      ReadMathConstString(LineP2YStr, LineP2YVal, False);
    end; {with}
  FuncCalc.ReadFromDisk(ReadGraphFile);
end; {procedure ReadVer2Fields;}

{---------------------------------}

procedure ReadVer3Fields;
var
  DiskRec : TDiskV3_FuncRotateRec;
begin {procedure ReadVer3Fields;}
  ReadGraphFile.ReadRecord(DiskRec, sizeof(DiskRec));
  BeginXVal := UnPackMathValue(DiskRec.BeginXVal);
  EndXVal := UnPackMathValue(DiskRec.EndXVal);
  RotAxis := DiskRec.RotAxis;
  LineP1XVal := UnPackMathValue(DiskRec.LineP1XVal);
  LineP1YVal := UnPackMathValue(DiskRec.LineP1YVal);
  LineP2XVal := UnPackMathValue(DiskRec.LineP2XVal);
  LineP2YVal := UnPackMathValue(DiskRec.LineP2YVal);
  Circles := DiskRec.Circles;
  CircleDetail := DiskRec.CircleDetail;
  Lines := DiskRec.Lines;
  LineDetail := DiskRec.LineDetail;
  FuncColor := DiskRec.FuncColor;
  RoundLargeValues := DiskRec.RoundLargeValues;
  with ReadGraphFile do
    begin
      ReadMathConstString(BeginXStr, BeginXVal, False);
      ReadMathConstString(EndXStr, EndXVal, False);
      ReadMathConstString(LineP1XStr, LineP1XVal, False);
      ReadMathConstString(LineP1YStr, LineP1YVal, False);
      ReadMathConstString(LineP2XStr, LineP2XVal, False);
      ReadMathConstString(LineP2YStr, LineP2YVal, False);
    end; {with}
  FuncCalc.ReadFromDisk(ReadGraphFile);
end; {procedure ReadVer3Fields;}

{---------------------------------}

begin {procedure TFuncRotateObj.ReadFromDisk}
  DidFieldsChange := true;
  if ReadGraphFile.FileVersion = 1 then
    ReadVer1Fields
  else
    if ReadGraphFile.FileVersion = 2 then
      ReadVer2Fields
    else {if ReadGraphFile.FileVersion >= 3 then}
      ReadVer3Fields;
end; {procedure TFuncRotateObj.ReadFromDisk}

{------------------------------------------------------------------------}

procedure TFuncRotateObj.Assign(Source: TFuncRotateObj);
begin {procedure TFuncRotateObj.Assign}
  FuncCalc.Assign(Source.FuncCalc);
  BeginXStr := Source.BeginXStr;
  EndXStr := Source.EndXStr;
  BeginXVal := Source.BeginXVal;
  EndXVal := Source.EndXVal;
  RotAxis := Source.RotAxis;
  LineP1XStr := Source.LineP1XStr;
  LineP1YStr := Source.LineP1YStr;
  LineP2XStr := Source.LineP2XStr;
  LineP2YStr := Source.LineP2YStr;
  LineP1XVal := Source.LineP1XVal;
  LineP1YVal := Source.LineP1YVal;
  LineP2XVal := Source.LineP2XVal;
  LineP2YVal := Source.LineP2YVal;
  Circles := Source.Circles;
  CircleDetail := Source.CircleDetail;
  Lines := Source.Lines;
  LineDetail := Source.LineDetail;
  FuncColor := Source.FuncColor;
  RoundLargeValues := Source.RoundLargeValues;
  {Make Memory Unique}
  UniqueString(BeginXStr);
  UniqueString(EndXStr);
  UniqueString(LineP1XStr);
  UniqueString(LineP1YStr);
  UniqueString(LineP2XStr);
  UniqueString(LineP2YStr);
end; {procedure TFuncRotateObj.Assign}

{------------------------------------------------------------------------}

function TFunc3DObj.GetListFuncStr : string;
begin {function TFunc3DObj.GetListFuncStr : string;}
  case FuncCalc.Graph3DFuncType of
    ft_3DX : GetListFuncStr := 'X=f(y,z)=' + FuncCalc.FuncString;
    ft_3DZ : GetListFuncStr := 'Z=f(x,y)=' + FuncCalc.FuncString;
    else{ft_3DY}GetListFuncStr := 'Y=f(x,z)=' + FuncCalc.FuncString;
  end; {case}
end; {function TFunc3DObj.GetListFuncStr : string;}

{-----------------------------------------------------------------------}

constructor TFunc3DObj.Create;
begin {constructor TFunc3DObj.Create;}
  inherited Create;
  FuncCalc := TMath3DFunc.Create;
  FuncColor := clBlue;
  {Var1}
  BeginVar1Str := '-1';
  EndVar1Str := '1';
  BeginVar1Val := LongintToMathValue(-1);
  EndVar1Val  := LongintToMathValue(1);
  Var1Lines := 12;
  Var1Detail := 6;
  {Var2}
  BeginVar2Str := '-1';
  EndVar2Str := '1';
  BeginVar2Val := LongintToMathValue(-1);
  EndVar2Val  := LongintToMathValue(1);
  Var2Lines := 12;
  Var2Detail := 6;
  RoundLargeValues := false;
end; {constructor TFunc3DObj.Create;}

{-----------------------------------------------------------------------}

destructor TFunc3DObj.Destroy;
begin {destructor TFunc3DObj.Destroy;}
  FuncCalc.Free;
  inherited Destroy;
end; {destructor TFunc3DObj.Destroy;}

{-----------------------------------------------------------------------}

function TFunc3DObj.AddLinesToList(TheFuncNum : integer;
                                   LineObj : TLineObj) : TLineListError;
  {these routines simulate an array of function points
     to make calculations and traversing easier }
type
  TBuild3DRec = record
    PointRec : TPointRec;
    IsDefined : boolean;
    ListPointer : pointer;
  end; {TBuild3DRec}
  PBuild3DArray = ^TBuild3DArray;
    {dummy type used for casting}
  TBuild3DArray = array[1..3800] of TBuild3DRec;

  PColCordArray = ^TColCordArray;
    {dummy type used for casting}
  TColCordArray = array[1..3800] of TMathFloat;

var
  BuildLine1, BuildLine2, SwapLine : PBuild3DArray;
  ColCordArray : PColCordArray;
  BuildLineSize : integer;
  RowCount, ColCount : integer;
  RowDetail, ColDetail : integer;
  RowBegin, RowEnd, ColBegin, ColEnd : TMathFloat;
  RowEndMinusRowBegin : TMathFloat;
  CurrRow, CurrCol : integer;
  RowCountDetail, ColCountDetail : integer;
  CurrBuild1, CurrBuild2 : TBuild3DRec;
  CurrHighestRow : integer;
  LineListError : TLineListError;

{-----------------------------------------------------}

function Initalizations : boolean;
var
  I : integer;
  ColEndMinusColBegin : TMathFloat;
begin {function Initalizations;}
  Result := true;
  if (Var1Lines >= 2) and (Var2Lines >= 2) then
    begin
      {the fallowing code assumes atleast 2
           lines in each axis direction}
      RowCount := ((Var1Lines-1)*Var2Detail)+Var1Lines;
      ColCount := ((Var2Lines-1)*Var1Detail)+Var2Lines;
      RowDetail := Var2Detail;
      ColDetail := Var1Detail;
    end {if}
  else
    begin
      {cover all 7 special case}
      {Case #1 if (Var1Lines = 0)and(Var2Lines = 1) then}
      {Case #2 if (Var1Lines = 0)and(Var2Lines >= 2) then}
      if (Var1Lines = 0)and(Var2Lines >= 1) then {cover cases #1,2}
        begin
          RowCount := Var2Detail + 2;
          ColCount := Var2Lines;
          RowDetail := 32767;
          ColDetail := -1;
        end {if}
      else
      {Case #3 if (Var1Lines = 1)and(Var2Lines = 0) then}
      {Case #6 if (Var1Lines >= 2)and(Var2Lines = 0) then}
      if (Var1Lines >= 1)and(Var2Lines = 0) then {cover cases #3,6}
        begin
          RowCount := Var1Lines;
          ColCount := Var1Detail + 2;
          RowDetail := -1;
          ColDetail := 32767;
        end {if}
      else
      {Case #4}
      if (Var1Lines = 1)and(Var2Lines = 1) then
        begin
          RowCount := Var2Detail + 2;
          ColCount := Var1Detail + 2;
          RowDetail := 10000;
          ColDetail := 10000;
        end {if}
      else
      {Case #5}
      if (Var1Lines = 1)and(Var2Lines >= 2) then
        begin
          RowCount := Var2Detail + 2;
          ColCount := ((Var2Lines-1)*Var1Detail)+Var2Lines;
          RowDetail := 10000;
          ColDetail := Var1Detail;
        end {if}
      else
      {Case #7}
      if (Var1Lines >= 2)and(Var2Lines = 1) then
        begin
          RowCount := ((Var1Lines-1)*Var2Detail)+Var1Lines;
          ColCount := Var1Detail + 2;
          RowDetail := Var2Detail;
          ColDetail := 10000;
        end {if}
      else
        begin {all case's should have been covered}
          Result := false; {this code should never execute}
          exit;
        end; {else}
    end; {else}
  RowBegin := BeginVar2Val.FloatValue;
  RowEnd := EndVar2Val.FloatValue;
  ColBegin := BeginVar1Val.FloatValue;
  ColEnd := EndVar1Val.FloatValue;
  BuildLineSize := ColCount*sizeof(TBuild3DRec);
  try
    GetMem(BuildLine1,BuildLineSize);
  except
    on EOutOfMemory do Result := false;
  end; {except}
  if Result then
    begin
      try
        GetMem(BuildLine2,BuildLineSize);
      except
        on EOutOfMemory do
          begin
            FreeMem(BuildLine1,BuildLineSize);
            Result := false;
          end; {on}
      end; {except}
    end; {if}
  if Result then
    begin
      try
        GetMem(ColCordArray,ColCount*sizeof(TMathFloat));
      except
        on EOutOfMemory do
          begin
            FreeMem(BuildLine2,BuildLineSize);
            FreeMem(BuildLine1,BuildLineSize);
            Result := false;
          end; {on}
      end; {except}
    end; {if}
  if Result then
    begin
      ColEndMinusColBegin := ColEnd-ColBegin;
      RowEndMinusRowBegin := RowEnd-RowBegin;
      if ColCount >= 1 then
        ColCordArray^[1] := ColBegin;
      if ColCount >= 2 then
        ColCordArray^[ColCount] := ColEnd;
      if ColCount >= 3 then
        for I := 2 to ColCount-1 do
           ColCordArray^[I] :=
                 (((I-1)/(ColCount-1))*(ColEndMinusColBegin))+ColBegin;
    end; {if}
  CurrHighestRow := 0;
end; {function Initalizations;}

{-----------------------------------------------------}

procedure CalcAFuncPoint(TheRowCord, TheColCord : TMathFloat;
                         var TheBuild3DRec : TBuild3DRec);
var
  TmpFuncResult : TMathFloat;
begin {procedure CalcAFuncPoint}
  {begin calculate the point}
  with FuncCalc do
    begin
      FloatVarValues[0] := TheColCord;
      FloatVarValues[1] := TheRowCord;
      TheBuild3DRec.IsDefined := EvaluateMathFunc;
      TmpFuncResult := FloatResultValue;
    end; {with}
  {end calculate the point}
  if TheBuild3DRec.IsDefined then
    begin
      if abs(TmpFuncResult) > Max3DDomain then
        begin
          if RoundLargeValues then
            begin
              if TmpFuncResult > Max3DDomain then
                TmpFuncResult := Max3DDomain
              else
                TmpFuncResult := Min3DDomain;
            end
          else
            begin
              TheBuild3DRec.IsDefined := false;
              Exit;
            end; {else}
        end; {if}
      with TheBuild3DRec.PointRec do
        case FuncCalc.Graph3DFuncType of
          ft_3DX : begin
                     Y := Trunc(TheColCord*FixedMul);
                     Z := Trunc(TheRowCord*FixedMul);
                     X := Trunc(TmpFuncResult*FixedMul);
                   end;
          ft_3DY : begin
                     X := Trunc(TheColCord*FixedMul);
                     Z := Trunc(TheRowCord*FixedMul);
                     Y := Trunc(TmpFuncResult*FixedMul);
                   end;
          else  begin  {ft_3DZ}
                     X := Trunc(TheColCord*FixedMul);
                     Y := Trunc(TheRowCord*FixedMul);
                     Z := Trunc(TmpFuncResult*FixedMul);
                   end;
        end; {case}
    end; {if}
end; {procedure CalcAFuncPoint}

{--------------------------------------------------}

procedure PointArray(Row, Col : integer;
                     var TheBuild3DRec : TBuild3DRec);
var
  I : integer;
  RowCord : TMathFloat;
  TmpBuild3DRec : TBuild3DRec;
begin {procedure PointArray}
  if Row > CurrHighestRow then
    begin
      inc(CurrHighestRow);
      {Swap the Row lines}
      SwapLine := BuildLine1;
      BuildLine1 := BuildLine2;
      BuildLine2 := SwapLine;
      {calc a new row}
      if Row <= 1 then
        RowCord := RowBegin
      else
        if Row >= RowCount then
          RowCord := RowEnd
        else
          RowCord :=
             (((Row-1)/(RowCount-1))*(RowEndMinusRowBegin))+RowBegin;
      for I := 1 to ColCount do
        begin
          CalcAFuncPoint(RowCord,ColCordArray^[I],TmpBuild3DRec);
          TmpBuild3DRec.ListPointer := nil;
          BuildLine2^[I] := TmpBuild3DRec;
        end; {for}
    end; {if}
  if Row = CurrHighestRow then
    TheBuild3DRec := BuildLine2^[Col]
  else
    TheBuild3DRec := BuildLine1^[Col];
end; {procedure PointArray}

{-------------------------------}

procedure CleanUpAndDispose;
begin {procedure CleanUpAndDispose;}
  FreeMem(ColCordArray,ColCount*sizeof(TMathFloat));
  FreeMem(BuildLine2,BuildLineSize);
  FreeMem(BuildLine1,BuildLineSize);
end; {procedure CleanUpAndDispose;}

{-----------------------------------------------------}

begin {procedure TFunc3DObj.AddLinesToList}
  Result := ll_NoError;
  LineObj.RegisterColor(TheFuncNum, FuncColor);
  LineObj.MarkBeginFunc;
  if not((Var1Lines <= 0) and (Var2Lines <= 0)) then
    begin
      if not Initalizations then
        begin
          Result := ll_NoMem;
          Exit;
        end; {if}
      LineListError := ll_NoError;
      RowCountDetail := 20000; {set to huge number}
      for CurrRow := 1 to RowCount do
        begin
          if RowCountDetail < RowDetail then
            inc(RowCountDetail)
          else
            RowCountDetail := 0;
          ColCountDetail := 20000; {set to huge number}
          for CurrCol := 1 to ColCount do
             begin
               if ColCountDetail < ColDetail then
                 inc(ColCountDetail)
               else
                 ColCountDetail := 0;
               PointArray(CurrRow, CurrCol, CurrBuild1);
               if CurrBuild1.IsDefined then
                 begin
                   if (CurrCol <> 1)and(RowCountDetail = 0) then
                     begin
                       PointArray(CurrRow, CurrCol-1, CurrBuild2);
                       if CurrBuild2.IsDefined then
                         LineListError := LineObj.AddItem_Points(
                             TheFuncNum,CurrBuild1.PointRec,
                             CurrBuild2.PointRec,
                             BuildLine2^[CurrCol].ListPointer,
                             BuildLine2^[CurrCol-1].ListPointer);
                     end;
                   if (CurrRow <> 1)and(ColCountDetail = 0) then
                     begin
                       PointArray(CurrRow-1, CurrCol, CurrBuild2);
                       if CurrBuild2.IsDefined then
                         LineListError := LineObj.AddItem_Points(
                           TheFuncNum,CurrBuild1.PointRec,
                           CurrBuild2.PointRec,
                           BuildLine2^[CurrCol].ListPointer,
                           BuildLine1^[CurrCol].ListPointer);
                     end;
                 end; {if}
             end; {for}
          if LineListError <> ll_NoError then
            begin
              Result := LineListError;
              CleanUpAndDispose;
              Exit;
            end; {if}
        end; {for}
      CleanUpAndDispose;
    end; {if}
end; {procedure TFunc3DObj.AddLinesToList}

{----------------------------------------------------------------------}

procedure TFunc3DObj.SaveToDisk(WriteGraphFile : TWriteGraphFile);
var
  DiskRec : TDiskV3_Func3DRec;
begin {procedure TFunc3DObj.SaveToDisk}
  WriteGraphFile.WriteLongint(f3D_FileType_3D);
  DiskRec.FuncColor := FuncColor;
  {Var1}
  DiskRec.BeginVar1Val := PackMathValue(BeginVar1Val);
  DiskRec.EndVar1Val := PackMathValue(EndVar1Val);
  DiskRec.Var1Lines := Var1Lines;
  DiskRec.Var1Detail := Var1Detail;
  {Var2}
  DiskRec.BeginVar2Val := PackMathValue(BeginVar2Val);
  DiskRec.EndVar2Val := PackMathValue(EndVar2Val);
  DiskRec.Var2Lines := Var2Lines;
  DiskRec.Var2Detail := Var2Detail;
  DiskRec.RoundLargeValues := RoundLargeValues;
  with WriteGraphFile do
    begin
      WriteRecord(DiskRec, sizeof(DiskRec));
      WriteMathString(BeginVar1Str);
      WriteMathString(EndVar1Str);
      WriteMathString(BeginVar2Str);
      WriteMathString(EndVar2Str);
    end; {with}
  FuncCalc.SaveToDisk(WriteGraphFile);
end; {procedure TFunc3DObj.SaveToDisk}

{----------------------------------------------------------------------}

procedure TFunc3DObj.ReadFromDisk(ReadGraphFile : TReadGraphFile);

{-----------------------------------}

procedure ReadVer1Fields;
const
  Ver1_ft_2D = 0;
  Ver1_ft_Polar = 1;
  Ver1_ft_3DX = 2;
  Ver1_ft_3DY = 3;
  Ver1_ft_3DZ = 4;
var
  DiskRec : TDiskV1_Func3DRec;
  DependantVar : byte; {'Y=f(x,z)=' etc}
begin {procedure ReadVer1Fields;}
  ReadGraphFile.ReadRecord(DiskRec, sizeof(DiskRec));
  DependantVar := DiskRec.DependantVar;
  FuncColor := DiskRec.FuncColor;
  {Var1}
  BeginVar1Str := DiskRec.BeginVar1Str;
  EndVar1Str := DiskRec.EndVar1Str;
  BeginVar1Val := FloatToMathValue(DiskRec.BeginVar1Val);
  EndVar1Val := FloatToMathValue(DiskRec.EndVar1Val);
  Var1Lines := DiskRec.Var1Lines;
  Var1Detail := DiskRec.Var1Detail;
  {Var2}
  BeginVar2Str := DiskRec.BeginVar2Str;
  EndVar2Str := DiskRec.EndVar2Str;
  BeginVar2Val := FloatToMathValue(DiskRec.BeginVar2Val);
  EndVar2Val := FloatToMathValue(DiskRec.EndVar2Val);
  Var2Lines := DiskRec.Var2Lines;
  Var2Detail := DiskRec.Var2Detail;
  RoundLargeValues := DiskRec.RoundLargeValues;
  case DependantVar of
    Ver1_ft_3DX: FuncCalc.Graph3DFuncType := ft_3DX;
    Ver1_ft_3DY: FuncCalc.Graph3DFuncType := ft_3DY;
    Ver1_ft_3DZ: FuncCalc.Graph3DFuncType := ft_3DZ;
  end; {case}
  FuncCalc.ReadFromDisk(ReadGraphFile);
  {fix and calc math strings}
  with ReadGraphFile do
    begin
      FixCalcMathConstString(BeginVar1Str, BeginVar1Val);
      FixCalcMathConstString(EndVar1Str, EndVar1Val);
      FixCalcMathConstString(BeginVar2Str, BeginVar2Val);
      FixCalcMathConstString(EndVar2Str, EndVar2Val);
    end; {with}
end; {procedure ReadVer1Fields;}

{-----------------------------------}

procedure ReadVer2Fields;
var
  DiskRec : TDiskV2_Func3DRec;
begin {procedure ReadVer2Fields;}
  ReadGraphFile.ReadRecord(DiskRec, sizeof(DiskRec));
  FuncColor := DiskRec.FuncColor;
  {Var1}
  BeginVar1Val := FloatToMathValue(DiskRec.BeginVar1Val);
  EndVar1Val := FloatToMathValue(DiskRec.EndVar1Val);
  Var1Lines := DiskRec.Var1Lines;
  Var1Detail := DiskRec.Var1Detail;
  {Var2}
  BeginVar2Val := FloatToMathValue(DiskRec.BeginVar2Val);
  EndVar2Val := FloatToMathValue(DiskRec.EndVar2Val);
  Var2Lines := DiskRec.Var2Lines;
  Var2Detail := DiskRec.Var2Detail;
  RoundLargeValues := DiskRec.RoundLargeValues;
  with ReadGraphFile do
    begin
      ReadMathConstString(BeginVar1Str, BeginVar1Val, False);
      ReadMathConstString(EndVar1Str, EndVar1Val, False);
      ReadMathConstString(BeginVar2Str, BeginVar2Val, False);
      ReadMathConstString(EndVar2Str, EndVar2Val, False);
    end; {with}
  FuncCalc.ReadFromDisk(ReadGraphFile);
end; {procedure ReadVer2Fields;}

{-----------------------------------}

procedure ReadVer3Fields;
var
  DiskRec : TDiskV3_Func3DRec;
begin {procedure ReadVer3Fields;}
  ReadGraphFile.ReadRecord(DiskRec, sizeof(DiskRec));
  FuncColor := DiskRec.FuncColor;
  {Var1}
  BeginVar1Val := UnPackMathValue(DiskRec.BeginVar1Val);
  EndVar1Val := UnPackMathValue(DiskRec.EndVar1Val);
  Var1Lines := DiskRec.Var1Lines;
  Var1Detail := DiskRec.Var1Detail;
  {Var2}
  BeginVar2Val := UnPackMathValue(DiskRec.BeginVar2Val);
  EndVar2Val := UnPackMathValue(DiskRec.EndVar2Val);
  Var2Lines := DiskRec.Var2Lines;
  Var2Detail := DiskRec.Var2Detail;
  RoundLargeValues := DiskRec.RoundLargeValues;
  with ReadGraphFile do
    begin
      ReadMathConstString(BeginVar1Str, BeginVar1Val, False);
      ReadMathConstString(EndVar1Str, EndVar1Val, False);
      ReadMathConstString(BeginVar2Str, BeginVar2Val, False);
      ReadMathConstString(EndVar2Str, EndVar2Val, False);
    end; {with}
  FuncCalc.ReadFromDisk(ReadGraphFile);
end; {procedure ReadVer3Fields;}

{-----------------------------------}

begin {procedure TFunc3DObj.ReadFromDisk}
  DidFieldsChange := true;
  if ReadGraphFile.FileVersion = 1 then
    ReadVer1Fields
  else
    if ReadGraphFile.FileVersion = 2 then
      ReadVer2Fields
    else {if ReadGraphFile.FileVersion >= 3 then}
      ReadVer3Fields;
end; {procedure TFunc3DObj.ReadFromDisk}

{----------------------------------------------------------------------}

procedure TFunc3DObj.Assign(Source: TFunc3DObj);
begin {procedure TFunc3DObj.Assign}
  FuncCalc.Assign(Source.FuncCalc);
  FuncColor := Source.FuncColor;
  {Var1}
  BeginVar1Str := Source.BeginVar1Str;
  EndVar1Str := Source.EndVar1Str;
  BeginVar1Val := Source.BeginVar1Val;
  EndVar1Val := Source.EndVar1Val;
  Var1Lines := Source.Var1Lines;
  Var1Detail := Source.Var1Detail;
  {Var2}
  BeginVar2Str := Source.BeginVar2Str;
  EndVar2Str := Source.EndVar2Str;
  BeginVar2Val := Source.BeginVar2Val;
  EndVar2Val := Source.EndVar2Val;
  Var2Lines := Source.Var2Lines;
  Var2Detail := Source.Var2Detail;
  RoundLargeValues := Source.RoundLargeValues;
  {Make Memory Unique}
  {Var1}
  UniqueString(BeginVar1Str);
  UniqueString(EndVar1Str);
  {Var2}
  UniqueString(BeginVar2Str);
  UniqueString(EndVar2Str);
end; {procedure TFunc3DObj.Assign}

{----------------------------------------------------------------------}

procedure TFuncRotateObj.AssignSeqObj(Source: TObject);
begin {procedure TFuncRotateObj.AssignSeqObj}
  Assign(Source as TFuncRotateObj);
end; {procedure TFuncRotateObj.AssignSeqObj}

{----------------------------------------------------------------------}

function TFuncRotateObj.CreateNewSeqObj: TObject;
begin {function TFuncRotateObj.CreateNewSeqObj}
  Result := TFuncRotateObj.Create;
end; {function TFuncRotateObj.CreateNewSeqObj}

{----------------------------------------------------------------------}

procedure TFunc3DObj.AssignSeqObj(Source: TObject);
begin {procedure TFunc3DObj.AssignSeqObj}
  Assign(Source as TFunc3DObj);
end; {procedure TFunc3DObj.AssignSeqObj}

{----------------------------------------------------------------------}

function TFunc3DObj.CreateNewSeqObj: TObject;
begin {function TFunc3DObj.CreateNewSeqObj}
  Result := TFunc3DObj.Create;
end; {function TFunc3DObj.CreateNewSeqObj}

{----------------------------------------------------------------------}

end.
