unit funcobj;

(*
Copyright 1999-2011 Greg VanMullem
This program is distributed under the terms of the GNU General Public License Version 3

This file is part of MathGV.

MathGV is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

MathGV is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MathGV.  If not, see <http://www.gnu.org/licenses/>.
*)

interface

uses SeqList, MathCalc, MathFunc, FileSave;

type
  TMathGraphFunc = class(TMathFunction)
  private
    { Private declarations }
  public
    { Public declarations }
    procedure SaveToDisk(WriteGraphFile : TWriteGraphFile); virtual;
    procedure ReadFromDisk(ReadGraphFile : TReadGraphFile); virtual;
  end; {TMathGraphFunc = class(TMathFunction)}

  TBaseFuncObj = class(TSequenceListObj)
    private
    { Private declarations }
    protected
    { Protected declarations }
      function GetListFuncStr: string; virtual; abstract;
    public
    { Public declarations }
      property ListFuncStr : string read GetListFuncStr;
  end; {TBaseFuncObj = class}

implementation

{-----------------------------------------------------------------------}

procedure TMathGraphFunc.SaveToDisk(WriteGraphFile : TWriteGraphFile);
begin {procedure TMathGraphFunc.SaveToDisk}
  WriteGraphFile.WriteMathString(FuncString);
end; {procedure TMathGraphFunc.SaveToDisk}

{-----------------------------------------------------------------------}

procedure TMathGraphFunc.ReadFromDisk(ReadGraphFile : TReadGraphFile);
var
  DidOK, VerifySuccessful : boolean;
  TmpMathStr : string;
begin {procedure TMathGraphFunc.ReadFromDisk}
  with ReadGraphFile do
    begin
      if FileVersion = 1 then
        begin
          SkipBytes(1);
          TmpMathStr := Read16BitString(255);
          FixMathString(TmpMathStr);
        end
      else {if FileVersion >= 2 then}
        TmpMathStr := ReadMathString;
    end; {with}
  FuncString := TmpMathStr;
  if VerifyMathFunc then
    VerifySuccessful := true
  else
    begin
      VerifySuccessful := false;
      repeat
        DidOK := ReadGraphFile.EditStrReadErrorDlg(GetLastErrorString, TmpMathStr);
        if DidOK then
          begin
            FuncString := TmpMathStr;
            VerifySuccessful := VerifyMathFunc;
          end; {if}
      until (not DidOK) or (VerifySuccessful);
    end; {else}
  if not VerifySuccessful then
    ReadGraphFile.RaiseExceptionLoading;
end; {procedure TMathGraphFunc.ReadFromDisk}

{-----------------------------------------------------------------------}

end.
