unit labeldlg;

(*
Copyright 1999-2011 Greg VanMullem
This program is distributed under the terms of the GNU General Public License Version 3

This file is part of MathGV.

MathGV is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

MathGV is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MathGV.  If not, see <http://www.gnu.org/licenses/>.
*)

interface

uses WinTypes, WinProcs, Classes, Graphics, Forms, Controls, Buttons,
  StdCtrls, ExtCtrls, Spin, DrawObjs, LbGlobal, GrDraw, CentrDlg;

type
  TLabelListDlg = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    HelpBtn: TBitBtn;
    FListBox: TListBox;
    SpinButton1: TSpinButton;
    ModifyButton: TBitBtn;
    DeleteButton: TBitBtn;
    OkSelectButton: TBitBtn;
    procedure SpinButton1DownClick(Sender: TObject);
    procedure SpinButton1UpClick(Sender: TObject);
    procedure ModifyButtonClick(Sender: TObject);
    procedure DeleteButtonClick(Sender: TObject);
    procedure FListBoxDblClick(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure OkSelectButtonClick(Sender: TObject);
  private
    { Private declarations }
    DeleteLabelList : TList;
  public
    { Public declarations }
    FDidListChange : boolean;
    FOkSelectWasClicked : boolean;
    FBmpAttribRec : TBitmapAttribRec;
    FBitmap : TBitmap;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  end;

function DisplayLabelListDlg(LabelObjList : TList;
                             const TheBmpAttribRec : TBitmapAttribRec;
                             TheBitmap : TBitmap;
                             var IsLabelSelected : boolean;
                             var TheCurrSelectNdx : integer) : boolean;

implementation

{$R *.DFM}

{------------------------------------------------------------------------}

function DisplayLabelListDlg(LabelObjList : TList;
                             const TheBmpAttribRec : TBitmapAttribRec;
                             TheBitmap : TBitmap;
                             var IsLabelSelected : boolean;
                             var TheCurrSelectNdx : integer) : boolean;
var
  I : integer;
  LabelListDlg : TLabelListDlg;
  TmpSelected : boolean;
begin {function DisplayLabelListDlg}
  TmpSelected := IsLabelSelected;
  Result := false;
  IsLabelSelected := false;
  LabelListDlg := TLabelListDlg.Create(Application);
  try
    LabelListDlg.FBmpAttribRec := TheBmpAttribRec;
    LabelListDlg.FBitmap := TheBitmap;
    SetDlgDefaultPos(LabelListDlg, nil);
    LabelListDlg.FListBox.Items.Clear;
    if LabelObjList.Count > 0 then
      begin
        for I := 0 to LabelObjList.Count -1 do
          LabelListDlg.FListBox.Items.AddObject(
            TDrawBaseClass(LabelObjList[I]).GetLabelNameString,
            LabelObjList[I]);
        if TmpSelected then
          LabelListDlg.FListBox.ItemIndex := TheCurrSelectNdx
        else
          LabelListDlg.FListBox.ItemIndex := 0;
      end; {if}
    if LabelListDlg.ShowModal = idOK then
      begin
        if LabelListDlg.FDidListChange then
          begin
            Result := true;
            LabelObjList.Clear;
            if LabelListDlg.FListBox.Items.Count > 0 then
              for I := 0 to (LabelListDlg.FListBox.Items.Count -1) do
                 LabelObjList.Add(LabelListDlg.FListBox.Items.objects[I]);
          end; {if}
        I := LabelListDlg.FListBox.ItemIndex;
        if LabelListDlg.FOkSelectWasClicked and (I >= 0) then
          begin
            IsLabelSelected := true;
            TheCurrSelectNdx := I;
          end; {if}
      end; {if}
  finally
    LabelListDlg.Free;
  end; {try...Finally}
end; {function DisplayLabelListDlg}

{------------------------------------------------------------------------}

constructor TLabelListDlg.Create(AOwner: TComponent);
begin {constructor TFuncListDlg.Create}
  inherited Create(AOwner);
  DeleteLabelList := TList.Create;
  FDidListChange := false;
  FOkSelectWasClicked := false;
  FBitmap := nil;
end; {constructor TFuncListDlg.Create}

{------------------------------------------------------------------------}

destructor TLabelListDlg.Destroy;
var
  I : integer;
begin {destructor TFuncListDlg.Destroy;}
  if ModalResult = IDOK then
    for I := 0 to (DeleteLabelList.Count -1) do
      TObject(DeleteLabelList[I]).free;
  DeleteLabelList.Free;
  inherited Destroy;
end; {destructor TFuncListDlg.Destroy;}

{------------------------------------------------------------------------}

procedure TLabelListDlg.ModifyButtonClick(Sender: TObject);
var
  I : integer;
  TmpObj : TDrawBaseClass;
begin
  I := FListBox.ItemIndex;
  if I > -1 then
    begin
      TmpObj := TDrawBaseClass(FListBox.Items.objects[I]);
      if ModifyLabelObj(TmpObj, FBmpAttribRec, FBitmap, Self) then
        begin
          FDidListChange := true;
          FListBox.Items.Strings[I] := TmpObj.GetLabelNameString;
          if FListBox.Items.Count >= 1 then
            FListBox.ItemIndex := I;
        end; {if}
    end; {if}
end;

procedure TLabelListDlg.DeleteButtonClick(Sender: TObject);
var
  I : integer;
begin
  I := FListBox.ItemIndex;
  if I > -1 then
    begin
      FDidListChange := true;
      DeleteLabelList.Add(FListBox.Items.objects[I]);
      FListBox.Items.Delete(I);
      if FListBox.Items.Count >= 1 then
        begin
          if I > (FListBox.Items.Count-1) then
            FListBox.ItemIndex := I-1
          else
            FListBox.ItemIndex := I;
        end;
    end; {if}
end;

procedure TLabelListDlg.SpinButton1DownClick(Sender: TObject);
var
  I : integer;
begin
  I := FListBox.ItemIndex;
  if I > -1 then
    if I < (FListBox.Items.Count-1) then
      begin
        FDidListChange := true;
        FListBox.Items.Exchange(I, (I+1));
        FListBox.ItemIndex := I+1;
      end; {if}
end;

procedure TLabelListDlg.SpinButton1UpClick(Sender: TObject);
var
  I : integer;
begin
  I := FListBox.ItemIndex;
  if I > 0 then
    begin
      FDidListChange := true;
      FListBox.Items.Exchange(I, (I-1));
      FListBox.ItemIndex := I-1;
    end; {if}
end;

procedure TLabelListDlg.FListBoxDblClick(Sender: TObject);
begin
  ModifyButtonClick(Sender);
end;

procedure TLabelListDlg.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (Key = VK_DELETE) then
    DeleteButtonClick(Sender);
end;

procedure TLabelListDlg.OkSelectButtonClick(Sender: TObject);
begin
  FOkSelectWasClicked := true;
end;

end.
