unit optdlg;

(*
Copyright 1999-2011 Greg VanMullem
This program is distributed under the terms of the GNU General Public License Version 3

This file is part of MathGV.

MathGV is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

MathGV is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MathGV.  If not, see <http://www.gnu.org/licenses/>.
*)

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls,
  Buttons, ComCtrls, ExtCtrls, Spin, editmath, FloatStr, Status,
  HotKey2D, HotKeyPo, HotKey3D, GenOptns, MathCalc, MathFunc, CentrDlg,
  ComnDlgs, DefFont;

type
  TOptionsDlg = class(TForm)
    Panel1: TPanel;
    Panel2: TPanel;
    PageControl1: TPageControl;
    GeneralTab: TTabSheet;
    StatusLineTab: TTabSheet;
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    HelpBtn: TBitBtn;
    Box2D: TGroupBox;
    BoxCenter2D: TGroupBox;
    EView2DCart: TRadioButton;
    EView2DDeg: TRadioButton;
    EView2DRad: TRadioButton;
    EView2DPix: TRadioButton;
    BoxMouse2D: TGroupBox;
    EMouse2DCart: TRadioButton;
    EMouse2DRad: TRadioButton;
    EMouse2DPix: TRadioButton;
    EMouse2DBmp: TRadioButton;
    EMouse2DDeg: TRadioButton;
    BoxMouse3D: TGroupBox;
    E3DPix: TRadioButton;
    E3DBmp: TRadioButton;
    E3DDontShow: TRadioButton;
    E3DCart: TRadioButton;
    BoxPolar: TGroupBox;
    BoxCenterPolar: TGroupBox;
    EViewPolCart: TRadioButton;
    EViewPolDeg: TRadioButton;
    EViewPolRad: TRadioButton;
    EViewPolPix: TRadioButton;
    BoxMousePolar: TGroupBox;
    EMousePolCart: TRadioButton;
    EMousePolRad: TRadioButton;
    EMousePolPix: TRadioButton;
    EMousePolBmp: TRadioButton;
    EMousePolDeg: TRadioButton;
    HotKey2DTab: TTabSheet;
    HotKeyPolarTab: TTabSheet;
    HotKey3DTab: TTabSheet;
    ZoomIncBox_2D: TGroupBox;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Edit1_2D: TEditMathConst;
    Edit2_2D: TEditMathConst;
    Edit3_2D: TEditMathConst;
    Edit4_2D: TEditMathConst;
    Edit5_2D: TEditMathConst;
    Edit6_2D: TEditMathConst;
    ScrollAmtBox_2D: TGroupBox;
    EScrollAmt_2D: TSpinEdit;
    ZoomIncBox_Pol: TGroupBox;
    Label7: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    Label10: TLabel;
    Label11: TLabel;
    Label12: TLabel;
    Edit1_Pol: TEditMathConst;
    Edit2_Pol: TEditMathConst;
    Edit3_Pol: TEditMathConst;
    Edit4_Pol: TEditMathConst;
    Edit5_Pol: TEditMathConst;
    Edit6_Pol: TEditMathConst;
    ScrollAmtBox_Pol: TGroupBox;
    EScrollAmt_Pol: TSpinEdit;
    RotationAmtBox_3D: TGroupBox;
    ERotAmount_3D: TEditMathConst;
    ZoomIncRangeBox_3D: TGroupBox;
    Label13: TLabel;
    Label14: TLabel;
    Label15: TLabel;
    Label16: TLabel;
    SpinEdit1_3D: TSpinEdit;
    SpinEdit2_3D: TSpinEdit;
    SpinEdit3_3D: TSpinEdit;
    SpinEdit4_3D: TSpinEdit;
    EDefaultChBox: TCheckBox;
    MiscellaneousBox_Gen: TGroupBox;
    RepeatingBox_Gen: TGroupBox;
    BmpAttribBox_Gen: TGroupBox;
    FlatToolBars_Gen: TCheckBox;
    DefaultFontButton: TButton;
    LocationsBox_Gen: TGroupBox;
    CenterDesktopRadio_Gen: TRadioButton;
    CenterMainWinRadio_Gen: TRadioButton;
    Label17: TLabel;
    Label18: TLabel;
    DefaultBmpWidth_Gen: TSpinEdit;
    Label19: TLabel;
    DefaultBmpHeight_Gen: TSpinEdit;
    RepeatingTBButtons_Gen: TCheckBox;
    Label20: TLabel;
    Label21: TLabel;
    DefaultColorDepth_Gen: TComboBox;
    Label22: TLabel;
    RepeatDelay_Gen: TTrackBar;
    Label23: TLabel;
    Label24: TLabel;
    Label25: TLabel;
    Label26: TLabel;
    RepeatRate_Gen: TTrackBar;
    CenterScreenRadio_Gen: TRadioButton;
    CenterParentRadio_Gen: TRadioButton;
    CenterComnChkBox_Gen: TCheckBox;
    BoxShowStatusLine: TGroupBox;
    EShowStatusLine: TCheckBox;
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
    procedure DefaultFontButtonClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    FTmpDefaultFont : TFont;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SetStatusLineData(SLObj : TStatusLineObj);
    procedure GetStatusLineData(SLObj : TStatusLineObj);
    procedure SetHotKey2DData(HK2DObj : THotKey2DObj);
    procedure GetHotKey2DData(HK2DObj : THotKey2DObj);
    function ValidateHotKey2D : boolean;
    procedure SetHotKeyPolarData(HKPolarObj : THotKeyPolarObj);
    procedure GetHotKeyPolarData(HKPolarObj : THotKeyPolarObj);
    function ValidateHotKeyPolar : boolean;
    procedure SetHotKey3DData(HK3DObj : THotKey3DObj);
    procedure GetHotKey3DData(HK3DObj : THotKey3DObj);
    function ValidateHotKey3D : boolean;
    procedure SetGeneralOptionsData(GenOptnsObj : TGeneralOptionsObj);
    procedure GetGeneralOptionsData(GenOptnsObj : TGeneralOptionsObj);
  end;

function DisplayOptionsDlg(ShowStatusTab : boolean;
                           GenOptsObj : TGeneralOptionsObj;
                           SLObj: TStatusLineObj;
                           HK2DObj : THotKey2DObj;
                           HKPolarObj : THotKeyPolarObj;
                           HK3DObj : THotKey3DObj) : boolean;

implementation

{$R *.DFM}

function DisplayOptionsDlg(ShowStatusTab : boolean;
                           GenOptsObj : TGeneralOptionsObj;
                           SLObj: TStatusLineObj;
                           HK2DObj : THotKey2DObj;
                           HKPolarObj : THotKeyPolarObj;
                           HK3DObj : THotKey3DObj) : boolean;
var
  OptionsDlg: TOptionsDlg;
begin {procedure DisplayOptionsDlg;}
  Result := false;
  OptionsDlg := TOptionsDlg.Create(Application);
  try
    SetDlgDefaultPos(OptionsDlg, nil);
    if ShowStatusTab then
      OptionsDlg.PageControl1.ActivePageIndex := 1
    else
      OptionsDlg.PageControl1.ActivePageIndex := 0;
    DefaultFontHolder.GetFontAttributes(OptionsDlg.FTmpDefaultFont);
    OptionsDlg.SetGeneralOptionsData(GenOptsObj);
    OptionsDlg.SetStatusLineData(SLObj);
    OptionsDlg.SetHotKey2DData(HK2DObj);
    OptionsDlg.SetHotKeyPolarData(HKPolarObj);
    OptionsDlg.SetHotKey3DData(HK3DObj);
    if (OptionsDlg.ShowModal = IDOK) then
      begin
        Result := true;
        if OptionsDlg.EDefaultChBox.Checked then
          begin
            DefaultFontHolder.ResetToDefault;
            GenOptsObj.ResetDefaults;
            SLObj.ResetDefaults;
            HK2DObj.ResetDefaults;
            HKPolarObj.ResetDefaults;
            HK3DObj.ResetDefaults;
          end
        else
          begin
            DefaultFontHolder.SetFontAttributes(OptionsDlg.FTmpDefaultFont);
            OptionsDlg.GetGeneralOptionsData(GenOptsObj);
            OptionsDlg.GetStatusLineData(SLObj);
            OptionsDlg.GetHotKey2DData(HK2DObj);
            OptionsDlg.GetHotKeyPolarData(HKPolarObj);
            OptionsDlg.GetHotKey3DData(HK3DObj);
          end; {else}
      end; {if}
  finally
    OptionsDlg.free;
  end; {try..finally}
end; {procedure DisplayOptionsDlg;}

{ TOptionsDlg }

constructor TOptionsDlg.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FTmpDefaultFont := TFont.Create;
end;

destructor TOptionsDlg.Destroy;
begin
  FTmpDefaultFont.free;
  inherited;
end;

procedure TOptionsDlg.GetGeneralOptionsData(
  GenOptnsObj: TGeneralOptionsObj);
begin
  with GenOptnsObj do
    begin
      EnableRepTBButtons := RepeatingTBButtons_Gen.Checked;
      RepeatDelay := RepeatDelay_Gen.Position;
      RepeatRate := RepeatRate_Gen.Position;
      DefaultBmpWidth := DefaultBmpWidth_Gen.Value;
      DefaultBmpHeight := DefaultBmpHeight_Gen.Value;
      case DefaultColorDepth_Gen.ItemIndex of
        0: DefaultColorDepth := pfDevice;
        1: DefaultColorDepth := pf4bit;
        2: DefaultColorDepth := pf8bit;
        3: DefaultColorDepth := pf15bit;
        4: DefaultColorDepth := pf16bit;
        5: DefaultColorDepth := pf24bit;
        6: DefaultColorDepth := pf32bit;
        else DefaultColorDepth := pfDevice;
      end; {case}
      FlatToolBars := FlatToolBars_Gen.Checked;
      if CenterScreenRadio_Gen.Checked then CenterDlgsOnPosition := dpScreenCenter else
      if CenterDesktopRadio_Gen.Checked then CenterDlgsOnPosition := dpDesktopCenter else
      if CenterMainWinRadio_Gen.Checked then CenterDlgsOnPosition := dpMainFormCenter else
      if CenterParentRadio_Gen.Checked then CenterDlgsOnPosition := dpSpecifiedFormCenter else
        CenterDlgsOnPosition := dpSpecifiedFormCenter;
      CenterCommonDlgs := CenterComnChkBox_Gen.Checked;
    end; {with}
end;

procedure TOptionsDlg.GetHotKey2DData(HK2DObj: THotKey2DObj);
begin
  with HK2DObj do
    begin
      ZoomRangeFloatArray[1] := Edit1_2D.LastValue;
      ZoomRangeFloatArray[2] := Edit2_2D.LastValue;
      ZoomRangeFloatArray[3] := Edit3_2D.LastValue;
      ZoomRangeFloatArray[4] := Edit4_2D.LastValue;
      ZoomRangeFloatArray[5] := Edit5_2D.LastValue;
      ZoomRangeFloatArray[6] := Edit6_2D.LastValue;
      GraphScrollViewAmount := EScrollAmt_2D.Value;
    end; {with}
end;

procedure TOptionsDlg.GetHotKey3DData(HK3DObj: THotKey3DObj);
begin
  with HK3DObj do
    begin
      Rotation3DIncAmt := ERotAmount_3D.LastValue;
      ZoomRange3DIntArray[1] := SpinEdit1_3D.Value;
      ZoomRange3DIntArray[2] := SpinEdit2_3D.Value;
      ZoomRange3DIntArray[3] := SpinEdit3_3D.Value;
      ZoomRange3DIntArray[4] := SpinEdit4_3D.Value;
    end; {with}
end;

procedure TOptionsDlg.GetHotKeyPolarData(HKPolarObj: THotKeyPolarObj);
begin
  with HKPolarObj do
    begin
      ZoomRangeFloatArray[1] := Edit1_Pol.LastValue;
      ZoomRangeFloatArray[2] := Edit2_Pol.LastValue;
      ZoomRangeFloatArray[3] := Edit3_Pol.LastValue;
      ZoomRangeFloatArray[4] := Edit4_Pol.LastValue;
      ZoomRangeFloatArray[5] := Edit5_Pol.LastValue;
      ZoomRangeFloatArray[6] := Edit6_Pol.LastValue;
      GraphScrollViewAmount := EScrollAmt_Pol.Value;
    end; {with}
end;

procedure TOptionsDlg.GetStatusLineData(SLObj: TStatusLineObj);
begin
  with SLObj do
    begin
      {Get New 2D Center}
      if EView2DCart.Checked then CenterFormat2D := vcCartesian else
      if EView2DDeg.Checked then CenterFormat2D := vcDegrees else
      if EView2DRad.Checked then CenterFormat2D := vcRadians else
      if EView2DPix.Checked then CenterFormat2D := vcPixels;
      {Get New Polar Center}
      if EViewPolCart.Checked then CenterFormatPolar := vcCartesian else
      if EViewPolDeg.Checked then CenterFormatPolar := vcDegrees else
      if EViewPolRad.Checked then CenterFormatPolar := vcRadians else
      if EViewPolPix.Checked then CenterFormatPolar := vcPixels;
      {Get New 2D Mouse}
      if EMouse2DCart.Checked then MouseFormat2D := mpCartesian else
      if EMouse2DDeg.Checked then MouseFormat2D := mpDegrees else
      if EMouse2DRad.Checked then MouseFormat2D := mpRadians else
      if EMouse2DPix.Checked then MouseFormat2D := mpPixels else
      if EMouse2DBmp.Checked then MouseFormat2D := mpBitMap;
      {Get New Polar Mouse}
      if EMousePolCart.Checked then MouseFormatPolar := mpCartesian else
      if EMousePolDeg.Checked then MouseFormatPolar := mpDegrees else
      if EMousePolRad.Checked then MouseFormatPolar := mpRadians else
      if EMousePolPix.Checked then MouseFormatPolar := mpPixels else
      if EMousePolBmp.Checked then MouseFormatPolar := mpBitMap;
      {Get New 3D Mouse}
      if E3DCart.Checked then MouseFormat3D := m3Cartesian else
      if E3DPix.Checked then MouseFormat3D := m3Pixels else
      if E3DBmp.Checked then MouseFormat3D := m3BitMap else
      if E3DDontShow.Checked then MouseFormat3D := m3DontShow;
      {is status line visible}
      ShowStatusLine := EShowStatusLine.Checked;
    end; {with}
end;

procedure TOptionsDlg.SetGeneralOptionsData(
  GenOptnsObj: TGeneralOptionsObj);
begin
  with GenOptnsObj do
    begin
      RepeatingTBButtons_Gen.Checked := EnableRepTBButtons;
      RepeatDelay_Gen.Position := RepeatDelay;
      RepeatRate_Gen.Position := RepeatRate;
      DefaultBmpWidth_Gen.Value := DefaultBmpWidth;
      DefaultBmpHeight_Gen.Value := DefaultBmpHeight;
      case DefaultColorDepth of
        pfDevice: DefaultColorDepth_Gen.ItemIndex := 0;
        pf4bit: DefaultColorDepth_Gen.ItemIndex := 1;
        pf8bit: DefaultColorDepth_Gen.ItemIndex := 2;
        pf15bit: DefaultColorDepth_Gen.ItemIndex := 3;
        pf16bit: DefaultColorDepth_Gen.ItemIndex := 4;
        pf24bit: DefaultColorDepth_Gen.ItemIndex := 5;
        pf32bit: DefaultColorDepth_Gen.ItemIndex := 6;
        else DefaultColorDepth_Gen.ItemIndex := -1;
      end; {case}
      FlatToolBars_Gen.Checked := FlatToolBars;
      case CenterDlgsOnPosition of
        dpScreenCenter: CenterScreenRadio_Gen.Checked := true;
        dpDesktopCenter: CenterDesktopRadio_Gen.Checked := true;
        dpMainFormCenter: CenterMainWinRadio_Gen.Checked := true;
        dpSpecifiedFormCenter: CenterParentRadio_Gen.Checked := true;
      end; {case}
      CenterComnChkBox_Gen.Checked := CenterCommonDlgs;
    end; {with}
end;

procedure TOptionsDlg.SetHotKey2DData(HK2DObj: THotKey2DObj);
begin
  with HK2DObj do
    begin
      Edit1_2D.text := Float2DZoomToStrF(ZoomRangeFloatArray[1]);
      Edit2_2D.text := Float2DZoomToStrF(ZoomRangeFloatArray[2]);
      Edit3_2D.text := Float2DZoomToStrF(ZoomRangeFloatArray[3]);
      Edit4_2D.text := Float2DZoomToStrF(ZoomRangeFloatArray[4]);
      Edit5_2D.text := Float2DZoomToStrF(ZoomRangeFloatArray[5]);
      Edit6_2D.text := Float2DZoomToStrF(ZoomRangeFloatArray[6]);
      EScrollAmt_2D.Value := GraphScrollViewAmount;
    end; {with}
end;

procedure TOptionsDlg.SetHotKey3DData(HK3DObj: THotKey3DObj);
begin
  with HK3DObj do
    begin
      ERotAmount_3D.text := RotateNumToStr(Rotation3DIncAmt);
      SpinEdit1_3D.Value := ZoomRange3DIntArray[1];
      SpinEdit2_3D.Value := ZoomRange3DIntArray[2];
      SpinEdit3_3D.Value := ZoomRange3DIntArray[3];
      SpinEdit4_3D.Value := ZoomRange3DIntArray[4];
    end; {with}
end;

procedure TOptionsDlg.SetHotKeyPolarData(HKPolarObj: THotKeyPolarObj);
begin
  with HKPolarObj do
    begin
      Edit1_Pol.text := Float2DZoomToStrF(ZoomRangeFloatArray[1]);
      Edit2_Pol.text := Float2DZoomToStrF(ZoomRangeFloatArray[2]);
      Edit3_Pol.text := Float2DZoomToStrF(ZoomRangeFloatArray[3]);
      Edit4_Pol.text := Float2DZoomToStrF(ZoomRangeFloatArray[4]);
      Edit5_Pol.text := Float2DZoomToStrF(ZoomRangeFloatArray[5]);
      Edit6_Pol.text := Float2DZoomToStrF(ZoomRangeFloatArray[6]);
      EScrollAmt_Pol.Value := GraphScrollViewAmount;
    end; {with}
end;

procedure TOptionsDlg.SetStatusLineData(SLObj: TStatusLineObj);
begin
  {Set 2D Center}
  case SLObj.CenterFormat2D of
    vcCartesian : EView2DCart.Checked := true;
    vcDegrees : EView2DDeg.Checked := true;
    vcRadians : EView2DRad.Checked := true;
    vcPixels : EView2DPix.Checked := true;
    else
      EView2DCart.Checked := true;
  end; {case}
  {Set Polar Center}
  case SLObj.CenterFormatPolar of
    vcCartesian : EViewPolCart.Checked := true;
    vcDegrees : EViewPolDeg.Checked := true;
    vcRadians : EViewPolRad.Checked := true;
    vcPixels : EViewPolPix.Checked := true;
    else
      EViewPolDeg.Checked := true;
  end; {case}
  {Set 2D Mouse}
  case SLObj.MouseFormat2D of
    mpCartesian : EMouse2DCart.Checked := true;
    mpDegrees : EMouse2DDeg.Checked := true;
    mpRadians : EMouse2DRad.Checked := true;
    mpPixels : EMouse2DPix.Checked := true;
    mpBitMap : EMouse2DBmp.Checked := true;
    else
      EMouse2DCart.Checked := true;
  end; {case}
  {Set Polar Mouse}
  case SLObj.MouseFormatPolar of
    mpCartesian : EMousePolCart.Checked := true;
    mpDegrees : EMousePolDeg.Checked := true;
    mpRadians : EMousePolRad.Checked := true;
    mpPixels : EMousePolPix.Checked := true;
    mpBitMap : EMousePolBmp.Checked := true;
    else
      EMousePolDeg.Checked := true;
  end; {case}
  {Set 3D Mouse}
  case SLObj.MouseFormat3D of
    m3Cartesian : E3DCart.Checked := true;
    m3Pixels : E3DPix.Checked := true;
    m3BitMap : E3DBmp.Checked := true;
    m3DontShow : E3DDontShow.Checked := true;
    else
      E3DDontShow.Checked := true;
  end; {case}
  EShowStatusLine.Checked := SLObj.ShowStatusLine;
end;

function TOptionsDlg.ValidateHotKey2D: boolean;
begin
  Result := false;
  if Edit1_2D.ValidateMathStr then
  if Edit2_2D.ValidateMathStr then
  if Edit3_2D.ValidateMathStr then
  if Edit4_2D.ValidateMathStr then
  if Edit5_2D.ValidateMathStr then
  if Edit6_2D.ValidateMathStr then
    Result := true;
end;

function TOptionsDlg.ValidateHotKey3D: boolean;
begin
  Result := ERotAmount_3D.ValidateMathStr;
end;

function TOptionsDlg.ValidateHotKeyPolar: boolean;
begin
  Result := false;
  if Edit1_Pol.ValidateMathStr then
  if Edit2_Pol.ValidateMathStr then
  if Edit3_Pol.ValidateMathStr then
  if Edit4_Pol.ValidateMathStr then
  if Edit5_Pol.ValidateMathStr then
  if Edit6_Pol.ValidateMathStr then
    Result := true;
end;

procedure TOptionsDlg.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  CanClose := false;
  if ModalResult = idOK then
    begin
      if ValidateHotKey2D then
      if ValidateHotKeyPolar then
      if ValidateHotKey3D then
        CanClose := true;
    end
  else
    CanClose := true;
end;

procedure TOptionsDlg.FormKeyPress(Sender: TObject; var Key: Char);
begin
  FormKeyPressSpinEditClose(self, Key);
end;

procedure TOptionsDlg.DefaultFontButtonClick(Sender: TObject);
begin
  ShowComnFontDlg(FTmpDefaultFont, Self);
end;

end.

