unit status;

(*
Copyright 1999-2011 Greg VanMullem
This program is distributed under the terms of the GNU General Public License Version 3

This file is part of MathGV.

MathGV is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

MathGV is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MathGV.  If not, see <http://www.gnu.org/licenses/>.
*)

interface

uses
  sysutils, Vw2dObj, VwPoObj, Vw3dObj, FloatStr, MathCalc, MathFunc,
  GrGlobal, iniFiles;

type
  TspViewCenter = (vcCartesian, vcDegrees, vcRadians, vcPixels);
  TspMousePos = (mpCartesian, mpDegrees, mpRadians, mpPixels, mpBitMap);
  TspMouse3D = (m3Cartesian, m3Pixels, m3BitMap, m3DontShow);

  TStatusLineObj = class
  private
    { Private declarations }
    PrevZoomStr : string;
    PrevCenterStr : string;
    PrevMouseStr : string;
    procedure DrawZoomStatus(const TheStatusStr : string);
    procedure DrawCenterStatus(const TheStatusStr : string);
    procedure DrawMouseStatus(const TheStatusStr : string);
    procedure CenterCartesian2D(View2DZoomObj : TView2DZoomObj);
    procedure CenterCartesianPolar(ViewPolarZoomObj : TViewPolarZoomObj);
    procedure CenterPolDeg2D(View2DZoomObj : TView2DZoomObj);
    procedure CenterPolDegPolar(ViewPolarZoomObj : TViewPolarZoomObj);
    procedure CenterPolRad2D(View2DZoomObj : TView2DZoomObj);
    procedure CenterPolRadPolar(ViewPolarZoomObj : TViewPolarZoomObj);
    procedure CenterPixels2D(View2DZoomObj : TView2DZoomObj);
    procedure CenterPixelsPolar(ViewPolarZoomObj : TViewPolarZoomObj);
    procedure MousePixels(PixelX, PixelY : longint);
    procedure MouseCartesian2D(View2DZoomObj : TView2DZoomObj;
                               PixelX, PixelY : longint);
    procedure MouseCartesianPolar(ViewPolarZoomObj : TViewPolarZoomObj;
                                  PixelX, PixelY : longint);
    procedure MousePolDeg2D(View2DZoomObj : TView2DZoomObj;
                            PixelX, PixelY : longint);
    procedure MousePolDegPolar(ViewPolarZoomObj : TViewPolarZoomObj;
                               PixelX, PixelY : longint);
    procedure MousePolRad2D(View2DZoomObj : TView2DZoomObj;
                            PixelX, PixelY : longint);
    procedure MousePolRadPolar(ViewPolarZoomObj : TViewPolarZoomObj;
                               PixelX, PixelY : longint);
    procedure MouseBmp(MouseX, MouseY : integer);
  public
    { Public declarations }
    CenterFormat2D : TspViewCenter;
    MouseFormat2D : TspMousePos;
    CenterFormatPolar : TspViewCenter;
    MouseFormatPolar : TspMousePos;
    MouseFormat3D : TspMouse3D;
    ShowStatusLine : boolean;
    constructor Create;
    procedure ResetDefaults;
    function ReadFromIniFile(TheIniFile : TMemIniFile) : boolean;
    function SaveToIniFile(TheIniFile : TMemIniFile) : boolean;
    //add exception handeling to lower
    procedure BlankStatusLine;
    procedure UpDate2DStatusLine(View2DZoomObj : TView2DZoomObj;
                                 BmpWidth, BmpHeight : integer;
                                 MouseX, MouseY : integer);
    procedure UpDatePolarStatusLine(ViewPolarZoomObj : TViewPolarZoomObj;
                                 BmpWidth, BmpHeight : integer;
                                 MouseX, MouseY : integer);
    procedure UpDate3DStatusLine(View3DZoomObj : TView3DZoomObj;
                                 BmpWidth, BmpHeight : integer;
                                 MouseX, MouseY : integer);
  end; {TStatusLineObj}

implementation

uses
  fsMain;

const
  //key strings
  sct_StatusLine = 'StatusLine';
    key_CenterFormat2D = 'CenterFormat2D';
    key_MouseFormat2D = 'MouseFormat2D';
    key_CenterFormatPolar = 'CenterFormatPolar';
    key_MouseFormatPolar = 'MouseFormatPolar';
    key_MouseFormat3D = 'MouseFormat3D';
    key_ShowStatusLine = 'ShowStatusLine';
  //enumerated type string
  //TspViewCenter
  enu_vcCartesian = 'vcCartesian';
  enu_vcDegrees = 'vcDegrees';
  enu_vcRadians = 'vcRadians';
  enu_vcPixels = 'vcPixels';
  //TspMousePos
  enu_mpCartesian = 'mpCartesian';
  enu_mpDegrees = 'mpDegrees';
  enu_mpRadians = 'mpRadians';
  enu_mpPixels = 'mpPixels';
  enu_mpBitMap = 'mpBitMap';
  //TspMouse3D
  enu_m3Cartesian = 'm3Cartesian';
  enu_m3Pixels = 'm3Pixels';
  enu_m3BitMap = 'm3BitMap';
  enu_m3DontShow = 'm3DontShow';

{----------------------------------------------------------------------}

function StatusLineFloatToStr(TheFloatCord : TMathFloat): string;
begin {function StatusLineFloatToStr}
  //old conversion; str(TheFloatCord:15:5,Result);
  Result := FloatToStrF(TheFloatCord, ffFixed, 30, 5);
  CleanFloatString(Result);
end; {function StatusLineFloatToStr}

{----------------------------------------------------------------------}

function ReadIniTspViewCenter(TheIniFile : TMemIniFile; DefaultValue : TspViewCenter;
                              const TheSection, TheKey : string) : TspViewCenter;
var
  TmpStr : string;
begin {function ReadIniTspViewCenter}
  TmpStr := TheIniFile.ReadString(TheSection, TheKey, '');
  if TmpStr = enu_vcCartesian then Result := vcCartesian else
  if TmpStr = enu_vcDegrees then Result := vcDegrees else
  if TmpStr = enu_vcRadians then Result := vcRadians else
  if TmpStr = enu_vcPixels then Result := vcPixels else
    Result := DefaultValue;
end; {function ReadIniTspViewCenter}

{----------------------------------------------------------------------}

procedure WriteIniTspViewCenter(TheIniFile : TMemIniFile; TheValue : TspViewCenter;
                                const TheSection, TheKey : string);
var
  TmpStr : string;
begin {procedure WriteIniTspViewCenter}
  case TheValue of
    vcCartesian: TmpStr := enu_vcCartesian;
    vcDegrees: TmpStr := enu_vcDegrees;
    vcRadians: TmpStr := enu_vcRadians;
    vcPixels: TmpStr := enu_vcPixels;
    else
      TmpStr := enu_vcCartesian;
  end; {case}
  TheIniFile.WriteString(TheSection, TheKey, TmpStr);
end; {procedure WriteIniTspViewCenter}

{----------------------------------------------------------------------}

function ReadIniTspMousePos(TheIniFile : TMemIniFile; DefaultValue : TspMousePos;
                            const TheSection, TheKey : string) : TspMousePos;
var
  TmpStr : string;
begin {function ReadIniTspMousePos}
  TmpStr := TheIniFile.ReadString(TheSection, TheKey, '');
  if TmpStr = enu_mpCartesian then Result := mpCartesian else
  if TmpStr = enu_mpDegrees then Result := mpDegrees else
  if TmpStr = enu_mpRadians then Result := mpRadians else
  if TmpStr = enu_mpPixels then Result := mpPixels else
  if TmpStr = enu_mpBitMap then Result := mpBitMap else
    Result := DefaultValue;
end; {function ReadIniTspMousePos}

{----------------------------------------------------------------------}

procedure WriteIniTspMousePos(TheIniFile : TMemIniFile; TheValue : TspMousePos;
                              const TheSection, TheKey : string);
var
  TmpStr : string;
begin {procedure WriteIniTspMousePos}
  case TheValue of
    mpCartesian: TmpStr := enu_mpCartesian;
    mpDegrees: TmpStr := enu_mpDegrees;
    mpRadians: TmpStr := enu_mpRadians;
    mpPixels: TmpStr := enu_mpPixels;
    mpBitMap: TmpStr := enu_mpBitMap;
    else
      TmpStr := enu_vcCartesian;
  end; {case}
  TheIniFile.WriteString(TheSection, TheKey, TmpStr);
end; {procedure WriteIniTspMousePos}

{----------------------------------------------------------------------}

function ReadIniTspMouse3D(TheIniFile : TMemIniFile; DefaultValue : TspMouse3D;
                           const TheSection, TheKey : string) : TspMouse3D;
var
  TmpStr : string;
begin {function ReadIniTspMouse3D}
  TmpStr := TheIniFile.ReadString(TheSection, TheKey, '');
  if TmpStr = enu_m3Cartesian then Result := m3Cartesian else
  if TmpStr = enu_m3Pixels then Result := m3Pixels else
  if TmpStr = enu_m3BitMap then Result := m3BitMap else
  if TmpStr = enu_m3DontShow then Result := m3DontShow else
    Result := DefaultValue;
end; {function ReadIniTspMouse3D}

{----------------------------------------------------------------------}

procedure WriteIniTspMouse3D(TheIniFile : TMemIniFile; TheValue : TspMouse3D;
                             const TheSection, TheKey : string);
var
  TmpStr : string;
begin {procedure WriteIniTspMouse3D}
  case TheValue of
    m3Cartesian: TmpStr := enu_m3Cartesian;
    m3Pixels: TmpStr := enu_m3Pixels;
    m3BitMap: TmpStr := enu_m3BitMap;
    m3DontShow: TmpStr := enu_m3DontShow;
    else
      TmpStr := enu_m3Cartesian;
  end; {case}
  TheIniFile.WriteString(TheSection, TheKey, TmpStr);
end; {procedure WriteIniTspMouse3D}

{----------------------------------------------------------------------}

{ TStatusLineObj }

constructor TStatusLineObj.Create;
begin {constructor TStatusLineObj.Create;}
  inherited;
  PrevZoomStr   := '^?~1'; //set to string that will never be used
  PrevCenterStr := '^?~2'; //set to string that will never be used
  PrevMouseStr  := '^?~3'; //set to string that will never be used
  ResetDefaults;
end; {constructor TStatusLineObj.Create;}

{----------------------------------------------------------------------}

function TStatusLineObj.ReadFromIniFile(TheIniFile: TMemIniFile) : boolean;
begin {function TStatusLineObj.ReadFromIniFile}
  Result := true;
  try
    CenterFormat2D := ReadIniTspViewCenter(TheIniFile, vcCartesian, sct_StatusLine, key_CenterFormat2D);
    MouseFormat2D := ReadIniTspMousePos(TheIniFile, mpCartesian, sct_StatusLine, key_MouseFormat2D);
    CenterFormatPolar := ReadIniTspViewCenter(TheIniFile, vcDegrees, sct_StatusLine, key_CenterFormatPolar);
    MouseFormatPolar := ReadIniTspMousePos(TheIniFile, mpDegrees, sct_StatusLine, key_MouseFormatPolar);
    MouseFormat3D := ReadIniTspMouse3D(TheIniFile, m3DontShow, sct_StatusLine, key_MouseFormat3D);
    ShowStatusLine := TheIniFile.ReadBool(sct_StatusLine, key_ShowStatusLine, true);
  except
    Result := false;
  end; {try..except}
end; {function TStatusLineObj.ReadFromIniFile}

{----------------------------------------------------------------------}

procedure TStatusLineObj.ResetDefaults;
begin {procedure TStatusLineObj.ResetDefaults;}
  CenterFormat2D := vcCartesian;
  MouseFormat2D := mpCartesian;
  CenterFormatPolar := vcDegrees;
  MouseFormatPolar := mpDegrees;
  MouseFormat3D := m3DontShow;
  ShowStatusLine := true;
end; {procedure TStatusLineObj.ResetDefaults;}

{----------------------------------------------------------------------}

function TStatusLineObj.SaveToIniFile(TheIniFile: TMemIniFile) : boolean;
begin {function TStatusLineObj.SaveToIniFile}
  Result := true;
  try
    WriteIniTspViewCenter(TheIniFile, CenterFormat2D, sct_StatusLine, key_CenterFormat2D);
    WriteIniTspMousePos(TheIniFile, MouseFormat2D, sct_StatusLine, key_MouseFormat2D);
    WriteIniTspViewCenter(TheIniFile, CenterFormatPolar, sct_StatusLine, key_CenterFormatPolar);
    WriteIniTspMousePos(TheIniFile, MouseFormatPolar, sct_StatusLine, key_MouseFormatPolar);
    WriteIniTspMouse3D(TheIniFile, MouseFormat3D, sct_StatusLine, key_MouseFormat3D);
    TheIniFile.WriteBool(sct_StatusLine, key_ShowStatusLine, ShowStatusLine);
  except
    Result := false;
  end; {try..except}
end; {function TStatusLineObj.SaveToIniFile}

{----------------------------------------------------------------------}

procedure TStatusLineObj.DrawZoomStatus(const TheStatusStr : string);
begin {procedure TStatusLineObj.DrawZoomStatus}
  if TheStatusStr <> PrevZoomStr then
    begin
      PrevZoomStr := TheStatusStr;
      MainForm.StatusBar.Panels[0].text := PrevZoomStr;
    end; {if}
end; {procedure TStatusLineObj.DrawZoomStatus}

{----------------------------------------------------------------------}

procedure TStatusLineObj.DrawCenterStatus(const TheStatusStr : string);
begin {procedure TStatusLineObj.DrawCenterStatus}
  if TheStatusStr <> PrevCenterStr then
    begin
      PrevCenterStr := TheStatusStr;
      MainForm.StatusBar.Panels[1].text := PrevCenterStr;
    end; {if}
end; {procedure TStatusLineObj.DrawCenterStatus}

{----------------------------------------------------------------------}

procedure TStatusLineObj.DrawMouseStatus(const TheStatusStr : string);
begin {procedure TStatusLineObj.DrawMouseStatus}
  if TheStatusStr <> PrevMouseStr then
    begin
      PrevMouseStr := TheStatusStr;
      MainForm.StatusBar.Panels[2].text := PrevMouseStr;
    end; {if}
end; {procedure TStatusLineObj.DrawMouseStatus}

{----------------------------------------------------------------------}

procedure TStatusLineObj.CenterCartesian2D(View2DZoomObj : TView2DZoomObj);
begin {procedure TStatusLineObj.CenterCartesian2D}
  with View2DZoomObj do
    DrawCenterStatus('Center X:'+
       StatusLineFloatToStr(ViewCenterX/XViewZoom)+
       ' Y:' + StatusLineFloatToStr(ViewCenterY/YViewZoom));
end; {procedure TStatusLineObj.CenterCartesian2D}

{----------------------------------------------------------------------}

procedure TStatusLineObj.CenterCartesianPolar(ViewPolarZoomObj : TViewPolarZoomObj);
begin {procedure TStatusLineObj.CenterCartesianPolar}
  with ViewPolarZoomObj do
    DrawCenterStatus('Center X:'+
       StatusLineFloatToStr(ViewCenterX/ViewZoom)+
       ' Y:' + StatusLineFloatToStr(ViewCenterY/ViewZoom));
end; {procedure TStatusLineObj.CenterCartesianPolar}

{----------------------------------------------------------------------}

procedure TStatusLineObj.CenterPolDeg2D(View2DZoomObj : TView2DZoomObj);
var
  PR, PTheta : TMathFloat;
begin {procedure TStatusLineObj.CenterPolDeg2D}
  with View2DZoomObj do
    ConvCartesianToPolar((ViewCenterX/XViewZoom),
                         (ViewCenterY/YViewZoom), PR, PTheta);
  DrawCenterStatus('Center R:'+
     StatusLineFloatToStr(PR)+
     ' T:' + StatusLineFloatToStr(PTheta*(180/pi)) + 'd');
end; {procedure TStatusLineObj.CenterPolDeg2D}

{----------------------------------------------------------------------}

procedure TStatusLineObj.CenterPolDegPolar(ViewPolarZoomObj : TViewPolarZoomObj);
var
  PR, PTheta : TMathFloat;
begin {procedure TStatusLineObj.CenterPolDegPolar}
  with ViewPolarZoomObj do
    ConvCartesianToPolar((ViewCenterX/ViewZoom),
                         (ViewCenterY/ViewZoom), PR, PTheta);
  DrawCenterStatus('Center R:'+
     StatusLineFloatToStr(PR)+
     ' T:' + StatusLineFloatToStr(PTheta*(180/pi)) + 'd');
end; {procedure TStatusLineObj.CenterPolDegPolar}

{----------------------------------------------------------------------}

procedure TStatusLineObj.CenterPolRad2D(View2DZoomObj : TView2DZoomObj);
var
  PR, PTheta : TMathFloat;
begin {procedure TStatusLineObj.CenterPolRad2D}
  with View2DZoomObj do
    ConvCartesianToPolar((ViewCenterX/XViewZoom),
                         (ViewCenterY/YViewZoom), PR, PTheta);
  DrawCenterStatus('Center R:'+
     StatusLineFloatToStr(PR)+
     ' T:' + StatusLineFloatToStr(PTheta) + 'r');
end; {procedure TStatusLineObj.CenterPolRad2D}

{----------------------------------------------------------------------}

procedure TStatusLineObj.CenterPolRadPolar(ViewPolarZoomObj : TViewPolarZoomObj);
var
  PR, PTheta : TMathFloat;
begin {procedure TStatusLineObj.CenterPolRadPolar}
  with ViewPolarZoomObj do
    ConvCartesianToPolar((ViewCenterX/ViewZoom),
                         (ViewCenterY/ViewZoom), PR, PTheta);
  DrawCenterStatus('Center R:'+
     StatusLineFloatToStr(PR)+
     ' T:' + StatusLineFloatToStr(PTheta) + 'r');
end; {procedure TStatusLineObj.CenterPolRadPolar}

{----------------------------------------------------------------------}

procedure TStatusLineObj.CenterPixels2D(View2DZoomObj : TView2DZoomObj);
begin {procedure TStatusLineObj.CenterPixels2D}
  DrawCenterStatus('Center(Pix) X:'+
                   FloatToStr(View2DZoomObj.ViewCenterX)+
                   ' Y:' + FloatToStr(View2DZoomObj.ViewCenterY));
end; {procedure TStatusLineObj.CenterPixels2D}

{----------------------------------------------------------------------}

procedure TStatusLineObj.CenterPixelsPolar(ViewPolarZoomObj : TViewPolarZoomObj);
begin {procedure TStatusLineObj.CenterPixelsPolar}
  DrawCenterStatus('Center(Pix) X:'+
                   FloatToStr(ViewPolarZoomObj.ViewCenterX)+
                   ' Y:' + FloatToStr(ViewPolarZoomObj.ViewCenterY));
end; {procedure TStatusLineObj.CenterPixelsPolar}

{----------------------------------------------------------------------}

procedure TStatusLineObj.MousePixels(PixelX, PixelY : longint);
begin {procedure TStatusLineObj.MousePixels}
  DrawMouseStatus('Mouse(Pix) X:' + IntToStr(PixelX) + ' Y:'+
                  IntToStr(PixelY));
end; {procedure TStatusLineObj.MousePixels}

{----------------------------------------------------------------------}

procedure TStatusLineObj.MouseCartesian2D(View2DZoomObj : TView2DZoomObj;
                                          PixelX, PixelY : longint);
begin {procedure TStatusLineObj.MouseCartesian2D}
  with View2DZoomObj do
    DrawMouseStatus('Mouse X:'+
               StatusLineFloatToStr(PixelX/XViewZoom)+' Y:'+
               StatusLineFloatToStr(PixelY/YViewZoom));
end; {procedure TStatusLineObj.MouseCartesian2D}

{----------------------------------------------------------------------}

procedure TStatusLineObj.MouseCartesianPolar(ViewPolarZoomObj : TViewPolarZoomObj;
                                             PixelX, PixelY : longint);
begin {procedure TStatusLineObj.MouseCartesianPolar}
  with ViewPolarZoomObj do
    DrawMouseStatus('Mouse X:'+
               StatusLineFloatToStr(PixelX/ViewZoom)+' Y:'+
               StatusLineFloatToStr(PixelY/ViewZoom));
end; {procedure TStatusLineObj.MouseCartesianPolar}

{----------------------------------------------------------------------}

procedure TStatusLineObj.MousePolDeg2D(View2DZoomObj : TView2DZoomObj;
                                       PixelX, PixelY : longint);
var
  PR, PTheta : TMathFloat;
begin {procedure TStatusLineObj.MousePolDeg2D}
  with View2DZoomObj do
  ConvCartesianToPolar((PixelX/XViewZoom),
                         (PixelY/YViewZoom), PR, PTheta);
  DrawMouseStatus('Mouse R:'+
     StatusLineFloatToStr(PR)+
     ' T:' + StatusLineFloatToStr(PTheta*(180/pi)) + 'd');
end; {procedure TStatusLineObj.MousePolDeg2D}

{----------------------------------------------------------------------}

procedure TStatusLineObj.MousePolDegPolar(ViewPolarZoomObj : TViewPolarZoomObj;
                                          PixelX, PixelY : longint);
var
  PR, PTheta : TMathFloat;
begin {procedure TStatusLineObj.MousePolDegPolar}
  with ViewPolarZoomObj do
  ConvCartesianToPolar((PixelX/ViewZoom),
                         (PixelY/ViewZoom), PR, PTheta);
  DrawMouseStatus('Mouse R:'+
     StatusLineFloatToStr(PR)+
     ' T:' + StatusLineFloatToStr(PTheta*(180/pi)) + 'd');
end; {procedure TStatusLineObj.MousePolDegPolar}

{----------------------------------------------------------------------}

procedure TStatusLineObj.MousePolRad2D(View2DZoomObj : TView2DZoomObj;
                                       PixelX, PixelY : longint);
var
  PR, PTheta : TMathFloat;
begin {procedure TStatusLineObj.MousePolRad2D}
  with View2DZoomObj do
    ConvCartesianToPolar((PixelX/XViewZoom),
                           (PixelY/YViewZoom), PR, PTheta);
  DrawMouseStatus('Mouse R:'+
     StatusLineFloatToStr(PR)+
     ' T:' + StatusLineFloatToStr(PTheta) + 'r');
end; {procedure TStatusLineObj.MousePolRad2D}

{----------------------------------------------------------------------}

procedure TStatusLineObj.MousePolRadPolar(ViewPolarZoomObj : TViewPolarZoomObj;
                                          PixelX, PixelY : longint);
var
  PR, PTheta : TMathFloat;
begin {procedure TStatusLineObj.MousePolRadPolar}
with ViewPolarZoomObj do
  ConvCartesianToPolar((PixelX/ViewZoom),
                         (PixelY/ViewZoom), PR, PTheta);
  DrawMouseStatus('Mouse R:'+
     StatusLineFloatToStr(PR)+
     ' T:' + StatusLineFloatToStr(PTheta) + 'r');
end; {procedure TStatusLineObj.MousePolRadPolar}

{----------------------------------------------------------------------}

procedure TStatusLineObj.MouseBmp(MouseX, MouseY : integer);
begin {procedure TStatusLineObj.MouseBmp}
  DrawMouseStatus('Mouse W:'+ IntToStr(MouseX)+ ' H:'+ IntToStr(MouseY));
end; {procedure TStatusLineObj.MouseBmp}

{----------------------------------------------------------------------}

procedure TStatusLineObj.BlankStatusLine;
begin {procedure TStatusLineObj.BlankStatusLine;}
  try
    DrawZoomStatus('');
    DrawCenterStatus('');
    DrawMouseStatus('');
  except
    PrevZoomStr := '';
    PrevCenterStr := '';
    PrevMouseStr := '';
  end; {try..except}
end; {procedure TStatusLineObj.BlankStatusLine;}

{----------------------------------------------------------------------}

procedure TStatusLineObj.UpDate2DStatusLine(View2DZoomObj : TView2DZoomObj;
                                            BmpWidth, BmpHeight : integer;
                                            MouseX, MouseY : integer);
var
  PixX, PixY : longint;
begin {procedure TStatusLineObj.UpDate2DStatusLine}
  try
    DrawZoomStatus('Zoom X:' + Float2DZoomToStrF(View2DZoomObj.XViewZoom) +
                       ' Y:' + Float2DZoomToStrF(View2DZoomObj.YViewZoom));
    case CenterFormat2D of
      vcCartesian : CenterCartesian2D(View2DZoomObj);
      vcDegrees : CenterPolDeg2D(View2DZoomObj);
      vcRadians : CenterPolRad2D(View2DZoomObj);
      vcPixels : CenterPixels2D(View2DZoomObj);
      else
        DrawCenterStatus('');
    end; {case}
    if (MouseX <= -1) or (MouseY <= -1) or
       (MouseX >= BmpWidth) or (MouseY >= BmpHeight) then
      DrawMouseStatus('')
    else
      begin
        PixX := (MouseX -((BMPWidth div 2)-1))+
                 Trunc(View2DZoomObj.ViewCenterX);
        PixY := (((BMPHeight div 2)-1) - MouseY)+
                 Trunc(View2DZoomObj.ViewCenterY);
        case MouseFormat2D of
          mpCartesian : MouseCartesian2D(View2DZoomObj, PixX, PixY);
          mpDegrees : MousePolDeg2D(View2DZoomObj, PixX, PixY);
          mpRadians : MousePolRad2D(View2DZoomObj, PixX, PixY);
          mpPixels : MousePixels(PixX, PixY);
          mpBitMap : MouseBmp(MouseX, MouseY);
          else
            DrawMouseStatus('');
        end; {case}
      end; {else}
  except
    BlankStatusLine;
  end; {try..except}
end; {procedure TStatusLineObj.UpDate2DStatusLine}

{----------------------------------------------------------------------}

procedure TStatusLineObj.UpDatePolarStatusLine(ViewPolarZoomObj : TViewPolarZoomObj;
                                               BmpWidth, BmpHeight : integer;
                                               MouseX, MouseY : integer);
var
  PixX, PixY : longint;
begin {procedure TStatusLineObj.UpDatePolarStatusLine}
  try
    DrawZoomStatus('Zoom: ' + Float2DZoomToStrF(ViewPolarZoomObj.ViewZoom));
    case CenterFormatPolar of
      vcCartesian : CenterCartesianPolar(ViewPolarZoomObj);
      vcDegrees : CenterPolDegPolar(ViewPolarZoomObj);
      vcRadians : CenterPolRadPolar(ViewPolarZoomObj);
      vcPixels : CenterPixelsPolar(ViewPolarZoomObj);
      else
        DrawCenterStatus('');
    end; {case}
    if (MouseX <= -1) or (MouseY <= -1) or
       (MouseX >= BmpWidth) or (MouseY >= BmpHeight) then
      DrawMouseStatus('')
    else
      begin
        PixX := (MouseX -((BMPWidth div 2)-1))+
                 Trunc(ViewPolarZoomObj.ViewCenterX);
        PixY := (((BMPHeight div 2)-1) - MouseY)+
                 Trunc(ViewPolarZoomObj.ViewCenterY);
        case MouseFormatPolar of
          mpCartesian : MouseCartesianPolar(ViewPolarZoomObj, PixX, PixY);
          mpDegrees : MousePolDegPolar(ViewPolarZoomObj, PixX, PixY);
          mpRadians : MousePolRadPolar(ViewPolarZoomObj, PixX, PixY);
          mpPixels : MousePixels(PixX, PixY);
          mpBitMap : MouseBmp(MouseX, MouseY);
          else
            DrawMouseStatus('');
        end; {case}
      end; {else}
  except
    BlankStatusLine;
  end; {try..except}
end; {procedure TStatusLineObj.UpDatePolarStatusLine}

{----------------------------------------------------------------------}

procedure TStatusLineObj.UpDate3DStatusLine(View3DZoomObj : TView3DZoomObj;
                                            BmpWidth, BmpHeight : integer;
                                            MouseX, MouseY : integer);
var
  PixX, PixY : longint;
begin {procedure TStatusLineObj.UpDate3DStatusLine}
  try
    DrawZoomStatus('Zoom: ' + IntToStr(View3DZoomObj.ViewZoom));
    with View3DZoomObj do
      DrawCenterStatus('Rotate X:'+
         StatusLineFloatToStr(RotateX)+
         ' Y:' + StatusLineFloatToStr(RotateY)+
         ' Z:' + StatusLineFloatToStr(RotateZ));
    if (MouseX <= -1) or (MouseY <= -1) or
       (MouseX >= BmpWidth) or (MouseY >= BmpHeight) then
      DrawMouseStatus('')
    else
      begin
        if MouseFormat3D = m3DontShow then
          DrawMouseStatus('')
        else
          begin
            PixX := (MouseX -((BMPWidth div 2)-1));
            PixY := (((BMPHeight div 2)-1) - MouseY);
            case MouseFormat3D of
              m3Cartesian : DrawMouseStatus('Mouse X:'+
                      StatusLineFloatToStr(PixX/View3DZoomObj.ViewZoom)+' Y:'+
                      StatusLineFloatToStr(PixY/View3DZoomObj.ViewZoom));
              m3Pixels : MousePixels(PixX, PixY);
              m3BitMap : MouseBmp(MouseX, MouseY);
              else
                DrawMouseStatus('');
            end; {case}
          end; {if}
      end; {else}
  except
    BlankStatusLine;
  end; {try..except}
end; {procedure TStatusLineObj.UpDate3DStatusLine}

{----------------------------------------------------------------------}

end.
